/**
   ##############################################################################
   ##                                                                          ##
   ## Appearance3DChooser                                                      ##
   ##                                                                          ##
   ## Copyright (C) 2009  Frederic Roudaut  <frederic.roudaut@free.fr>         ##
   ##                                                                          ##
   ##                                                                          ##
   ## This program is free software: you can redistribute it and/or modify     ##
   ## it under the terms of the GNU General Public License as published by     ##
   ## the Free Software Foundation, either version 3 of the License, or        ##
   ## (at your option) any later version.                                      ##
   ##                                                                          ##
   ## This program is distributed in the hope that it will be useful,          ##
   ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
   ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
   ## GNU General Public License for more details.                             ##
   ##                                                                          ##
   ## You should have received a copy of the GNU General Public License        ##
   ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
   ##                                                                          ##
   ##                                                                          ##
   ##############################################################################
 **/

package com.appearance3Dchooser;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.io.*;
import javax.swing.JApplet;
import javax.swing.JOptionPane;


/**
 * Applet Example for Appearence3DChooser.
 * 
 */
public class Appearance3DChooserApplet extends JApplet 
{

	private static final long serialVersionUID = 1L;

	/*
      public static void main(final String[] args) 
      {
      final Appearance3DChooserApplet demo = new Appearance3DChooserApplet();
      demo.setVisible(true);
      }
	 */

	/**
	 * Called when the Applet is first loaded.
	 * 
	 */
	public void init ()
	{

		// if loadDLL is set to true, then it means that the dynamic library has to be loaded 
		// into the client JRE Path. In this case the associated jar has to be signed first. 
		// Used to Automaticaly copy the DLL in the java path. Nevertheless they have to be in 
		// in a root dll directory
		if (getParameter("loadDLL") != null && getParameter("loadDLL").compareTo("true") == 0)
		{
			// load Windows DLL for Java3D 
			if(System.getProperty("os.name").indexOf("Win") != -1)
			{
				copyResourceFromJarToBinDir("j3dcore-d3d.dll");
				copyResourceFromJarToBinDir("j3dcore-ogl.dll");
				copyResourceFromJarToBinDir("j3dcore-ogl-cg.dll");
				copyResourceFromJarToBinDir("j3dcore-ogl-chk.dll");
			}        	
			else // Consider it is Linux		    
			{
				copyResourceFromJarToBinDir("libj3dcore-ogl.so");
				copyResourceFromJarToBinDir("libj3dcore-ogl-cg.so");			
			}        	
		}

		// if askRenderer is set to true, then it means that a popup will we run prior
		// in order to set the Renderer
		if (getParameter("askRenderer") != null && getParameter("askRenderer").compareTo("true") == 0)
		{

			if (System.getProperty("os.name").indexOf("Win") != -1) 
			{ 			
				String[] options = {"OpenGL","Direct3D"}; 
				String rend = "DEFAULT";
				if(System.getProperty("j3d.rend")!=null)
					rend = System.getProperty("j3d.rend");
				{
					if(rend.compareTo("ogl") == 0)
						rend = options[0];
					else if (rend.compareTo("d3d") == 0)
						rend = options[1];
				}

				int i = JOptionPane.showOptionDialog(null,
						Info.ressources.getObject("3D_Motor_choice_1").toString() + " : " + rend + "\n" +
						Info.ressources.getObject("3D_Motor_choice_2").toString(),				
						Info.ressources.getObject("3D_Motor_choice_3").toString(),
						JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE,
						null,
						options,
						options[0]); // OpenGL by default
				if (i == 0) { // OpenGL
					System.setProperty("j3d.rend", "ogl");
				} else 
				{ // Direct3D
					System.setProperty("j3d.rend","d3d");				
				}
			}
		}


		final Appearance3DChooser m = new Appearance3DChooser("test", true, true, true, true);

		m.toBack();
		m.setVisible(false);
		setContentPane(m.getRootPane());	

		m.runListeners();
		m.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) 
			{
				if (evt.getPropertyName().compareTo(Appearance3DChooser.SPECULAR) == 0)
				{
					System.out.println("SPECULAR : (" + m.getSpecularColor()[0]+ "," 
							+ m.getSpecularColor()[1]+ "," 
							+ m.getSpecularColor()[2]+")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.AMBIENT) == 0)
				{
					System.out.println("AMBIENT : (" + m.getAmbientColor()[0]+ "," 
							+ m.getAmbientColor()[1] + "," 
							+ m.getAmbientColor()[2]+")");		    
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.EMISSIVE) == 0)
				{
					System.out.println("EMISSIVE : (" + m.getEmissiveColor()[0]+ "," 
							+ m.getEmissiveColor()[1] + "," 
							+ m.getEmissiveColor()[2]+")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.DIFFUSE) == 0)
				{
					System.out.println("DIFFUSE : (" + m.getDiffuseColor()[0] + "," 
							+ m.getDiffuseColor()[1] + "," 
							+ m.getDiffuseColor()[2]+")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.COLORTARGET) == 0)
				{
					System.out.println("COLORTARGET : (" + m.getColorTarget() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.SHININESS) == 0)
				{
					System.out.println("SHININESS : (" + m.getShininess() + ")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.LIGHTING) == 0)
				{
					System.out.println("LIGHTING : (" + m.getLighting() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TRANSPARENCY) == 0)
				{
					System.out.println("TRANSPARENCY : (" + m.getTransparency() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TRANSPARENCY_MODE) == 0)
				{
					System.out.println("TRANSPARENCY_MODE : (" + m.getTransparencyMode() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TRANSPARENCY_SRC_BLEND_FUNCTION) == 0)
				{
					System.out.println("TRANSPARENCY_SRC_BLEND_FUNCTION : (" + m.getTransparencySrcBlendFunction() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TRANSPARENCY_DST_BLEND_FUNCTION) == 0)
				{
					System.out.println("TRANSPARENCY_DST_BLEND_FUNCTION : (" + m.getTransparencyDstBlendFunction() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_USE) == 0)
				{
					System.out.println("TEXTURE_USE : (" + m.isTextureSet() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_FILE) == 0)
				{
					System.out.println("TEXTURE_FILE : (" + m.getTextureFile() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_TRANSFORM) == 0)
				{
					System.out.println("TEXTURE_TRANSFORM : \n" + m.getTextureTransform() );
					System.out.println("Scale : x=" + m.getTextureTransformScaleX() + ", y=" +m.getTextureTransformScaleY());
					System.out.println("Translation : x=" + m.getTextureTransformTranslatX() + ", y=" +m.getTextureTransformTranslatY());
					System.out.println("Rotation : z=" + m.getTextureTransformRotZ());		    
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_PERSPECTIVE_CORRECTION_MODE) == 0)
				{
					System.out.println("TEXTURE_PERSPECTIVE_CORRECTION_MODE : (" + m.getTexturePerspectiveCorrectionMode() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_MODE) == 0)
				{
					System.out.println("TEXTURE_MODE : (" + m.getTextureMode() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_BLEND) == 0)
				{
					System.out.println("TEXTURE_BLEND : (" + 
							m.getTextureBlendColor()[0] + "," +
							m.getTextureBlendColor()[1] + "," +
							m.getTextureBlendColor()[2] + "," +
							m.getTextureBlendColor()[3] 
							                         + ")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_MODE) == 0)
				{
					System.out.println("TEXTURE_COMBINE_RGB_MODE : (" + m.getTextureCombineRGBMode() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SOURCE) == 0)
				{
					System.out.println("TEXTURE_COMBINE_RGB_SOURCE : (" + 
							m.getTextureCombineRGBSource()[0] + "," +
							m.getTextureCombineRGBSource()[1] + "," +
							m.getTextureCombineRGBSource()[2] 
							                               + ")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_FUNCTION) == 0)
				{
					System.out.println("TEXTURE_COMBINE_RGB_FUNCTION : (" + 
							m.getTextureCombineRGBFunction()[0] + "," +
							m.getTextureCombineRGBFunction()[1] + "," +
							m.getTextureCombineRGBFunction()[2] 
							                                 + ")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SCALE) == 0)
				{
					System.out.println("TEXTURE_COMBINE_RGB_SCALE : (" + m.getTextureCombineRGBScale() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_MODE) == 0)
				{
					System.out.println("TEXTURE_COMBINE_ALPHA_MODE : (" + m.getTextureCombineAlphaMode() +")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SOURCE) == 0)
				{
					System.out.println("TEXTURE_COMBINE_ALPHA_SOURCE : (" + 
							m.getTextureCombineAlphaSource()[0] + "," +
							m.getTextureCombineAlphaSource()[1] + "," +
							m.getTextureCombineAlphaSource()[2]
							                                 + ")");
				}		    
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_FUNCTION) == 0)
				{
					System.out.println("TEXTURE_COMBINE_ALPHA_FUNCTION : (" + 
							m.getTextureCombineAlphaFunction()[0] + "," +
							m.getTextureCombineAlphaFunction()[1] + "," +
							m.getTextureCombineAlphaFunction()[2]
							                                   + ")");
				}
				else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SCALE) == 0)
				{
					System.out.println("TEXTURE_COMBINE_ALPHA_SCALE : (" + m.getTextureCombineAlphaScale() +")");
				}		    

			}});	


		// Set Values and reinit the defaults one since it leads to a 
		// problem because of the mouse focus from JFrame	
		m.setEmissiveColor(0f,0f,0f,false);
		m.setSpecularColor(0f,0f,0f,false);
		m.setAmbientColor(0f,0f,0f,false);
		m.setDiffuseColor(0f,0f,0f,false);
		m.setTextureBlendColor(0f,0f,0f,0f,false);

		m.setInitParameters();


	}


	/**
	 * Used to automaticaly download a DLL on the client Host.
	 * @param tDll the DLL to download. It Must be in a Root dll directory.
	 * 
	 */
	protected void copyResourceFromJarToBinDir(String tDll)  // Name of the DLL
	{
		try 
		{
			//InputStream dllInputStream = getClass().getResourceAsStream("/dll/"+tDll);
			InputStream dllInputStream = new FileInputStream("dll\\"+tDll);

			String javaHome = System.getProperty("java.home");
			String fileSeparator = "" + System.getProperty("file.separator").charAt(0);
			File outFile = null;

			BufferedInputStream  binStream  = new BufferedInputStream(dllInputStream);
			outFile = new File(javaHome+fileSeparator+"bin"+fileSeparator+tDll);

			BufferedOutputStream boutStream = new BufferedOutputStream(new FileOutputStream(outFile));
			int b = -1;
			while((b = binStream.read()) != -1) 
			{ 
				boutStream.write(b);
			}
			binStream.close();
			binStream.close();
			boutStream.flush();
			boutStream.close();

			System.out.println("Copy " + tDll + " into " + outFile.getPath()); 
		} 

		catch(Exception e) 
		{
			System.err.println("Error : cannot download " + tDll); 
			e.printStackTrace();
		}
	}

}