/**
   ##############################################################################
   ##                                                                          ##
   ## Appearance3DChooser                                                      ##
   ##                                                                          ##
   ## Copyright (C) 2009  Frederic Roudaut  <frederic.roudaut@free.fr>         ##
   ##                                                                          ##
   ##                                                                          ##
   ## This program is free software: you can redistribute it and/or modify     ##
   ## it under the terms of the GNU General Public License as published by     ##
   ## the Free Software Foundation, either version 3 of the License, or        ##
   ## (at your option) any later version.                                      ##
   ##                                                                          ##
   ## This program is distributed in the hope that it will be useful,          ##
   ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
   ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
   ## GNU General Public License for more details.                             ##
   ##                                                                          ##
   ## You should have received a copy of the GNU General Public License        ##
   ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
   ##                                                                          ##
   ##                                                                          ##
   ##############################################################################
**/



package com.appearance3Dchooser;

import javax.swing.*; 
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Font;
import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.BorderLayout;
import java.awt.event.*;

import javax.vecmath.Matrix4d;   
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransparencyAttributes;
import javax.media.j3d.TextureAttributes;
import javax.media.j3d.Material;

import com.awt.GridBagLayoutConstraint;


/**
 * Create a Frame with all the Parameters Values 
 * It will dynamically update them every 500ms.
 *
 */
public class Appearance3DChooserInfo
{	  
    private static final long serialVersionUID = 1L;

    protected JLabel entryTextSPECULAR = null;
    protected JLabel entryTextAMBIENT = null;
    protected JLabel entryTextEMISSIVE = null;
    protected JLabel entryTextDIFFUSE = null;
    protected JLabel entryTextCOLORTARGET = null;
    protected JLabel entryTextSHININESS = null;
    protected JLabel entryTextLIGHTING = null;
    protected JLabel entryTextTRANSPARENCY = null;
    protected JLabel entryTextTRANSPARENCY_MODE = null;
    protected JLabel entryTextTRANSPARENCY_SRC_BLEND_FUNCTION = null;
    protected JLabel entryTextTRANSPARENCY_DST_BLEND_FUNCTION = null;
    protected JLabel entryTextTEXTURE_USE = null;
    protected JLabel entryTextTEXTURE_FILE = null;
    protected JLabel entryTextTEXTURE_TRANSFORM1 = null;
    protected JLabel entryTextTEXTURE_TRANSFORM2 = null;
    protected JLabel entryTextTEXTURE_TRANSFORM3 = null;
    protected JLabel entryTextRotation = null;
    protected JLabel entryTextScale = null;
    protected JLabel entryTextTranslation = null;
    protected JLabel entryTextTEXTURE_PERSPECTIVE_CORRECTION_MODE = null;
    protected JLabel entryTextTEXTURE_MODE = null;
    protected JLabel entryTextTEXTURE_BLEND = null;
    protected JLabel entryTextTEXTURE_COMBINE_RGB_MODE = null;
    protected JLabel entryTextTEXTURE_COMBINE_RGB_SOURCE = null;
    protected JLabel entryTextTEXTURE_COMBINE_RGB_FUNCTION = null;
    protected JLabel entryTextTEXTURE_COMBINE_RGB_SCALE = null;
    protected JLabel entryTextTEXTURE_COMBINE_ALPHA_MODE = null;
    protected JLabel entryTextTEXTURE_COMBINE_ALPHA_SOURCE = null;
    protected JLabel entryTextTEXTURE_COMBINE_ALPHA_FUNCTION = null;
    protected JLabel entryTextTEXTURE_COMBINE_ALPHA_SCALE = null;

    protected  Appearance3DChooser app;

    /**
     * Constructor that draw a Frame with all the Appearance3DChooser Parameters values.
     * @param appear the Appearance3DChooser Object
     *
     */      
    public Appearance3DChooserInfo(Appearance3DChooser appear)
    {
	this(appear, new Dimension(900,520));
    }


    /**
     * Constructor that draw a Frame with all the Appearance3DChooser Parameters values.
     * @param appear the Appearance3DChooser Object
     * @param d the text Panel Dimension
     *
     */      
    public Appearance3DChooserInfo(Appearance3DChooser appear, Dimension d)
    {
	this.app = appear;

	/**
	 * Class used to update the Parameters Info every 500ms.
	 *
	 */       
	class updateAppearance3DChooserInfo extends Thread 
	{
	    	    	    
	    /**
	     * Start a thread for updating Appareance3DChooser Parameters Info every 500ms.
	     *
	     */       
	    public void run()
	    {		
		while(true)
		    {
			try {
			    Thread.sleep(500);
			}
			catch (Exception e)
			    {
			    }
			
			updateAppearance3DChooserInfo();
			
		    }
	    }
	}

	final updateAppearance3DChooserInfo updtInfo = new updateAppearance3DChooserInfo();

	final JFrame parametersInfoBox = new JFrame(Info.ressources.getObject("Parameters_Info").toString());
	parametersInfoBox.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);	
	parametersInfoBox.setVisible(true);

	JPanel parametersInfoBoxPanel = new JPanel(new BorderLayout());
	GridBagLayout gbmain = new GridBagLayout();
	parametersInfoBoxPanel.setLayout(gbmain);
	parametersInfoBox.getContentPane().add(parametersInfoBoxPanel, BorderLayout.CENTER);	
			
	JPanel textPanel = new JPanel();
	GridBagLayout gbtextPanel = new GridBagLayout();	
	textPanel.setLayout(gbtextPanel);
	JScrollPane textScrolled = new JScrollPane(textPanel);	
	textScrolled.setPreferredSize(d);   
		
	JLabel entryText;
	GridBagLayout gb;	

	// Create Material Panel
	JPanel materialPanel = new JPanel();
	materialPanel.setBorder(BorderFactory.createTitledBorder(Info.ressources.getObject("Material").toString()));
	gb = new GridBagLayout();
	materialPanel.setLayout(gb);	

	entryText = new JLabel("EMISSIVE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	materialPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,1,
						      new Insets(0,8,0,15)));

	entryTextEMISSIVE = new JLabel();
	entryTextEMISSIVE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	materialPanel.add(entryTextEMISSIVE);
	gb.setConstraints(entryTextEMISSIVE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,1,
							      new Insets(0,8,0,15)));

	entryText = new JLabel("AMBIENT");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	materialPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,2,
						      new Insets(0,8,0,15)));

	entryTextAMBIENT = new JLabel();
	entryTextAMBIENT.setFont(new Font("SansSerif", Font.PLAIN, 10));
	materialPanel.add(entryTextAMBIENT);
	gb.setConstraints(entryTextAMBIENT, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,2,
							     new Insets(0,8,0,15)));

	entryText = new JLabel("DIFFUSE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	materialPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,3,
						      new Insets(0,8,0,15)));
	
	entryTextDIFFUSE = new JLabel();
	entryTextDIFFUSE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	materialPanel.add(entryTextDIFFUSE);
	gb.setConstraints(entryTextDIFFUSE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,3,
							     new Insets(0,8,0,15)));
			
	entryText = new JLabel("SPECULAR");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	materialPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,4,
						      new Insets(0,8,0,15)));
	
	entryTextSPECULAR = new JLabel();
	entryTextSPECULAR.setFont(new Font("SansSerif", Font.PLAIN, 10));
	materialPanel.add(entryTextSPECULAR);
	gb.setConstraints(entryTextSPECULAR, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,4,
							      new Insets(0,8,0,15)));				
       
	entryText = new JLabel("LIGHTING");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	materialPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,5,
						      new Insets(0,8,0,15)));
			
	entryTextLIGHTING = new JLabel();
	entryTextLIGHTING.setFont(new Font("SansSerif", Font.PLAIN, 10));
	materialPanel.add(entryTextLIGHTING);
	gb.setConstraints(entryTextLIGHTING, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,5,
							      new Insets(0,8,0,15)));

	entryText = new JLabel("SHININESS");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	materialPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,6,
						      new Insets(0,8,0,15)));
			
	entryTextSHININESS = new JLabel();
	entryTextSHININESS.setFont(new Font("SansSerif", Font.PLAIN, 10));
	materialPanel.add(entryTextSHININESS);
	gb.setConstraints(entryTextSHININESS, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,6,
							       new Insets(0,8,0,15)));
			
	entryText = new JLabel("COLORTARGET");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	materialPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,7,
						      new Insets(0,8,0,15)));
			
	entryTextCOLORTARGET = new JLabel();
	entryTextCOLORTARGET.setFont(new Font("SansSerif", Font.PLAIN, 10));
	materialPanel.add(entryTextCOLORTARGET);
	gb.setConstraints(entryTextCOLORTARGET, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,7,
								 new Insets(0,8,0,15)));
	
	// Add Material Panel
	textPanel.add(materialPanel);	
	gbtextPanel.setConstraints(materialPanel, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,0,
								   new Insets(0,15,0,0)));

	// Create Transparency Panel
	JPanel transparencyPanel = new JPanel();
	transparencyPanel.setBorder(BorderFactory.createTitledBorder(Info.ressources.getObject("Transparency").toString()));
	gb = new GridBagLayout();
	transparencyPanel.setLayout(gb);	

	entryText = new JLabel("TRANSPARENCY_MODE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	transparencyPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,8,
						      new Insets(0,8,0,15)));
	
	entryTextTRANSPARENCY_MODE = new JLabel();
	entryTextTRANSPARENCY_MODE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	transparencyPanel.add(entryTextTRANSPARENCY_MODE);
	gb.setConstraints(entryTextTRANSPARENCY_MODE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,8,
								       new Insets(0,8,0,15)));
		
	entryText = new JLabel("TRANSPARENCY_SRC_BLEND_FUNCTION");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	transparencyPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,9,
						      new Insets(0,8,0,15)));
			
	entryTextTRANSPARENCY_SRC_BLEND_FUNCTION = new JLabel();
	entryTextTRANSPARENCY_SRC_BLEND_FUNCTION.setFont(new Font("SansSerif", Font.PLAIN, 10));
	transparencyPanel.add(entryTextTRANSPARENCY_SRC_BLEND_FUNCTION);
	gb.setConstraints(entryTextTRANSPARENCY_SRC_BLEND_FUNCTION, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,9,
										     new Insets(0,8,0,15)));

	entryText = new JLabel("TRANSPARENCY_DST_BLEND_FUNCTION");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	transparencyPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,10,
						      new Insets(0,8,0,15)));
	
	entryTextTRANSPARENCY_DST_BLEND_FUNCTION = new JLabel();
	entryTextTRANSPARENCY_DST_BLEND_FUNCTION.setFont(new Font("SansSerif", Font.PLAIN, 10));
	transparencyPanel.add(entryTextTRANSPARENCY_DST_BLEND_FUNCTION);
	gb.setConstraints(entryTextTRANSPARENCY_DST_BLEND_FUNCTION, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,10,
										     new Insets(0,8,0,15)));

	entryText = new JLabel("TRANSPARENCY");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	transparencyPanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,7,
						      new Insets(0,8,0,15)));
	
	entryTextTRANSPARENCY = new JLabel();
	entryTextTRANSPARENCY.setFont(new Font("SansSerif", Font.PLAIN, 10));
	transparencyPanel.add(entryTextTRANSPARENCY);
	gb.setConstraints(entryTextTRANSPARENCY, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,7,
								  new Insets(0,8,0,15)));

	// Add Transparency Panel
	textPanel.add(transparencyPanel);	
	gbtextPanel.setConstraints(transparencyPanel, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,0,
								       new Insets(0,15,0,0)));

	// Create Texture Panel
	JPanel texturePanel = new JPanel();
	texturePanel.setBorder(BorderFactory.createTitledBorder(Info.ressources.getObject("Texture").toString()));
	gb = new GridBagLayout();
	texturePanel.setLayout(gb);	

	entryText = new JLabel("TEXTURE_USE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,11,
						      new Insets(0,8,0,15)));

	entryTextTEXTURE_USE = new JLabel();
	entryTextTEXTURE_USE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_USE);
	gb.setConstraints(entryTextTEXTURE_USE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,11,
								 new Insets(0,8,0,15)));

	entryText = new JLabel("TEXTURE_FILE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,12,
						      new Insets(0,8,0,15)));
	
	entryTextTEXTURE_FILE = new JLabel();
	entryTextTEXTURE_FILE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_FILE);
	gb.setConstraints(entryTextTEXTURE_FILE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,12,
								  new Insets(0,8,0,15)));
	
	entryText = new JLabel("TEXTURE_TRANSFORM");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,13,
						      new Insets(0,8,0,15)));
	
	entryTextTEXTURE_TRANSFORM1 = new JLabel();
	entryTextTEXTURE_TRANSFORM1.setFont(new Font("Courier", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_TRANSFORM1);
	gb.setConstraints(entryTextTEXTURE_TRANSFORM1, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,13,
									new Insets(0,8,0,15)));
	entryTextTEXTURE_TRANSFORM2 = new JLabel();   
	entryTextTEXTURE_TRANSFORM2.setFont(new Font("Courier", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_TRANSFORM2);
	gb.setConstraints(entryTextTEXTURE_TRANSFORM2, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,14,
									new Insets(0,8,0,15)));

	entryTextTEXTURE_TRANSFORM3 = new JLabel();
	entryTextTEXTURE_TRANSFORM3.setFont(new Font("Courier", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_TRANSFORM3);
	gb.setConstraints(entryTextTEXTURE_TRANSFORM3, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,15,
									new Insets(0,8,0,15)));

	entryTextRotation = new JLabel();
	entryTextRotation.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextRotation);
	gb.setConstraints(entryTextRotation, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,17,
							      new Insets(8,8,0,15)));
	
	entryTextScale = new JLabel();
	entryTextScale.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextScale);
	gb.setConstraints(entryTextScale, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,18,
							   new Insets(0,8,0,15)));
	
	entryTextTranslation = new JLabel();
	entryTextTranslation.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTranslation);
	gb.setConstraints(entryTextTranslation, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,19,
								 new Insets(0,8,0,15)));
			
	entryText = new JLabel("TEXTURE_PERSPECTIVE_CORRECTION_MODE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,20,
						      new Insets(0,8,0,15)));

	entryTextTEXTURE_PERSPECTIVE_CORRECTION_MODE = new JLabel();
	entryTextTEXTURE_PERSPECTIVE_CORRECTION_MODE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_PERSPECTIVE_CORRECTION_MODE);
	gb.setConstraints(entryTextTEXTURE_PERSPECTIVE_CORRECTION_MODE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,20,
											 new Insets(0,8,0,15)));

	entryText = new JLabel("TEXTURE_MODE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,21,
						      new Insets(0,8,0,15)));
	
	entryTextTEXTURE_MODE = new JLabel();
	entryTextTEXTURE_MODE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_MODE);
	gb.setConstraints(entryTextTEXTURE_MODE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,21,
								  new Insets(0,8,0,15)));

	entryText = new JLabel("TEXTURE_BLEND");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,22,
						      new Insets(0,8,0,15)));
			
	entryTextTEXTURE_BLEND = new JLabel();
	entryTextTEXTURE_BLEND.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_BLEND);
	gb.setConstraints(entryTextTEXTURE_BLEND, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,22,
								   new Insets(0,8,0,15)));

	entryText = new JLabel("TEXTURE_COMBINE_RGB_MODE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,23,
						      new Insets(0,8,0,15)));
			
	entryTextTEXTURE_COMBINE_RGB_MODE = new JLabel();
	entryTextTEXTURE_COMBINE_RGB_MODE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_COMBINE_RGB_MODE);
	gb.setConstraints(entryTextTEXTURE_COMBINE_RGB_MODE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,23,
									      new Insets(0,8,0,15)));


	entryText = new JLabel("TEXTURE_COMBINE_RGB_SOURCE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,24,
						      new Insets(0,8,0,15)));
						    
	entryTextTEXTURE_COMBINE_RGB_SOURCE = new JLabel();
	entryTextTEXTURE_COMBINE_RGB_SOURCE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_COMBINE_RGB_SOURCE);
	gb.setConstraints(entryTextTEXTURE_COMBINE_RGB_SOURCE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,24,
										new Insets(0,8,0,15)));


	entryText = new JLabel("TEXTURE_COMBINE_RGB_FUNCTION");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,25,
						      new Insets(0,8,0,15)));
				
	entryTextTEXTURE_COMBINE_RGB_FUNCTION = new JLabel();
	entryTextTEXTURE_COMBINE_RGB_FUNCTION.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_COMBINE_RGB_FUNCTION);
	gb.setConstraints(entryTextTEXTURE_COMBINE_RGB_FUNCTION, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,25,
										  new Insets(0,8,0,15)));



	entryText = new JLabel("TEXTURE_COMBINE_RGB_SCALE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,26,
						      new Insets(0,8,0,15)));
			
	entryTextTEXTURE_COMBINE_RGB_SCALE = new JLabel();
	entryTextTEXTURE_COMBINE_RGB_SCALE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_COMBINE_RGB_SCALE);
	gb.setConstraints(entryTextTEXTURE_COMBINE_RGB_SCALE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,26,
									       new Insets(0,8,0,15)));

	entryText = new JLabel("TEXTURE_COMBINE_ALPHA_MODE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,27,
						      new Insets(0,8,0,15)));
						    
 	entryTextTEXTURE_COMBINE_ALPHA_MODE = new JLabel();
	entryTextTEXTURE_COMBINE_ALPHA_MODE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_COMBINE_ALPHA_MODE);
	gb.setConstraints(entryTextTEXTURE_COMBINE_ALPHA_MODE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,27,
										new Insets(0,8,0,15)));

	entryText = new JLabel("TEXTURE_COMBINE_ALPHA_SOURCE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,28,
						      new Insets(0,8,0,15)));			
			    			    
	entryTextTEXTURE_COMBINE_ALPHA_SOURCE = new JLabel();
	entryTextTEXTURE_COMBINE_ALPHA_SOURCE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_COMBINE_ALPHA_SOURCE);
	gb.setConstraints(entryTextTEXTURE_COMBINE_ALPHA_SOURCE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,28,
										  new Insets(0,8,0,15)));

	entryText = new JLabel("TEXTURE_COMBINE_ALPHA_FUNCTION");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,29,
						      new Insets(0,8,0,15)));
			
	entryTextTEXTURE_COMBINE_ALPHA_FUNCTION = new JLabel();
	entryTextTEXTURE_COMBINE_ALPHA_FUNCTION.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_COMBINE_ALPHA_FUNCTION);
	gb.setConstraints(entryTextTEXTURE_COMBINE_ALPHA_FUNCTION, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,29,
										    new Insets(0,8,0,15)));

	entryText = new JLabel("TEXTURE_COMBINE_ALPHA_SCALE");
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	texturePanel.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,30,
						      new Insets(0,8,0,15)));			

 	entryTextTEXTURE_COMBINE_ALPHA_SCALE = new JLabel();
	entryTextTEXTURE_COMBINE_ALPHA_SCALE.setFont(new Font("SansSerif", Font.PLAIN, 10));
	texturePanel.add(entryTextTEXTURE_COMBINE_ALPHA_SCALE);
	gb.setConstraints(entryTextTEXTURE_COMBINE_ALPHA_SCALE, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,30,
										 new Insets(0,8,0,15)));

	//Add Texture Panel
	textPanel.add(texturePanel);	
	GridBagConstraints gc = new GridBagLayoutConstraint(GridBagConstraints.WEST,0,1,
						 new Insets(15,15,0,0));
	gc.gridwidth=2;
	gbtextPanel.setConstraints(texturePanel, gc);

	parametersInfoBoxPanel.add(textScrolled);
	gbmain.setConstraints(textScrolled, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
							     new Insets(8,8,8,8)));		
		    
	JButton okbutton = new JButton(Info.ressources.getObject("OK").toString());
	parametersInfoBoxPanel.add(okbutton);
	gbmain.setConstraints(okbutton, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,1,
							 new Insets(15,8,8,8)));
	okbutton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {		    
		    updtInfo.stop();
		    parametersInfoBox.setVisible(false);
		    parametersInfoBox.dispose();
		}
	    });
	 
	updtInfo.start();

	parametersInfoBox.pack();
	parametersInfoBox.setResizable(false);
	parametersInfoBox.setVisible(true);
	
    }


    /**
     * Function that updates all the parameters info.
     *
     */       
    protected void updateAppearance3DChooserInfo()
    {
	entryTextSPECULAR.setText("= " + "(" + app.getSpecularColor()[0]+ ", " 
				  + app.getSpecularColor()[1]+ ", " 
				  + app.getSpecularColor()[2]+")");
	
	entryTextAMBIENT.setText("= " + "(" + app.getAmbientColor()[0]+ ", " 
				 + app.getAmbientColor()[1]+ ", " 
				 + app.getAmbientColor()[2]+")");
	
	entryTextEMISSIVE.setText("= " + "(" + app.getEmissiveColor()[0]+ ", " 
				  + app.getEmissiveColor()[1]+ ", " 
				  + app.getEmissiveColor()[2]+")");
	
	entryTextDIFFUSE.setText("= " + "(" + app.getDiffuseColor()[0]+ ", " 
				 + app.getDiffuseColor()[1]+ ", " 
				 + app.getDiffuseColor()[2]+")");
			
	entryTextCOLORTARGET.setText("= " + colorTargetToString(app.getColorTarget()));

	entryTextSHININESS.setText("= " + app.getShininess());
			
	entryTextLIGHTING.setText("= " + app.getLighting());

	entryTextTRANSPARENCY.setText("= " + app.getTransparency());

	entryTextTRANSPARENCY_MODE.setText("= " + transparencyModeToString(app.getTransparencyMode()));
		
	entryTextTRANSPARENCY_SRC_BLEND_FUNCTION.setText("= " + transparencyBlendFunctionToString(app.getTransparencySrcBlendFunction()));

	entryTextTRANSPARENCY_DST_BLEND_FUNCTION.setText("= " + transparencyBlendFunctionToString(app.getTransparencyDstBlendFunction()));

	entryTextTEXTURE_USE.setText("= " + app.isTextureSet());

	entryTextTEXTURE_FILE.setText("= " + app.getTextureFile());
				
	Matrix4d m = new Matrix4d();
	Transform3D t3d = new Transform3D(app.getTextureTransform());
	t3d.get(m);

	String str="       "; 

	entryTextTEXTURE_TRANSFORM1.setText("= " +  
					    str.substring(String.valueOf((float)Math.round(m.getElement(0,0) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(0,0) * 100) / 100 + ", " +
					    str.substring(String.valueOf((float)Math.round(m.getElement(0,1) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(0,1) * 100) / 100 + ", " +
					    str.substring(String.valueOf((float)Math.round(m.getElement(0,2) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(0,2) * 100) / 100
					    );
	
	entryTextTEXTURE_TRANSFORM2.setText("  " +  
					    str.substring(String.valueOf((float)Math.round(m.getElement(1,0) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(1,0) * 100) / 100 + ", " +
					    str.substring(String.valueOf((float)Math.round(m.getElement(1,1) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(1,1) * 100) / 100 + ", " +
					    str.substring(String.valueOf((float)Math.round(m.getElement(1,2) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(1,2) * 100) / 100
					    );   
	
	entryTextTEXTURE_TRANSFORM3.setText("  " +  
					    str.substring(String.valueOf((float)Math.round(m.getElement(2,0) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(2,0) * 100) / 100 + ", " +
					    str.substring(String.valueOf((float)Math.round(m.getElement(2,1) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(2,1) * 100) / 100 + ", " +
					    str.substring(String.valueOf((float)Math.round(m.getElement(2,2) * 100) / 100).length()) 
					    + (float)Math.round(m.getElement(2,2) * 100) / 100
					    );
	
	entryTextRotation.setText("Rotation : z= " + app.getTextureTransformRotZ());
	
	entryTextScale.setText("Scale : x= " + app.getTextureTransformScaleX() + ", y= " +app.getTextureTransformScaleY());
	
	entryTextTranslation.setText("Translation : x= " + app.getTextureTransformTranslatX() + ", y= " +app.getTextureTransformTranslatY());
	
	entryTextTEXTURE_PERSPECTIVE_CORRECTION_MODE.setText("= " + texturePerspectiveCorrectionModeToString(app.getTexturePerspectiveCorrectionMode()));
	
	entryTextTEXTURE_MODE.setText("= " + textureModeToString(app.getTextureMode()));
			
	entryTextTEXTURE_BLEND.setText("= " + "(" + 
				       app.getTextureBlendColor()[0] + ", " +
				       app.getTextureBlendColor()[1] + ", " +
				       app.getTextureBlendColor()[2] + ", " +
				       app.getTextureBlendColor()[3] 
				       + ")");
			
	entryTextTEXTURE_COMBINE_RGB_MODE.setText("= " + textureCombineModeToString(app.getTextureCombineRGBMode()));
						    
	entryTextTEXTURE_COMBINE_RGB_SOURCE.setText("= " +"(" + 
						    textureCombineSourceToString(app.getTextureCombineRGBSource()[0]) + ", " +
						    textureCombineSourceToString(app.getTextureCombineRGBSource()[1]) + ", " +
						    textureCombineSourceToString(app.getTextureCombineRGBSource()[2]) 
						    + ")");
				
	entryTextTEXTURE_COMBINE_RGB_FUNCTION.setText("= " +"(" + 
						      textureCombineFunctionToString(app.getTextureCombineRGBFunction()[0]) + ", " +
						      textureCombineFunctionToString(app.getTextureCombineRGBFunction()[1]) + ", " +
						      textureCombineFunctionToString(app.getTextureCombineRGBFunction()[2]) 
						      + ")");
			
	entryTextTEXTURE_COMBINE_RGB_SCALE.setText("= " + app.getTextureCombineRGBScale());
						    
 	entryTextTEXTURE_COMBINE_ALPHA_MODE.setText("= " + textureCombineModeToString(app.getTextureCombineAlphaMode()));
			    			    
	entryTextTEXTURE_COMBINE_ALPHA_SOURCE.setText("= " + "(" + 
						      textureCombineSourceToString(app.getTextureCombineAlphaSource()[0]) + ", " +
						      textureCombineSourceToString(app.getTextureCombineAlphaSource()[1]) + ", " +
						      textureCombineSourceToString(app.getTextureCombineAlphaSource()[2])
						      + ")");
			
	entryTextTEXTURE_COMBINE_ALPHA_FUNCTION.setText("= " + "(" + 
							textureCombineFunctionToString(app.getTextureCombineAlphaFunction()[0]) + ", " +
							textureCombineFunctionToString(app.getTextureCombineAlphaFunction()[1]) + ", " +
							textureCombineFunctionToString(app.getTextureCombineAlphaFunction()[2])
							+ ")");
	
 	entryTextTEXTURE_COMBINE_ALPHA_SCALE.setText("= " + app.getTextureCombineAlphaScale());	
    }
  
    
    /**
     * Convert TransparencyAttributes Mode to String.
     * @return a String for the TransparencyAttributes Mode.
     */	
    public static String transparencyModeToString (int m)
    {
	switch (m)
	    {
	    case TransparencyAttributes.BLENDED:
		return "BLENDED";
	    case TransparencyAttributes.FASTEST:
		return "FASTEST";
	    case TransparencyAttributes.NICEST:
		return "NICEST"; 
	    case TransparencyAttributes.NONE:
		return "NONE";
	    case TransparencyAttributes.SCREEN_DOOR:
		return "SCREEN_DOOR";
	    default :
		return "-1";
	    }
    }


    /**
     * Convert TransparencyAttributes Source or Destination Blend Function to String.
     * @return the corresponding String.
     */	
    public static String transparencyBlendFunctionToString (int f)
    {
	switch (f)
	    {
	    case TransparencyAttributes.BLEND_ZERO:
		return "BLEND_ZERO";
	    case TransparencyAttributes.BLEND_ONE: 
		return "BLEND_ONE";
	    case TransparencyAttributes.BLEND_SRC_ALPHA:
		return "BLEND_SRC_ALPHA";
	    case TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA:
		return "BLEND_ONE_MINUS_SRC_ALPHA";
	    case TransparencyAttributes.BLEND_DST_COLOR:
		return "BLEND_DST_COLOR";
	    case TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR:
		return "BLEND_ONE_MINUS_DST_COLOR";
	    case TransparencyAttributes.BLEND_SRC_COLOR:
		return "BLEND_SRC_COLOR";
	    case TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR:
		return "BLEND_ONE_MINUS_SRC_COLOR";
	    default :
		return "-1";		
	    }
    }

    
    /**
     * Convert TextureAttributes Perspective Correction Mode to String.
     * @return the corresponding String.
     */	
    public static String texturePerspectiveCorrectionModeToString (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.NICEST:
		return "NICEST";
	    case TextureAttributes.FASTEST:
		return "FASTEST";	
	    default :
		return "-1";
	    }
    }


    /**
     * Convert TextureAttributes Mode to String.
     * @return the corresponding String.
     */	
    public static String textureModeToString (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.MODULATE:
		return "MODULATE";
	    case TextureAttributes.DECAL:
		return "DECAL";	
	    case TextureAttributes.BLEND:
		return "BLEND";	
	    case TextureAttributes.REPLACE:
		return "REPLACE";	
	    case TextureAttributes.COMBINE:
		return "COMBINE";	
	    default :
		return "-1";
	    }
    }


    /**
     * Convert TextureAttributes Combine Mode to String.
     * @return the corresponding String.
     */	
    public static String textureCombineModeToString (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.COMBINE_REPLACE:
		return "COMBINE_REPLACE";
	    case TextureAttributes.COMBINE_MODULATE:
		return "COMBINE_MODULATE";	
	    case TextureAttributes.COMBINE_ADD:
		return "COMBINE_ADD";	
	    case TextureAttributes.COMBINE_ADD_SIGNED:
		return "COMBINE_ADD_SIGNED";	
	    case TextureAttributes.COMBINE_SUBTRACT:
		return "COMBINE_SUBTRACT";	
	    case TextureAttributes.COMBINE_INTERPOLATE:
		return "COMBINE_INTERPOLATE";	
	    case TextureAttributes.COMBINE_DOT3:
		return "COMBINE_DOT3";	
	    default :
		return "-1";
	    }
    }


    /**
     * Convert TextureAttributes Combine Source to String.
     * @return the corresponding String.
     */	
    public static String textureCombineSourceToString (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.COMBINE_OBJECT_COLOR:
		return "COMBINE_OBJECT_COLOR";
	    case TextureAttributes.COMBINE_TEXTURE_COLOR:
		return "COMBINE_TEXTURE_COLOR";	
	    case TextureAttributes.COMBINE_CONSTANT_COLOR:
		return "COMBINE_CONSTANT_COLOR";	
	    case TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE:
		return "COMBINE_PREVIOUS_TEXTURE_UNIT_STATE";		    
	    default :
		return "-1";
	    }
    }


    /**
     * Convert TextureAttributes Combine Function to String.
     * @return the corresponding String.
     */	
    public static String textureCombineFunctionToString (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.COMBINE_SRC_COLOR:
		return "COMBINE_SRC_COLOR";
	    case TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR:
		return "COMBINE_ONE_MINUS_SRC_COLOR";	
	    case TextureAttributes.COMBINE_SRC_ALPHA:
		return "COMBINE_SRC_ALPHA";	
	    case TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA:
		return "COMBINE_ONE_MINUS_SRC_ALPHA";		    
	    default :
		return "-1";
	    }
    }


    /**
     * Convert Material Color Target to String.
     * @return the corresponding String.
     */	
    public static String colorTargetToString (int m)
    {
	switch (m)
	    {
	    case Material.AMBIENT:
		return "AMBIENT";
	    case Material.EMISSIVE:
		return "EMISSIVE";	
	    case Material.DIFFUSE:
		return "DIFFUSE";	
	    case Material.SPECULAR:
		return "SPECULAR";		
	    case Material.AMBIENT_AND_DIFFUSE:
		return "AMBIENT_AND_DIFFUSE";
	    default :
		return "-1";
	    }
    }

}