/**
   ##############################################################################
   ##                                                                          ##
   ## Appearance3DChooser                                                      ##
   ##                                                                          ##
   ## Copyright (C) 2009  Frederic Roudaut  <frederic.roudaut@free.fr>         ##
   ##                                                                          ##
   ##                                                                          ##
   ##                                                                          ##
   ## This program is free software: you can redistribute it and/or modify     ##
   ## it under the terms of the GNU General Public License as published by     ##
   ## the Free Software Foundation, either version 3 of the License, or        ##
   ## (at your option) any later version.                                      ##
   ##                                                                          ##
   ## This program is distributed in the hope that it will be useful,          ##
   ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
   ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
   ## GNU General Public License for more details.                             ##
   ##                                                                          ##
   ## You should have received a copy of the GNU General Public License        ##
   ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
   ##                                                                          ##
   ##                                                                          ##
   ##############################################################################
**/

package com.appearance3Dchooser;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.net.URL;

import javax.media.j3d.TransparencyAttributes;
import javax.media.j3d.TextureAttributes;
import javax.media.j3d.Material;
import javax.media.j3d.Transform3D;


/**
 *  Appearance3DChooser Class.<br/>
 *   This is perhaps the Class you want to use for updating your objects appearances. It includes :
 *   <ul>
 *   <li>A main part for updating following Java3D attributes on a defined object:
 *      <ul>
 *       <li><strong>Material</strong></li>
 *       <li><strong>TransparencyAttributes</strong></li>
 *       <li><strong>TextureAttributes</strong></li>
 *       <li><strong>Lighting</strong></li>
 *      </ul>
 *   </li>
 *   <li>A menu bar with some submenus for the Help, the java3D characteristics, the user info, the java info, the system info, the parameters values info, and a About Panel.</li>
 *   <li>three more buttons : one for OK, one for CANCEL, and one for RESET.</li>
 *   </ul>
 * <br/> An instance of this object fire a Property Event when a dedicated parameter is modified. If Cancel is clicked, all parameters are reseted and
 * associated Property Events are also fired. It distinguishes the following events :
 * <ul>
 * <br/>
 * <li>Material Panel 
 *     <ul>
 *     <li>SPECULAR </li>
 *     <li>AMBIENT </li>
 *     <li>EMISSIVE </li>
 *     <li>DIFFUSE </li>
 *     <li>COLORTARGET </li>
 *     <li>SHININESS  </li>
 *     <li>LIGHTING  </li>
 *     </ul>  
 *     <br/>
 *     </li>
 *
 *  <li>Transparency Panel
 *     <ul>
 *     <li>TRANSPARENCY  </li>
 *     <li>TRANSPARENCY_MODE  </li>
 *     <li>TRANSPARENCY_SRC_BLEND_FUNCTION  </li>
 *     <li>TRANSPARENCY_DST_BLEND_FUNCTION  </li>
 *     </ul>
 *     <br/>
 *     </li>
 *
 * <li> Texture Panel
 *     <ul>
 *     <li>TEXTURE_USE  </li>
 *     <li>TEXTURE_FILE  </li>
 *     <li>TEXTURE_TRANSFORM  </li>
 *     <li>TEXTURE_PERSPECTIVE_CORRECTION_MODE  </li>
 *     <li>TEXTURE_MODE  </li>
 *     <li>TEXTURE_BLEND  </li> 
 *     <li>TEXTURE_COMBINE_RGB_MODE  </li> 
 *     <li>TEXTURE_COMBINE_RGB_SOURCE  </li> 
 *     <li>TEXTURE_COMBINE_RGB_FUNCTION  </li> 
 *     <li>TEXTURE_COMBINE_RGB_SCALE  </li>
 *     <li>TEXTURE_COMBINE_ALPHA_MODE  </li> 
 *     <li>TEXTURE_COMBINE_ALPHA_SOURCE  </li> 
 *     <li>TEXTURE_COMBINE_ALPHA_FUNCTION  </li> 
 *     <li>TEXTURE_COMBINE_ALPHA_SCALE  </li>
 *     </ul>
 *     <br/>
 *     </li>
 * </li>
 * </ul>
 * 
 */    
public class Appearance3DChooser extends JFrame implements ActionListener, WindowListener 
{   
    private static final long serialVersionUID = 1L;
    
    private Appearance3DChooserPanel appearance3DPan;
   
    protected JButton ok;
    protected JButton cancel;  
    protected JButton reset;  
    protected JMenuItem info3DMenuItem;
    protected JMenuItem aboutMenuItem;
    protected JMenuItem helpMenuItem;  
    protected JMenuItem infoUserMenuItem;  
    protected JMenuItem infoSystemMenuItem;
    protected JMenuItem infoJavaMenuItem;
    protected JMenuItem infoParametersMenuItem;
        
    protected float []emissiveColorInit = {0.36f,0.85f,0.80f};
    protected float []specularColorInit = {1.0f,1.0f,1.0f};
    protected float []ambientColorInit = {0.2f,0.2f,0.2f};
    protected float []diffuseColorInit = {1.0f,1.0f,1.0f};
    protected float shininessInit = 64.0f;
    protected int colorTargetInit = Material.DIFFUSE;
    protected boolean lightingInit = true;
    protected float transparencyInit = 0.0f;    
    protected int transparencyModeInit = TransparencyAttributes.NONE;
    protected int transparencySrcBlendFunctionInit = TransparencyAttributes.BLEND_SRC_ALPHA;
    protected int transparencyDstBlendFunctionInit = TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA;
    protected Transform3D textureTransformInit = new Transform3D();
    protected boolean textureUseInit = false;
    protected URL textureFileInit = (new Info()).getClass().getResource("resources/DefaultTexture.png");
    protected int texturePerspectiveCorrectionModeInit = TextureAttributes.NICEST;
    protected int textureModeInit = TextureAttributes.REPLACE;
    protected float []textureBlendColorInit = {0.5f,0.5f,0.5f,0.5f};
    protected int textureCombineRGBModeInit = TextureAttributes.COMBINE_MODULATE;
    protected int []textureCombineRGBSourceInit = {TextureAttributes.COMBINE_TEXTURE_COLOR, 
						   TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE, 
						   TextureAttributes.COMBINE_CONSTANT_COLOR};
    protected int []textureCombineRGBFunctionInit = {TextureAttributes.COMBINE_SRC_COLOR, 
						     TextureAttributes.COMBINE_SRC_COLOR, 
						     TextureAttributes.COMBINE_SRC_COLOR};
    protected int textureCombineRGBScaleInit = 1;
    protected int textureCombineAlphaModeInit = TextureAttributes.COMBINE_MODULATE;
    protected int []textureCombineAlphaSourceInit = {TextureAttributes.COMBINE_TEXTURE_COLOR, 
						     TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE, 
						     TextureAttributes.COMBINE_CONSTANT_COLOR};
    protected int []textureCombineAlphaFunctionInit = {TextureAttributes.COMBINE_SRC_ALPHA, 
						       TextureAttributes.COMBINE_SRC_ALPHA, 
						       TextureAttributes.COMBINE_SRC_ALPHA};
    protected int textureCombineAlphaScaleInit = 1;


    protected float []emissiveColor = {0.36f,0.85f,0.80f};
    protected float []specularColor = {1.0f,1.0f,1.0f};
    protected float []ambientColor = {0.2f,0.2f,0.2f};
    protected float []diffuseColor = {1.0f,1.0f,1.0f};
    protected float shininess = 64.0f;
    protected int colorTarget = Material.DIFFUSE;
    protected boolean lighting = true;
    protected float transparency = 0.0f;
    protected int transparencyMode = TransparencyAttributes.NONE;
    protected int transparencySrcBlendFunction = TransparencyAttributes.BLEND_SRC_ALPHA;
    protected int transparencyDstBlendFunction = TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA;
    protected Transform3D textureTransform = new Transform3D();
    protected boolean textureUse = false;
    protected URL textureFile = (new Info()).getClass().getResource("resources/DefaultTexture.png");
    protected int texturePerspectiveCorrectionMode = TextureAttributes.NICEST;
    protected int textureMode = TextureAttributes.REPLACE;
    protected float []textureBlendColor = {0.5f,0.5f,0.5f,0.5f};
    protected int textureCombineRGBMode = TextureAttributes.COMBINE_MODULATE;
    protected int []textureCombineRGBSource = {TextureAttributes.COMBINE_TEXTURE_COLOR, 
					       TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE, 
					       TextureAttributes.COMBINE_CONSTANT_COLOR};
    protected int []textureCombineRGBFunction = {TextureAttributes.COMBINE_SRC_COLOR, 
						 TextureAttributes.COMBINE_SRC_COLOR, 
						 TextureAttributes.COMBINE_SRC_COLOR};
    protected int textureCombineRGBScale = 1;
    protected int textureCombineAlphaMode = TextureAttributes.COMBINE_MODULATE;
    protected int []textureCombineAlphaSource = {TextureAttributes.COMBINE_TEXTURE_COLOR, 
						 TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE, 
						 TextureAttributes.COMBINE_CONSTANT_COLOR};
    protected int []textureCombineAlphaFunction = {TextureAttributes.COMBINE_SRC_ALPHA, 
						   TextureAttributes.COMBINE_SRC_ALPHA, 
						   TextureAttributes.COMBINE_SRC_ALPHA};
    protected int textureCombineAlphaScale = 1;        


    public static final String SPECULAR = Appearance3DChooserPanel.SPECULAR;
    public static final String AMBIENT = Appearance3DChooserPanel.AMBIENT;
    public static final String EMISSIVE = Appearance3DChooserPanel.EMISSIVE;
    public static final String DIFFUSE = Appearance3DChooserPanel.DIFFUSE;
    public static final String COLORTARGET = Appearance3DChooserPanel.COLORTARGET;
    public static final String SHININESS = Appearance3DChooserPanel.SHININESS;
    public static final String LIGHTING = Appearance3DChooserPanel.LIGHTING;
    public static final String TRANSPARENCY = Appearance3DChooserPanel.TRANSPARENCY;
    public static final String TRANSPARENCY_MODE = Appearance3DChooserPanel.TRANSPARENCY_MODE;
    public static final String TRANSPARENCY_SRC_BLEND_FUNCTION = Appearance3DChooserPanel.TRANSPARENCY_SRC_BLEND_FUNCTION;
    public static final String TRANSPARENCY_DST_BLEND_FUNCTION = Appearance3DChooserPanel.TRANSPARENCY_DST_BLEND_FUNCTION;    
    public static final String TEXTURE_USE = Appearance3DChooserPanel.TEXTURE_USE;
    public static final String TEXTURE_FILE = Appearance3DChooserPanel.TEXTURE_FILE;
    public static final String TEXTURE_TRANSFORM = Appearance3DChooserPanel.TEXTURE_TRANSFORM;
    public static final String TEXTURE_PERSPECTIVE_CORRECTION_MODE = Appearance3DChooserPanel.TEXTURE_PERSPECTIVE_CORRECTION_MODE;
    public static final String TEXTURE_MODE = Appearance3DChooserPanel.TEXTURE_MODE;
    public static final String TEXTURE_BLEND = Appearance3DChooserPanel.TEXTURE_BLEND;
    public static final String TEXTURE_COMBINE_RGB_MODE = Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_MODE;
    public static final String TEXTURE_COMBINE_RGB_SOURCE = Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SOURCE;
    public static final String TEXTURE_COMBINE_RGB_FUNCTION = Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_FUNCTION;
    public static final String TEXTURE_COMBINE_RGB_SCALE = Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SCALE;
    public static final String TEXTURE_COMBINE_ALPHA_MODE = Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_MODE;
    public static final String TEXTURE_COMBINE_ALPHA_SOURCE = Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SOURCE;
    public static final String TEXTURE_COMBINE_ALPHA_FUNCTION = Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_FUNCTION;
    public static final String TEXTURE_COMBINE_ALPHA_SCALE = Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SCALE;


    protected PropertyChangeSupport boundSupport;


    /**
     * Main Appearance3DChooser constructor. Generate a Frame with all the Panes associated to control 
     * some of the JAVA 3D Appearance attributes.  
     *
     * @param title Title for the frame.
     * @param showMaterial set it to true if you want to see the Material Panel.
     * @param showTransparency set it to true if you want to see the Transparency Panel.
     * @param showTexture set it to true if you want to see the Texture Panel.
     * @param showRendering set it to true if you want to see the Rendering Panel.
     */
    public Appearance3DChooser(String title, boolean showMaterial, boolean showTransparency, boolean showTexture, boolean showRendering)
    {
	this(title, showMaterial, showTransparency,  showTexture, showRendering, true);
    }
    
    
    
    /**
     * Main Appearance3DChooser constructor. Generate a Frame with all the Panes associated to control 
     * some of the JAVA 3D Appearance attributes.  
     *
     * @param title Title for the frame.
     * @param showMaterial set it to true if you want to see the Material Panel.
     * @param showTransparency set it to true if you want to see the Transparency Panel.
     * @param showTexture set it to true if you want to see the Texture Panel.
     * @param showRendering set it to true if you want to see the Rendering Panel.
     */
    public Appearance3DChooser(String title, boolean showMaterial, boolean showTransparency, boolean showTexture, boolean showRendering, boolean show)
    {
	boundSupport  = new PropertyChangeSupport(this);
	appearance3DPan = new Appearance3DChooserPanel(showMaterial, showTransparency, showTexture, showRendering);

	// Initialize parameters
	setInitParameters();			    	

	//Add OK, Reset and Cancel Button
	ok = new JButton(Info.ressources.getObject("OK").toString());
	ok.addActionListener(this);
	cancel = new JButton(Info.ressources.getObject("Cancel").toString());
	cancel.addActionListener(this);	
	reset = new JButton(Info.ressources.getObject("Reset").toString());
	reset.addActionListener(this);	
	
	setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	setTitle(title);
	addWindowListener(this);
	getContentPane().setLayout(new GridBagLayout());

	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0; c.gridy = 0;
	c.weightx = 1; c.weighty = 1; c.fill = GridBagConstraints.BOTH;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.insets = new Insets(10,10,10,10);
	getContentPane().add(appearance3DPan,c);
	c.gridy++; c.gridwidth = 1;
	getContentPane().add(new JPanel(),c);
	c.gridx++; c.weightx = 0;
	c.insets = new Insets(8,8,8,2);
	getContentPane().add(reset,c);
	c.gridx++; c.weightx = 0;
	c.insets = new Insets(8,2,8,2);
	getContentPane().add(cancel,c);
	c.gridx++; c.weightx = 0;
	c.insets = new Insets(8,2,8,8);
	getContentPane().add(ok,c);
	getRootPane().setDefaultButton(ok);
	
	
	//Add a Menu Bar
	//Create the menu bar.
	JMenuBar menuBar = new JMenuBar();
	//Build the first menu.
	JMenu menu = new JMenu("?");
	//menu.setMnemonic(KeyEvent.VK_I);
	
	//a Submenu for System (CPU, OS, Memory ...) Info
	infoSystemMenuItem = new JMenuItem(Info.ressources.getObject("System_Info").toString(), KeyEvent.VK_S);
	infoSystemMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.ALT_MASK));
	infoSystemMenuItem.addActionListener(this);
	menu.add(infoSystemMenuItem);
	//a Submenu for 3D Characteristics
	info3DMenuItem = new JMenuItem(Info.ressources.getObject("3D_Info").toString(), KeyEvent.VK_I);
	info3DMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.ALT_MASK));
	info3DMenuItem.addActionListener(this);
	menu.add(info3DMenuItem);
	//a Submenu for Java Info
	infoJavaMenuItem = new JMenuItem(Info.ressources.getObject("Java_Info").toString(), KeyEvent.VK_J);
	infoJavaMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.ALT_MASK));
	infoJavaMenuItem.addActionListener(this);
	menu.add(infoJavaMenuItem);
	//a Submenu for User Info
	infoUserMenuItem = new JMenuItem(Info.ressources.getObject("User_Info").toString(), KeyEvent.VK_U);
	infoUserMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_4, ActionEvent.ALT_MASK));
	infoUserMenuItem.addActionListener(this);
	menu.add(infoUserMenuItem);
	//a Submenu for Parameters Values Info
	infoParametersMenuItem = new JMenuItem(Info.ressources.getObject("Parameters_Info").toString(), KeyEvent.VK_P);
	infoParametersMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_5, ActionEvent.ALT_MASK));
	infoParametersMenuItem.addActionListener(this);
	menu.add(infoParametersMenuItem);
	//a Submenu for Help on components
	helpMenuItem = new JMenuItem(Info.ressources.getObject("Help").toString(),KeyEvent.VK_E);
	helpMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_6, ActionEvent.ALT_MASK));
	helpMenuItem.addActionListener(this);
	menu.add(helpMenuItem);
	//a Submenu for About Info
	aboutMenuItem = new JMenuItem(Info.ressources.getObject("About").toString(), KeyEvent.VK_A);
	aboutMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_7, ActionEvent.ALT_MASK));
	aboutMenuItem.addActionListener(this);
	menu.add(aboutMenuItem);
	menuBar.add(Box.createHorizontalGlue());
	menuBar.add(menu);

	setJMenuBar(menuBar);

	pack();
	setResizable(false);
	setVisible(show);			
    }
    

    /**
     * Show or Hide the Windows.
     * @param on true if the windows has to be shown, falsed otherwise.     
     */  
    public void showWindow(boolean on)
    {
    	setVisible(on);    	
    }     
    
    /**
     * Set all the parameters to their init value.     
     */  
    public void setInitParameters()
    {
	appearance3DPan.setAmbientColor(ambientColorInit);
	appearance3DPan.setDiffuseColor(diffuseColorInit); 
	appearance3DPan.setEmissiveColor(emissiveColorInit);  
	appearance3DPan.setSpecularColor(specularColorInit);
	appearance3DPan.setShininess(shininessInit);
	appearance3DPan.setColorTarget(colorTargetInit);      
	appearance3DPan.setLighting(lightingInit);
	appearance3DPan.setTransparency(transparencyInit);
	appearance3DPan.setTransparencyMode(transparencyModeInit);
	appearance3DPan.setTransparencySrcBlendFunction(transparencySrcBlendFunctionInit);
	appearance3DPan.setTransparencyDstBlendFunction(transparencyDstBlendFunctionInit);
	appearance3DPan.setTextureUse(textureUseInit);
	appearance3DPan.setTextureTransform(textureTransformInit);		
	appearance3DPan.setDefaultTextureFile(textureFileInit);		    	
	appearance3DPan.setTexturePerspectiveCorrectionMode(texturePerspectiveCorrectionModeInit);
	appearance3DPan.setTextureMode(textureModeInit);
	appearance3DPan.setTextureBlendColor(textureBlendColorInit);	    
	appearance3DPan.setTextureCombineRGBMode(textureCombineRGBModeInit);
	appearance3DPan.setTextureCombineRGBSource(textureCombineRGBSourceInit);
	appearance3DPan.setTextureCombineRGBFunction(textureCombineRGBFunctionInit);
	appearance3DPan.setTextureCombineRGBScale(textureCombineRGBScaleInit);	
	appearance3DPan.setTextureCombineAlphaMode(textureCombineAlphaModeInit);
	appearance3DPan.setTextureCombineAlphaSource(textureCombineAlphaSourceInit);
	appearance3DPan.setTextureCombineAlphaFunction(textureCombineAlphaFunctionInit);
	appearance3DPan.setTextureCombineAlphaScale(textureCombineAlphaScaleInit);	
    }

    /**
     * 
     * Called when an action is performed : the OK/CANCEL/RESET button is clicked or a menu is choosen in the menu bar
     * @param e the related action.
     *
     */                       
    public void actionPerformed(ActionEvent e) 
    {
	Object source = e.getSource();

	if(source == helpMenuItem)
	    {
		Info.showJava3DHelp();
	    }

	else if (source == info3DMenuItem) 
	    {	
		Info.show3DInfo();
	    }

	else if (source == infoUserMenuItem) 
	    {
		new com.envInfo.UserInfo();
	    }

	else if (source == infoSystemMenuItem) 
	    {	
		new com.envInfo.SystemInfo();
	    }

	else if (source == infoJavaMenuItem) 
	    {			
		new com.envInfo.JavaInfo();
	    }

	else if (source == aboutMenuItem) 
	    {	
		Info.showAbout();
	    }
	
	else if (source == infoParametersMenuItem)
	    {
		new Appearance3DChooserInfo(this);
	    }

	else if(source == ok) 
	    {
		setVisible(false);
		dispose(); 
	    }

	else if (source == cancel) 
	    {	
		// Reset all the parameters if cancel
		setInitParameters();			    
		setVisible(false);		 
		dispose(); 				
	    }

	else if (source == reset) 
	    {	
		// Reset all the parameters 
		setInitParameters();			    		
	    }
    }

    /**
     * 
     * Called when the Main JFrame is in the process of being closed.
     * @param w the related event.
     *
     */                           
    public void windowClosing ( WindowEvent w )
    {	
	// called when user closes the JFrame.
	// Reset all the parameters if closing
	setInitParameters();
	//setVisible(false);
	dispose(); 
    }

    /**
     * 
     * Called when user Deactivates the Main JFrame.
     * @param w the related event.
     *
     */                           
    public void windowDeactivated ( WindowEvent w )
    {
    }

    /**
     * 
     * Called when user Activates the Main JFrame.
     * @param w the related event.
     *
     */                           
    public void windowActivated ( WindowEvent w )
    {
    }

    /**
     * 
     * Called when user Deiconifies the Main JFrame.
     * @param w the related event.
     *
     */                           
    public void windowDeiconified ( WindowEvent w )
    {
    }

    /**
     * 
     * Called when user iconifies the Main JFrame.
     * @param w the related event.
     *
     */                           
    public void windowIconified ( WindowEvent w )
    {
    }

    /**
     * 
     * Called when the Main JFrame is closed.
     * @param w the related event.
     *
     */                           
    public void windowClosed ( WindowEvent w )
    {
    }
    
    /**
     * 
     * Called when the Main JFrame has been opened.
     * @param w the related event.
     *
     */                           
    public void windowOpened ( WindowEvent w )
    {
    }

   
    /**
     * 
     * Must be called before listening any Property change. 
     *
     */                           
    public void runListeners()
    {
	if (appearance3DPan == null) return;

	appearance3DPan.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) 
		{
		    if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.SPECULAR) == 0)
			{
			    updateSpecularColor(getSpecularColor());				   
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.AMBIENT) == 0)
			{
			    updateAmbientColor(getAmbientColor());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.EMISSIVE) == 0)
			{
			    updateEmissiveColor(getEmissiveColor());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.DIFFUSE) == 0)
			{
			    updateDiffuseColor(getDiffuseColor());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.COLORTARGET) == 0)
			{
			    updateColorTarget(getColorTarget());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.SHININESS) == 0)
			{
			    updateShininess(getShininess());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.LIGHTING) == 0)
			{
			    updateLighting(getLighting());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TRANSPARENCY) == 0)
			{
			    updateTransparency(getTransparency());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TRANSPARENCY_MODE) == 0)
			{
			    updateTransparencyMode(getTransparencyMode());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TRANSPARENCY_SRC_BLEND_FUNCTION) == 0)
			{
			    updateTransparencySrcBlendFunction(getTransparencySrcBlendFunction());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TRANSPARENCY_DST_BLEND_FUNCTION) == 0)
			{
			    updateTransparencyDstBlendFunction(getTransparencyDstBlendFunction());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_USE) == 0)
			{
			    updateTextureUse(isTextureSet());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_FILE) == 0)
			{
			    updateTextureFile(getTextureFile());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_TRANSFORM) == 0)
			{
			    updateTextureTransform(getTextureTransform());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_PERSPECTIVE_CORRECTION_MODE) == 0)
			{
			    updateTexturePerspectiveCorrectionMode(getTexturePerspectiveCorrectionMode());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_MODE) == 0)
			{
			    updateTextureMode(getTextureMode());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_BLEND) == 0)
			{
			    updateTextureBlendColor(getTextureBlendColor());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_MODE) == 0)
			{			   
			    updateTextureCombineRGBMode(getTextureCombineRGBMode());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SOURCE) == 0)
			{
			    updateTextureCombineRGBSource(getTextureCombineRGBSource());			    
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_FUNCTION) == 0)
			{
			    updateTextureCombineRGBFunction(getTextureCombineRGBFunction());			    
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SCALE) == 0)
			{
			    updateTextureCombineRGBScale(getTextureCombineRGBScale());			    
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_MODE) == 0)
			{
			    updateTextureCombineAlphaMode(getTextureCombineAlphaMode());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SOURCE) == 0)
			{
			    updateTextureCombineAlphaSource(getTextureCombineAlphaSource());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_FUNCTION) == 0)
			{
			    updateTextureCombineAlphaFunction(getTextureCombineAlphaFunction());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SCALE) == 0)
			{
			    updateTextureCombineAlphaScale(getTextureCombineAlphaScale());
			}
		}});
    }
    

    /**
     * 
     * Add a Property Change Listener.
     * @param s the string describing it.
     * @param l the Property Change Listener.
     *
     */                           
    public void addPropertyChangeListener(String s, PropertyChangeListener l) 
    {
	boundSupport.addPropertyChangeListener(s, l);
    }

    /**
     * 
     * Add a Property Change Listener.
     * @param l the Property Change Listener.
     *
     */                           
    public void addPropertyChangeListener(PropertyChangeListener l) 
    {
	boundSupport.addPropertyChangeListener(l);
    }

    /**
     * 
     * Remove a Property Change Listener.
     * @param l the Property Change Listener.
     *
     */                           
    public void removePropertyChangeListener(PropertyChangeListener l) 
    {
	boundSupport.removePropertyChangeListener(l);
    }

    /**
     * 
     * Return the Apparence3DChooser Panel.
     * @return the Apparence3DChooser Panel.
     *
     */                           
    private Appearance3DChooserPanel getPickerPan()
    {
	return appearance3DPan;
    }
    
    
    /**
     * 
     * Return the Specular Color.
     * @return the Specular Color in (R,G,B).
     *
     */                           
    public float[] getSpecularColor()
    {
	return getPickerPan().getSpecularColor();
    }

    /**
     * 
     * Set the Specular Color.
     * @param r the Specular Color in R.
     * @param g the Specular Color in G.
     * @param b the Specular Color in B.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                           
    public void setSpecularColor(float r, float g, float b, boolean init)
    {
    	specularColor[0] = r;
    	specularColor[1] = g;
    	specularColor[2] = b;
    	
    	if(init)
	    {
    		specularColorInit[0] = r;
        	specularColorInit[1] = g;
        	specularColorInit[2] = b;        	 
		getPickerPan().setSpecularColor(specularColor);		          
	    }
	else
	    {
		getPickerPan().setSpecularColor(specularColor);		          
		boundSupport.firePropertyChange(Appearance3DChooser.SPECULAR,specularColorInit,specularColor);
	    }
    }


    /**
     * 
     * Internal update of the Specular Color Parameters and fire of an associated property.
     * @param c the Specular Color in (R,G,B).
     *
     */                           
    private void updateSpecularColor (float []c)
    {
	float []lastSpecular = specularColor;
	specularColor = c;
	boundSupport.firePropertyChange(Appearance3DChooser.SPECULAR,lastSpecular,specularColor);
    }
    

    /**
     * 
     * Return the Ambient Color.
     * @return the Ambient Color in (R,G,B).
     *
     */                               
    public float[] getAmbientColor()
    {
	return getPickerPan().getAmbientColor();
    }
    
    /**
     * 
     * Set the Ambient Color.
     * @param r the Ambient Color in R.
     * @param g the Ambient Color in G.
     * @param b the Ambient Color in B.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                           
    public void setAmbientColor(float r, float g, float b, boolean init)
    {    
    	ambientColor[0] = r;
    	ambientColor[1] = g;
    	ambientColor[2] = b;    	
    	
    	if(init)
	    {
    		ambientColorInit[0] = r;
        	ambientColorInit[1] = g;
        	ambientColorInit[2] = b;
        	getPickerPan().setAmbientColor(ambientColor);		        	
	    }

	else
	    {
		getPickerPan().setAmbientColor(ambientColor);		        	
		boundSupport.firePropertyChange(Appearance3DChooser.AMBIENT,ambientColorInit,ambientColor);
	    }
    }

    /**
     * 
     * Internal update of the Ambient Color Parameters and fire of an associated property.
     * @param c the Ambient Color in (R,G,B).
     *
     */                           
    private void updateAmbientColor (float []c)
    {
	float []lastAmbient = ambientColor;
	ambientColor = c;
	boundSupport.firePropertyChange(Appearance3DChooser.AMBIENT,lastAmbient,ambientColor);
    }

    /**
     * 
     * Return the Diffuse Color.
     * @return the Diffuse Color in (R,G,B).
     *
     */                               
    public float[] getDiffuseColor()
    {
	return getPickerPan().getDiffuseColor();
    }

    /**
     * 
     * Set the Diffuse Color
     * @param r the Diffuse Color in R.
     * @param g the Diffuse Color in G.
     * @param b the Diffuse Color in B.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                           
    public void setDiffuseColor(float r, float g, float b, boolean init)
    {
    	diffuseColor[0] = r;
    	diffuseColor[1] = g;
    	diffuseColor[2] = b;

    	if(init)
	    {
    		diffuseColorInit[0] = r;
        	diffuseColorInit[1] = g;
        	diffuseColorInit[2] = b;
        	getPickerPan().setDiffuseColor(diffuseColor); 		    
	    }
	else
	    {
		getPickerPan().setDiffuseColor(diffuseColor); 		    
		boundSupport.firePropertyChange(Appearance3DChooser.DIFFUSE,diffuseColorInit,diffuseColor);
	    }
    }

    /**
     * 
     * Internal update of the Diffuse Color Parameters and fire of an associated property.
     * @param c the Diffuse Color in (R,G,B).
     *
     */                           
    private void updateDiffuseColor (float []c)
    {
	float []lastDiffuse = diffuseColor;
	diffuseColor = c;
	boundSupport.firePropertyChange(Appearance3DChooser.DIFFUSE,lastDiffuse,diffuseColor);
    }
    
    /**
     * 
     * Return the Emissive Color.
     * @return the Emissive Color in (R,G,B).
     *
     */                               
    public float[] getEmissiveColor()
    {
	return getPickerPan().getEmissiveColor();
    }

    /**
     * 
     * Set the Emissive Color.
     * @param r the Emissive Color in R.
     * @param g the Emissive Color in G.
     * @param b the Emissive Color in B.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                           
    public void setEmissiveColor(float r, float g, float b, boolean init)
    {
    	emissiveColor[0] = r;
    	emissiveColor[1] = g;
    	emissiveColor[2] = b;
    	
    	if(init)
	    {
    		emissiveColorInit[0] = r;
        	emissiveColorInit[1] = g;
        	emissiveColorInit[2] = b;
        	getPickerPan().setEmissiveColor(emissiveColor);		    
	    }		
	else
	    {
		getPickerPan().setEmissiveColor(emissiveColor);		    
		boundSupport.firePropertyChange(Appearance3DChooser.EMISSIVE,emissiveColorInit,emissiveColor);
	    }
    }

    /**
     * 
     * Internal update of the Emissive Color Parameters and fire of an associated property.
     * @param c the Emissive Color in (R,G,B).
     *
     */                           
    private void updateEmissiveColor (float []c)
    {
	float []lastEmissiveColor = emissiveColor;
	emissiveColor = c;
	boundSupport.firePropertyChange(Appearance3DChooser.EMISSIVE,lastEmissiveColor,emissiveColor);
    }
    
    /**
     * 
     * Return the Color Target.
     * @return the Color Target. One of: Material.AMBIENT, Material.EMISSIVE, Material.DIFFUSE, Material.SPECULAR, 
     * or Material.AMBIENT_AND_DIFFUSE.
     *
     */                               
    public int getColorTarget()
    {
	return getPickerPan().getColorTarget();
    }

    /**
     * 
     * Set the Color Target.
     * @param c the Color Target. One of: Material.AMBIENT, Material.EMISSIVE, Material.DIFFUSE, Material.SPECULAR, 
     * or Material.AMBIENT_AND_DIFFUSE.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                               
    public void setColorTarget(int c, boolean init)
    {
    	colorTarget = c;    	
    	
    	if(init)
	    {
    		colorTargetInit = c;   
		getPickerPan().setColorTarget(c);
	    }	
	else
	    {
		getPickerPan().setColorTarget(c);
		boundSupport.firePropertyChange(Appearance3DChooser.COLORTARGET,colorTargetInit,colorTarget);
	    }
    }
    
    /**
     * 
     * Internal update of the Color Target Parameters and fire of an associated property.
     * @param c the Color Target. One of: Material.AMBIENT, Material.EMISSIVE, Material.DIFFUSE, Material.SPECULAR, 
     * or Material.AMBIENT_AND_DIFFUSE.
     *
     */                               
    private void updateColorTarget (int c)
    {
	int lastColorTarget = colorTarget;
	colorTarget = c;
	boundSupport.firePropertyChange(Appearance3DChooser.COLORTARGET,(int)lastColorTarget,(int)colorTarget);
    }

    /**
     * 
     * Return the Shininess.
     * @return the Shininess in the range [1.0, 128.0] with 1.0 being not shiny and 128.0 being very shiny.
     *
     */                               
    public float getShininess()
    {
	return getPickerPan().getShininess();
    }
    
    /**
     * 
     * Set the Shininess.
     * @param s the Shininess in the range [1.0, 128.0] with 1.0 being not shiny and 128.0 being very shiny.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                               
    public void setShininess(float s, boolean init)
    {
    	shininess = s;       	
    	
    	if(init)
	    {
    		shininessInit = s;           	
		getPickerPan().setShininess(s);		  
	    }
	else
	    {
		getPickerPan().setShininess(s);		  
		boundSupport.firePropertyChange(Appearance3DChooser.SHININESS,shininessInit,shininess);
	    }
    }

    /**
     * 
     * Internal update of the Shininess Parameters and fire of an associated property.
     * @param s the Shininess in the range [1.0, 128.0] with 1.0 being not shiny and 128.0 being very shiny.
     *
     */                               
    private void updateShininess (float s)
    {
	float lastShininess = shininess;
	shininess = s;
	boundSupport.firePropertyChange(Appearance3DChooser.SHININESS,lastShininess,shininess);
    }

    /**
     * 
     * Return the Lighting mode.
     * @return true if lighting is enabled for the object, false otherwise.
     *
     */                               
    public boolean getLighting()
    {
	return getPickerPan().getLighting();
    }
    
    /**
     * 
     * Set the Lighting mode.
     * @param l true lighting is enabled for the object, false otherwise.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                               
    public void setLighting(boolean l, boolean init)
    {
    	lighting = l;   

    	if(init)
	    {
    		lightingInit = l;       
		getPickerPan().setLighting(l);		
	    }
	else
	    {
		getPickerPan().setLighting(l);		
		boundSupport.firePropertyChange(Appearance3DChooser.LIGHTING,lightingInit,lighting);
	    }
    }

    /**
     * 
     * Internal update of the Lighting mode Parameters and fire of an associated property.
     * @param l true lighting is enabled for the object, false otherwise.
     *
     */                               
    private void updateLighting (boolean l)
    {
	boolean lastLighting = lighting;
	lighting = l;
	boundSupport.firePropertyChange(Appearance3DChooser.LIGHTING,lastLighting,lighting);
    }

    /**
     * 
     * Return the transparency value.
     * @return the amount of transparency to be applied to the Appearance component object. 
     * The transparency values are in the range [0.0, 1.0], with 0.0 being fully opaque and 1.0 being fully transparent. 
     *
     */                               
    public float getTransparency()
    {
	return getPickerPan().getTransparency();
    }
    
    /**
     * 
     * Set the transparency value.
     * @param t the amount of transparency to be applied to the Appearance component object. 
     * The transparency values are in the range [0.0, 1.0], with 0.0 being fully opaque and 1.0 being fully transparent. 
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                               
    public void setTransparency(float t, boolean init)
    {
    	transparency = t;   
    	
    	if(init)
	    {
    		transparencyInit = t;           	
		getPickerPan().setTransparency(t);		    
	    }
	else
	    {
		getPickerPan().setTransparency(t);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TRANSPARENCY,transparencyInit,transparency);
	    }
    }


    /**
     * 
     * Internal update of the transparency value Parameters and fire of an associated property.
     * @param f the amount of transparency to be applied to the Appearance component object. 
     * The transparency values are in the range [0.0, 1.0], with 0.0 being fully opaque and 1.0 being fully transparent. 
     *
     */                               
    private void updateTransparency (float f)
    {
	float lastTransparency = transparency;
	transparency = f;
	boundSupport.firePropertyChange(Appearance3DChooser.TRANSPARENCY,lastTransparency,transparency);
    }


    /**
     * 
     * Return the transparency Mode.
     * @return the transparency Mode between TransparencyAttributes.FASTEST, TransparencyAttributes.NICEST, TransparencyAttributes.SCREEN_DOOR, TransparencyAttributes.BLENDED, TransparencyAttributes.NONE 
     *
     */                               
    public int getTransparencyMode()
    {
	return getPickerPan().getTransparencyMode();
    }

    /**
     * 
     * Set the transparency Mode.
     * @param m the transparency Mode between TransparencyAttributes.FASTEST, TransparencyAttributes.NICEST, TransparencyAttributes.SCREEN_DOOR, TransparencyAttributes.BLENDED, TransparencyAttributes.NONE 
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */                                  
    public void setTransparencyMode(int m, boolean init)
    {
    	transparencyMode = m;   
    	
    	if(init)
	    {
    		transparencyModeInit = m;           	
		getPickerPan().setTransparencyMode(m);		
	    }
	else
	    {
		getPickerPan().setTransparencyMode(m);		
		boundSupport.firePropertyChange(Appearance3DChooser.TRANSPARENCY_MODE,transparencyModeInit,transparencyMode);
	    }
    }

    /**
     * 
     * Internal update of the transparency Mode Parameters and fire of an associated property.
     * @param m the transparency Mode between TransparencyAttributes.FASTEST, TransparencyAttributes.NICEST, TransparencyAttributes.SCREEN_DOOR, TransparencyAttributes.BLENDED, TransparencyAttributes.NONE 
     *
     */                                  
    private void updateTransparencyMode (int m)
    {
	int lastTransparencyMode = transparencyMode;
	transparencyMode = m;
	boundSupport.firePropertyChange(Appearance3DChooser.TRANSPARENCY_MODE,lastTransparencyMode,transparencyMode);
    }

    /**
     * 
     * Return the transparency Source Blend Function.
     * @return the transparency Source Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_DST_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR.
     *
     */    
    public int getTransparencySrcBlendFunction()
    {
	return getPickerPan().getTransparencySrcBlendFunction();
    }
    
    /**
     * 
     * Set the transparency Source Blend Function.
     * @param f the transparency Source Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_DST_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTransparencySrcBlendFunction(int f, boolean init)
    {
    	transparencySrcBlendFunction = f;   
    	
    	if(init)
	    {
    		transparencySrcBlendFunctionInit = f;           	
		getPickerPan().setTransparencySrcBlendFunction(f);		    
	    }	
	else
	    {
		getPickerPan().setTransparencySrcBlendFunction(f);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TRANSPARENCY_SRC_BLEND_FUNCTION,transparencySrcBlendFunctionInit,transparencySrcBlendFunction);
	    }
    }

    /**
     * 
     * Internal update of the transparency Source Blend Function Parameters and fire of an associated property.
     * @param f the transparency Source Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_DST_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR.
     *
     */    
    private void updateTransparencySrcBlendFunction (int f)
    {
	int lastTransparencySrcBlendFunction = transparencySrcBlendFunction;
	transparencySrcBlendFunction = f;
	boundSupport.firePropertyChange(Appearance3DChooser.TRANSPARENCY_SRC_BLEND_FUNCTION,lastTransparencySrcBlendFunction,transparencySrcBlendFunction);
    }

    /**
     * 
     * Return the transparency Destination Blend Function.
     * @return the transparency Destination Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_SRC_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR.
     *
     */    
    public int getTransparencyDstBlendFunction()
    {
	return getPickerPan().getTransparencyDstBlendFunction();
    }
    
    /**
     * 
     * Set the transparency Destination Blend Function.
     * @param f the transparency Destination Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_SRC_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTransparencyDstBlendFunction(int f, boolean init)
    {
    	transparencyDstBlendFunction = f;   

    	if(init)
	    {
    		transparencyDstBlendFunctionInit = f;           	
		getPickerPan().setTransparencyDstBlendFunction(f);		    
	    }
	else
	    {
		getPickerPan().setTransparencyDstBlendFunction(f);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TRANSPARENCY_DST_BLEND_FUNCTION,transparencyDstBlendFunctionInit,transparencyDstBlendFunction);
	    }
    }    

    /**
     * 
     * Internal update of the transparency Destination Blend Function Parameters and fire of an associated property.
     * @param f the transparency Destination Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_SRC_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR.
     *
     */    
    private void updateTransparencyDstBlendFunction (int f)
    {
	int lastTransparencyDstBlendFunction = transparencyDstBlendFunction;
	transparencyDstBlendFunction = f;
	boundSupport.firePropertyChange(Appearance3DChooser.TRANSPARENCY_DST_BLEND_FUNCTION,lastTransparencyDstBlendFunction,transparencyDstBlendFunction);
    }

    /**
     * 
     * Return the Texture Transform.
     * @return the Texture Transform.
     *
     */    
    public Transform3D getTextureTransform()
    {
	return getPickerPan().getTextureTransform();
    }

    /**
     * 
     * Set the Texture Transform.
     * @param t the Texture Transform.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureTransform(Transform3D t, boolean init)
    {
	textureTransform = new Transform3D(t);   
    	
    	if(init)
	    {
    		textureTransformInit = new Transform3D(t);              	
		getPickerPan().setTextureTransform(t);
	    }
	else
	    {		
		getPickerPan().setTextureTransform(t);
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_TRANSFORM,textureTransformInit,textureTransform);
	    }

    }

    /**
     * 
     * Internal update of the Texture Transform Parameters and fire of an associated property.
     * @param t the Texture Transform.
     *
     */    
    private void updateTextureTransform(Transform3D t)
    {
	Transform3D lastTextureTransform = new Transform3D(textureTransform);
	textureTransform = t;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_TRANSFORM,lastTextureTransform,textureTransform);
    }

    /**
     * 
     * Return the Texture Transform Scale on X axis.
     * @return the Texture Transform Scale on X axis.
     *
     */    
    public float getTextureTransformScaleX()
    {
	return getPickerPan().getTextureTransformScaleX();
    }

    /**
     * 
     * Set the Texture Transform Scale on X axis.
     * @param v the Texture Transform Scale on X axis.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureTransformScaleX(float v, boolean init)
    {
	getPickerPan().setTextureTransformScaleX(v);

	if (init)
	    {
		textureTransformInit = getTextureTransform();
	    }
    }

    /**
     * 
     * Return the Texture Transform Scale on Y axis.
     * @return the Texture Transform Scale on Y axis.
     *
     */    
    public float getTextureTransformScaleY()
    {
	return getPickerPan().getTextureTransformScaleY();
    }

    /**
     * 
     * Set the Texture Transform Scale on Y axis.
     * @param v the Texture Transform Scale on Y axis.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureTransformScaleY(float v, boolean init)
    {
	getPickerPan().setTextureTransformScaleY(v);
	
	if (init)
	    {
		textureTransformInit = getTextureTransform();
	    }
    }

    /**
     * 
     * Return the Texture Transform Translation on X axis.
     * @return the Texture Transform Translation on X axis.
     *
     */    
    public float getTextureTransformTranslatX()
    {
	return getPickerPan().getTextureTransformTranslatX();
    }

    /**
     * 
     * Set the Texture Transform Translation on X axis.
     * @param v the Texture Transform Translation on X axis.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureTransformTranslatX(float v, boolean init)
    {
	getPickerPan().setTextureTransformTranslatX(v);

	if (init)
	    {
		textureTransformInit = getTextureTransform();
	    }
    }

    /**
     * 
     * Return the Texture Transform Translation on Y axis.
     * @return the Texture Transform Translation on Y axis.
     *
     */    
    public float getTextureTransformTranslatY()
    {
	return getPickerPan().getTextureTransformTranslatY();
    }

    /**
     * 
     * Set the Texture Transform Translation on Y axis.
     * @param v the Texture Transform Translation on Y axis.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureTransformTranslatY(float v, boolean init)
    {
	getPickerPan().setTextureTransformTranslatY(v);

	if (init)
	    {
		textureTransformInit = getTextureTransform();
	    }
    }

    /**
     * 
     * Return the Texture Transform Rotation on Z axis.
     * @return the Texture Transform Rotation on Z axis.
     *
     */    
    public float getTextureTransformRotZ()
    {
	return getPickerPan().getTextureTransformRotZ();
    }

    /**
     * 
     * Set the Texture Transform Rotation on Z axis.
     * @param v the Texture Transform Rotation on Z axis.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureTransformRotZ(float v, boolean init)
    {
	getPickerPan().setTextureTransformRotZ(v);

	if (init)
	    {
		textureTransformInit = getTextureTransform();
	    }
    }

    /**
     * 
     * Return the Texture File URL.
     * @return the Texture File URL.
     *
     */    
    public URL getTextureFile()
    {
	return getPickerPan().getTextureFile();
    }

    /**
     * 
     * Set the Texture File URL.
     * @param f the Texture File URL.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureFile(URL f, boolean init)
    {
	textureFile = f;
	if (init)
	    {
		textureFileInit = f;
		getPickerPan().setDefaultTextureFile(f);		    
	    }
	else
	    {
		getPickerPan().setTextureFile(f);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_FILE,textureFileInit,textureFile);
	    }
    }

    /**
     * 
     * Internal update of the Texture File URL Parameters and fire of an associated property.
     * @param f the Texture File URL.
     *
     */    
    private void updateTextureFile (URL f)
    {
	URL lastTextureFile = textureFile;
	textureFile = f;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_FILE,lastTextureFile,textureFile);
    }

    /**
     * 
     * Return the Texture Mode.
     * @return the Texture Mode between TextureAttributes.MODULATE, TextureAttributes.DECAL, TextureAttributes.BLEND, 
     * TextureAttributes.REPLACE, TextureAttributes.COMBINE.
     *
     */    
    public int getTextureMode()
    {
	return getPickerPan().getTextureMode();
    }

    /**
     * 
     * Set the Texture Mode.
     * @param m the Texture Mode between TextureAttributes.MODULATE, TextureAttributes.DECAL, TextureAttributes.BLEND, 
     * TextureAttributes.REPLACE, TextureAttributes.COMBINE.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureMode(int m, boolean init)
    {
	textureMode = m;

	if(init)
	    {
    		textureModeInit = m;      
		getPickerPan().setTextureMode(m);	
	    }
	else
	    {
		getPickerPan().setTextureMode(m);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_MODE,textureModeInit,textureMode);
	    }
    }
        
    /**
     * 
     * Internal update of the Texture Mode Parameters and fire of an associated property.
     * @param m the Texture Mode between TextureAttributes.MODULATE, TextureAttributes.DECAL, TextureAttributes.BLEND, 
     * TextureAttributes.REPLACE, TextureAttributes.COMBINE.
     *
     */    
    private void updateTextureMode(int m)
    {
	int lastTextureMode = textureMode;
	textureMode = m;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_MODE,lastTextureMode,textureMode);
    }

    /**
     * 
     * Return the Texture Blend Color.
     * @return the Texture Blend Color in (R,G,B,A).
     *
     */  
    public float[] getTextureBlendColor()
    {
	return getPickerPan().getTextureBlendColor();
    }

    /**
     * 
     * Set the Texture Blend Color. 
     * @param r the Texture Blend Color in R.
     * @param g the Texture Blend Color in G.
     * @param b the Texture Blend Color in B.
     * @param a the Texture Blend Color in A.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */  
    public void setTextureBlendColor(float r, float g, float b, float a, boolean init)
    {
    	textureBlendColor[0] = r;
    	textureBlendColor[1] = g;
    	textureBlendColor[2] = b;
	textureBlendColor[3] = a;

    	if(init)
	    {
		textureBlendColorInit[0] = r;
		textureBlendColorInit[1] = g;
		textureBlendColorInit[2] = b;
		textureBlendColorInit[3] = a;
		getPickerPan().setTextureBlendColor(textureBlendColor); 		    
	    }	
	else
	    {
		getPickerPan().setTextureBlendColor(textureBlendColor); 		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_BLEND,textureBlendColorInit,textureBlendColor);
	    }	
    }


    /**
     * 
     * Internal update of the Texture Blend Color Parameters and fire of an associated property.
     * @param b the Texture Blend Color in (R,G,B,A).
     *
     */  
    private void updateTextureBlendColor(float []b)
    {
	float []lastTextureBlendColor = textureBlendColor;
	textureBlendColor = b;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_BLEND,lastTextureBlendColor,textureBlendColor);
    }
    
    /**
     * 
     * Return the Texture Perspective Correction Mode.
     * @return the Texture Perspective Correction Mode between TextureAttributes.NICEST, TextureAttributes.FASTEST.
     *
     */    
    public int getTexturePerspectiveCorrectionMode()
    {
	return getPickerPan().getTexturePerspectiveCorrectionMode();
    }

    /**
     * 
     * Set the Texture Perspective Correction Mode.
     * @param m the Texture Perspective Correction Mode between TextureAttributes.NICEST, TextureAttributes.FASTEST. 
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTexturePerspectiveCorrectionMode(int m, boolean init)
    {
	texturePerspectiveCorrectionMode = m;

	if(init)
	    {
    		texturePerspectiveCorrectionModeInit = m;              	
		getPickerPan().setTexturePerspectiveCorrectionMode(m);		    
	    }
	else
	    {
		getPickerPan().setTexturePerspectiveCorrectionMode(m);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_PERSPECTIVE_CORRECTION_MODE,texturePerspectiveCorrectionModeInit,texturePerspectiveCorrectionMode);
	    }
    }


    /**
     * 
     * Internal update of the Texture Perspective Correction Mode Parameters and fire of an associated property.
     * @param m the Texture Perspective Correction Mode between TextureAttributes.NICEST, TextureAttributes.FASTEST. 
     *
     */    
    private void updateTexturePerspectiveCorrectionMode(int m)
    {
	int lastTexturePerspectiveCorrectionMode = texturePerspectiveCorrectionMode;
	texturePerspectiveCorrectionMode = m;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_PERSPECTIVE_CORRECTION_MODE,
					lastTexturePerspectiveCorrectionMode,
					texturePerspectiveCorrectionMode);
    }


    /**
     * 
     * Return the Texture Combine RGB Mode.
     * @return the Texture Combine RGB Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3.
     *
     */    
    public int getTextureCombineRGBMode()
    {
	return getPickerPan().getTextureCombineRGBMode();
    }

    /**
     * 
     * Set the Texture Combine RGB Mode 
     * @param m the Texture Combine RGB Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3. 
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureCombineRGBMode(int m, boolean init)
    {
	textureCombineRGBMode = m;

	if(init)
	    {
    		textureCombineRGBModeInit = m;              	
		getPickerPan().setTextureCombineRGBMode(m);		    
	    }
	else
	    {
		getPickerPan().setTextureCombineRGBMode(m);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_RGB_MODE,textureCombineRGBModeInit,textureCombineRGBMode);
	    }
    }

    /**
     * 
     * Internal update of the Texture Combine RGB Mode Parameters and fire of an associated property.
     * @param m the Texture Combine RGB Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3. 
     *
     */    
    private void updateTextureCombineRGBMode(int m)
    {
	int lastTextureCombineRGBMode = textureCombineRGBMode;
	textureCombineRGBMode = m;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_RGB_MODE,
					lastTextureCombineRGBMode,
					textureCombineRGBMode);
    }

    /**
     * 
     * Return the Texture Combine RGB Source.
     * @return the Texture Combine RGB Source between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     *
     */    
    public int[] getTextureCombineRGBSource()
    {
	return getPickerPan().getTextureCombineRGBSource();
    }

    /**
     * 
     * Set the Texture Combine RGB Source.
     * @param c0 the Texture Combine RGB Source C0 between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     * @param c1 the Texture Combine RGB Source C1 between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     * @param c2 the Texture Combine RGB Source C2 between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureCombineRGBSource(int c0, int c1, int c2, boolean init)
    {
    	textureCombineRGBSource[0] = c0;
    	textureCombineRGBSource[1] = c1;
    	textureCombineRGBSource[2] = c2;

    	if(init)
	    {
		textureCombineRGBSourceInit[0] = c0;
		textureCombineRGBSourceInit[1] = c1;
		textureCombineRGBSourceInit[2] = c2;		
		getPickerPan().setTextureCombineRGBSource(textureCombineRGBSource); 		    
	    }	
	else
	    {
		getPickerPan().setTextureCombineRGBSource(textureCombineRGBSource); 		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_RGB_SOURCE,textureCombineRGBSourceInit,textureCombineRGBSource);
	    }
	
    }

    /**
     * 
     * Internal update of the Texture Combine RGB Source Parameters and fire of an associated property.
     * @param s the Texture Combine RGB Source (C0,C1,C2) between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.     
     *
     */    
    private void updateTextureCombineRGBSource(int[] s)
    {
	int []lastTextureCombineRGBSource = textureCombineRGBSource;
	textureCombineRGBSource = s;		
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_RGB_SOURCE,
					lastTextureCombineRGBSource,
					textureCombineRGBSource);
    }

    /**
     * 
     * Return the Texture Combine RGB Function.
     * @return the Texture Combine RGB Function between TextureAttributes.COMBINE_SRC_COLOR, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */    
    public int[] getTextureCombineRGBFunction()
    {
	return getPickerPan().getTextureCombineRGBFunction();
    }

    /**
     * 
     * Set the Texture Combine RGB Function.
     * @param c0 the Texture Combine RGB Function C0 between TextureAttributes.COMBINE_SRC_COLOR, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     * @param c1 the Texture Combine RGB Function C1 between TextureAttributes.COMBINE_SRC_COLOR, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     * @param c2 the Texture Combine RGB Function C2 between TextureAttributes.COMBINE_SRC_COLOR, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureCombineRGBFunction(int c0, int c1, int c2, boolean init)
    {
    	textureCombineRGBFunction[0] = c0;
    	textureCombineRGBFunction[1] = c1;
    	textureCombineRGBFunction[2] = c2;

    	if(init)
	    {
		textureCombineRGBFunctionInit[0] = c0;
		textureCombineRGBFunctionInit[1] = c1;
		textureCombineRGBFunctionInit[2] = c2;		        	
		getPickerPan().setTextureCombineRGBFunction(textureCombineRGBFunction); 		    
	    }	
	else
	    {
		getPickerPan().setTextureCombineRGBFunction(textureCombineRGBFunction); 		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_RGB_FUNCTION,textureCombineRGBFunctionInit,textureCombineRGBFunction);
	    }
	
    }


    /**
     * 
     * Internal update of the Texture Combine RGB Function Parameters and fire of an associated property.
     * @param s the Texture Combine RGB Function (C0,C1,C2) between TextureAttributes.COMBINE_SRC_COLOR, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */    
    private void updateTextureCombineRGBFunction(int[] s)
    {
	int []lastTextureCombineRGBFunction = textureCombineRGBFunction;
	textureCombineRGBFunction = s;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_RGB_FUNCTION,
					lastTextureCombineRGBFunction,
					textureCombineRGBFunction);
    }


    /**
     * 
     * Return the Texture Combine RGB Scale.
     * @return the Texture Combine RGB Scale.
     *
     */  
    public int getTextureCombineRGBScale()
    {
	return getPickerPan().getTextureCombineRGBScale();
    }

    /**
     * 
     * Set the Texture Combine RGB Scale.
     * @param s the Texture Combine RGB Scale.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */  
    public void setTextureCombineRGBScale(int s, boolean init)
    {
	textureCombineRGBScale = s;
	
	if(init)
	    {
    		textureCombineRGBScaleInit = s;      
		getPickerPan().setTextureCombineRGBScale(s);		    
	    }
	else
	    {
		getPickerPan().setTextureCombineRGBScale(s);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_RGB_SCALE,textureCombineRGBScaleInit,textureCombineRGBScale);
	    }
    }


    /**
     * 
     * Internal update of the Texture Combine RGB Scale Parameters and fire of an associated property.
     * @param s the Texture Combine RGB Scale.
     *
     */  
    private void updateTextureCombineRGBScale(int s)
    {
	int lastTextureCombineRGBScale = textureCombineRGBScale;
	textureCombineRGBScale = s;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_RGB_SCALE,
					lastTextureCombineRGBScale,
					textureCombineRGBScale);
    }

    /**
     * 
     * Return the Texture Combine Alpha Mode.
     * @return the Texture Combine Alpha Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3.
     *
     */    
    public int getTextureCombineAlphaMode()
    {
	return getPickerPan().getTextureCombineAlphaMode();
    }

    /**
     * 
     * Set the Texture Combine Alpha Mode/
     * @param m the Texture Combine Alpha Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureCombineAlphaMode(int m, boolean init)
    {
	textureCombineAlphaMode = m;

	if(init)
	    {
    		textureCombineAlphaModeInit = m;              	
		getPickerPan().setTextureCombineAlphaMode(m);		    
	    }
	else
	    {
		getPickerPan().setTextureCombineAlphaMode(m);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_ALPHA_MODE,textureCombineAlphaModeInit,textureCombineAlphaMode);
	    }
    }


    /**
     * 
     * Internal update of the Texture Combine Alpha Mode Parameters and fire of an associated property.
     * @param m the Texture Combine Alpha Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3.
     *
     */    
    private void updateTextureCombineAlphaMode(int m)
    {
	int lastTextureCombineAlphaMode = textureCombineAlphaMode;
	textureCombineAlphaMode = m;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_ALPHA_MODE,
					lastTextureCombineAlphaMode,
					textureCombineAlphaMode);
    }
    
    /**
     * 
     * Return the Texture Combine Alpha Source.
     * @return the Texture Combine Alpha Source between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     *
     */    
    public int[] getTextureCombineAlphaSource()
    {
	return getPickerPan().getTextureCombineAlphaSource();
    }

    /**
     * 
     * Set the Texture Combine Alpha Source.
     * @param c0 the Texture Combine Alpha Source C0 between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     * @param c1 the Texture Combine Alpha Source C1 between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     * @param c2 the Texture Combine Alpha Source C2 between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureCombineAlphaSource(int c0, int c1, int c2, boolean init)
    {
    	textureCombineAlphaSource[0] = c0;
    	textureCombineAlphaSource[1] = c1;
    	textureCombineAlphaSource[2] = c2;

    	if(init)
	    {
		textureCombineAlphaSourceInit[0] = c0;
		textureCombineAlphaSourceInit[1] = c1;
		textureCombineAlphaSourceInit[2] = c2;		        	
		getPickerPan().setTextureCombineAlphaSource(textureCombineAlphaSource); 		    
	    }	
	else
	    {
		getPickerPan().setTextureCombineAlphaSource(textureCombineAlphaSource);  
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_ALPHA_SOURCE,textureCombineAlphaSourceInit,textureCombineAlphaSource);		
	    }
    }

    /**
     * 
     * Internal update of the Texture Combine Alpha Source Parameters and fire of an associated property.
     * @param s the Texture Combine Alpha Source (C0,C1,C2) between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     *
     */    
    private void updateTextureCombineAlphaSource(int[] s)
    {
	int []lastTextureCombineAlphaSource = textureCombineAlphaSource;
	textureCombineAlphaSource = s;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_ALPHA_SOURCE,
					lastTextureCombineAlphaSource,
					textureCombineAlphaSource);
    }

    /**
     * 
     * Return the Texture Combine Alpha Function.
     * @return the Texture Combine Alpha Function between TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */    
    public int[] getTextureCombineAlphaFunction()
    {
	return getPickerPan().getTextureCombineAlphaFunction();
    }

    /**
     * 
     * Set the Texture Combine Alpha Function.
     * @param c0 the Texture Combine Alpha Function C0 between TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     * @param c1 the Texture Combine Alpha Function C1 between TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     * @param c2 the Texture Combine Alpha Function C2 between TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */    
    public void setTextureCombineAlphaFunction(int c0, int c1, int c2, boolean init)
    {
    	textureCombineAlphaFunction[0] = c0;
    	textureCombineAlphaFunction[1] = c1;
    	textureCombineAlphaFunction[2] = c2;

    	if(init)
	    {
		textureCombineAlphaFunctionInit[0] = c0;
		textureCombineAlphaFunctionInit[1] = c1;
		textureCombineAlphaFunctionInit[2] = c2;		
		getPickerPan().setTextureCombineAlphaFunction(textureCombineAlphaFunction); 		    
	    }	
	else
	    {
		getPickerPan().setTextureCombineAlphaFunction(textureCombineAlphaFunction); 		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_ALPHA_FUNCTION,textureCombineAlphaFunctionInit,textureCombineAlphaFunction);
	    }
    }

    /**
     * 
     * Internal update of the Texture Combine Alpha Function Parameters and fire of an associated property.
     * @param s the Texture Combine Alpha Function (C0,C1,C2) between TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */    
    private void updateTextureCombineAlphaFunction(int[] s)
    {
	int []lastTextureCombineAlphaFunction = textureCombineAlphaFunction;
	textureCombineAlphaFunction = s;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_ALPHA_FUNCTION,
					lastTextureCombineAlphaFunction,
					textureCombineAlphaFunction);
    }

    /**
     * 
     * Return the Texture Combine Alpha Scale.
     * @return the Texture Combine Alpha Scale.
     *
     */  
    public int getTextureCombineAlphaScale()
    {
	return getPickerPan().getTextureCombineAlphaScale();
    }

    /**
     * 
     * Set the Texture Combine Alpha Scale .
     * @param s the Texture Combine Alpha Scale.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */  
    public void setTextureCombineAlphaScale(int s, boolean init)
    {
	textureCombineAlphaScale = s;

	if(init)
	    {
    		textureCombineAlphaScaleInit = s;      
		getPickerPan().setTextureCombineAlphaScale(s);		    
	    }
	else
	    {
		getPickerPan().setTextureCombineAlphaScale(s);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_ALPHA_SCALE,textureCombineAlphaScaleInit,textureCombineAlphaScale);
	    }
    }

    /**
     * 
     * Internal update of the Texture Combine Alpha Scale Parameters and fire of an associated property.
     * @param s the Texture Combine Alpha Scale.
     *
     */  
    private void updateTextureCombineAlphaScale(int s)
    {
	int lastTextureCombineAlphaScale = textureCombineAlphaScale;
	textureCombineAlphaScale = s;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_COMBINE_ALPHA_SCALE,
					lastTextureCombineAlphaScale,
					textureCombineAlphaScale);
    }

    /**
     * 
     * Return the Texture Setting.
     * @return true if the texture is used, false otherwise.
     *
     */  
    public boolean isTextureSet()
    {
	return getPickerPan().isTextureSet();
    }

    /**
     * 
     * Set the Texture Setting.
     * @param b true if the texture is used, false otherwise.
     * @param init set to true if it replaces the init value. Ie the CANCEL button will set it to this value.
     *
     */  
    public void setTextureUse(boolean b, boolean init)
    {
	textureUse = b;

	if(init)
	    {
    		textureUseInit = b;              	
		getPickerPan().setTextureUse(b);		    
	    }
	else
	    {
		getPickerPan().setTextureUse(b);		    
		boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_USE,textureUseInit,textureUse);
	    }
    }

    /**
     * 
     * Internal update of the Texture Setting Parameters and fire of an associated property.
     * @param b true if the texture is used, false otherwise.
     *
     */  
    private void updateTextureUse(boolean b)
    {	
	boolean lastTextureUse = textureUse;
	textureUse = b;
	boundSupport.firePropertyChange(Appearance3DChooser.TEXTURE_USE,lastTextureUse,textureUse);
    }
    
    
    /**
     * 
     * A Main Exemple.
     *
     */  
    public static void main ( String args[] )
    {	
	/////////// Basic Exemple

	final Appearance3DChooser m1 = new Appearance3DChooser("test1", true, true, true, true);
	m1.runListeners();
	m1.addPropertyChangeListener(Appearance3DChooser.SPECULAR, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {		   		   
		    System.out.println("SPECULAR : (" + m1.getSpecularColor()[0]+ "," 
				       + m1.getSpecularColor()[1]+ "," 
				       + m1.getSpecularColor()[2]+")");
		}});
		
	m1.addPropertyChangeListener(Appearance3DChooser.AMBIENT, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("AMBIENT : (" + m1.getAmbientColor()[0]+ "," 
				       + m1.getAmbientColor()[1] + "," 
				       + m1.getAmbientColor()[2]+")");		    
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooser.EMISSIVE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("EMISSIVE : (" + m1.getEmissiveColor()[0]+ "," 
				       + m1.getEmissiveColor()[1] + "," 
				       + m1.getEmissiveColor()[2]+")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooser.DIFFUSE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("DIFFUSE : (" + m1.getDiffuseColor()[0] + "," 
				       + m1.getDiffuseColor()[1] + "," 
				       + m1.getDiffuseColor()[2]+")");
		}});       
		
	m1.addPropertyChangeListener(Appearance3DChooser.COLORTARGET, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("COLORTARGET : (" + m1.getColorTarget() +")");
		}});        
	
	m1.addPropertyChangeListener(Appearance3DChooser.SHININESS, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("SHININESS : (" + m1.getShininess() + ")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooser.LIGHTING, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("LIGHTING : (" + m1.getLighting() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TRANSPARENCY, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TRANSPARENCY : (" + m1.getTransparency() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TRANSPARENCY_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TRANSPARENCY_MODE : (" + m1.getTransparencyMode() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TRANSPARENCY_SRC_BLEND_FUNCTION, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TRANSPARENCY_SRC_BLEND_FUNCTION : (" + m1.getTransparencySrcBlendFunction() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TRANSPARENCY_DST_BLEND_FUNCTION, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TRANSPARENCY_DST_BLEND_FUNCTION : (" + m1.getTransparencyDstBlendFunction() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TEXTURE_USE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_USE : (" + m1.isTextureSet() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TEXTURE_FILE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_FILE : (" + m1.getTextureFile() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TEXTURE_TRANSFORM, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("\nTEXTURE_TRANSFORM2 : \n" + m1.getTextureTransform() );
		    System.out.println("Scale : x=" + m1.getTextureTransformScaleX() + ", y=" +m1.getTextureTransformScaleY());
		    System.out.println("Translation : x=" + m1.getTextureTransformTranslatX() + ", y=" +m1.getTextureTransformTranslatY());
		    System.out.println("Rotation : z=" + m1.getTextureTransformRotZ());		    
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TEXTURE_PERSPECTIVE_CORRECTION_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_PERSPECTIVE_CORRECTION_MODE : (" + m1.getTexturePerspectiveCorrectionMode() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TEXTURE_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_MODE : (" + m1.getTextureMode() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooser.TEXTURE_BLEND, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_BLEND : (" + 
				       m1.getTextureBlendColor()[0] + "," +
				       m1.getTextureBlendColor()[1] + "," +
				       m1.getTextureBlendColor()[2] + "," +
				       m1.getTextureBlendColor()[3] 
				       + ")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_RGB_MODE : (" + m1.getTextureCombineRGBMode() +")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SOURCE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_RGB_SOURCE : (" + 
				       m1.getTextureCombineRGBSource()[0]+"," +
				       m1.getTextureCombineRGBSource()[1]+"," +
				       m1.getTextureCombineRGBSource()[2] 
				       + ")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_FUNCTION, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_RGB_FUNCTION : (" + 
				       m1.getTextureCombineRGBFunction()[0]+"," +
				       m1.getTextureCombineRGBFunction()[1]+"," +
				       m1.getTextureCombineRGBFunction()[2] 
				       + ")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SCALE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_RGB_SCALE : (" + m1.getTextureCombineRGBScale() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_ALPHA_MODE : (" + m1.getTextureCombineAlphaMode() +")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SOURCE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_ALPHA_SOURCE : (" + 
				       m1.getTextureCombineAlphaSource()[0] + "," +
				       m1.getTextureCombineAlphaSource()[1] + "," +
				       m1.getTextureCombineAlphaSource()[2] 
				       + ")");		    
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_FUNCTION, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_ALPHA_FUNCTION : (" + 
				       m1.getTextureCombineAlphaFunction()[0] + "," +
				       m1.getTextureCombineAlphaFunction()[1] + "," +
				       m1.getTextureCombineAlphaFunction()[2] 
				       + ")");		    
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SCALE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_ALPHA_SCALE : (" + m1.getTextureCombineAlphaScale() +")");
		}});
	


	/////////// A more realistic Exemple

	final Appearance3DChooser m2 = new Appearance3DChooser("test2", true, true, true, true);
	m2.runListeners();
	m2.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) 
		{
		    if (evt.getPropertyName().compareTo(Appearance3DChooser.SPECULAR) == 0)
			{
			    System.out.println("SPECULAR2 : (" + m2.getSpecularColor()[0]+ "," 
					       + m2.getSpecularColor()[1]+ "," 
					       + m2.getSpecularColor()[2]+")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.AMBIENT) == 0)
			{
			    System.out.println("AMBIENT2 : (" + m2.getAmbientColor()[0]+ "," 
					       + m2.getAmbientColor()[1] + "," 
					       + m2.getAmbientColor()[2]+")");		    
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.EMISSIVE) == 0)
			{
			    System.out.println("EMISSIVE2 : (" + m2.getEmissiveColor()[0]+ "," 
					       + m2.getEmissiveColor()[1] + "," 
					       + m2.getEmissiveColor()[2]+")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.DIFFUSE) == 0)
			{
			    System.out.println("DIFFUSE2 : (" + m2.getDiffuseColor()[0] + "," 
					       + m2.getDiffuseColor()[1] + "," 
					       + m2.getDiffuseColor()[2]+")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.COLORTARGET) == 0)
			{
			    System.out.println("COLORTARGET2 : (" + m2.getColorTarget() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.SHININESS) == 0)
			{
			    System.out.println("SHININESS2 : (" + m2.getShininess() + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.LIGHTING) == 0)
			{
			    System.out.println("LIGHTING2 : (" + m2.getLighting() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TRANSPARENCY) == 0)
			{
			    System.out.println("TRANSPARENCY2 : (" + m2.getTransparency() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TRANSPARENCY_MODE) == 0)
			{
			    System.out.println("TRANSPARENCY_MODE2 : (" + m2.getTransparencyMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TRANSPARENCY_SRC_BLEND_FUNCTION) == 0)
			{
			    System.out.println("TRANSPARENCY_SRC_BLEND_FUNCTION2 : (" + m2.getTransparencySrcBlendFunction() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TRANSPARENCY_DST_BLEND_FUNCTION) == 0)
			{
			    System.out.println("TRANSPARENCY_DST_BLEND_FUNCTION2 : (" + m2.getTransparencyDstBlendFunction() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_USE) == 0)
			{
		    	    System.out.println("TEXTURE_USE2 : (" + m2.isTextureSet() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_FILE) == 0)
			{
		    	    System.out.println("TEXTURE_FILE2 : (" + m2.getTextureFile() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_TRANSFORM) == 0)
			{
			    System.out.println("\nTEXTURE_TRANSFORM2 : \n" + m2.getTextureTransform() );
			    System.out.println("Rotation : z=" + m2.getTextureTransformRotZ());	
			    System.out.println("Scale : x=" + m2.getTextureTransformScaleX() + ", y=" +m2.getTextureTransformScaleY());
			    System.out.println("Translation : x=" + m2.getTextureTransformTranslatX() + ", y=" +m2.getTextureTransformTranslatY());		  	    
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_PERSPECTIVE_CORRECTION_MODE) == 0)
			{
		    	    System.out.println("TEXTURE_PERSPECTIVE_CORRECTION_MODE2 : (" + m2.getTexturePerspectiveCorrectionMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_MODE) == 0)
			{
		    	    System.out.println("TEXTURE_MODE2 : (" + m2.getTextureMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooser.TEXTURE_BLEND) == 0)
			{
		    	    System.out.println("TEXTURE_BLEND2 : (" + 
					       m2.getTextureBlendColor()[0] + "," +
					       m2.getTextureBlendColor()[1] + "," +
					       m2.getTextureBlendColor()[2] + "," +
					       m2.getTextureBlendColor()[3] 
					       + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_MODE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_RGB_MODE2 : (" + m2.getTextureCombineRGBMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SOURCE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_RGB_SOURCE2 : (" + 
					       m2.getTextureCombineRGBSource()[0] + "," +
					       m2.getTextureCombineRGBSource()[1] + "," +
					       m2.getTextureCombineRGBSource()[2] 
					       + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_FUNCTION) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_RGB_FUNCTION2 : (" + 
					       m2.getTextureCombineRGBFunction()[0] + "," +
					       m2.getTextureCombineRGBFunction()[1] + "," +
					       m2.getTextureCombineRGBFunction()[2] 
					       + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SCALE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_RGB_SCALE2 : (" + m2.getTextureCombineRGBScale() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_MODE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_ALPHA_MODE2 : (" + m2.getTextureCombineAlphaMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SOURCE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_ALPHA_SOURCE2 : (" + 
					       m2.getTextureCombineAlphaSource()[0] + "," +
					       m2.getTextureCombineAlphaSource()[1] + "," +
					       m2.getTextureCombineAlphaSource()[2]
					       + ")");
			}		    
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_FUNCTION) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_ALPHA_FUNCTION2 : (" + 
					       m2.getTextureCombineAlphaFunction()[0] + "," +
					       m2.getTextureCombineAlphaFunction()[1] + "," +
					       m2.getTextureCombineAlphaFunction()[2]
					       + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SCALE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_ALPHA_SCALE2 : (" + m2.getTextureCombineAlphaScale() +")");
			}		    
		    		   
		}});


	// Test Initialisation

	float []emissiveColor = {1.0f,0.9f,0.8f};
	float []ambientColor = {0.7f,0.6f,0.5f};
	float []diffuseColor = {0.4f,0.3f,0.2f};
	float []specularColor = {0.1f,0.0f,1.0f};
	boolean lighting = false;
	float shininess = 85.0f;
	int colorTarget = Material.SPECULAR;
	float transparency = 0.5f;
	int transparencyMode = TransparencyAttributes.SCREEN_DOOR;
	int transparencySrcBlendFunction = TransparencyAttributes.BLEND_ONE;
	int transparencyDstBlendFunction = TransparencyAttributes.BLEND_SRC_ALPHA;
	Transform3D textureTransform = new Transform3D(); //////////////////////
	boolean textureUse = true;
	URL textureFile = (new Info()).getClass().getResource("resources/TexturePanel-2.png"); 
	int texturePerspectiveCorrectionMode = TextureAttributes.FASTEST;
	int textureMode = TextureAttributes.REPLACE;
	float []textureBlendColor = {0.1f,0.2f,0.3f,0.4f};
	int textureCombineRGBMode = TextureAttributes.COMBINE_INTERPOLATE;
	int []textureCombineRGBSource = {TextureAttributes.COMBINE_OBJECT_COLOR, 
					 TextureAttributes.COMBINE_CONSTANT_COLOR, 
					 TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE};
	int []textureCombineRGBFunction = {TextureAttributes.COMBINE_SRC_ALPHA, 
					   TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, 
					   TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA};
	int textureCombineRGBScale = 2;
	int textureCombineAlphaMode = TextureAttributes.COMBINE_DOT3;
	int []textureCombineAlphaSource = {TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE, 
					   TextureAttributes.COMBINE_TEXTURE_COLOR, 
					   TextureAttributes.COMBINE_OBJECT_COLOR};
	int []textureCombineAlphaFunction = {TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA, 
					     TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA, 
					     TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA};
	int textureCombineAlphaScale = 4;        


	// Set init Value
	
	m2.setAmbientColor(ambientColor[0],ambientColor[1],ambientColor[2], true);
	m2.setDiffuseColor(diffuseColor[0],diffuseColor[1],diffuseColor[2], true); 
	m2.setEmissiveColor(emissiveColor[0],emissiveColor[1],emissiveColor[2], true);  
	m2.setSpecularColor(specularColor[0],specularColor[1],specularColor[2], true);
	m2.setShininess(shininess, true);
	m2.setColorTarget(colorTarget, true);      
	m2.setLighting(lighting, true);
	m2.setTransparency(transparency, true);
	m2.setTransparencyMode(transparencyMode, true);
	m2.setTransparencySrcBlendFunction(transparencySrcBlendFunction, true);
	m2.setTransparencyDstBlendFunction(transparencyDstBlendFunction, true);
	m2.setTextureUse(textureUse, true);
	m2.setTextureTransform(textureTransform, true);	
	m2.setTextureTransformScaleX(4,true);
	m2.setTextureTransformScaleY(2,true);
	m2.setTextureTransformTranslatX(0.5f,true);
	m2.setTextureTransformTranslatY(1,true);
	m2.setTextureTransformRotZ(1.67f,true);
	m2.setTextureFile(textureFile, true);		    	
	m2.setTexturePerspectiveCorrectionMode(texturePerspectiveCorrectionMode, true);
	m2.setTextureMode(textureMode, true);
	m2.setTextureBlendColor(textureBlendColor[0],textureBlendColor[1],textureBlendColor[2],textureBlendColor[3], true);	    
	m2.setTextureCombineRGBMode(textureCombineRGBMode, true);
	m2.setTextureCombineRGBSource(textureCombineRGBSource[0],textureCombineRGBSource[1],textureCombineRGBSource[2], true);
	m2.setTextureCombineRGBFunction(textureCombineRGBFunction[0],textureCombineRGBFunction[1],textureCombineRGBFunction[2], true);
	m2.setTextureCombineRGBScale(textureCombineRGBScale, true);	
	m2.setTextureCombineAlphaMode(textureCombineAlphaMode, true);
	m2.setTextureCombineAlphaSource(textureCombineAlphaSource[0],textureCombineAlphaSource[1],textureCombineAlphaSource[2], true);
	m2.setTextureCombineAlphaFunction(textureCombineAlphaFunction[0],textureCombineAlphaFunction[1],textureCombineAlphaFunction[2], true);
	m2.setTextureCombineAlphaScale(textureCombineAlphaScale, true);	
	

	// Set new values that are not the default ones ... ie Cancel button will unset them
	/*
	  m2.setAmbientColor(ambientColor[0],ambientColor[1],ambientColor[2], false);
	  m2.setDiffuseColor(diffuseColor[0],diffuseColor[1],diffuseColor[2], false); 
	  m2.setEmissiveColor(emissiveColor[0],emissiveColor[1],emissiveColor[2], false);  
	  m2.setSpecularColor(specularColor[0],specularColor[1],specularColor[2], false);
	  m2.setShininess(shininess, false);
	  m2.setColorTarget(colorTarget, false);      
	  m2.setLighting(lighting, false);
	  m2.setTransparency(transparency, false);
	  m2.setTransparencyMode(transparencyMode, false);
	  m2.setTransparencySrcBlendFunction(transparencySrcBlendFunction, false);
	  m2.setTransparencyDstBlendFunction(transparencyDstBlendFunction, false);
	  m2.setTextureUse(textureUse, false);
	  m2.setTextureTransform(textureTransform, false);		
	  m2.setTextureTransformRotZ(2.674f,false);
	  m2.setTextureTransformScaleX(4,false);
	  m2.setTextureTransformScaleY(2,false);
	  m2.setTextureTransformTranslatX(0.5f,false);
	  m2.setTextureTransformTranslatY(1,false);
	  m2.setTextureFile(textureFile, false);		    	
	  m2.setTexturePerspectiveCorrectionMode(texturePerspectiveCorrectionMode, false);
	  m2.setTextureMode(textureMode, false);
	  m2.setTextureBlendColor(textureBlendColor[0],textureBlendColor[1],textureBlendColor[2],textureBlendColor[3], false);	    
	  m2.setTextureCombineRGBMode(textureCombineRGBMode, false);
	  m2.setTextureCombineRGBSource(textureCombineRGBSource[0],textureCombineRGBSource[1],textureCombineRGBSource[2], false);
	  m2.setTextureCombineRGBFunction(textureCombineRGBFunction[0],textureCombineRGBFunction[1],textureCombineRGBFunction[2], false);
	  m2.setTextureCombineRGBScale(textureCombineRGBScale, false);	
	  m2.setTextureCombineAlphaMode(textureCombineAlphaMode, false);
	  m2.setTextureCombineAlphaSource(textureCombineAlphaSource[0],textureCombineAlphaSource[1],textureCombineAlphaSource[2], false);
	  m2.setTextureCombineAlphaFunction(textureCombineAlphaFunction[0],textureCombineAlphaFunction[1],textureCombineAlphaFunction[2], false);
	  m2.setTextureCombineAlphaScale(textureCombineAlphaScale, false);	
	*/




	while(true)
	    {		
		try{		  
		    Thread.currentThread().sleep(60000); //sleep for 60000 ms		    
		}
		catch(Exception ie){
		    //If this thread was interrupted by nother thread 
		}
		
		if(!m1.isVisible() && !m2.isVisible())
		    System.exit(0);
	    }		
    }
    
}