/**
   ##############################################################################
   ##                                                                          ##
   ## Appearance3DChooser                                                      ##
   ##                                                                          ##
   ## Copyright (C) 2009  Frederic Roudaut  <frederic.roudaut@free.fr>         ##
   ##                                                                          ##
   ##                                                                          ##
   ## This program is free software: you can redistribute it and/or modify     ##
   ## it under the terms of the GNU General Public License as published by     ##
   ## the Free Software Foundation, either version 3 of the License, or        ##
   ## (at your option) any later version.                                      ##
   ##                                                                          ##
   ## This program is distributed in the hope that it will be useful,          ##
   ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
   ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
   ## GNU General Public License for more details.                             ##
   ##                                                                          ##
   ## You should have received a copy of the GNU General Public License        ##
   ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
   ##                                                                          ##
   ##                                                                          ##
   ##############################################################################
**/


package com.appearance3Dchooser;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.event.*;
import javax.swing.*;
import java.net.URL;

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;


import com.colorpicker.swing.*;
import javax.vecmath.Color3f;
import javax.media.j3d.Material;
import javax.media.j3d.TransparencyAttributes;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TextureAttributes;

/**
 *  Appearance3DChooserPanel Class.<br/>
 *   A basic class with a Panel for updating your objects appearances. It takes into account following attributes :
 *   <ul>
 *       <li><strong>Material</strong></li>
 *       <li><strong>TransparencyAttributes</strong></li>
 *       <li><strong>TextureAttributes</strong></li>
 *       <li><strong>Lighting</strong></li>
 *      </ul>
 *   </li>

 * <br/> An instance of this object fire a Property Event when a dedicated parameter is modified. It distinguishes the following events :
 * <ul>
 * <br/>
 * <li>Material Panel 
 *     <ul>
 *     <li>SPECULAR </li>
 *     <li>AMBIENT </li>
 *     <li>EMISSIVE </li>
 *     <li>DIFFUSE </li>
 *     <li>COLORTARGET </li>
 *     <li>SHININESS  </li>
 *     <li>LIGHTING  </li>
 *     </ul>  
 *     <br/>
 *     </li>
 *
 *  <li>Transparency Panel
 *     <ul>
 *     <li>TRANSPARENCY  </li>
 *     <li>TRANSPARENCY_MODE  </li>
 *     <li>TRANSPARENCY_SRC_BLEND_FUNCTION  </li>
 *     <li>TRANSPARENCY_DST_BLEND_FUNCTION  </li>
 *     </ul>
 *     <br/>
 *     </li>
 *
 * <li> Texture Panel
 *     <ul>
 *     <li>TEXTURE_USE  </li>
 *     <li>TEXTURE_FILE  </li>
 *     <li>TEXTURE_TRANSFORM  </li>
 *     <li>TEXTURE_PERSPECTIVE_CORRECTION_MODE  </li>
 *     <li>TEXTURE_MODE  </li>
 *     <li>TEXTURE_BLEND  </li> 
 *     <li>TEXTURE_COMBINE_RGB_MODE  </li> 
 *     <li>TEXTURE_COMBINE_RGB_SOURCE  </li> 
 *     <li>TEXTURE_COMBINE_RGB_FUNCTION  </li> 
 *     <li>TEXTURE_COMBINE_RGB_SCALE  </li>
 *     <li>TEXTURE_COMBINE_ALPHA_MODE  </li> 
 *     <li>TEXTURE_COMBINE_ALPHA_SOURCE  </li> 
 *     <li>TEXTURE_COMBINE_ALPHA_FUNCTION  </li> 
 *     <li>TEXTURE_COMBINE_ALPHA_SCALE  </li>
 *     </ul>
 *     <br/>
 *     </li>
 * </li>
 * </ul>
 * 
 */    
public class Appearance3DChooserPanel extends JPanel implements ActionListener, ChangeListener, ItemListener
{	
    private static final long serialVersionUID = 1L;
	
    public static final String SPECULAR = "specular";
    public static final String AMBIENT = "ambient";
    public static final String EMISSIVE = "emissive";
    public static final String DIFFUSE = "diffuse";
    public static final String COLORTARGET = "colortarget";
    public static final String SHININESS = "shininess";
    public static final String LIGHTING = "lighting";
    public static final String TRANSPARENCY = "transparency";
    public static final String TRANSPARENCY_MODE = "transparency_mode";
    public static final String TRANSPARENCY_SRC_BLEND_FUNCTION = "transparency_src_blend_function";
    public static final String TRANSPARENCY_DST_BLEND_FUNCTION = "transparency_dst_blend_function";
    public static final String TEXTURE_FILE = TexturePane.TEXTURE_FILE;
    public static final String TEXTURE_USE = TexturePane.TEXTURE_USE;
    public static final String TEXTURE_TRANSFORM = TexturePane.TEXTURE_TRANSFORM;
    public static final String TEXTURE_PERSPECTIVE_CORRECTION_MODE = TexturePane.TEXTURE_PERPSPECTIVE_CORRECTION_MODE;
    public static final String TEXTURE_MODE = TexturePane.TEXTURE_MODE;
    public static final String TEXTURE_BLEND = TexturePane.TEXTURE_BLEND;
    public static final String TEXTURE_COMBINE_RGB_MODE = TexturePane.TEXTURE_COMBINE_RGB_MODE;
    public static final String TEXTURE_COMBINE_RGB_SOURCE = TexturePane.TEXTURE_COMBINE_RGB_SOURCE;
    public static final String TEXTURE_COMBINE_RGB_FUNCTION = TexturePane.TEXTURE_COMBINE_RGB_FUNCTION;
    public static final String TEXTURE_COMBINE_RGB_SCALE = TexturePane.TEXTURE_COMBINE_RGB_SCALE;
    public static final String TEXTURE_COMBINE_ALPHA_MODE = TexturePane.TEXTURE_COMBINE_ALPHA_MODE;
    public static final String TEXTURE_COMBINE_ALPHA_SOURCE = TexturePane.TEXTURE_COMBINE_ALPHA_SOURCE;
    public static final String TEXTURE_COMBINE_ALPHA_FUNCTION = TexturePane.TEXTURE_COMBINE_ALPHA_FUNCTION;
    public static final String TEXTURE_COMBINE_ALPHA_SCALE = TexturePane.TEXTURE_COMBINE_ALPHA_SCALE;

    public static final int COLORTARGET_AMBIENT=Material.AMBIENT; 
    public static final int COLORTARGET_EMISSIVE=Material.EMISSIVE;
    public static final int COLORTARGET_DIFFUSE=Material.DIFFUSE;
    public static final int COLORTARGET_SPECULAR=Material.SPECULAR;
    public static final int COLORTARGET_AMBIENT_AND_DIFFUSE=Material.AMBIENT_AND_DIFFUSE;     
        
    public static final int TRANSPARENCY_MODE_FASTEST=0;
    public static final int TRANSPARENCY_MODE_NICEST=1; 
    public static final int TRANSPARENCY_MODE_BLENDED=2;
    public static final int TRANSPARENCY_MODE_SCREEN_DOOR=3;
    public static final int TRANSPARENCY_MODE_NONE=4;

    public static final int TRANSPARENCY_SRC_BLEND_ZERO=0;
    public static final int TRANSPARENCY_SRC_BLEND_ONE=1;
    public static final int TRANSPARENCY_SRC_BLEND_SRC_ALPHA=2;
    public static final int TRANSPARENCY_SRC_BLEND_ONE_MINUS_SRC_ALPHA=3;
    public static final int TRANSPARENCY_SRC_BLEND_DST_COLOR=4;
    public static final int TRANSPARENCY_SRC_BLEND_ONE_MINUS_DST_COLOR=5;
    public static final int TRANSPARENCY_DST_BLEND_ZERO=0;
    public static final int TRANSPARENCY_DST_BLEND_ONE=1;
    public static final int TRANSPARENCY_DST_BLEND_SRC_ALPHA=2;
    public static final int TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_ALPHA=3;
    public static final int TRANSPARENCY_DST_BLEND_SRC_COLOR=4;
    public static final int TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_COLOR=5;

    
    protected float[] ambientColor = {0.2f,0.2f,0.2f};
    protected float[] diffuseColor = {1.0f,1.0f,1.0f};	
    protected float[] specularColor = {1.0f,1.0f,1.0f};	
    protected float[] emissiveColor = {0.36f,0.85f,0.80f};
    protected float shininess = 64;
    protected int colorTarget = COLORTARGET_EMISSIVE;
    protected boolean lighting = true;
    protected int transparencyMode = TRANSPARENCY_MODE_NONE;
    protected int transparencySrcBlendFunction = TRANSPARENCY_SRC_BLEND_SRC_ALPHA;
    protected int transparencyDstBlendFunction = TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_ALPHA;    
    protected float transparency = 0;
    protected URL DefaultTextureName = (new Info()).getClass().getResource("resources/DefaultTexture.png");
    protected URL textureName = DefaultTextureName;
    protected boolean isTextureSet;
    protected Transform3D textureTransform;
    protected int texturePerspectiveCorrectionMode = TextureAttributes.NICEST;
    protected int textureMode = TextureAttributes.REPLACE;
    protected float[] textureBlendColor = {0.5f,0.5f,0.5f,0.5f};
    protected int[] textureCombineRGBSource = {TextureAttributes.COMBINE_TEXTURE_COLOR, 
					       TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE, 
					       TextureAttributes.COMBINE_CONSTANT_COLOR};
    protected int[] textureCombineRGBFunction = {TextureAttributes.COMBINE_SRC_COLOR, 
						 TextureAttributes.COMBINE_SRC_COLOR, 
						 TextureAttributes.COMBINE_SRC_COLOR};
    protected int textureCombineRGBMode = TextureAttributes.COMBINE_MODULATE;
    protected int textureCombineRGBScale = 1;
    protected int textureCombineAlphaMode = TextureAttributes.COMBINE_MODULATE;
    protected int[] textureCombineAlphaSource = {TextureAttributes.COMBINE_TEXTURE_COLOR, 
						 TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE, 
						 TextureAttributes.COMBINE_CONSTANT_COLOR};
    protected int[] textureCombineAlphaFunction = {TextureAttributes.COMBINE_SRC_ALPHA, 
						   TextureAttributes.COMBINE_SRC_ALPHA, 
						   TextureAttributes.COMBINE_SRC_ALPHA};
    protected int textureCombineAlphaScale = 1;

    protected JTabbedPane mainTabbedPane;
    protected JTabbedPane MaterialTabPane;
    protected JPanel ambientColorPanel;
    protected JPanel diffuseColorPanel; 
    protected JPanel specularColorPanel; 
    protected JPanel emissiveColorPanel;
    protected JPanel miscellaneousPanel;
    protected JPanel transparencyPanel;  
    protected TexturePane texturePane;
    protected RenderingViewPanel renderingViewPanel;
    
    protected ColorPicker ambientColorPicker;
    protected ColorPicker diffuseColorPicker;
    protected ColorPicker emissiveColorPicker;
    protected ColorPicker specularColorPicker;
    protected JComboBox listColorTarget;
    protected JComboBox listTransparencyMode;
    protected JComboBox listTransparencySrcBlendFunction;
    protected JComboBox listTransparencyDstBlendFunction;
    protected JSlider sliderShininess;
    protected JSpinner spinnerShininess;
    protected JCheckBox checkBoxLighting;
    protected JSlider sliderTransparency;
    protected JSpinner spinnerTransparency;

    protected boolean showMaterial, showTransparency, showTexture, showRendering;

    /**
     * Main Appearance3DChooserPanel constructor. Generate a Panel with all the Panes associated to control 
     * some of the JAVA 3D Appearence attributes.  
     *
     * @param showMaterial set it to true if you want to see the Material Panel.
     * @param showTransparency set it to true if you want to see the Transparency Panel.
     * @param showTexture set it to true if you want to see the Texture Panel.
     * @param showRendering set it to true if you want to see the Rendering Panel.
     */
    public Appearance3DChooserPanel(boolean showMaterial, boolean showTransparency, boolean showTexture, boolean showRendering)
    {		
	this.showMaterial = showMaterial;
	this.showTransparency = showTransparency;
	this.showTexture = showTexture; 
	this.showRendering = showRendering;
	mainTabbedPane = new JTabbedPane();
		
	makeMaterialPane();	
	makeTransparencyPane();
	makeTexturePane();
	makeRenderPane();

	add(mainTabbedPane);
    }

    /**
     *
     * Create a Rendering Pane.
     *
     */
    private void makeRenderPane()
    {		
	// Show Rendering
	renderingViewPanel = new RenderingViewPanel(DefaultTextureName);
	if(showRendering)
	    mainTabbedPane.addTab(Info.ressources.getObject("Render").toString(),renderingViewPanel);			
    }

    /**
     *
     * Create a Texture Pane.
     *
     */
    private void makeTexturePane()
    {		
	DefaultTextureName = (new Info()).getClass().getResource("resources/DefaultTexture.png");	

	// Texture Pane
	texturePane = new TexturePane(DefaultTextureName);


	textureTransform = new Transform3D(); 

	texturePane.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    if(evt.getPropertyName().compareTo(TexturePane.TEXTURE_FILE) == 0)
			{
			    updateTexture(texturePane.getTextureFile());				
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_USE) == 0)
			{
			    updateIsTextureSet(texturePane.isTextureSet());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_TRANSFORM) == 0)
			{
			    updateTextureTransform(texturePane.getTextureTransform());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_PERPSPECTIVE_CORRECTION_MODE) == 0)
			{
			    updateTexturePerspectiveCorrectionMode(texturePane.getTexturePerspectiveCorrectionMode());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_MODE) == 0)
			{
			    updateTextureMode(texturePane.getTextureMode());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_BLEND) == 0)
			{
			    updateTextureBlendColor(texturePane.getTextureBlendColor());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_COMBINE_RGB_MODE) == 0)
			{			   
			    updateTextureCombineRGBMode(texturePane.getTextureCombineRGBMode());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_COMBINE_RGB_SOURCE) == 0)
			{
			    updateTextureCombineRGBSource(texturePane.getTextureCombineRGBSource());  	    
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_COMBINE_RGB_FUNCTION) == 0)
			{
			    updateTextureCombineRGBFunction(texturePane.getTextureCombineRGBFunction());       
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_COMBINE_RGB_SCALE) == 0)
			{
			    updateTextureCombineRGBScale(texturePane.getTextureCombineRGBScale());  	       
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_COMBINE_ALPHA_MODE) == 0)
			{
			    updateTextureCombineAlphaMode(texturePane.getTextureCombineAlphaMode());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_COMBINE_ALPHA_SOURCE) == 0)
			{
			    updateTextureCombineAlphaSource(texturePane.getTextureCombineAlphaSource());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_COMBINE_ALPHA_FUNCTION) == 0)
			{
			    updateTextureCombineAlphaFunction(texturePane.getTextureCombineAlphaFunction());
			}
		    else if (evt.getPropertyName().compareTo(TexturePane.TEXTURE_COMBINE_ALPHA_SCALE) == 0)
			{
			    updateTextureCombineAlphaScale(texturePane.getTextureCombineAlphaScale());
			}
		}});
	
	if(showTexture)
	    mainTabbedPane.addTab(Info.ressources.getObject("Texture").toString(),texturePane);				
    }


    /**
     *
     * Create a Transparency Pane.
     *
     */    
    private void makeTransparencyPane()
    {
	// Transparency Panel
	transparencyPanel = new JPanel();
	GridBagConstraints c = new GridBagConstraints();

	transparencyPanel.setLayout(new GridBagLayout());
	c.gridx = 0; c.gridy = 0;
	c.weightx = 0; c.weighty = 0; 
	c.insets = new Insets(0,0,0,0);
	c.fill = GridBagConstraints.WEST;
	c.anchor = GridBagConstraints.CENTER;
	
	JLabel transparencyModeLabel = new JLabel(Info.ressources.getObject("Transparency_Mode").toString());
	c.gridwidth=2;
	transparencyPanel.add(transparencyModeLabel, c);
	
	String [] transparencyModeChoices = new String[5];
	transparencyModeChoices[TRANSPARENCY_MODE_BLENDED] = "BLENDED";
	transparencyModeChoices[TRANSPARENCY_MODE_FASTEST] = "FASTEST";
	transparencyModeChoices[TRANSPARENCY_MODE_NICEST] = "NICEST";
	transparencyModeChoices[TRANSPARENCY_MODE_NONE] = "NONE";
	transparencyModeChoices[TRANSPARENCY_MODE_SCREEN_DOOR] = "SCREEN_DOOR";	           
	listTransparencyMode = new JComboBox(transparencyModeChoices);
	listTransparencyMode.setSelectedIndex(transparencyMode);
	listTransparencyMode.addActionListener(this);		
	c.gridy++;
	transparencyPanel.add(listTransparencyMode, c);
		
	JLabel transparencySrcBlendFunctionLabel = new JLabel(Info.ressources.getObject("Source_Blend_Function").toString());
	c.gridy++;
	c.insets = new Insets(30,0,0,0);
	transparencyPanel.add(transparencySrcBlendFunctionLabel, c);
	c.insets = new Insets(0,0,0,0);
		
	String [] transparencySrcBlendFunctionChoice = new String[6];
	transparencySrcBlendFunctionChoice[TRANSPARENCY_SRC_BLEND_ZERO] = "BLEND_ZERO";
	transparencySrcBlendFunctionChoice[TRANSPARENCY_SRC_BLEND_ONE] = "BLEND_ONE";
	transparencySrcBlendFunctionChoice[TRANSPARENCY_SRC_BLEND_SRC_ALPHA] = "BLEND_SRC_ALPHA";
	transparencySrcBlendFunctionChoice[TRANSPARENCY_SRC_BLEND_ONE_MINUS_SRC_ALPHA] = "BLEND_ONE_MINUS_SRC_ALPHA";
	transparencySrcBlendFunctionChoice[TRANSPARENCY_SRC_BLEND_DST_COLOR] = "BLEND_DST_COLOR";	           
	transparencySrcBlendFunctionChoice[TRANSPARENCY_SRC_BLEND_ONE_MINUS_DST_COLOR]="BLEND_ONE_MINUS_DST_COLOR";
	listTransparencySrcBlendFunction = new JComboBox(transparencySrcBlendFunctionChoice);
	listTransparencySrcBlendFunction.setSelectedIndex(transparencySrcBlendFunction);
	listTransparencySrcBlendFunction.addActionListener(this);
	c.gridy++;
	transparencyPanel.add(listTransparencySrcBlendFunction, c);			

	JLabel transparencyDstBlendFunctionLabel = new JLabel(Info.ressources.getObject("Destination_Blend_Function").toString());
	c.gridy++;
	c.insets = new Insets(30,0,0,0);
	transparencyPanel.add(transparencyDstBlendFunctionLabel, c);
	c.insets = new Insets(0,0,0,0);

	String [] transparencyDstBlendFunctionChoice = new String[6];
	transparencyDstBlendFunctionChoice[TRANSPARENCY_DST_BLEND_ZERO] = "BLEND_ZERO";
	transparencyDstBlendFunctionChoice[TRANSPARENCY_DST_BLEND_ONE] = "BLEND_ONE";
	transparencyDstBlendFunctionChoice[TRANSPARENCY_DST_BLEND_SRC_ALPHA] = "BLEND_SRC_ALPHA";
	transparencyDstBlendFunctionChoice[TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_ALPHA] = "BLEND_ONE_MINUS_SRC_ALPHA";
	transparencyDstBlendFunctionChoice[TRANSPARENCY_DST_BLEND_SRC_COLOR] = "BLEND_SRC_COLOR"; 
	transparencyDstBlendFunctionChoice[TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_COLOR]="BLEND_ONE_MINUS_SRC_COLOR";
	listTransparencyDstBlendFunction = new JComboBox(transparencyDstBlendFunctionChoice);
	listTransparencyDstBlendFunction.setSelectedIndex(transparencyDstBlendFunction);
	listTransparencyDstBlendFunction.addActionListener(this);
	c.gridy++;
	transparencyPanel.add(listTransparencyDstBlendFunction, c);
	


	JLabel transparencyLabel = new JLabel(Info.ressources.getObject("Transparency").toString());
	c.gridy++;
	c.insets = new Insets(30,0,0,0);
	transparencyPanel.add(transparencyLabel, c);
	c.insets = new Insets(0,0,0,0);
	sliderTransparency = new JSlider(JSlider.HORIZONTAL,0,100,0);
	sliderTransparency.addChangeListener(this);
	c.gridy ++;
	c.gridwidth=1;
	transparencyPanel.add(sliderTransparency, c);

	SpinnerModel transparencySpinnerModel = new SpinnerNumberModel(Float.valueOf(0.0f), //initial value
								       Float.valueOf(0.0f), //min
								       Float.valueOf(1.0f), //max
								       Float.valueOf(0.01f)); //step
	spinnerTransparency = new JSpinner(transparencySpinnerModel);
	((JSpinner.DefaultEditor)spinnerTransparency.getEditor()).getTextField().setColumns(2);
	spinnerTransparency.addChangeListener(this);

	c.gridx ++;
	transparencyPanel.add(spinnerTransparency,c);

	if(showTransparency)
	    mainTabbedPane.addTab(Info.ressources.getObject("Transparency").toString(),transparencyPanel);		
    }


    /**
     *
     * Create a Material Pane.
     *
     */        
    private void makeMaterialPane()
    {
	GridBagConstraints c;
	MaterialTabPane = new JTabbedPane();
	MaterialTabPane.setTabPlacement(JTabbedPane.LEFT);

	// Panel For emissive color
	emissiveColorPanel = new JPanel();
	emissiveColorPanel.setLayout(new GridBagLayout());
	c = new GridBagConstraints();
	c.fill = GridBagConstraints.CENTER;
	emissiveColorPicker = new ColorPicker(true,false);
	emissiveColorPicker.setRGB((int)(emissiveColor[0]*255), (int)(emissiveColor[1]*255), (int)(emissiveColor[2]*255));
	emissiveColorPanel.add(emissiveColorPicker,c);
	emissiveColorPicker.addPropertyChangeListener(ColorPicker.HEX_COLOR_PROPERTY, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {				
		    float rgb[] = new float[3];
		    rgb[0]=(float)emissiveColorPicker.getRGB()[0] / 255;
		    rgb[1]=(float)emissiveColorPicker.getRGB()[1] / 255;
		    rgb[2]=(float)emissiveColorPicker.getRGB()[2] / 255;
		    updateEmissiveColor(rgb);		    
		}
	    });
	
	MaterialTabPane.addTab(Info.ressources.getObject("Emissive").toString(),emissiveColorPanel);

	// Panel For ambient color
	ambientColorPanel = new JPanel();
	ambientColorPanel.setLayout(new GridBagLayout());
	c = new GridBagConstraints();
	c.fill = GridBagConstraints.CENTER;
	ambientColorPicker = new ColorPicker(true,false); 
	ambientColorPicker.setRGB((int)(ambientColor[0]*255), (int)(ambientColor[1]*255), (int)(ambientColor[2]*255));
	ambientColorPanel.add(ambientColorPicker,c);
	ambientColorPicker.addPropertyChangeListener(ColorPicker.HEX_COLOR_PROPERTY, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {	
		    float rgb[] = new float[3];
		    rgb[0]=(float)ambientColorPicker.getRGB()[0] / 255;
		    rgb[1]=(float)ambientColorPicker.getRGB()[1] / 255;
		    rgb[2]=(float)ambientColorPicker.getRGB()[2] / 255;				
		    updateAmbientColor(rgb);				
		}
	    });
	MaterialTabPane.addTab(Info.ressources.getObject("Ambient").toString(),ambientColorPanel);

	// Panel For diffuse color
	diffuseColorPanel = new JPanel();
	diffuseColorPanel.setLayout(new GridBagLayout());
	c = new GridBagConstraints();
	c.fill = GridBagConstraints.CENTER;
	diffuseColorPicker = new ColorPicker(true,false);
	diffuseColorPicker.setRGB((int)(diffuseColor[0]*255), (int)(diffuseColor[1]*255), (int)(diffuseColor[2]*255));
	diffuseColorPanel.add(diffuseColorPicker,c);
	diffuseColorPicker.addPropertyChangeListener(ColorPicker.HEX_COLOR_PROPERTY, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    float rgb[] = new float[3];
		    rgb[0]=(float)diffuseColorPicker.getRGB()[0] / 255;
		    rgb[1]=(float)diffuseColorPicker.getRGB()[1] / 255;
		    rgb[2]=(float)diffuseColorPicker.getRGB()[2] / 255;				
		    updateDiffuseColor(rgb);
		}				   
	    });
	MaterialTabPane.addTab(Info.ressources.getObject("Diffuse").toString(),diffuseColorPanel);
	
	// Panel For specular color
	specularColorPanel = new JPanel();
	specularColorPanel.setLayout(new GridBagLayout());
	c = new GridBagConstraints();
	c.fill = GridBagConstraints.CENTER;
	specularColorPicker = new ColorPicker(true,false);     
	specularColorPicker.setRGB((int)(specularColor[0]*255), (int)(specularColor[1]*255), (int)(specularColor[2]*255));
	specularColorPanel.add(specularColorPicker,c);
	specularColorPicker.addPropertyChangeListener(ColorPicker.HEX_COLOR_PROPERTY, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {		
		    float rgb[] = new float[3];
		    rgb[0]=(float)specularColorPicker.getRGB()[0] / 255;
		    rgb[1]=(float)specularColorPicker.getRGB()[1] / 255;
		    rgb[2]=(float)specularColorPicker.getRGB()[2] / 255;				
		    updateSpecularColor(rgb);
		}
	    });

	MaterialTabPane.addTab(Info.ressources.getObject("Specular").toString(),specularColorPanel);
	
	// Miscellaneous Panel : Lighting, Shininess, Target Color
	miscellaneousPanel = new JPanel();		
	c = new GridBagConstraints();
	miscellaneousPanel.setLayout(new GridBagLayout());
	c.gridx = 0; c.gridy = 0;
	c.weightx = 0; c.weighty = 0; 
	c.insets = new Insets(0,0,0,0);
	c.fill = GridBagConstraints.CENTER;
	c.anchor = GridBagConstraints.CENTER;
	c.gridwidth = 2;
	
	checkBoxLighting = new JCheckBox(Info.ressources.getObject("Lighting").toString());
	checkBoxLighting.setSelected(true);
	checkBoxLighting.addItemListener(this);		
	
	miscellaneousPanel.add(checkBoxLighting, c);
	c.gridx = 0; c.gridy++; 
	c.insets = new Insets(30,0,0,0);
	c.gridwidth = 1;
	

	JLabel sliderShininessLabel = new JLabel(Info.ressources.getObject("Shininess").toString(), JLabel.CENTER);
        sliderShininessLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	miscellaneousPanel.add(sliderShininessLabel, c);
	c.gridy++; 
	c.insets = new Insets(0,0,0,0);

	sliderShininess = new JSlider(JSlider.HORIZONTAL, 0, 128, (int)shininess);
	sliderShininess.setMajorTickSpacing(32);
	sliderShininess.setMinorTickSpacing(4);
	sliderShininess.setPaintTicks(true);
	sliderShininess.setPaintLabels(true);
	sliderShininess.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
	Font font = new Font("Serif", Font.ITALIC, 15);
	sliderShininess.setFont(font);
	sliderShininess.addChangeListener(this);
	miscellaneousPanel.add(sliderShininess, c);

	SpinnerModel shininessSpinnerModel = new SpinnerNumberModel(Float.valueOf(shininess), //initial value
								    Float.valueOf(0.0f), //min
								    Float.valueOf(128.0f), //max
								    Float.valueOf(0.1f)); //step
	spinnerShininess = new JSpinner(shininessSpinnerModel);
	((JSpinner.DefaultEditor)spinnerShininess.getEditor()).getTextField().setColumns(3);
	spinnerShininess.addChangeListener(this);

	c.gridx ++;
	c.insets = new Insets(0,0,20,0);
	miscellaneousPanel.add(spinnerShininess,c);
	c.insets = new Insets(0,0,0,0);
	c.gridx =0;

	c.gridy++; 	
	c.insets = new Insets(30,0,0,0);
	c.gridwidth = 2;

	JLabel listColorTargetLabel = new JLabel(Info.ressources.getObject("Color_Target").toString(), JLabel.CENTER);
        listColorTargetLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
	miscellaneousPanel.add(listColorTargetLabel, c);
	c.gridy++; 
	c.insets = new Insets(0,0,0,0);	

	String [] ColorTargetChoices = new String[5];
	ColorTargetChoices[COLORTARGET_AMBIENT] = "AMBIENT";
	ColorTargetChoices[COLORTARGET_EMISSIVE] = "EMISSIVE";
	ColorTargetChoices[COLORTARGET_DIFFUSE] = "DIFFUSE";
	ColorTargetChoices[COLORTARGET_SPECULAR] = "SPECULAR";
	ColorTargetChoices[COLORTARGET_AMBIENT_AND_DIFFUSE] = "AMBIENT_AND_DIFFUSE";
	           
	listColorTarget = new JComboBox(ColorTargetChoices);
	listColorTarget.setSelectedIndex(colorTarget);
	listColorTarget.addActionListener(this);
	miscellaneousPanel.add(listColorTarget, c);


	MaterialTabPane.addTab(Info.ressources.getObject("Miscellaneous").toString(),miscellaneousPanel);

	if(showMaterial)
	    mainTabbedPane.addTab(Info.ressources.getObject("Material").toString(),MaterialTabPane);
    }


    /**
     * 
     * Called when an action is performed on the Transparency Pane mainly.
     * @param e the related action.
     *
     */                       
    public void actionPerformed(ActionEvent e) 
    {	
	Object source = e.getSource();
	if(source == listTransparencyMode)
	    {
		updateTransparencyMode(listTransparencyMode.getSelectedIndex());	    
	    }
	else if(source == listTransparencySrcBlendFunction)
	    {
		updateTransparencySrcBlendFunction(listTransparencySrcBlendFunction.getSelectedIndex());
	    }
	else if(source == listTransparencyDstBlendFunction)
	    {
		updateTransparencyDstBlendFunction(listTransparencyDstBlendFunction.getSelectedIndex());
	    }
	else if(source == listColorTarget)
	    {
		updateColorTarget(listColorTarget.getSelectedIndex());
	    }
    }


    /**
     * 
     * Called when a state changes on the Transparency and the Material Pane mainly (ie a slider is used ...).
     * @param e the related event.
     *
     */                       
    public void stateChanged(ChangeEvent e) 
    {
	Object source = e.getSource();
	if(source == sliderTransparency)
	    {      
		float o = Float.valueOf(sliderTransparency.getValue()) / 100;
		//For safety reason
		o = ((float) (Math.round (o*100))) / 100;
		updateTransparency(o);
		spinnerTransparency.setValue(o);
	    }
	else if(source == spinnerTransparency)
	    {
		float o = (Float)spinnerTransparency.getValue();
		o = ((float) (Math.round (o*100))) / 100;
		updateTransparency(o);		    
		sliderTransparency.setValue(Math.round(o*100));
		spinnerTransparency.setValue(o);
	    }
	else if(source == sliderShininess)
	    {
		float s = Float.valueOf(sliderShininess.getValue());
		//For safety reason
		s = ((float) (Math.round (s*10))) / 10;
		updateShininess(s);
		spinnerShininess.setValue(s);
	    }
	else if(source == spinnerShininess)
	    {
		float s = (Float)(spinnerShininess.getValue());
		s = ((float) (Math.round (s*10))) / 10;
		updateShininess(s);		   
		sliderShininess.setValue(Math.round(s));
	    }
    }


    /**
     * 
     * Called when the Lighting checkbox in the Material Pane is clicked.
     * @param e the related event.
     *
     */                           
    public void itemStateChanged(ItemEvent e)
    {			
	Object source = e.getSource();
	if(source == checkBoxLighting)
	    {
		updateLighting(checkBoxLighting.isSelected());
	    }
    }	        
	

    /**
     * 
     * Return the transparency value.
     * @return the amount of transparency to be applied to the Appearance component object. 
     * The transparency values are in the range [0.0, 1.0], with 0.0 being fully opaque and 1.0 being fully transparent. 
     *
     */                               
    public float getTransparency ()
    {
	return transparency;
    }
    
    /**
     * 
     * Set the transparency value.
     * @param o the amount of transparency to be applied to the Appearance component object. 
     * The transparency values are in the range [0.0, 1.0], with 0.0 being fully opaque and 1.0 being fully transparent. 
     *
     */                               
    public void setTransparency (float o)
    {
	sliderTransparency.setValue((int)(o*100));
	spinnerTransparency.setValue(o);
	transparency = o;
    }
    

    /**
     * 
     * Internal update of the transparency value Parameters and fire of an associated property.
     * @param o the amount of transparency to be applied to the Appearance component object. 
     * The transparency values are in the range [0.0, 1.0], with 0.0 being fully opaque and 1.0 being fully transparent. 
     *
     */                               
    private void updateTransparency (float o)
    {
	float lastTransparency = transparency;
	transparency = o;
	renderingViewPanel.setObjectsTransparency(o);
	firePropertyChange(TRANSPARENCY,lastTransparency,transparency);
    }
   
    /**
     * 
     * Return the transparency Mode.
     * @return the transparency Mode between TransparencyAttributes.FASTEST, TransparencyAttributes.NICEST, TransparencyAttributes.SCREEN_DOOR, TransparencyAttributes.BLENDED, TransparencyAttributes.NONE 
     *
     */                               
    public int getTransparencyMode ()
    {
	return convertExternTransparencyMode(transparencyMode);
    }
    
    /**
     * 
     * Set the transparency Mode.
     * @param m the transparency Mode between TransparencyAttributes.FASTEST, TransparencyAttributes.NICEST, TransparencyAttributes.SCREEN_DOOR, TransparencyAttributes.BLENDED, TransparencyAttributes.NONE 
     *
     */                                  
    public void setTransparencyMode (int m)
    {
	listTransparencyMode.setSelectedIndex(convertInternTransparencyMode(m));
	transparencyMode = convertInternTransparencyMode(m);
    }
    
    /**
     * 
     * Internal update of the transparency Mode Parameters and fire of an associated property.
     * @param m the transparency Mode between TransparencyAttributes.FASTEST, TransparencyAttributes.NICEST, TransparencyAttributes.SCREEN_DOOR, TransparencyAttributes.BLENDED, TransparencyAttributes.NONE 
     *
     */                                  
    private void updateTransparencyMode (int m)
    {
	int lastTransparencyMode = transparencyMode;
	transparencyMode = m;
	renderingViewPanel.setObjectsTransparencyMode(convertExternTransparencyMode(m));
	firePropertyChange(TRANSPARENCY_MODE,lastTransparencyMode,transparencyMode);
    }

    /**
     * 
     * Return the transparency Source Blend Function.
     * @return the transparency Source Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_DST_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR.
     *
     */    
    public int getTransparencySrcBlendFunction ()
    {
	return convertExternTransparencySrcBlendFunction(transparencySrcBlendFunction);
    }
    
    /**
     * 
     * Set the transparency Source Blend Function.
     * @param m the transparency Source Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_DST_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR.
     *
     */    
    public void setTransparencySrcBlendFunction (int m)
    {
	listTransparencySrcBlendFunction.setSelectedIndex(convertInternTransparencySrcBlendFunction(m));
	transparencySrcBlendFunction = convertInternTransparencySrcBlendFunction(m);
    }

    /**
     * 
     * Internal update of the transparency Source Blend Function Parameters and fire of an associated property.
     * @param f the transparency Source Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_DST_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR.
     *
     */    
    private void updateTransparencySrcBlendFunction(int f)
    {
	int lastTransparencySrcBlendFunction = transparencySrcBlendFunction; 
	transparencySrcBlendFunction = f;
	renderingViewPanel.setObjectsTransparencySrcBlendFunction(convertExternTransparencySrcBlendFunction(f));
	firePropertyChange(TRANSPARENCY_SRC_BLEND_FUNCTION,lastTransparencySrcBlendFunction,transparencySrcBlendFunction);
    }

    /**
     * 
     * Return the transparency Destination Blend Function.
     * @return the transparency Destination Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_SRC_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR.
     *
     */    
    public int getTransparencyDstBlendFunction ()
    {
	return convertExternTransparencyDstBlendFunction(transparencyDstBlendFunction);
    }
    
    /**
     * 
     * Set the transparency Destination Blend Function.
     * @param m the transparency Destination Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_SRC_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR.
     *
     */    
    public void setTransparencyDstBlendFunction (int m)
    {
	listTransparencyDstBlendFunction.setSelectedIndex(convertInternTransparencyDstBlendFunction(m));
	transparencyDstBlendFunction = convertInternTransparencyDstBlendFunction(m);
    }

    /**
     * 
     * Internal update of the transparency Destination Blend Function Parameters and fire of an associated property.
     * @param f the transparency Destination Blend Function between TransparencyAttributes.BLEND_ZERO, TransparencyAttributes.BLEND_ONE, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA, 
     * TransparencyAttributes.BLEND_SRC_COLOR, TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR.
     *
     */    
    private void updateTransparencyDstBlendFunction(int f)
    {
	int lastTransparencyDstBlendFunction = transparencyDstBlendFunction; 
	transparencyDstBlendFunction = f;
	renderingViewPanel.setObjectsTransparencyDstBlendFunction(convertExternTransparencyDstBlendFunction(f));
	firePropertyChange(TRANSPARENCY_DST_BLEND_FUNCTION,lastTransparencyDstBlendFunction,transparencyDstBlendFunction);
    }

    /**
     * 
     * Return the Lighting mode.
     * @return true if lighting is enabled for the object, false otherwise.
     *
     */                               
    public boolean getLighting ()
    {
	return lighting;
    }
    
    /**
     * 
     * Set the Lighting mode.
     * @param l true lighting is enabled for the object, false otherwise.
     *
     */                               
    public void setLighting (boolean l)
    {
	checkBoxLighting.setSelected(l);
	lighting = l;
    }

    /**
     * 
     * Internal update of the Lighting mode Parameters and fire of an associated property.
     * @param l true lighting is enabled for the object, false otherwise.
     *
     */                               
    private void updateLighting (boolean l)
    {
	boolean lastLighting = lighting;
	lighting = l;	
	renderingViewPanel.setObjectsLighting(l);
	firePropertyChange(LIGHTING,lastLighting,lighting);
    }

    /**
     * 
     * Return the Color Target.
     * @return the Color Target. One of: Material.AMBIENT, Material.EMISSIVE, Material.DIFFUSE, Material.SPECULAR, 
     * or Material.AMBIENT_AND_DIFFUSE.
     *
     */                               
    public int getColorTarget ()
    {
	return colorTarget;
    }

    /**
     * 
     * Set the Color Target.
     * @param c the Color Target. One of: Material.AMBIENT, Material.EMISSIVE, Material.DIFFUSE, Material.SPECULAR, 
     * or Material.AMBIENT_AND_DIFFUSE.
     *
     */                               
    public void setColorTarget (int c)
    {
	listColorTarget.setSelectedIndex(c);
	colorTarget = c;
    }

    /**
     * 
     * Internal update of the Color Target Parameters and fire of an associated property.
     * @param c the Color Target. One of: Material.AMBIENT, Material.EMISSIVE, Material.DIFFUSE, Material.SPECULAR, 
     * or Material.AMBIENT_AND_DIFFUSE.
     *
     */                               
    private void updateColorTarget (int c)
    {
	int lastColorTarget = colorTarget;
	colorTarget = c;
	firePropertyChange(COLORTARGET,lastColorTarget,colorTarget);
    }

    /**
     * 
     * Return the Shininess.
     * @return the Shininess in the range [1.0, 128.0] with 1.0 being not shiny and 128.0 being very shiny.
     *
     */                               
    public float getShininess ()
    {
	return shininess;
    }

    /**
     * 
     * Set the Shininess.
     * @param s the Shininess in the range [1.0, 128.0] with 1.0 being not shiny and 128.0 being very shiny.
     *
     */                               
    public void setShininess (float s)
    {
	sliderShininess.setValue((int)(s));
	spinnerShininess.setValue(s);
	shininess = s;
    }

    /**
     * 
     * Internal update of the Shininess Parameters and fire of an associated property.
     * @param s the Shininess in the range [1.0, 128.0] with 1.0 being not shiny and 128.0 being very shiny.
     *
     */                               
    private void updateShininess (float s)
    {
	float lastShininess = shininess;
	shininess = s;
	renderingViewPanel.setObjectsShininess((float)s);
	firePropertyChange(SHININESS,lastShininess,shininess);
    }
		
    /**
     * 
     * Return the Ambient Color.
     * @return the Ambient Color in (R,G,B).
     *
     */                               
    public float [] getAmbientColor ()
    {
	return ambientColor;
    }
	
    /**
     * 
     * Set the Ambient Color
     * @param c the Ambient Color in (R,G,B).
     *
     */
    public void setAmbientColor (float []c)
    {	
	ambientColorPicker.setRGB((int)(c[0]*255), (int)(c[1]*255), (int)(c[2]*255));
	ambientColor = c;
    }

    /**
     * 
     * Internal update of the Ambient Color Parameters and fire of an associated property.
     * @param c the Ambient Color in (R,G,B).
     *
     */                           
    private void updateAmbientColor (float []c)
    {
	float []lastAmbient = ambientColor;
	ambientColor = c;
	renderingViewPanel.setObjectsAmbientColor(new Color3f(c[0],c[1],c[2]));
	firePropertyChange(AMBIENT,lastAmbient,ambientColor);
    }
	
    /**
     * 
     * Return the Diffuse Color.
     * @return the Diffuse Color in (R,G,B).
     *
     */                               
    public float [] getDiffuseColor ()
    {
	return diffuseColor;
    }

    /**
     * 
     * Set the Diffuse Color
     * @param c the Diffuse Color in (R,G,B).
     *
     */
    public void setDiffuseColor (float []c)
    {	
	diffuseColorPicker.setRGB((int)(c[0]*255), (int)(c[1]*255), (int)(c[2]*255));
	diffuseColor = c;
    }

    /**
     * 
     * Internal update of the Diffuse Color Parameters and fire of an associated property.
     * @param c the Diffuse Color in (R,G,B).
     *
     */                           
    private void updateDiffuseColor (float []c)
    {
	float []lastDiffuse = diffuseColor;
	diffuseColor = c;
	renderingViewPanel.setObjectsDiffuseColor(new Color3f(c[0],c[1],c[2]));
	firePropertyChange(DIFFUSE,lastDiffuse,diffuseColor);
    }
		
    /**
     * 
     * Return the Specular Color.
     * @return the Specular Color in (R,G,B).
     *
     */                           
    public float [] getSpecularColor ()
    {
	return specularColor;
    }
	
    /**
     * 
     * Set the Specular Color.
     * @param c the Specular Color in (R,G,B).
     *
     */                           
    public void setSpecularColor (float []c)
    {
	specularColorPicker.setRGB((int)(c[0]*255), (int)(c[1]*255), (int)(c[2]*255));
	specularColor = c;	
    }

    private void updateSpecularColor (float []c)
    {
	float []lastSpecular = specularColor;
	specularColor = c;
	renderingViewPanel.setObjectsSpecularColor(new Color3f(c[0],c[1],c[2]));
	firePropertyChange(SPECULAR,lastSpecular,specularColor);
    }
	
    /**
     * 
     * Return the Emissive Color.
     * @return the Emissive Color in (R,G,B).
     *
     */                               	
    public float [] getEmissiveColor ()
    {
	return emissiveColor;
    }
	
    /**
     * 
     * Set the Emissive Color.
     * @param c the Emissive Color in (R,G,B).
     *
     */
    public void setEmissiveColor (float []c)
    { 
	emissiveColorPicker.setRGB((int)(c[0]*255), (int)(c[1]*255), (int)(c[2]*255));
	emissiveColor = c;      
    }

    /**
     * 
     * Internal update of the Emissive Color Parameters and fire of an associated property.
     * @param c the Emissive Color in (R,G,B).
     *
     */                           
    private void updateEmissiveColor (float []c)
    {
	float []lastEmissiveColor = emissiveColor;
	emissiveColor = c;
	renderingViewPanel.setObjectsEmissiveColor(new Color3f(c[0],c[1],c[2]));
	firePropertyChange(EMISSIVE,lastEmissiveColor,emissiveColor);
    }

    /**
     * 
     * Return the Texture Setting.
     * @return true if the texture is used, false otherwise.
     *
     */  
    public boolean isTextureSet()
    {
	return isTextureSet;
    }

    /**
     * 
     * Set the Texture Setting.
     * @param b true if the texture is used, false otherwise.
     *
     */  
    public void setTextureUse(boolean b)
    {
	texturePane.setTextureUse(b);
	isTextureSet = b;
    }

    /**
     * 
     * Internal update of the Texture Setting Parameters and fire of an associated property.
     * @param on true if the texture is used, false otherwise.
     *
     */  
    private void updateIsTextureSet(boolean on)
    {
	boolean lastIsTextureSet = isTextureSet;
	isTextureSet = on;
	
	if(on)
	    {
		renderingViewPanel.setTexture(textureName);
	    }	
	else
	    renderingViewPanel.unsetTexture();

	firePropertyChange(TEXTURE_USE,lastIsTextureSet,on);
    }

    /**
     * 
     * Return the Texture File URL.
     * @return the Texture File URL.
     *
     */    
    public URL getTextureFile()
    {
	return texturePane.getTextureFile();
    }

    /**
     * 
     * Set the Texture File URL.
     * @param t the Texture File URL.
     *
     */    
    public void setTextureFile(URL t)
    {
	texturePane.setTextureFile(t);
    }

    /**
     * 
     * Set the Default Texture File URL.
     * @param t the Default Texture File URL.
     *
     */    
    public void setDefaultTextureFile(URL t)
    {
	texturePane.setDefaultTextureFile(t);
    }


    /**
     * 
     * Internal update of the Texture File URL Parameters and fire of an associated property.
     * @param t the Texture File URL.
     *
     */    
    private void updateTexture(URL t)
    {
	URL lastTextureName = textureName;
	textureName = t;	
	if (textureName != null && textureName.getPath().compareTo("") != 0 && isTextureSet)
	    {
		renderingViewPanel.setTexture(textureName);	       
	    }

	firePropertyChange(TEXTURE_FILE,lastTextureName,textureName);
    }				

    /**
     * 
     * Return the Texture Transform.
     * @return the Texture Transform.
     *
     */    
    public Transform3D getTextureTransform()
    {
	return new Transform3D(textureTransform);
    }

    /**
     * 
     * Set the Texture Transform.
     * @param t the Texture Transform.
     *
     */    
    public void setTextureTransform(Transform3D t)
    {
	texturePane.setTextureTransform(t);
	textureTransform = t;
    }

    /**
     * 
     * Internal update of the Texture Transform Parameters and fire of an associated property.
     * @param t the Texture Transform.
     *
     */    
    private void updateTextureTransform(Transform3D t)
    {
	Transform3D lastTextureTransform = new Transform3D(textureTransform);
	textureTransform = new Transform3D(t);		
	renderingViewPanel.setTextureTransform(textureTransform);	       
	firePropertyChange(TEXTURE_TRANSFORM,lastTextureTransform,textureTransform);
    }				

    /**
     * 
     * Return the Texture Transform Scale on X axis.
     * @return the Texture Transform Scale on X axis.
     *
     */    
    public float getTextureTransformScaleX()
    {
	return texturePane.getTextureTransformScaleX();
    }

    /**
     * 
     * Set the Texture Transform Scale on X axis.
     * @param v the Texture Transform Scale on X axis.
     *
     */    
    public void setTextureTransformScaleX(float v)
    {
	
	texturePane.setTextureTransformScaleX(v);
    }
    
    /**
     * 
     * Return the Texture Transform Scale on Y axis.
     * @return the Texture Transform Scale on Y axis.
     *
     */    
    public float getTextureTransformScaleY()
    {
	return texturePane.getTextureTransformScaleY();
    }    

    /**
     * 
     * Set the Texture Transform Scale on Y axis.
     * @param v the Texture Transform Scale on Y axis.
     *
     */    
    public void setTextureTransformScaleY(float v)
    {
	
	texturePane.setTextureTransformScaleY(v);
    }
    
    /**
     * 
     * Return the Texture Transform Translation on X axis.
     * @return the Texture Transform Translation on X axis.
     *
     */    
    public float getTextureTransformTranslatX()
    {
	return texturePane.getTextureTransformTranslatX();
    }

    /**
     * 
     * Set the Texture Transform Translation on X axis.
     * @param v the Texture Transform Translation on X axis.
     *
     */    
    public void setTextureTransformTranslatX(float v)
    {
	
	texturePane.setTextureTransformTranslatX(v);
    }
        
    /**
     * 
     * Return the Texture Transform Translation on Y axis.
     * @return the Texture Transform Translation on Y axis.
     *
     */    
    public float getTextureTransformTranslatY()
    {
	return texturePane.getTextureTransformTranslatY();
    }

    /**
     * 
     * Set the Texture Transform Translation on Y axis.
     * @param v the Texture Transform Translation on Y axis.
     *
     */    
    public void setTextureTransformTranslatY(float v)
    {
	
	texturePane.setTextureTransformTranslatY(v);
    }

    /**
     * 
     * Return the Texture Transform Rotation on Z axis.
     * @return the Texture Transform Rotation on Z axis.
     *
     */    
    public float getTextureTransformRotZ()
    {
	return texturePane.getTextureTransformRotZ();
    }

    /**
     * 
     * Set the Texture Transform Rotation on Z axis.
     * @param v the Texture Transform Rotation on Z axis.
     *
     */    
    public void setTextureTransformRotZ(float v)
    {	
	texturePane.setTextureTransformRotZ(v);
    }

    /**
     * 
     * Return the Texture Perspective Correction Mode.
     * @return the Texture Perspective Correction Mode between TextureAttributes.NICEST, TextureAttributes.FASTEST.
     *
     */    
    public int getTexturePerspectiveCorrectionMode()
    {
	return texturePerspectiveCorrectionMode;	
    }

    /**
     * 
     * Set the Texture Perspective Correction Mode.
     * @param m the Texture Perspective Correction Mode between TextureAttributes.NICEST, TextureAttributes.FASTEST. 
     *
     */    
    public void setTexturePerspectiveCorrectionMode(int m)
    {
	texturePane.setTexturePerspectiveCorrectionMode(m);
	texturePerspectiveCorrectionMode = m;      
    } 

    protected void updateTexturePerspectiveCorrectionMode(int m)
    {	
	int lastTexturePerspectiveCorrectionMode = texturePerspectiveCorrectionMode;
	texturePerspectiveCorrectionMode = m;
	renderingViewPanel.setObjectsTexturePerspectiveCorrectionMode(m);	       
	firePropertyChange(TEXTURE_PERSPECTIVE_CORRECTION_MODE,lastTexturePerspectiveCorrectionMode,texturePerspectiveCorrectionMode);	
    }

    /**
     * 
     * Return the Texture Mode.
     * @return the Texture Mode between TextureAttributes.MODULATE, TextureAttributes.DECAL, TextureAttributes.BLEND, 
     * TextureAttributes.REPLACE, TextureAttributes.COMBINE.
     *
     */    
    public int getTextureMode()
    {
	return textureMode;	
    }

    /**
     * 
     * Set the Texture Mode.
     * @param m the Texture Mode between TextureAttributes.MODULATE, TextureAttributes.DECAL, TextureAttributes.BLEND, 
     * TextureAttributes.REPLACE, TextureAttributes.COMBINE.
     *
     */    
    public void setTextureMode(int m)
    {
	texturePane.setTextureMode(m);
	textureMode = m;
    }

    /**
     * 
     * Internal update of the Texture Mode Parameters and fire of an associated property.
     * @param m the Texture Mode between TextureAttributes.MODULATE, TextureAttributes.DECAL, TextureAttributes.BLEND, 
     * TextureAttributes.REPLACE, TextureAttributes.COMBINE.
     *
     */    
    protected void updateTextureMode(int m)
    {	
	int lastTextureMode = textureMode;
	textureMode = m;	
	renderingViewPanel.setObjectsTextureMode(m);	       
	firePropertyChange(TEXTURE_MODE,lastTextureMode,textureMode);	
    }

    /**
     * 
     * Return the Texture Blend Color.
     * @return the Texture Blend Color in (R,G,B,A).
     *
     */  
    public float [] getTextureBlendColor ()
    {
	return textureBlendColor;
    }
	
    /**
     * 
     * Set the Texture Blend Color. 
     * @param c the Texture Blend Color in (R,G,B,A).
     *
     */  
    public void setTextureBlendColor (float []c)
    {
	texturePane.setTextureBlendColor(c);	       
	textureBlendColor = c;	
    }

    /**
     * 
     * Internal update of the Texture Blend Color Parameters and fire of an associated property.
     * @param c the Texture Blend Color in (R,G,B,A).
     *
     */  
    private void updateTextureBlendColor (float []c)
    {
	float []lastTextureBlendColor = textureBlendColor;
	textureBlendColor = c;
	renderingViewPanel.setObjectsTextureBlendColor(c[0],c[1],c[2],c[3]);
	firePropertyChange(TEXTURE_BLEND,lastTextureBlendColor, textureBlendColor);
    }

    /**
     * 
     * Return the Texture Combine RGB Mode.
     * @return the Texture Combine RGB Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3 
     *
     */    
    public int getTextureCombineRGBMode ()
    {
	return textureCombineRGBMode;
    }

    /**
     * 
     * Set the Texture Combine RGB Mode.
     * @param m the Texture Combine RGB Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3.
     *
     */    
    public void setTextureCombineRGBMode (int m)
    {
	texturePane.setTextureCombineRGBMode(m);	
	textureCombineRGBMode = m;
    }

    /**
     * 
     * Internal update of the Texture Combine RGB Mode Parameters and fire of an associated property.
     * @param m the Texture Combine RGB Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3. 
     *
     */    
    private void updateTextureCombineRGBMode(int m)
    {
	int lastTextureCombineRGBMode = textureCombineRGBMode;
	textureCombineRGBMode = m;	
	renderingViewPanel.setObjectsTextureCombineRGBMode(m);
	firePropertyChange(TEXTURE_COMBINE_RGB_MODE,lastTextureCombineRGBMode, textureCombineRGBMode);
    }

    /**
     * 
     * Return the Texture Combine RGB Source.
     * @return the Texture Combine RGB Source between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     *
     */    
    public int[] getTextureCombineRGBSource ()
    {
	return textureCombineRGBSource;
    }

    /**
     * 
     * Set the Texture Combine RGB Source
     * @param m the Texture Combine RGB Source (C0,C1,C2) between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE
     *
     */
    public void setTextureCombineRGBSource (int []m)
    {
	texturePane.setTextureCombineRGBSource(m);		       
	textureCombineRGBSource = m;
    }

    /**
     * 
     * Internal update of the Texture Combine RGB Source Parameters and fire of an associated property.
     * @param m the Texture Combine RGB Source (C0,C1,C2) between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.     
     *
     */    
    private void updateTextureCombineRGBSource(int []m)
    {
	int []lastTextureCombineRGBSource = textureCombineRGBSource;
	textureCombineRGBSource = m;
	renderingViewPanel.setObjectsTextureCombineRGBSource(m);
	firePropertyChange(TEXTURE_COMBINE_RGB_SOURCE,lastTextureCombineRGBSource, textureCombineRGBSource);
    }

    /**
     * 
     * Return the Texture Combine RGB Function.
     * @return the Texture Combine RGB Function between TextureAttributes.COMBINE_SRC_COLOR, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */    
    public int[] getTextureCombineRGBFunction ()
    {
	return textureCombineRGBFunction;
    }

    /**
     * 
     * Set the Texture Combine RGB Function.
     * @param m the Texture Combine RGB Function (C0,C1,C2) between TextureAttributes.COMBINE_SRC_COLOR, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */
    public void setTextureCombineRGBFunction (int []m)
    {
	texturePane.setTextureCombineRGBFunction(m);	
	textureCombineRGBFunction = m;
    }

    /**
     * 
     * Internal update of the Texture Combine RGB Function Parameters and fire of an associated property.
     * @param m the Texture Combine RGB Function (C0,C1,C2) between TextureAttributes.COMBINE_SRC_COLOR, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */    
    private void updateTextureCombineRGBFunction(int []m)
    {
	int []lastTextureCombineRGBFunction = textureCombineRGBFunction;
	textureCombineRGBFunction = m;
	renderingViewPanel.setObjectsTextureCombineRGBFunction(m);
	firePropertyChange(TEXTURE_COMBINE_RGB_FUNCTION,lastTextureCombineRGBFunction, textureCombineRGBFunction);
    }

    /**
     * 
     * Return the Texture Combine RGB Scale.
     * @return the Texture Combine RGB Scale.
     *
     */  
    public int getTextureCombineRGBScale ()
    {
	return textureCombineRGBScale;
    }

    /**
     * 
     * Set the Texture Combine RGB Scale.
     * @param s the Texture Combine RGB Scale.
     *
     */  
    public void setTextureCombineRGBScale (int s)
    {
	texturePane.setTextureCombineRGBScale(s);	
	textureCombineRGBScale = s;
    }

    /**
     * 
     * Internal update of the Texture Combine RGB Scale Parameters and fire of an associated property.
     * @param m the Texture Combine RGB Scale.
     *
     */  
    private void updateTextureCombineRGBScale(int m)
    {
	int lastTextureCombineRGBScale = textureCombineRGBScale;
	textureCombineRGBScale = m;
	renderingViewPanel.setObjectsTextureCombineRGBScale(m);
	firePropertyChange(TEXTURE_COMBINE_RGB_SCALE,lastTextureCombineRGBScale, textureCombineRGBScale);
    }

    /**
     * 
     * Return the Texture Combine Alpha Mode.
     * @return the Texture Combine Alpha Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3.
     *
     */    
    public int getTextureCombineAlphaMode ()
    {
	return textureCombineAlphaMode;
    }
    
    /**
     * 
     * Set the Texture Combine Alpha Mode.
     * @param m the Texture Combine Alpha Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3.
     *
     */    
    public void setTextureCombineAlphaMode (int m)
    {
	texturePane.setTextureCombineAlphaMode(m);	
	textureCombineAlphaMode = m;
    }

    /**
     * 
     * Internal update of the Texture Combine Alpha Mode Parameters and fire of an associated property.
     * @param m the Texture Combine Alpha Mode between TextureAttributes.COMBINE_REPLACE, TextureAttributes.COMBINE_MODULATE, 
     * TextureAttributes.COMBINE_ADD, TextureAttributes.COMBINE_ADD_SIGNED, TextureAttributes.COMBINE_SUBTRACT, 
     * TextureAttributes.COMBINE_INTERPOLATE, TextureAttributes.COMBINE_DOT3.
     *
     */    
    private void updateTextureCombineAlphaMode(int m)
    {
	int lastTextureCombineAlphaMode = textureCombineAlphaMode;
	textureCombineAlphaMode = m;	
	renderingViewPanel.setObjectsTextureCombineAlphaMode(m);
	firePropertyChange(TEXTURE_COMBINE_ALPHA_MODE,lastTextureCombineAlphaMode, textureCombineAlphaMode);
    }

    /**
     * 
     * Return the Texture Combine Alpha Source.
     * @return the Texture Combine Alpha Source between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     *
     */    
    public int[] getTextureCombineAlphaSource ()
    {
	return textureCombineAlphaSource;
    }

    /**
     * 
     * Set the Texture Combine Alpha Source.
     * @param m the Texture Combine Alpha Source (C0,C1,C2) between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     *
     */
    public void setTextureCombineAlphaSource (int []m)
    {
	texturePane.setTextureCombineAlphaSource(m);	       
	textureCombineAlphaSource = m;
    }

    /**
     * 
     * Internal update of the Texture Combine Alpha Source Parameters and fire of an associated property.
     * @param m the Texture Combine Alpha Source (C0,C1,C2) between TextureAttributes.COMBINE_OBJECT_COLOR, 
     * TextureAttributes.COMBINE_TEXTURE_COLOR, TextureAttributes.COMBINE_CONSTANT_COLOR, 
     * TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE.
     *
     */    
    private void updateTextureCombineAlphaSource(int []m)
    {
	int []lastTextureCombineAlphaSource = textureCombineAlphaSource;
	textureCombineAlphaSource = m;
	renderingViewPanel.setObjectsTextureCombineAlphaSource(m);
	firePropertyChange(TEXTURE_COMBINE_ALPHA_SOURCE,lastTextureCombineAlphaSource, textureCombineAlphaSource);
    }

    /**
     * 
     * Return the Texture Combine Alpha Function.
     * @return the Texture Combine Alpha Function between TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */    
    public int[] getTextureCombineAlphaFunction ()
    {
	return textureCombineAlphaFunction;
    }

    /**
     * 
     * Set the Texture Combine Alpha Function.
     * @param m the Texture Combine Alpha Function (C0,C1,C2) between TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */
    public void setTextureCombineAlphaFunction (int []m)
    {
	texturePane.setTextureCombineAlphaFunction(m);	
	textureCombineAlphaFunction = m;
    }

    /**
     * 
     * Internal update of the Texture Combine Alpha Function Parameters and fire of an associated property.
     * @param m the Texture Combine Alpha Function (C0,C1,C2) between TextureAttributes.COMBINE_SRC_ALPHA, 
     * TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA.
     *
     */    
    private void updateTextureCombineAlphaFunction(int []m)
    {
	int []lastTextureCombineAlphaFunction = textureCombineAlphaFunction;
	textureCombineAlphaFunction = m;
	renderingViewPanel.setObjectsTextureCombineAlphaFunction(m);	
	firePropertyChange(TEXTURE_COMBINE_ALPHA_FUNCTION,lastTextureCombineAlphaFunction, textureCombineAlphaFunction);
    }

    /**
     * 
     * Return the Texture Combine Alpha Scale.
     * @return the Texture Combine Alpha Scale.
     *
     */  
    public int getTextureCombineAlphaScale ()
    {
	return textureCombineAlphaScale;
    }

    /**
     * 
     * Set the Texture Combine Alpha Scale.
     * @param s the Texture Combine Alpha Scale.
     *
     */  
    public void setTextureCombineAlphaScale (int s)
    {
	texturePane.setTextureCombineAlphaScale(s);	
	textureCombineAlphaScale = s;
    }

    /**
     * 
     * Internal update of the Texture Combine Alpha Scale Parameters and fire of an associated property.
     * @param m the Texture Combine Alpha Scale.
     *
     */  
    private void updateTextureCombineAlphaScale(int m)
    {
	int lastTextureCombineAlphaScale = textureCombineAlphaScale;
	textureCombineAlphaScale = m;
	renderingViewPanel.setObjectsTextureCombineAlphaScale(m);
	firePropertyChange(TEXTURE_COMBINE_ALPHA_SCALE,lastTextureCombineAlphaScale, textureCombineAlphaScale);
    }

    /**
     * 
     * Convert a Transparency Mode from TransparencyAttributes representation to Internal representation.
     * @param m a Transparency Mode in TransparencyAttributes representation.
     * @return a Transparency Mode in Internal representation.
     *
     */                	
    public static int convertInternTransparencyMode (int m)
    {
	switch (m)
	    {
	    case TransparencyAttributes.BLENDED:
		return TRANSPARENCY_MODE_BLENDED;
	    case TransparencyAttributes.FASTEST:
		return TRANSPARENCY_MODE_FASTEST;
	    case TransparencyAttributes.NICEST:
		return TRANSPARENCY_MODE_NICEST; 
	    case TransparencyAttributes.NONE:
		return TRANSPARENCY_MODE_NONE;
	    case TransparencyAttributes.SCREEN_DOOR:
		return TRANSPARENCY_MODE_SCREEN_DOOR;
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Transparency Mode from Internal representation to TransparencyAttributes representation.
     * @param m a Transparency Mode in Internal representation.
     * @return a Transparency Mode in TransparencyAttributes representation.
     *
     */                	
    public static int convertExternTransparencyMode (int m)
    {
	switch (m)
	    {
	    case TRANSPARENCY_MODE_BLENDED:
		return TransparencyAttributes.BLENDED;
	    case TRANSPARENCY_MODE_FASTEST:
		return TransparencyAttributes.FASTEST;
	    case TRANSPARENCY_MODE_NICEST:
		return TransparencyAttributes.NICEST; 
	    case TRANSPARENCY_MODE_NONE:
		return TransparencyAttributes.NONE;
	    case TRANSPARENCY_MODE_SCREEN_DOOR:
		return TransparencyAttributes.SCREEN_DOOR;
	    default :
		return -1;
	    }
    }   

    /**
     * 
     * Convert a Transparency Source Blend Function from Internal representation to TransparencyAttributes representation.
     * @param f a Transparency Source Blend Function in Internal representation.
     * @return a Transparency Source Blend Function in TransparencyAttributes representation.
     *
     */                	
    public static int convertExternTransparencySrcBlendFunction (int f)
    {
	switch (f)
	    {
	    case TRANSPARENCY_SRC_BLEND_ZERO:
		return TransparencyAttributes.BLEND_ZERO;
	    case TRANSPARENCY_SRC_BLEND_ONE: 
		return TransparencyAttributes.BLEND_ONE;
	    case TRANSPARENCY_SRC_BLEND_SRC_ALPHA:
		return TransparencyAttributes.BLEND_SRC_ALPHA;
	    case TRANSPARENCY_SRC_BLEND_ONE_MINUS_SRC_ALPHA:
		return TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA;
	    case TRANSPARENCY_SRC_BLEND_DST_COLOR:
		return TransparencyAttributes.BLEND_DST_COLOR;
	    case TRANSPARENCY_SRC_BLEND_ONE_MINUS_DST_COLOR:
		return TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR;
	    default :
		return -1;		
	    }
    }
    
    /**
     * 
     * Convert a Transparency Source Blend Function from TransparencyAttributes representation to Internal representation.
     * @param f a Transparency Source Blend Function in TransparencyAttributes representation.
     * @return a Transparency Source Blend Function in Internal representation.
     *
     */                	
    public static int convertInternTransparencySrcBlendFunction (int f)
    {
	switch (f)
	    {
	    case TransparencyAttributes.BLEND_ZERO:
		return TRANSPARENCY_SRC_BLEND_ZERO;
	    case TransparencyAttributes.BLEND_ONE: 
		return TRANSPARENCY_SRC_BLEND_ONE;
	    case TransparencyAttributes.BLEND_SRC_ALPHA:
		return TRANSPARENCY_SRC_BLEND_SRC_ALPHA;
	    case TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA:
		return TRANSPARENCY_SRC_BLEND_ONE_MINUS_SRC_ALPHA;
	    case TransparencyAttributes.BLEND_DST_COLOR:
		return TRANSPARENCY_SRC_BLEND_DST_COLOR;
	    case TransparencyAttributes.BLEND_ONE_MINUS_DST_COLOR:
		return TRANSPARENCY_SRC_BLEND_ONE_MINUS_DST_COLOR;
	    default :
		return -1;		
	    }
    }

    /**
     * 
     * Convert a Transparency Destination Blend Function from Internal representation to TransparencyAttributes representation.
     * @param f a Transparency Destination Blend Function in Internal representation.
     * @return a Transparency Destination Blend Function in TransparencyAttributes representation.
     *
     */                	
    public static int convertExternTransparencyDstBlendFunction (int f)
    {
	switch (f)
	    {
	    case TRANSPARENCY_DST_BLEND_ZERO:
		return TransparencyAttributes.BLEND_ZERO;
	    case TRANSPARENCY_DST_BLEND_ONE: 
		return TransparencyAttributes.BLEND_ONE;
	    case TRANSPARENCY_DST_BLEND_SRC_ALPHA:
		return TransparencyAttributes.BLEND_SRC_ALPHA;
	    case TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_ALPHA:
		return TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA;
	    case TRANSPARENCY_DST_BLEND_SRC_COLOR:
		return TransparencyAttributes.BLEND_SRC_COLOR;
	    case TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_COLOR:
		return TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR;
	    default :
		return -1;		
	    }
    }

    /**
     * 
     * Convert a Transparency Destination Blend Function from TransparencyAttributes representation to Internal representation.
     * @param f a Transparency Destination Blend Function in TransparencyAttributes representation.
     * @return a Transparency Destination Blend Function in Internal representation.
     *
     */                	
    public static int convertInternTransparencyDstBlendFunction (int f)
    {
	switch (f)
	    {
	    case TransparencyAttributes.BLEND_ZERO:
		return TRANSPARENCY_DST_BLEND_ZERO;
	    case TransparencyAttributes.BLEND_ONE: 
		return TRANSPARENCY_DST_BLEND_ONE;
	    case TransparencyAttributes.BLEND_SRC_ALPHA:
		return TRANSPARENCY_DST_BLEND_SRC_ALPHA;
	    case TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA:
		return TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_ALPHA;
	    case TransparencyAttributes.BLEND_SRC_COLOR:
		return TRANSPARENCY_DST_BLEND_SRC_COLOR;
	    case TransparencyAttributes.BLEND_ONE_MINUS_SRC_COLOR:
		return TRANSPARENCY_DST_BLEND_ONE_MINUS_SRC_COLOR;
	    default :
		return -1;		
	    }
    }

	
    /**
     * 
     * A Main Exemple.
     *
     */  
    public static void main ( String args[] )
    {
	final Appearance3DChooserPanel m1 = new Appearance3DChooserPanel(true, true, true, true);
	final Appearance3DChooserPanel m2 = new Appearance3DChooserPanel(true, true, true, true);

	m1.addPropertyChangeListener(Appearance3DChooserPanel.AMBIENT, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("AMBIENT : (" + m1.getAmbientColor()[0]+ "," + m1.getAmbientColor()[1] + "," + m1.getAmbientColor()[2]+")");		    
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.SPECULAR, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("SPECULAR : (" + m1.getSpecularColor()[0]+ "," + m1.getSpecularColor()[1]+ "," + m1.getSpecularColor()[2]+")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.EMISSIVE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("EMISSIVE : (" + m1.getEmissiveColor()[0]+ "," + m1.getEmissiveColor()[1] + "," + m1.getEmissiveColor()[2]+")");
		  
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.DIFFUSE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("DIFFUSE : (" + m1.getDiffuseColor()[0] + "," + m1.getDiffuseColor()[1] + "," + m1.getDiffuseColor()[2]+")");
		}});       
		
	m1.addPropertyChangeListener(Appearance3DChooserPanel.COLORTARGET, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("COLORTARGET : (" + m1.getColorTarget() +")");
		}});        
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.SHININESS, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("SHININESS : (" + m1.getShininess() + ")");
		}});	
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.LIGHTING, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("LIGHTING : (" + m1.getLighting() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TRANSPARENCY, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TRANSPARENCY : (" + m1.getTransparency() +")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TRANSPARENCY_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TRANSPARENCY_MODE : (" + m1.getTransparencyMode() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TRANSPARENCY_SRC_BLEND_FUNCTION, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TRANSPARENCY_SRC_BLEND_FUNCTION : (" + m1.getTransparencySrcBlendFunction() +")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TRANSPARENCY_DST_BLEND_FUNCTION, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TRANSPARENCY_DST_BLEND_FUNCTION : (" + m1.getTransparencyDstBlendFunction() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_USE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_USE : (" + m1.isTextureSet() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_FILE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_FILE : (" + m1.getTextureFile() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_TRANSFORM, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("\nTEXTURE_TRANSFORM : \n" + m1.getTextureTransform());
		    System.out.println("Scale : x=" + m1.getTextureTransformScaleX() + ", y=" +m1.getTextureTransformScaleY());
		    System.out.println("Translation : x=" + m1.getTextureTransformTranslatX() + ", y=" +m1.getTextureTransformTranslatY());
		    System.out.println("Rotation : z=" + m1.getTextureTransformRotZ());
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_PERSPECTIVE_CORRECTION_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_PERSPECTIVE_CORRECTION_MODE : (" + m1.getTexturePerspectiveCorrectionMode() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_MODE : (" + m1.getTextureMode() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_BLEND, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_BLEND : (" + 
				       m1.getTextureBlendColor()[0] + "," +
				       m1.getTextureBlendColor()[1] + "," +
				       m1.getTextureBlendColor()[2] + "," +
				       m1.getTextureBlendColor()[3] 
				       + ")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_RGB_MODE : (" + m1.getTextureCombineRGBMode() +")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SOURCE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_RGB_SOURCE : (" + 
				       m1.getTextureCombineRGBSource()[0]+"," +
				       m1.getTextureCombineRGBSource()[1]+"," +
				       m1.getTextureCombineRGBSource()[2] 
				       + ")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_FUNCTION, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_RGB_FUNCTION : (" + 
				       m1.getTextureCombineRGBFunction()[0]+"," +
				       m1.getTextureCombineRGBFunction()[1]+"," +
				       m1.getTextureCombineRGBFunction()[2] 
				       + ")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SCALE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_RGB_SCALE : (" + m1.getTextureCombineRGBScale() +")");
		}});

	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_MODE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_ALPHA_MODE : (" + m1.getTextureCombineAlphaMode() +")");
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SOURCE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_ALPHA_SOURCE : (" + 
				       m1.getTextureCombineAlphaSource()[0] + "," +
				       m1.getTextureCombineAlphaSource()[1] + "," +
				       m1.getTextureCombineAlphaSource()[2] 
				       + ")");		    
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_FUNCTION, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_ALPHA_FUNCTION : (" + 
				       m1.getTextureCombineAlphaFunction()[0] + "," +
				       m1.getTextureCombineAlphaFunction()[1] + "," +
				       m1.getTextureCombineAlphaFunction()[2] 
				       + ")");		    
		}});
	
	m1.addPropertyChangeListener(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SCALE, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    System.out.println("TEXTURE_COMBINE_ALPHA_SCALE : (" + m1.getTextureCombineAlphaScale() +")");
		}});
		
	JFrame frame1 = new JFrame();
	frame1.getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0; c.gridy = 0;
	c.weightx = 1; c.weighty = 1; c.fill = GridBagConstraints.BOTH;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.insets = new Insets(10,10,10,10);
	frame1.getContentPane().add(m1,c);
	c.gridy++; c.gridwidth = 1;	
	frame1.pack();
	frame1.setVisible(true);		

	
	m2.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    if(evt.getPropertyName().compareTo(Appearance3DChooserPanel.AMBIENT) == 0)
			{
			    System.out.println("AMBIENT2 : (" + m2.getAmbientColor()[0]+ "," + m2.getAmbientColor()[1] + "," + m2.getAmbientColor()[2]+")");		    
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.EMISSIVE) == 0)
			{
			    System.out.println("EMISSIVE2 : (" + m2.getEmissiveColor()[0]+ "," + m2.getEmissiveColor()[1] + "," + m2.getEmissiveColor()[2]+")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.SPECULAR) == 0)
			{
			    System.out.println("SPECULAR2 : (" + m2.getSpecularColor()[0]+ "," + m2.getSpecularColor()[1]+ "," + m2.getSpecularColor()[2]+")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.DIFFUSE) == 0)
			{
			    System.out.println("DIFFUSE2 : (" + m2.getDiffuseColor()[0] + "," + m2.getDiffuseColor()[1] + "," + m2.getDiffuseColor()[2]+")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.COLORTARGET) == 0)
			{
			    System.out.println("COLORTARGET2 : (" + m2.getColorTarget() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.SHININESS) == 0)
			{
			    System.out.println("SHININESS2 : (" + m2.getShininess() + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.LIGHTING) == 0)
			{
			    System.out.println("LIGHTING2 : (" + m2.getLighting() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TRANSPARENCY) == 0)
			{
			    System.out.println("TRANSPARENCY2 : (" + m2.getTransparency() +")");			    
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TRANSPARENCY_MODE) == 0)
			{
			    System.out.println("TRANSPARENCY_MODE2 : (" + m2.getTransparencyMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TRANSPARENCY_SRC_BLEND_FUNCTION) == 0)
			{
		    	    System.out.println("TRANSPARENCY_SRC_BLEND_FUNCTION2 : (" + m2.getTransparencySrcBlendFunction() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TRANSPARENCY_DST_BLEND_FUNCTION) == 0)
			{
		    	    System.out.println("TRANSPARENCY_DST_BLEND_FUNCTION2 : (" + m2.getTransparencyDstBlendFunction() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_USE) == 0)
			{
		    	    System.out.println("TEXTURE_USE2 : (" + m2.isTextureSet() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_FILE) == 0)
			{
		    	    System.out.println("TEXTURE_FILE2 : (" + m2.getTextureFile() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_TRANSFORM) == 0)
			{
		    	    System.out.println("\nTEXTURE_TRANSFORM2 : \n" + m2.getTextureTransform() );
			    System.out.println("Scale : x=" + m2.getTextureTransformScaleX() + ", y=" +m2.getTextureTransformScaleY());
			    System.out.println("Translation : x=" + m2.getTextureTransformTranslatX() + ", y=" +m2.getTextureTransformTranslatY());
			    System.out.println("Rotation : z=" + m2.getTextureTransformRotZ());
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_PERSPECTIVE_CORRECTION_MODE) == 0)
			{
		    	    System.out.println("TEXTURE_PERSPECTIVE_CORRECTION_MODE2 : (" + m2.getTexturePerspectiveCorrectionMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_MODE) == 0)
			{
		    	    System.out.println("TEXTURE_MODE2 : (" + m2.getTextureMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_BLEND) == 0)
			{
		    	    System.out.println("TEXTURE_BLEND2 : (" + 
					       m2.getTextureBlendColor()[0] + "," +
					       m2.getTextureBlendColor()[1] + "," +
					       m2.getTextureBlendColor()[2] + "," +
					       m2.getTextureBlendColor()[3] 
					       + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_MODE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_RGB_MODE2 : (" + m2.getTextureCombineRGBMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SOURCE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_RGB_SOURCE2 : (" + 
					       m2.getTextureCombineRGBSource()[0] + "," +
					       m2.getTextureCombineRGBSource()[1] + "," +
					       m2.getTextureCombineRGBSource()[2] 
					       + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_FUNCTION) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_RGB_FUNCTION2 : (" + 
					       m2.getTextureCombineRGBFunction()[0] + "," +
					       m2.getTextureCombineRGBFunction()[1] + "," +
					       m2.getTextureCombineRGBFunction()[2] 
					       + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_RGB_SCALE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_RGB_SCALE2 : (" + m2.getTextureCombineRGBScale() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_MODE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_ALPHA_MODE2 : (" + m2.getTextureCombineAlphaMode() +")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SOURCE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_ALPHA_SOURCE2 : (" + 
					       m2.getTextureCombineAlphaSource()[0] + "," +
					       m2.getTextureCombineAlphaSource()[1] + "," +
					       m2.getTextureCombineAlphaSource()[2]
					       + ")");
			}		    
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_FUNCTION) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_ALPHA_FUNCTION2 : (" + 
					       m2.getTextureCombineAlphaFunction()[0] + "," +
					       m2.getTextureCombineAlphaFunction()[1] + "," +
					       m2.getTextureCombineAlphaFunction()[2]
					       + ")");
			}
		    else if (evt.getPropertyName().compareTo(Appearance3DChooserPanel.TEXTURE_COMBINE_ALPHA_SCALE) == 0)
			{
			    System.out.println("TEXTURE_COMBINE_ALPHA_SCALE2 : (" + m2.getTextureCombineAlphaScale() +")");
			}		    

		}});


	// Test Initialisation

	float []emissiveColor = {1.0f,0.9f,0.8f};
	float []ambientColor = {0.7f,0.6f,0.5f};
	float []diffuseColor = {0.4f,0.3f,0.2f};
	float []specularColor = {0.1f,0.0f,1.0f};
	boolean lighting = false;
	float shininess = 85.0f;
	int colorTarget = Material.SPECULAR;
	float transparency = 0.5f;
	int transparencyMode = TransparencyAttributes.SCREEN_DOOR;
	int transparencySrcBlendFunction = TransparencyAttributes.BLEND_ONE;
	int transparencyDstBlendFunction = TransparencyAttributes.BLEND_SRC_ALPHA;
	Transform3D textureTransform = new Transform3D(); //////////////////////
	boolean textureUse = true;
	URL textureFile = (new Info()).getClass().getResource("resources/TexturePanel-2.png"); 
	int texturePerspectiveCorrectionMode = TextureAttributes.FASTEST;
	int textureMode = TextureAttributes.REPLACE;
	float []textureBlendColor = {0.1f,0.2f,0.3f,0.4f};
	int textureCombineRGBMode = TextureAttributes.COMBINE_INTERPOLATE;
	int []textureCombineRGBSource = {TextureAttributes.COMBINE_OBJECT_COLOR, 
					 TextureAttributes.COMBINE_CONSTANT_COLOR, 
					 TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE};
	int []textureCombineRGBFunction = {TextureAttributes.COMBINE_SRC_ALPHA, 
					   TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR, 
					   TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA};
	int textureCombineRGBScale = 2;
	int textureCombineAlphaMode = TextureAttributes.COMBINE_DOT3;
	int []textureCombineAlphaSource = {TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE, 
					   TextureAttributes.COMBINE_TEXTURE_COLOR, 
					   TextureAttributes.COMBINE_OBJECT_COLOR};
	int []textureCombineAlphaFunction = {TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA, 
					     TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA, 
					     TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA};
	int textureCombineAlphaScale = 4;        


	// Set init Value
	
	m2.setAmbientColor(ambientColor);
	m2.setDiffuseColor(diffuseColor); 
	m2.setEmissiveColor(emissiveColor);  
	m2.setSpecularColor(specularColor);
	m2.setShininess(shininess);
	m2.setColorTarget(colorTarget);      
	m2.setLighting(lighting);
	m2.setTransparency(transparency);
	m2.setTransparencyMode(transparencyMode);
	m2.setTransparencySrcBlendFunction(transparencySrcBlendFunction);
	m2.setTransparencyDstBlendFunction(transparencyDstBlendFunction);
	m2.setTextureUse(textureUse);
	m2.setTextureTransform(textureTransform);	
	m2.setTextureTransformScaleX(4);
	m2.setTextureTransformScaleY(2);
	m2.setTextureTransformTranslatX(0.5f);
	m2.setTextureTransformTranslatY(1);
	m2.setTextureTransformRotZ(1.67f);
	m2.setTextureFile(textureFile);		    	
	m2.setTexturePerspectiveCorrectionMode(texturePerspectiveCorrectionMode);
	m2.setTextureMode(textureMode);
	m2.setTextureBlendColor(textureBlendColor);	    
	m2.setTextureCombineRGBMode(textureCombineRGBMode);
	m2.setTextureCombineRGBSource(textureCombineRGBSource);
	m2.setTextureCombineRGBFunction(textureCombineRGBFunction);
	m2.setTextureCombineRGBScale(textureCombineRGBScale);	
	m2.setTextureCombineAlphaMode(textureCombineAlphaMode);
	m2.setTextureCombineAlphaSource(textureCombineAlphaSource);
	m2.setTextureCombineAlphaFunction(textureCombineAlphaFunction);
	m2.setTextureCombineAlphaScale(textureCombineAlphaScale);

	    
	JFrame frame2 = new JFrame();
	frame2.getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints c2 = new GridBagConstraints();
	c2.gridx = 0; c2.gridy = 0;
	c2.weightx = 1; c2.weighty = 1; c2.fill = GridBagConstraints.BOTH;
	c2.gridwidth = GridBagConstraints.REMAINDER;
	c2.insets = new Insets(10,10,10,10);
	frame2.getContentPane().add(m2,c2);
	frame2.pack();
	frame2.setVisible(true);		

	while(true)
	    {		
		try{		   
		    Thread.currentThread();
			Thread.sleep(60000); //sleep for 60000 ms		    
		}
		catch(Exception ie){
		    //If this thread was intrrupted by nother thread 
		}
		
		if(!frame1.isVisible() && !frame2.isVisible())
		    System.exit(0);
	    }	

    }


}