/**
   ##############################################################################
   ##                                                                          ##
   ## Appearance3DChooser                                                      ##
   ##                                                                          ##
   ## Copyright (C) 2009  Frederic Roudaut  <frederic.roudaut@free.fr>         ##
   ##                                                                          ##
   ##                                                                          ##
   ## This program is free software: you can redistribute it and/or modify     ##
   ## it under the terms of the GNU General Public License as published by     ##
   ## the Free Software Foundation, either version 3 of the License, or        ##
   ## (at your option) any later version.                                      ##
   ##                                                                          ##
   ## This program is distributed in the hope that it will be useful,          ##
   ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
   ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
   ## GNU General Public License for more details.                             ##
   ##                                                                          ##
   ## You should have received a copy of the GNU General Public License        ##
   ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
   ##                                                                          ##
   ##                                                                          ##
   ##############################################################################
 **/



package com.appearance3Dchooser;

import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.*;
import java.awt.Insets;
import java.awt.Font;
import java.awt.Dimension;
import java.util.Map;
import java.util.ResourceBundle;
import javax.swing.*;
import java.io.*;
import javax.swing.event.HyperlinkEvent;

import javax.media.j3d.Canvas3D;
import javax.media.j3d.GraphicsConfigTemplate3D;
import javax.media.j3d.VirtualUniverse;

import com.awt.GridBagLayoutConstraint;



/**
 * An helpful class for tierce information, for showing 3D Info, a Help Page, an about panel ...
 *
 */
public class Info 
{

	private static final long serialVersionUID = 1L;

	/** 
	 * The localized strings used in this (and related) panel(s). 
	 */
	public static ResourceBundle ressources = ResourceBundle.getBundle("com.appearance3Dchooser.resources.Appearance3DChooser");
	/** 
	 * The Frame used to show the GPL License. 
	 */
	protected static JFrame licenseFrame;

	/**
	 * Draw a Frame with some useful helps on Java3D.
	 *
	 */   
	public static void showJava3DHelp()
	{
		showJava3DHelp(new Dimension(800, 800));
	}

	/**
	 * Draw a Frame with some useful helps on Java3D.
	 * @param d the text Panel Dimension
	 *
	 */   
	public static void showJava3DHelp(Dimension d)
	{
		final JFrame helpFrame;
		JEditorPane helpPane = new JEditorPane();
		javax.swing.event.HyperlinkListener helpPaneListener;	

		helpPane.setEditable(false);
		java.net.URL helpURL = (new Info()).getClass().getResource("resources/Appearance3DChooserHelp.html");
		if (helpURL != null) {
			try {
				helpPane.setPage(helpURL);
				helpPane.setContentType("text/html");
			} catch (IOException e) {
				System.err.println("Attempted to read a bad URL: " + helpURL);
			}
		} else {
			System.err.println("Couldn't find file: Appearance3DChooserHelp.html");
		}

		// Add a listener for following Links       
		helpPaneListener = new javax.swing.event.HyperlinkListener() {
			public void hyperlinkUpdate(javax.swing.event.HyperlinkEvent e) {
				if (e.getEventType() ==  HyperlinkEvent.EventType.ACTIVATED) {
					JEditorPane pane = (JEditorPane)e.getSource();
					try {
						pane.setPage(e.getURL());
					}catch (IOException ex) {
						pane.setText("Error : "+ ex.getMessage());
					}
				}
			}
		};

		helpPane.addHyperlinkListener(helpPaneListener);

		//Put the editor pane in a scroll pane.
		JScrollPane helpScrollPane = new JScrollPane(helpPane);
		helpScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		helpScrollPane.setPreferredSize(d);
		helpScrollPane.setMinimumSize(new Dimension(10, 10));

		helpFrame= new JFrame(Info.ressources.getObject("Help").toString());
		GridBagLayout gb = new GridBagLayout();
		helpFrame.setLayout(gb);
		helpFrame.add(helpScrollPane);		
		gb.setConstraints(helpScrollPane, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
				new Insets(15,15,15,15)));

		JButton okbutton = new JButton(Info.ressources.getObject("OK").toString());
		helpFrame.add(okbutton);
		gb.setConstraints(okbutton, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,1,
				new Insets(15,8,8,8)));
		okbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				helpFrame.setVisible(false);
				helpFrame.dispose();
			}
		});


		helpFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		helpFrame.pack();
		helpFrame.setVisible(true);
	}


	/**
	 * Draw a Frame with the version, author and content information.
	 *
	 */   
	public static void showAbout() 
	{		
		final JFrame aboutFrame = new JFrame(Info.ressources.getObject("About").toString());
		JPanel aboutPanel = new JPanel(new BorderLayout());
		JPanel gplPanel = new JPanel(new BorderLayout());	
		aboutFrame.getContentPane().add(aboutPanel, BorderLayout.CENTER);
		JPanel textPanel = new JPanel();	
		aboutPanel.add(textPanel, BorderLayout.EAST);	
		GridBagLayout gb; 

		java.net.URL url = (new Info()).getClass().getResource("resources/about.png");	
		if (url != null) {
			ImageIcon aboutPicture = new ImageIcon(url, "Appearance 3D Chooser");
			if (aboutPicture != null) {
				JLabel aboutLabel = new JLabel(aboutPicture);
				JPanel aboutPicturePanel = new JPanel();
				gb = new GridBagLayout();
				aboutPicturePanel.setLayout(gb);
				aboutPicturePanel.add(aboutLabel, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
						new Insets(8,8,8,15)));		
				aboutPanel.add(aboutPicturePanel, BorderLayout.WEST);
			}
		}

		gb = new GridBagLayout();
		textPanel.setLayout(gb);
		JLabel entryText = new JLabel("Appearance3DChooser");
		entryText.setFont(new Font("SansSerif", Font.BOLD, 18));
		textPanel.add(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
				new Insets(60,15,0,15)) );
		entryText = new JLabel(Info.ressources.getObject("Appearance3DChooserVersion").toString());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 12));
		textPanel.add(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,1,
				new Insets(0,15,0,15)) );

		entryText = new JLabel(Info.ressources.getObject("About_content1").toString());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,2,
				new Insets(30,15,0,15)) );
		entryText = new JLabel(Info.ressources.getObject("About_content2").toString());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,3,
				new Insets(0,15,0,15)) );
		entryText = new JLabel(Info.ressources.getObject("About_content3").toString());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,4,
				new Insets(30,15,0,15)) );
		entryText = new JLabel(Info.ressources.getObject("About_content4").toString());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,5,
				new Insets(30,15,0,15)) );
		entryText = new JLabel(Info.ressources.getObject("About_content5").toString());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,6,
				new Insets(0,15,0,15)) );


		url = (new Info()).getClass().getResource("resources/gplv3.png");	
		if (url != null) {
			ImageIcon gplPicture = new ImageIcon(url, "License");
			if (gplPicture != null) {
				JLabel gplLabel = new JLabel(gplPicture);
				JPanel gplPicturePanel = new JPanel();
				gb = new GridBagLayout();
				gplPicturePanel.setLayout(gb);
				gplPicturePanel.add(gplLabel, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
						new Insets(5,10,0,0)));		
				gplPanel.add(gplPicturePanel, BorderLayout.WEST);
				createLicense();
				gplLabel.addMouseListener(new MouseListener() {
					public void mouseClicked(MouseEvent e) {
						licenseFrame.setVisible(!licenseFrame.isVisible());
					}
					public void mouseExited(MouseEvent e) {
					}
					public void mouseEntered(MouseEvent e) {
					}
					public void mousePressed(MouseEvent e) {			    
					}
					public void mouseReleased(MouseEvent e) {
					}
				});		
			}
		}
		textPanel.add(gplPanel, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,7,	
				new Insets(0,0,0,0)) );

		JButton okbutton = new JButton(Info.ressources.getObject("OK").toString());
		textPanel.add(okbutton, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,9, new Insets(90,8,8,8)));
		okbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				aboutFrame.setVisible(false);
				aboutFrame.dispose();
			}
		});

		aboutFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		aboutFrame.setResizable(false);
		aboutFrame.pack();
		aboutFrame.setVisible(true);
	}



	/**
	 * Create a Frame for showing GPL License.     
	 *
	 */   
	protected static void createLicense()
	{
		createLicense(new Dimension(650, 800));
	}


	/**
	 * Create a Frame for showing GPL License.
	 * @param d the text Panel Dimension
	 *
	 */   
	protected static void createLicense(Dimension d)
	{
		licenseFrame = new JFrame("GPL");	
		JTextArea text = new JTextArea();
		JPanel textPanel = new JPanel();
		JPanel licensePanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		licensePanel.setLayout(gb);

		text.setFont(new Font("SansSerif", Font.PLAIN, 12));       
		InputStream ips = (new Info()).getClass().getResourceAsStream("resources/license.txt");
		if (ips != null)
		{
			try
			{			
				InputStreamReader ipsr=new InputStreamReader(ips);
				BufferedReader br=new BufferedReader(ipsr);
				String ligne;
				String textContent = "";
				while ((ligne = br.readLine())!= null)
				{			
					textContent+= "\t" + ligne + "\n";		    
				}  

				text.append(textContent);
			}
			catch ( Exception e)
			{
				System.err.println("Error : License File not found ");
			}
		}

		text.setCaretPosition(0);
		JScrollPane textScrolled = new JScrollPane(text);	
		textScrolled.setPreferredSize(d);   
		textPanel.add(textScrolled);	
		licensePanel.add(textPanel);
		gb.setConstraints(textPanel, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
				new Insets(15,15,15,15)));
		

		JButton okbutton = new JButton(Info.ressources.getObject("OK").toString());
		licensePanel.add(okbutton);
		gb.setConstraints(okbutton, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,1,
				new Insets(15,8,8,8)));
		okbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				licenseFrame.setVisible(false);
				licenseFrame.dispose();
			}
		});



		licenseFrame.getContentPane().add(licensePanel);	
		licenseFrame.pack();
		licenseFrame.setVisible(false);
	}


	/**
	 * Draw a Frame with some useful information on Java3D and your 3D Characteristics.
	 *
	 */   
	public static void show3DInfo() {
		show3DInfo(new Dimension(580, 500));	
	}

	/**
	 * Draw a Frame with some useful information on Java3D and your 3D Characteristics.
	 * @param d the text Panel Dimension
	 *
	 */   
	public static void show3DInfo(Dimension d) 
	{	
		Map vuMap = VirtualUniverse.getProperties();
		GraphicsConfigTemplate3D template = new GraphicsConfigTemplate3D();
		template.setStereo(GraphicsConfigTemplate3D.PREFERRED);
		template.setSceneAntialiasing(GraphicsConfigTemplate3D.REQUIRED);
		template.setDoubleBuffer(GraphicsConfigTemplate3D.PREFERRED);
		GraphicsConfiguration config = GraphicsEnvironment
		.getLocalGraphicsEnvironment().getDefaultScreenDevice()
		.getBestConfiguration(template);
		Map c3dMap = new Canvas3D(config).queryProperties();	

		final JFrame show3DBox = new JFrame(Info.ressources.getObject("About_Java3D").toString());
		show3DBox.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		show3DBox.setVisible(true);

		JPanel show3DPanel = new JPanel(new BorderLayout());
		JLabel entryText;

		JPanel textPanel = new JPanel();
		show3DPanel.add(textPanel, BorderLayout.WEST);
		GridBagLayout gb = new GridBagLayout();
		textPanel.setLayout(gb);

		entryText = new JLabel(Info.ressources.getObject("Version").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + vuMap.get("j3d.version"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,0,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("Vendor").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,1,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + vuMap.get("j3d.vendor"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,1,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("Specification_Version").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,2,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + vuMap.get("j3d.specification.version"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,2,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("Specification_Vendor").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,3,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + vuMap.get("j3d.specification.vendor"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,3,
				new Insets(0,8,0,15)));



		entryText = new JLabel(Info.ressources.getObject("Rendering_Pipeline").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,4,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + vuMap.get("j3d.pipeline"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,4,
				new Insets(0,8,0,15)));


		entryText = new JLabel(Info.ressources.getObject("Rendering_Library").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,5,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + vuMap.get("j3d.renderer"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,5,
				new Insets(0,8,0,15)));


		entryText = new JLabel(Info.ressources.getObject("Rendering_Version").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,6,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("native.version"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,6,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("doubleBufferAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,7,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("doubleBufferAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,7,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("stereoAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,8,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("stereoAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,8,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("sceneAntialiasingAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,9,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("sceneAntialiasingAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,9,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("sceneAntialiasingNumPasses").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,10,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("sceneAntialiasingNumPasses"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,10,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureColorTableSize").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,11,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureColorTableSize"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,11,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureEnvCombineAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,12,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureEnvCombineAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,12,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureCombineDot3Available").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,13,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureCombineDot3Available"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,13,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureCombineSubtractAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,14,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureCombineSubtractAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,14,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("texture3DAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,15,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("texture3DAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,15,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureCubeMapAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,16,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureCubeMapAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,16,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureSharpenAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,17,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureSharpenAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,17,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureFilter4Available").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,18,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureFilter4Available"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,18,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureDetailAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,19,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureDetailAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,19,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureAnisotropicFilterDegreeMax").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,20,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureAnisotropicFilterDegreeMax"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,20,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureBoundaryWidthMax").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,21,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureBoundaryWidthMax"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,21,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureWidthMax").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,22,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureWidthMax"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,22,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureHeightMax").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,23,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureHeightMax"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,23,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureLodOffsetAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,24,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureLodOffsetAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,24,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureLodRangeAvailable").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,25,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureLodRangeAvailable"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,25,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("textureUnitStateMax").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,26,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("textureUnitStateMax"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,26,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("compressedGeometry.majorVersionNumber").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,27,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("compressedGeometry.majorVersionNumber"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,27,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("compressedGeometry.minorVersionNumber").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,28,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("compressedGeometry.minorVersionNumber"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,28,
				new Insets(0,8,0,15)));

		entryText = new JLabel(Info.ressources.getObject("compressedGeometry.minorMinorVersionNumber").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,29,
				new Insets(0,15,0,15)));

		entryText = new JLabel("= " + c3dMap.get("compressedGeometry.minorMinorVersionNumber"));
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,29,
				new Insets(0,8,0,15)));


		JScrollPane textScrolled = new JScrollPane(textPanel);	
		textScrolled.setPreferredSize(d);	
		show3DPanel.add(textScrolled, BorderLayout.WEST);
		GridBagLayout gbMain = new GridBagLayout();
		show3DPanel.setLayout(gbMain);
		show3DPanel.add(textScrolled);
		GridBagConstraints gbc = new GridBagLayoutConstraint(GridBagConstraints.EAST,0,0,
				new Insets(15,15,15,15));
		gbc.gridwidth = 2;
		gbMain.setConstraints(textScrolled, gbc);


		JButton okbutton = new JButton(Info.ressources.getObject("OK").toString());
		show3DPanel.add(okbutton);
		gbMain.setConstraints(okbutton, new GridBagLayoutConstraint(GridBagConstraints.EAST,1,1,
				new Insets(15,8,8,8)));
		okbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				show3DBox.setVisible(false);
				show3DBox.dispose();
			}
		});

		show3DBox.getContentPane().add(show3DPanel, BorderLayout.CENTER);
		show3DBox.pack();
		show3DBox.setResizable(false);
		show3DBox.setVisible(true);
	}

}