/**
   ##############################################################################
   ##                                                                          ##
   ## Appearance3DChooser                                                      ##
   ##                                                                          ##
   ## Copyright (C) 2009  Frederic Roudaut  <frederic.roudaut@free.fr>         ##
   ##                                                                          ##
   ##                                                                          ##
   ## The Basic View Model has been found on Internet. Nevertheless, I do not  ##
   ## know where, thus if you have any idea, tell me and I will add a          ##
   ## reference for it.                                                        ##
   ##                                                                          ##
   ##                                                                          ##
   ## This program is free software: you can redistribute it and/or modify     ##
   ## it under the terms of the GNU General Public License as published by     ##
   ## the Free Software Foundation, either version 3 of the License, or        ##
   ## (at your option) any later version.                                      ##
   ##                                                                          ##
   ## This program is distributed in the hope that it will be useful,          ##
   ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
   ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
   ## GNU General Public License for more details.                             ##
   ##                                                                          ##
   ## You should have received a copy of the GNU General Public License        ##
   ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
   ##                                                                          ##
   ##                                                                          ##
   ##############################################################################
**/


package com.appearance3Dchooser;


import java.awt.*;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.util.Vector;
import java.util.Iterator;
import java.util.BitSet;
import java.net.URL;


// Java 3D Packages
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.image.TextureLoader;
import javax.media.j3d.*;
import javax.vecmath.*;


/**
 * Panel creation for having a Rendering View.
 * 
 */
public class RenderingViewPanel extends JPanel implements ActionListener, ChangeListener
{
    private static final long serialVersionUID = 1L;

    public static final float SPINNER_COORDINATE_MIN = -100.0f;
    public static final float SPINNER_COORDINATE_MAX = 100.0f;
    public static final float SPINNER_COORDINATE_INIT = 0.0f;
    public static final float SPINNER_COORDINATE_STEP = 0.10f;
    public static final float SPINNER_ANGLE_MIN = 0.0f;
    public static final float SPINNER_ANGLE_MAX = 180.0f;
    public static final float SPINNER_ANGLE_INIT = 180.0f;
    public static final float SPINNER_ANGLE_STEP = 0.10f;
    public static final float SPINNER_CONCENTRATION_MIN = 0.0f;
    public static final float SPINNER_CONCENTRATION_MAX = 128.0f;
    public static final float SPINNER_CONCENTRATION_INIT = 0.0f;
    public static final float SPINNER_CONCENTRATION_STEP = 0.10f;
    public static final int SPINNER_COLOR_MIN = 0;
    public static final int SPINNER_COLOR_MAX = 255;
    public static final int SPINNER_COLOR_INIT = 128;    
    public static final int SPINNER_COLOR_STEP = 1;
    public static final int SPINNER_BACKGROUNDCOLOR_R_INIT = 180;
	public static final int SPINNER_BACKGROUNDCOLOR_G_INIT = 180;
	public static final int SPINNER_BACKGROUNDCOLOR_B_INIT = 120;

    protected Switch objectsSwitch;
    protected Switch axisFrontend;
    protected BranchGroup scene; 
    protected TransformGroup sceneTransform;
    protected SceneView vueTop;
    protected SceneView vueFront;
    protected SceneView vueLeft;
    protected SceneView vueRight;
    protected SceneView bigVue;

    protected JRadioButton antialiasingButton;
    protected JRadioButton axisButton;    
    protected JSpinner backgroundColorRSpinner, backgroundColorGSpinner, backgroundColorBSpinner;
    protected JRadioButton bigVueButton;
    protected JRadioButton boxButton;
    protected JRadioButton sphereButton;
    protected JRadioButton cylinderButton;
    protected JRadioButton coneButton;    
    protected JRadioButton addAmbientLightButton;
    protected JRadioButton addDirectionalLightButton;
    protected JRadioButton addSpotLightButton;
    protected JRadioButton addPointLightButton;

    protected JPanel scenePanel;
    protected JPanel addingLightPanel;
    protected JPanel lightsPanel = new JPanel();		
    protected JScrollPane lightsPanelScrollPane;
    protected JFrame bigVueFrame;

    public static final int LIGHTPANEL_X = 415;
    public static final int LIGHTPANEL_Y = 370;
    protected int NbLignsLights = 0;    
    protected GridBagConstraints lightsConstraint = new GridBagConstraints();
    protected Texture texture;
    protected URL DefaultTexture;
    protected TextureAttributes textureAttributes;
    protected Transform3D textureTransform;    
    protected Background background;

    protected Vector ambientLightsVector;
    protected Vector directionalLightsVector;
    protected Vector pointLightsVector;
    protected Vector spotLightsVector;
    
    private Appearance appearanceObjects = new Appearance();

    /**
     * Constructor for creating a rendering view panel
     * @param DefaultTextureName Default Texture File.
     */    
    public RenderingViewPanel(URL DefaultTextureName) {
          
	DefaultTexture = DefaultTextureName;

	GridBagConstraints c = new GridBagConstraints();
	this.setLayout(new GridBagLayout());
	c.gridx = 0; c.gridy = 0;
	c.weightx = 0; c.weighty = 0; 
	c.insets = new Insets(0,0,0,10);
	c.fill = GridBagConstraints.CENTER;
	c.anchor = GridBagConstraints.CENTER;
	this.add(makeScenePanel(),c);
	c.gridx = 0; c.gridy = 1; 
	this.add(makeObjectChooserPanel(),c);
	c.gridx = 1; c.gridy = 0; 
	c.insets = new Insets(10,0,0,0);
	this.add(makeLightsPanel(),c);
	c.gridx = 1; c.gridy = 1; 
	this.add(makeAddingLightsPanel(),c);
	initLights();
    }

    /**
     * Panel creation for adding new lights.
     * 
     */
    protected JPanel makeAddingLightsPanel()
    {
	addingLightPanel = new JPanel();
	GridBagConstraints c = new GridBagConstraints();
	addingLightPanel.setLayout(new GridBagLayout());

	c.gridx = 0; c.gridy = 0;
	c.gridheight = 2;
	c.insets = new Insets(0,0,0,25);
	JLabel addAddLightLabel = new JLabel(Info.ressources.getObject("ADD").toString()+":");
	addingLightPanel.add(addAddLightLabel,c);

	c.gridx = 1; 
	c.gridheight = 1;
	c.anchor = GridBagConstraints.CENTER;
	c.fill = GridBagConstraints.CENTER;
	c.insets = new Insets(0,0,0,20);
	addAmbientLightButton = new JRadioButton();
	addAmbientLightButton.setSelected(false);
	addAmbientLightButton.addActionListener(this);
	addingLightPanel.add(addAmbientLightButton,c);
	
	c.gridx = 2; 
	addPointLightButton = new JRadioButton();
	addPointLightButton.setSelected(false);
	addPointLightButton.addActionListener(this);
	addingLightPanel.add(addPointLightButton,c);
	
	c.gridx = 3; 
	addDirectionalLightButton = new JRadioButton();
	addDirectionalLightButton.setSelected(false);
	addDirectionalLightButton.addActionListener(this);
	addingLightPanel.add(addDirectionalLightButton,c);
	
	c.gridx = 4; 
	addSpotLightButton = new JRadioButton();
	addSpotLightButton.setSelected(false);
	addSpotLightButton.addActionListener(this);
	addingLightPanel.add(addSpotLightButton,c);

	c.gridx = 1; c.gridy = 1;
	JLabel addAmbientLightLabel = new JLabel(Info.ressources.getObject("Ambient").toString());
	addingLightPanel.add(addAmbientLightLabel,c);
	
	c.gridx = 2; 
	JLabel addPointLightLabel = new JLabel(Info.ressources.getObject("Point").toString());
	addingLightPanel.add(addPointLightLabel,c);
	
	c.gridx = 3; 
	JLabel addDirectionalLightLabel = new JLabel(Info.ressources.getObject("Directional").toString());
	addingLightPanel.add(addDirectionalLightLabel,c);
	
	c.gridx = 4; 
	JLabel addSpotLightLabel = new JLabel(Info.ressources.getObject("Spot").toString());
	addingLightPanel.add(addSpotLightLabel,c);

	return addingLightPanel;
    }



    /**
     * Panel creation for controlling lights in the scene.
     * 
     */
    protected JScrollPane makeLightsPanel()
    {
	lightsPanel.setLayout(new GridBagLayout());	
	lightsConstraint.weightx = 0; lightsConstraint.weighty = 0; 
	lightsConstraint.insets = new Insets(0,0,0,0);
	lightsConstraint.fill = GridBagConstraints.WEST;
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.gridx = 0;
	lightsConstraint.gridy = NbLignsLights;	
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	// add a button to set antialiasing
	antialiasingButton = new JRadioButton(Info.ressources.getObject("Antialiasing").toString());
	lightsConstraint.gridwidth =2;
	antialiasingButton.setSelected(false);
	antialiasingButton.addActionListener(this);
	lightsPanel.add(antialiasingButton,lightsConstraint);	
	NbLignsLights++;

	// add a button to set Axis
	axisButton = new JRadioButton(Info.ressources.getObject("Axis").toString());
	lightsConstraint.gridx ++;
	axisButton.setSelected(false);
	axisButton.addActionListener(this);
	lightsPanel.add(axisButton,lightsConstraint);	

	// add a button to show the big view
	bigVueButton = new JRadioButton(Info.ressources.getObject("Big_View").toString());
	lightsConstraint.gridx +=2;
	bigVueButton.setSelected(false);
	bigVueButton.addActionListener(this);
	lightsPanel.add(bigVueButton,lightsConstraint);	

	lightsConstraint.gridwidth =1;
	NbLignsLights++;

	// add a Label for the Background Color				
	// Background Color 		
	JLabel backgroundColorLabel = new JLabel(Info.ressources.getObject("Background_Color").toString()+":");
	lightsConstraint.gridy ++;
	lightsConstraint.gridx = 0; 
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(10,20,0,0);
	lightsPanel.add(backgroundColorLabel,lightsConstraint);
	lightsConstraint.insets = new Insets(10,0,0,0);
	lightsConstraint.anchor = GridBagConstraints.WEST;

	JLabel backgroundColorR = new JLabel(Info.ressources.getObject("R").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(10,10,0,5);
	lightsPanel.add(backgroundColorR,lightsConstraint);
	lightsConstraint.insets = new Insets(10,0,0,0);

	SpinnerModel backgroundColorSpinnerRModel = new SpinnerNumberModel(SPINNER_BACKGROUNDCOLOR_R_INIT, //initial value
			SPINNER_COLOR_MIN, //min
			SPINNER_COLOR_MAX, //max
			SPINNER_COLOR_STEP); //step
	backgroundColorRSpinner = new JSpinner(backgroundColorSpinnerRModel);
	((JSpinner.DefaultEditor)backgroundColorRSpinner.getEditor()).getTextField().setColumns(3);
	backgroundColorRSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(backgroundColorRSpinner,lightsConstraint);

	JLabel backgroundColorG = new JLabel(Info.ressources.getObject("G").toString());
	lightsConstraint.gridx++;
	lightsConstraint.insets = new Insets(10,10,0,5);
	lightsPanel.add(backgroundColorG,lightsConstraint);
	lightsConstraint.insets = new Insets(10,0,0,0);

	SpinnerModel backgroundColorSpinnerGModel = new SpinnerNumberModel(SPINNER_BACKGROUNDCOLOR_G_INIT , //initial value
			SPINNER_COLOR_MIN, //min
			SPINNER_COLOR_MAX, //max
			SPINNER_COLOR_STEP); //step
	backgroundColorGSpinner = new JSpinner(backgroundColorSpinnerGModel);
	((JSpinner.DefaultEditor)backgroundColorGSpinner.getEditor()).getTextField().setColumns(3);
	backgroundColorGSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(backgroundColorGSpinner,lightsConstraint);

	JLabel backgroundColorB = new JLabel(Info.ressources.getObject("B").toString());
	lightsConstraint.gridx++;
	lightsConstraint.insets = new Insets(10,10,0,5);
	lightsPanel.add(backgroundColorB,lightsConstraint);
	lightsConstraint.insets = new Insets(10,0,0,0);

	SpinnerModel backgroundColorSpinnerBModel = new SpinnerNumberModel(SPINNER_BACKGROUNDCOLOR_B_INIT , //initial value
			SPINNER_COLOR_MIN, //min
			SPINNER_COLOR_MAX, //max
			SPINNER_COLOR_STEP); //step
	backgroundColorBSpinner = new JSpinner(backgroundColorSpinnerBModel);
	((JSpinner.DefaultEditor)backgroundColorBSpinner.getEditor()).getTextField().setColumns(3);
	backgroundColorBSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(backgroundColorBSpinner,lightsConstraint);

	NbLignsLights += 3;
		
	ambientLightsVector = new Vector();
	directionalLightsVector = new Vector();
	pointLightsVector = new Vector();
	spotLightsVector = new Vector();

	CreateAmbientLightControl();
	CreateDirectionalLightControl();
	CreatePointLightControl();
	CreateSpotLightControl();
	 
	lightsPanelScrollPane = new JScrollPane(lightsPanel);
	lightsPanelScrollPane.setPreferredSize(new Dimension(LIGHTPANEL_X, LIGHTPANEL_Y));
	return lightsPanelScrollPane;	
    }

    /**
     * Panel creation for controlling an Ambient light in the scene.
     * 
     */
    private void CreateAmbientLightControl()
    {
	// Ambient Light 	
	JRadioButton ambientLightButton = new JRadioButton(Info.ressources.getObject("Ambient_Light").toString());
	ambientLightButton.setSelected(false);
	ambientLightButton.addActionListener(this);
	lightsConstraint.gridx = 0;
	lightsConstraint.gridy = NbLignsLights; 
	lightsPanel.add(ambientLightButton,lightsConstraint);	
	
	JLabel ambientLightsColor = new JLabel(Info.ressources.getObject("Color").toString()+":");
	lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(ambientLightsColor,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	lightsConstraint.anchor = GridBagConstraints.WEST;

	JLabel ambientLightsColorR = new JLabel(Info.ressources.getObject("R").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(ambientLightsColorR,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel ambientLightsSpinnerRModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
									 SPINNER_COLOR_MIN, //min
									 SPINNER_COLOR_MAX, //max
									 SPINNER_COLOR_STEP); //step
	JSpinner ambientLightsColorRSpinner = new JSpinner(ambientLightsSpinnerRModel);
	((JSpinner.DefaultEditor)ambientLightsColorRSpinner.getEditor()).getTextField().setColumns(3);
	ambientLightsColorRSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(ambientLightsColorRSpinner,lightsConstraint);
	
	JLabel ambientLightsColorG = new JLabel(Info.ressources.getObject("G").toString());
	lightsConstraint.gridx++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(ambientLightsColorG,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel ambientLightsSpinnerGModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
									 SPINNER_COLOR_MIN, //min
									 SPINNER_COLOR_MAX, //max
									 SPINNER_COLOR_STEP); //step
	JSpinner ambientLightsColorGSpinner = new JSpinner(ambientLightsSpinnerGModel);
	((JSpinner.DefaultEditor)ambientLightsColorGSpinner.getEditor()).getTextField().setColumns(3);
	ambientLightsColorGSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(ambientLightsColorGSpinner,lightsConstraint);

	JLabel ambientLightsColorB = new JLabel(Info.ressources.getObject("B").toString());
	lightsConstraint.gridx++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(ambientLightsColorB,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel ambientLightsSpinnerBModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
									 SPINNER_COLOR_MIN, //min
									 SPINNER_COLOR_MAX, //max
									 SPINNER_COLOR_STEP); //step
	JSpinner ambientLightsColorBSpinner = new JSpinner(ambientLightsSpinnerBModel);
	((JSpinner.DefaultEditor)ambientLightsColorBSpinner.getEditor()).getTextField().setColumns(3);
	ambientLightsColorBSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(ambientLightsColorBSpinner,lightsConstraint);
	
	ambientLightsVector.add(new AmbientLightObject(ambientLightButton, 
						       ambientLightsColorRSpinner, 
						       ambientLightsColorGSpinner, 
						       ambientLightsColorBSpinner,
						       sceneTransform
						       ));
	NbLignsLights += 3;
    }
    

    /**
     * Panel creation for controlling a Directional light in the scene.
     * 
     */
    private void CreateDirectionalLightControl()
    {
	// Directional Light 
	JRadioButton directionalLightButton = new JRadioButton(Info.ressources.getObject("Directional_Light").toString());
	directionalLightButton.setSelected(false);
	directionalLightButton.addActionListener(this);
	lightsConstraint.gridx = 0; 
	lightsConstraint.gridy = NbLignsLights;
	lightsPanel.add(directionalLightButton,lightsConstraint);

	JLabel directionalLightsColor = new JLabel(Info.ressources.getObject("Color").toString()+":");
	lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(directionalLightsColor,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel directionalLightsColorR = new JLabel(Info.ressources.getObject("R").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(directionalLightsColorR,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel directionalLightsSpinnerRModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
									     SPINNER_COLOR_MIN, //min
									     SPINNER_COLOR_MAX, //max
									     SPINNER_COLOR_STEP); //step      
	JSpinner directionalLightsColorRSpinner = new JSpinner(directionalLightsSpinnerRModel);
	((JSpinner.DefaultEditor)directionalLightsColorRSpinner.getEditor()).getTextField().setColumns(3);
	directionalLightsColorRSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(directionalLightsColorRSpinner,lightsConstraint);

	JLabel directionalLightsColorG = new JLabel(Info.ressources.getObject("G").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(directionalLightsColorG,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel directionalLightsSpinnerGModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
									     SPINNER_COLOR_MIN, //min
									     SPINNER_COLOR_MAX, //max
									     SPINNER_COLOR_STEP); //step      
	JSpinner directionalLightsColorGSpinner = new JSpinner(directionalLightsSpinnerGModel);
	((JSpinner.DefaultEditor)directionalLightsColorGSpinner.getEditor()).getTextField().setColumns(3);
	directionalLightsColorGSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(directionalLightsColorGSpinner,lightsConstraint);

	JLabel directionalLightsColorB = new JLabel(Info.ressources.getObject("B").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(directionalLightsColorB,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel directionalLightsSpinnerBModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
									     SPINNER_COLOR_MIN, //min
									     SPINNER_COLOR_MAX, //max
									     SPINNER_COLOR_STEP); //step      
	JSpinner directionalLightsColorBSpinner = new JSpinner(directionalLightsSpinnerBModel);
	((JSpinner.DefaultEditor)directionalLightsColorBSpinner.getEditor()).getTextField().setColumns(3);
	directionalLightsColorBSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(directionalLightsColorBSpinner,lightsConstraint);

	JLabel directionalLightsDirection = new JLabel(Info.ressources.getObject("Direction").toString()+":");
	lightsConstraint.gridx = 0;
	lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(directionalLightsDirection,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel directionalLightsDirectionX = new JLabel("X");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(directionalLightsDirectionX,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel directionalLightsSpinnerXModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
									     Float.valueOf(SPINNER_COORDINATE_MIN), //min
									     Float.valueOf(SPINNER_COORDINATE_MAX), //max
									     Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner directionalLightsDirectionXSpinner = new JSpinner(directionalLightsSpinnerXModel);
	((JSpinner.DefaultEditor)directionalLightsDirectionXSpinner.getEditor()).getTextField().setColumns(3);
	directionalLightsDirectionXSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(directionalLightsDirectionXSpinner,lightsConstraint);

	JLabel directionalLightsDirectionY = new JLabel("Y");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(directionalLightsDirectionY,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel directionalLightsSpinnerYModel = new SpinnerNumberModel(Float.valueOf(-0.5f), //initial value
									     Float.valueOf(SPINNER_COORDINATE_MIN), //min
									     Float.valueOf(SPINNER_COORDINATE_MAX), //max
									     Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner directionalLightsDirectionYSpinner = new JSpinner(directionalLightsSpinnerYModel);
	((JSpinner.DefaultEditor)directionalLightsDirectionYSpinner.getEditor()).getTextField().setColumns(3);
	directionalLightsDirectionYSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(directionalLightsDirectionYSpinner,lightsConstraint);

	JLabel directionalLightsDirectionZ = new JLabel("Z");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(directionalLightsDirectionZ,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel directionalLightsSpinnerZModel = new SpinnerNumberModel(Float.valueOf(-0.1f), //initial value
									     Float.valueOf(SPINNER_COORDINATE_MIN), //min
									     Float.valueOf(SPINNER_COORDINATE_MAX), //max
									     Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner directionalLightsDirectionZSpinner = new JSpinner(directionalLightsSpinnerZModel);
	((JSpinner.DefaultEditor)directionalLightsDirectionZSpinner.getEditor()).getTextField().setColumns(3);
	directionalLightsDirectionZSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(directionalLightsDirectionZSpinner,lightsConstraint);
	
	directionalLightsVector.add(new DirectionalLightObject(directionalLightButton, 
							       directionalLightsColorRSpinner, 
							       directionalLightsColorGSpinner, 
							       directionalLightsColorBSpinner,
							       directionalLightsDirectionXSpinner,
							       directionalLightsDirectionYSpinner,
							       directionalLightsDirectionZSpinner,
							       sceneTransform
							       ));
	NbLignsLights += 4;
	
    }


    /**
     * Panel creation for controlling a Point light in the scene.
     * 
     */
    private void CreatePointLightControl()
    {
	// Point Light
	JRadioButton pointLightButton = new JRadioButton(Info.ressources.getObject("Point_Light").toString());
	pointLightButton.setSelected(false);
	pointLightButton.addActionListener(this);
	lightsConstraint.gridx = 0; 
	lightsConstraint.gridy = NbLignsLights;
	lightsPanel.add(pointLightButton,lightsConstraint);

	JLabel pointLightsColor = new JLabel(Info.ressources.getObject("Color").toString()+":");
	lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(pointLightsColor,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel pointLightsColorR = new JLabel(Info.ressources.getObject("R").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsColorR,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel pointLightsSpinnerRModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
								       SPINNER_COLOR_MIN, //min
								       SPINNER_COLOR_MAX, //max
								       SPINNER_COLOR_STEP); //step      
	JSpinner pointLightsColorRSpinner = new JSpinner(pointLightsSpinnerRModel);
	((JSpinner.DefaultEditor)pointLightsColorRSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsColorRSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(pointLightsColorRSpinner,lightsConstraint);

	JLabel pointLightsColorG = new JLabel(Info.ressources.getObject("G").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsColorG,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel pointLightsSpinnerGModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
								       SPINNER_COLOR_MIN, //min
								       SPINNER_COLOR_MAX, //max
								       SPINNER_COLOR_STEP); //step      
	JSpinner pointLightsColorGSpinner = new JSpinner(pointLightsSpinnerGModel);
	((JSpinner.DefaultEditor)pointLightsColorGSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsColorGSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(pointLightsColorGSpinner,lightsConstraint);

	JLabel pointLightsColorB = new JLabel(Info.ressources.getObject("B").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsColorB,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel pointLightsSpinnerBModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
								       SPINNER_COLOR_MIN, //min
								       SPINNER_COLOR_MAX, //max
								       SPINNER_COLOR_STEP); //step      
	JSpinner pointLightsColorBSpinner = new JSpinner(pointLightsSpinnerBModel);
	((JSpinner.DefaultEditor)pointLightsColorBSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsColorBSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(pointLightsColorBSpinner,lightsConstraint);
	
	JLabel pointLightsPosition = new JLabel(Info.ressources.getObject("Position").toString()+":");
	lightsConstraint.gridx = 0;
	lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(pointLightsPosition,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel pointLightsPositionX = new JLabel("X");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsPositionX,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel pointLightsSpinnerXModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
								       Float.valueOf(SPINNER_COORDINATE_MIN), //min
								       Float.valueOf(SPINNER_COORDINATE_MAX), //max
								       Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner pointLightsPositionXSpinner = new JSpinner(pointLightsSpinnerXModel);
	((JSpinner.DefaultEditor)pointLightsPositionXSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsPositionXSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(pointLightsPositionXSpinner,lightsConstraint);

	JLabel pointLightsPositionY = new JLabel("Y");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsPositionY,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel pointLightsSpinnerYModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
								       Float.valueOf(SPINNER_COORDINATE_MIN), //min
								       Float.valueOf(SPINNER_COORDINATE_MAX), //max
								       Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner pointLightsPositionYSpinner = new JSpinner(pointLightsSpinnerYModel);
	((JSpinner.DefaultEditor)pointLightsPositionYSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsPositionYSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(pointLightsPositionYSpinner,lightsConstraint);

	JLabel pointLightsPositionZ = new JLabel("Z");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsPositionZ,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel pointLightsSpinnerZModel = new SpinnerNumberModel(Float.valueOf(1.5f), //initial value
								       Float.valueOf(SPINNER_COORDINATE_MIN), //min
								       Float.valueOf(SPINNER_COORDINATE_MAX), //max
								       Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner pointLightsPositionZSpinner = new JSpinner(pointLightsSpinnerZModel);
	((JSpinner.DefaultEditor)pointLightsPositionZSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsPositionZSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(pointLightsPositionZSpinner,lightsConstraint);
	
	JLabel pointLightsAttenuation = new JLabel(Info.ressources.getObject("Attenuation").toString()+":");
	lightsConstraint.gridx = 0; lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(pointLightsAttenuation,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel pointLightsAttenuationX = new JLabel("X");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsAttenuationX,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel pointLightsSpinnerXAttenuationModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
										  Float.valueOf(SPINNER_COORDINATE_MIN), //min
										  Float.valueOf(SPINNER_COORDINATE_MAX), //max
										  Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner pointLightsAttenuationXSpinner = new JSpinner(pointLightsSpinnerXAttenuationModel);
	((JSpinner.DefaultEditor)pointLightsAttenuationXSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsAttenuationXSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(pointLightsAttenuationXSpinner,lightsConstraint);

	JLabel pointLightsAttenuationY = new JLabel("Y");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsAttenuationY,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel pointLightsSpinnerYAttenuationModel = new SpinnerNumberModel(Float.valueOf(0.5f), //initial value
										  Float.valueOf(SPINNER_COORDINATE_MIN), //min
										  Float.valueOf(SPINNER_COORDINATE_MAX), //max
										  Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner pointLightsAttenuationYSpinner = new JSpinner(pointLightsSpinnerYAttenuationModel);
	((JSpinner.DefaultEditor)pointLightsAttenuationYSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsAttenuationYSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(pointLightsAttenuationYSpinner,lightsConstraint);

	JLabel pointLightsAttenuationZ = new JLabel("Z");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(pointLightsAttenuationZ,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel pointLightsSpinnerZAttenuationModel = new SpinnerNumberModel(Float.valueOf(1.0f), //initial value
										  Float.valueOf(SPINNER_COORDINATE_MIN), //min
										  Float.valueOf(SPINNER_COORDINATE_MAX), //max
										  Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner pointLightsAttenuationZSpinner = new JSpinner(pointLightsSpinnerZAttenuationModel);
	((JSpinner.DefaultEditor)pointLightsAttenuationZSpinner.getEditor()).getTextField().setColumns(3);
	pointLightsAttenuationZSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(pointLightsAttenuationZSpinner,lightsConstraint);

	pointLightsVector.add(new PointLightObject(pointLightButton, 
						   pointLightsColorRSpinner, 
						   pointLightsColorGSpinner, 
						   pointLightsColorBSpinner,
						   pointLightsPositionXSpinner,
						   pointLightsPositionYSpinner,
						   pointLightsPositionZSpinner,
						   pointLightsAttenuationXSpinner,
						   pointLightsAttenuationYSpinner,
						   pointLightsAttenuationZSpinner,
						   sceneTransform
						   ));

	NbLignsLights += 5;
    }



    /**
     * Panel creation for controlling a Spot light in the scene.
     * 
     */
    private void CreateSpotLightControl()
    {
	// Spot Light 
	JRadioButton spotLightButton = new JRadioButton(Info.ressources.getObject("Spot_Light").toString());
	spotLightButton.setSelected(false);
	spotLightButton.addActionListener(this);
	lightsConstraint.gridx = 0; 
	lightsConstraint.gridy = NbLignsLights;
	lightsPanel.add(spotLightButton,lightsConstraint);

	JLabel spotLightsColor = new JLabel(Info.ressources.getObject("Color").toString()+":");
	lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(spotLightsColor,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel spotLightsColorR = new JLabel(Info.ressources.getObject("R").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsColorR,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel spotLightsSpinnerRModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
								      SPINNER_COLOR_MIN, //min
								      SPINNER_COLOR_MAX, //max
								      SPINNER_COLOR_STEP); //step      
	JSpinner spotLightsColorRSpinner = new JSpinner(spotLightsSpinnerRModel);
	((JSpinner.DefaultEditor)spotLightsColorRSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsColorRSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(spotLightsColorRSpinner,lightsConstraint);

	JLabel spotLightsColorG = new JLabel(Info.ressources.getObject("G").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsColorG,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel spotLightsSpinnerGModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
								      SPINNER_COLOR_MIN, //min
								      SPINNER_COLOR_MAX, //max
								      SPINNER_COLOR_STEP); //step      
	JSpinner spotLightsColorGSpinner = new JSpinner(spotLightsSpinnerGModel);
	((JSpinner.DefaultEditor)spotLightsColorGSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsColorGSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(spotLightsColorGSpinner,lightsConstraint);

	JLabel spotLightsColorB = new JLabel(Info.ressources.getObject("B").toString());
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsColorB,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel spotLightsSpinnerBModel = new SpinnerNumberModel(SPINNER_COLOR_INIT, //initial value
								      SPINNER_COLOR_MIN, //min
								      SPINNER_COLOR_MAX, //max
								      SPINNER_COLOR_STEP); //step      
	JSpinner spotLightsColorBSpinner = new JSpinner(spotLightsSpinnerBModel);
	((JSpinner.DefaultEditor)spotLightsColorBSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsColorBSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(spotLightsColorBSpinner,lightsConstraint);

	JLabel spotLightsPosition = new JLabel(Info.ressources.getObject("Position").toString()+":");
	lightsConstraint.gridx = 0;
	lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(spotLightsPosition,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel spotLightsPositionX = new JLabel("X");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsPositionX,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel spotLightsSpinnerXModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
								      Float.valueOf(SPINNER_COORDINATE_MIN), //min
								      Float.valueOf(SPINNER_COORDINATE_MAX), //max
								      Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsPositionXSpinner = new JSpinner(spotLightsSpinnerXModel);
	((JSpinner.DefaultEditor)spotLightsPositionXSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsPositionXSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(spotLightsPositionXSpinner,lightsConstraint);

	JLabel spotLightsPositionY = new JLabel("Y");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsPositionY,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel spotLightsSpinnerYModel = new SpinnerNumberModel(Float.valueOf(1.7f), //initial value
								      Float.valueOf(SPINNER_COORDINATE_MIN), //min
								      Float.valueOf(SPINNER_COORDINATE_MAX), //max
								      Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsPositionYSpinner = new JSpinner(spotLightsSpinnerYModel);
	((JSpinner.DefaultEditor)spotLightsPositionYSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsPositionYSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(spotLightsPositionYSpinner,lightsConstraint);

	JLabel spotLightsPositionZ = new JLabel("Z");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsPositionZ,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel spotLightsSpinnerZModel = new SpinnerNumberModel(Float.valueOf(1.5f), //initial value
								      Float.valueOf(SPINNER_COORDINATE_MIN), //min
								      Float.valueOf(SPINNER_COORDINATE_MAX), //max
								      Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsPositionZSpinner = new JSpinner(spotLightsSpinnerZModel);
	((JSpinner.DefaultEditor)spotLightsPositionZSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsPositionZSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(spotLightsPositionZSpinner,lightsConstraint);
	
	JLabel spotLightsAttenuation = new JLabel(Info.ressources.getObject("Attenuation").toString()+":");
	lightsConstraint.gridx = 0; lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(spotLightsAttenuation,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel spotLightsAttenuationX = new JLabel("X");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsAttenuationX,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel spotLightsSpinnerXAttenuationModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
										 Float.valueOf(SPINNER_COORDINATE_MIN), //min
										 Float.valueOf(SPINNER_COORDINATE_MAX), //max
										 Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsAttenuationXSpinner = new JSpinner(spotLightsSpinnerXAttenuationModel);
	((JSpinner.DefaultEditor)spotLightsAttenuationXSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsAttenuationXSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(spotLightsAttenuationXSpinner,lightsConstraint);

	JLabel spotLightsAttenuationY = new JLabel("Y");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsAttenuationY,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel spotLightsSpinnerYAttenuationModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
										 Float.valueOf(SPINNER_COORDINATE_MIN), //min
										 Float.valueOf(SPINNER_COORDINATE_MAX), //max
										 Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsAttenuationYSpinner = new JSpinner(spotLightsSpinnerYAttenuationModel);
	((JSpinner.DefaultEditor)spotLightsAttenuationYSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsAttenuationYSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(spotLightsAttenuationYSpinner,lightsConstraint);

	JLabel spotLightsAttenuationZ = new JLabel("Z");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsAttenuationZ,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel spotLightsSpinnerZAttenuationModel = new SpinnerNumberModel(Float.valueOf(1.5f), //initial value
										 Float.valueOf(SPINNER_COORDINATE_MIN), //min
										 Float.valueOf(SPINNER_COORDINATE_MAX), //max
										 Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsAttenuationZSpinner = new JSpinner(spotLightsSpinnerZAttenuationModel);
	((JSpinner.DefaultEditor)spotLightsAttenuationZSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsAttenuationZSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(spotLightsAttenuationZSpinner,lightsConstraint);

	JLabel spotLightsDirection = new JLabel(Info.ressources.getObject("Direction").toString()+":");
	lightsConstraint.gridx = 0;
	lightsConstraint.gridy ++;
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsPanel.add(spotLightsDirection,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);

	JLabel spotLightsDirectionX = new JLabel("X");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsDirectionX,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel spotLightsSpinnerXDirectionModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
									       Float.valueOf(SPINNER_COORDINATE_MIN), //min
									       Float.valueOf(SPINNER_COORDINATE_MAX), //max
									       Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsDirectionXSpinner = new JSpinner(spotLightsSpinnerXDirectionModel);
	((JSpinner.DefaultEditor)spotLightsDirectionXSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsDirectionXSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(spotLightsDirectionXSpinner,lightsConstraint);

	JLabel spotLightsDirectionY = new JLabel("Y");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsDirectionY,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);
	
	SpinnerModel spotLightsSpinnerYDirectionModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
									       Float.valueOf(SPINNER_COORDINATE_MIN), //min
									       Float.valueOf(SPINNER_COORDINATE_MAX), //max
									       Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsDirectionYSpinner = new JSpinner(spotLightsSpinnerYDirectionModel);
	((JSpinner.DefaultEditor)spotLightsDirectionYSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsDirectionYSpinner.addChangeListener(this);
	lightsConstraint.gridx++;
	lightsPanel.add(spotLightsDirectionYSpinner,lightsConstraint);

	JLabel spotLightsDirectionZ = new JLabel("Z");
	lightsConstraint.gridx ++;
	lightsConstraint.insets = new Insets(0,10,0,5);
	lightsPanel.add(spotLightsDirectionZ,lightsConstraint);
	lightsConstraint.insets = new Insets(0,0,0,0);

	SpinnerModel spotLightsSpinnerZDirectionModel = new SpinnerNumberModel(Float.valueOf(SPINNER_COORDINATE_INIT), //initial value
									       Float.valueOf(SPINNER_COORDINATE_MIN), //min
									       Float.valueOf(SPINNER_COORDINATE_MAX), //max
									       Float.valueOf(SPINNER_COORDINATE_STEP)); //step
	JSpinner spotLightsDirectionZSpinner = new JSpinner(spotLightsSpinnerZDirectionModel);
	((JSpinner.DefaultEditor)spotLightsDirectionZSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsDirectionZSpinner.addChangeListener(this);
	lightsConstraint.gridx ++;
	lightsPanel.add(spotLightsDirectionZSpinner,lightsConstraint);

	JLabel spotLightsSpreadAngle = new JLabel(Info.ressources.getObject("SpreadAngle").toString()+":");
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsConstraint.gridx = 0;
	lightsConstraint.gridy ++;
	lightsPanel.add(spotLightsSpreadAngle,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);
	lightsConstraint.gridx +=2;

	SpinnerModel spotLightsSpinnerSpreadAngleModel = new SpinnerNumberModel(Float.valueOf(SPINNER_ANGLE_INIT), //initial value
										Float.valueOf(SPINNER_ANGLE_MIN), //min
										Float.valueOf(SPINNER_ANGLE_MAX), //max
										Float.valueOf(SPINNER_ANGLE_STEP)); //step
	JSpinner spotLightsSpreadAngleSpinner = new JSpinner(spotLightsSpinnerSpreadAngleModel);
	((JSpinner.DefaultEditor)spotLightsSpreadAngleSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsSpreadAngleSpinner.addChangeListener(this);
	lightsPanel.add(spotLightsSpreadAngleSpinner,lightsConstraint);

	JLabel spotLightsConcentration = new JLabel(Info.ressources.getObject("Concentration").toString()+":");
	lightsConstraint.anchor = GridBagConstraints.EAST;
	lightsConstraint.insets = new Insets(0,0,0,10);
	lightsConstraint.gridx = 0;
	lightsConstraint.gridy ++;
	lightsPanel.add(spotLightsConcentration,lightsConstraint);
	lightsConstraint.anchor = GridBagConstraints.WEST;
	lightsConstraint.insets = new Insets(0,0,0,0);
	lightsConstraint.gridx +=2;

	SpinnerModel spotLightsSpinnerConcentrationModel = new SpinnerNumberModel(Float.valueOf(SPINNER_CONCENTRATION_INIT), //initial value
										  Float.valueOf(SPINNER_CONCENTRATION_MIN), //min
										  Float.valueOf(SPINNER_CONCENTRATION_MAX), //max
										  Float.valueOf(SPINNER_CONCENTRATION_STEP)); //step
	JSpinner spotLightsConcentrationSpinner = new JSpinner(spotLightsSpinnerConcentrationModel);
	((JSpinner.DefaultEditor)spotLightsConcentrationSpinner.getEditor()).getTextField().setColumns(3);
	spotLightsConcentrationSpinner.addChangeListener(this);
	lightsPanel.add(spotLightsConcentrationSpinner,lightsConstraint);
	
	
	spotLightsVector.add(new SpotLightObject(spotLightButton, 
						 spotLightsColorRSpinner, 
						 spotLightsColorGSpinner, 
						 spotLightsColorBSpinner,
						 spotLightsPositionXSpinner,
						 spotLightsPositionYSpinner,
						 spotLightsPositionZSpinner,
						 spotLightsAttenuationXSpinner,
						 spotLightsAttenuationYSpinner,
						 spotLightsAttenuationZSpinner,
						 spotLightsDirectionXSpinner,
						 spotLightsDirectionYSpinner,
						 spotLightsDirectionZSpinner,
						 spotLightsSpreadAngleSpinner,
						 spotLightsConcentrationSpinner,
						 sceneTransform
						 ));

	NbLignsLights += 8;
    }


    /**
     * Panel for the 3D scene.
     * 
     */
    protected JPanel makeScenePanel()
    {
	scenePanel = new JPanel();
	scenePanel.setLayout(new GridLayout(2, 2));
	scenePanel.setPreferredSize(new Dimension(350, 350));
	
	// Creation of Universe and Local for the 3D scene and the cameras
	VirtualUniverse universe = new VirtualUniverse();
	Locale locale = new Locale(universe);

	// Creation of the Views 
	// Front View
	vueFront = new SceneView();
	TransformGroup vpTG = vueFront.getViewPlatformTransformGroup();
	Transform3D transform = new Transform3D();
	transform.setTranslation(new Vector3f(0f, 0f, 3.7f));
	vpTG.setTransform(transform);

	// Right View
	vueRight = new SceneView();
	vpTG = vueRight.getViewPlatformTransformGroup();
	transform = new Transform3D();
	transform.setTranslation(new Vector3f(-3.7f, 0f, 0f));
	Transform3D transform2 = new Transform3D();
	transform2.rotY(-Math.PI / 2.0);
	transform.mul(transform2);
	vpTG.setTransform(transform);

	// Left View
	vueLeft = new SceneView();
	vpTG = vueLeft.getViewPlatformTransformGroup();
	transform = new Transform3D();
	transform.setTranslation(new Vector3f(3.7f, 0f, 0f));
	transform2 = new Transform3D();
	transform2.rotY(Math.PI / 2.0);
	transform.mul(transform2);
	vpTG.setTransform(transform);

	// Top View
	vueTop = new SceneView();
	vpTG = vueTop.getViewPlatformTransformGroup();
	transform = new Transform3D();
	transform.setTranslation(new Vector3f(0f, 3.7f, 0f));	
	transform2 = new Transform3D();
	transform2.rotX(-Math.PI / 2.0);
	transform.mul(transform2);
	vpTG.setTransform(transform);
	
	// Big View 
	bigVue = new SceneView();
	vpTG = bigVue.getViewPlatformTransformGroup();
	transform = new Transform3D();
	transform.setTranslation(new Vector3f(0f, 0f, 3.7f));

	/*transform.setTranslation(new Vector3f(-2.4f, 1.7f, 1.7f));
	  transform2 = new Transform3D();
	  transform2.rotX(-Math.PI / 4.0);
	  transform.mul(transform2);
	  transform2 = new Transform3D();
	  transform2.rotY(-Math.PI / 4.0);
	  transform.mul(transform2);	*/
	vpTG.setTransform(transform);

	// Attach the views to the locale object
	locale.addBranchGraph(vueFront.getParent());
	locale.addBranchGraph(vueRight.getParent());
	locale.addBranchGraph(vueLeft.getParent());
	locale.addBranchGraph(vueTop.getParent());
	locale.addBranchGraph(bigVue.getParent());

	// 3D Scene Creation        
	scene = createSceneGraph(vueTop.getCanvas3D(),
				 vueRight.getCanvas3D(),
				 vueLeft.getCanvas3D(),
				 vueFront.getCanvas3D(),
				 bigVue.getCanvas3D());
		
	// Compilation of the 3D Scene
	scene.compile();

	// Attach the Scene to the locale object
	locale.addBranchGraph(scene);

	// Add all the canvas 3D (correspondant a chaque vue) to the main frame
	// Top view
	JPanel panelTop = new JPanel();
	panelTop.setLayout(new BorderLayout());
	TitledBorder borderTop = new TitledBorder(new EtchedBorder(),
						  Info.ressources.getObject("Top_View").toString());
	panelTop.setBorder(borderTop);
	panelTop.add(vueTop.getCanvas3D(), BorderLayout.CENTER);

	// Right view
	JPanel panelRight = new JPanel();
	panelRight.setLayout(new BorderLayout());
	TitledBorder borderRight = new TitledBorder(new EtchedBorder(),
						    Info.ressources.getObject("Right_View").toString());
	panelRight.setBorder(borderRight);
	panelRight.add(vueRight.getCanvas3D(), BorderLayout.CENTER);

	// Left View
	JPanel panelLeft = new JPanel();
	panelLeft.setLayout(new BorderLayout());
	TitledBorder borderLeft = new TitledBorder(new EtchedBorder(),
						   Info.ressources.getObject("Left_View").toString());
	panelLeft.setBorder(borderLeft);
	panelLeft.add(vueLeft.getCanvas3D(), BorderLayout.CENTER);

	// Front View
	JPanel panelFront = new JPanel();
	panelFront.setLayout(new BorderLayout());
	TitledBorder borderFront = new TitledBorder(new EtchedBorder(),
						    Info.ressources.getObject("Front_View").toString());
	panelFront.setBorder(borderFront);
	panelFront.add(vueFront.getCanvas3D(), BorderLayout.CENTER);

	// Add all the 4 vues to the main Frame
	scenePanel.add(panelTop);
	scenePanel.add(panelRight);
	scenePanel.add(panelLeft);
	scenePanel.add(panelFront);

	// Big view
	JPanel panelBigVue = new JPanel();
	panelBigVue.setLayout(new BorderLayout());
	TitledBorder borderBigVue = new TitledBorder(new EtchedBorder(),Info.ressources.getObject("Big_View").toString());
	panelBigVue.setBorder(borderBigVue);
	panelBigVue.add(bigVue.getCanvas3D(), BorderLayout.CENTER);

	// Add the Big View to a dedicated Frame
	bigVueFrame = new JFrame();
	bigVueFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	bigVueFrame.add(panelBigVue);

	Dimension d = new Dimension(550, 550);
	bigVueFrame.setSize(d);
	//bigVueFrame.setResizable(false);
	bigVueFrame.setVisible(false);       

	return scenePanel;
    }


    /**
     * Panel creation for choosing the object in the Scene : Cube, Cylinder, Sphere, Cone.
     * 
     */
    protected JPanel makeObjectChooserPanel()
    {
	JPanel objectChooserPanel = new JPanel();

	boxButton = new JRadioButton(Info.ressources.getObject("Cube").toString());
	boxButton.setSelected(true);
	boxButton.addActionListener(this);

	sphereButton = new JRadioButton(Info.ressources.getObject("Sphere").toString());
	sphereButton.addActionListener(this);

	cylinderButton = new JRadioButton(Info.ressources.getObject("Cylinder").toString());
	cylinderButton.addActionListener(this);

	coneButton = new JRadioButton(Info.ressources.getObject("Cone").toString());
	coneButton.addActionListener(this);

	//Group the Objects selection buttons.
        ButtonGroup group = new ButtonGroup();
        group.add(boxButton);
        group.add(sphereButton);
        group.add(cylinderButton);
        group.add(coneButton);      

	//Attach the Objects selection buttons
	objectChooserPanel.add(boxButton);
	objectChooserPanel.add(sphereButton);
	objectChooserPanel.add(cylinderButton);
	objectChooserPanel.add(coneButton);

	return objectChooserPanel;
    }


    /**
     * redraw all the rendering panels
     * 
     */
    public void repaint()
    {
	super.repaint();   
	validate(); 
	if(lightsPanel!=null)
	    lightsPanel.repaint();
	if(lightsPanelScrollPane!=null)
	    lightsPanelScrollPane.repaint();
	if(scenePanel!=null)
	    scenePanel.repaint();
    }

    
    /**
     * run when the user modifies a light parameter on a running light. It permits to update the scene based on the user choice.     
     * @param e the associated event
     * 
     */
    public void stateChanged(ChangeEvent e) 
    {
	Object source = e.getSource();
	
	if(source == backgroundColorRSpinner || source == backgroundColorGSpinner || source == backgroundColorBSpinner)
	{
		setBackgroundColor(new Color3f((float)(Integer)backgroundColorRSpinner.getModel().getValue() /255, 
				(float)(Integer)backgroundColorGSpinner.getModel().getValue() /255,
				(float)(Integer)backgroundColorBSpinner.getModel().getValue() /255));
	}

	else
	{
	// Look at Selection Button First
	// And then if it is set, look at parameters inside
	// Then the event may not be found if the associated Selection Button is not set
	// This algorithm should be quicker

	// Look at Ambient Lights Button
	for (Iterator it = ambientLightsVector.iterator(); it.hasNext();) 
	    {
		AmbientLightObject obj = (AmbientLightObject)it.next();
		if(obj.getButton().isSelected())
		    {
			if(source == obj.getColorRSpinner() || source == obj.getColorGSpinner() || source == obj.getColorBSpinner())
			    {
				obj.updateColor();
				return;
			    }			
		    }				
	    }

	// Look at Directional Lights Button
	for (Iterator it = directionalLightsVector.iterator(); it.hasNext();) 
	    {
		DirectionalLightObject obj = (DirectionalLightObject)it.next();
		if(obj.getButton().isSelected())
		    {
			if(source == obj.getColorRSpinner() || source == obj.getColorGSpinner() || source == obj.getColorBSpinner())
			    {
				obj.updateColor();
				return;
			    }
			else if (source == obj.getDirectionXSpinner() || source == obj.getDirectionYSpinner() || source == obj.getDirectionZSpinner())
			    {
				obj.updateDirection();
				return;
			    }			
		    }				
	    }

	// Look at Point Lights Button
	for (Iterator it = pointLightsVector.iterator(); it.hasNext();) 
	    {
		PointLightObject obj = (PointLightObject)it.next();
		if(obj.getButton().isSelected())
		    {
			if(source == obj.getColorRSpinner() || source == obj.getColorGSpinner() || source == obj.getColorBSpinner())
			    {
				obj.updateColor();
				return;
			    }
			
			else if (source == obj.getPositionXSpinner() || source == obj.getPositionYSpinner() || source == obj.getPositionZSpinner())
			    {
				obj.updatePosition();
				return;
			    }			

			else if (source == obj.getAttenuationXSpinner() || source == obj.getAttenuationYSpinner() || source == obj.getAttenuationZSpinner())
			    {
				obj.updateAttenuation();
				return;
			    }			
		    }				
	    }

	// Look at Spot Lights Button
	for (Iterator it = spotLightsVector.iterator(); it.hasNext();) 
	    {
		SpotLightObject obj = (SpotLightObject)it.next();
		if(obj.getButton().isSelected())
		    {
			if(source == obj.getColorRSpinner() || source == obj.getColorGSpinner() || source == obj.getColorBSpinner())
			    {
				obj.updateColor();
				return;
			    }
			
			else if (source == obj.getPositionXSpinner() || source == obj.getPositionYSpinner() || source == obj.getPositionZSpinner())
			    {
				obj.updatePosition();
				return;
			    }			

			else if (source == obj.getAttenuationXSpinner() || source == obj.getAttenuationYSpinner() || source == obj.getAttenuationZSpinner())
			    {
				obj.updateAttenuation();
				return;
			    }			

			else if (source == obj.getDirectionXSpinner() || source == obj.getDirectionYSpinner() || source == obj.getDirectionZSpinner())
			    {
				obj.updateDirection();
				return;
			    }			

			else if (source == obj.getSpreadAngleSpinner())
			    {
				obj.updateSpreadAngle();
				return;
			    }			
			
			else if (source == obj.getConcentrationSpinner())
			    {
				obj.updateConcentration();
				return;
			    }			
		    }				
	    }
	}
    }


 
    /**
     * run when the user chooses another object in the scene or wants a 3D Axis, Antialiasing or a Big View.
     * @param e the associated event
     * 
     */
    public void actionPerformed(ActionEvent e) 
    {
	Object source = e.getSource();
	
	if(source == boxButton)
	    setObject(0);
	else if (source == sphereButton)
	    setObject(1);
	else if (source == cylinderButton)
	    setObject(2);
	else if (source == coneButton)
	    setObject(3);
	else if (source == antialiasingButton)
	    setSceneAntialiasing(antialiasingButton.isSelected());
	else if (source == axisButton)
	    showAxis();
	else if (source == bigVueButton)
	    bigVueFrame.setVisible(bigVueButton.isSelected());
	else if (source == addAmbientLightButton)
	    {
		CreateAmbientLightControl();
		addAmbientLightButton.setSelected(false);
		repaint();	
	    }
	else if (source == addPointLightButton)
	    {
		CreatePointLightControl();
		addPointLightButton.setSelected(false);		
		repaint();
	    }
	else if (source == addDirectionalLightButton)
	    {
		CreateDirectionalLightControl();
		addDirectionalLightButton.setSelected(false);
		repaint();
	    }
	else if (source == addSpotLightButton)
	    {
		CreateSpotLightControl();
		addSpotLightButton.setSelected(false);
		repaint();
	    }

	else 
	    {
		// Look at Ambient Lights Button
		for (Iterator it = ambientLightsVector.iterator(); it.hasNext();) 
		    {
			AmbientLightObject obj = (AmbientLightObject)it.next();
			if(source == obj.getButton())
			    {
				if(obj.getButton().isSelected()) 
				    obj.set(true);				    
				else
				    obj.set(false);				    
				    
				return;			       
			    }					
		    }

		// Look at Directional Lights Button
		for (Iterator it = directionalLightsVector.iterator(); it.hasNext();) 
		    {
			DirectionalLightObject obj = (DirectionalLightObject) it.next();
			if(source == obj.getButton())
			    {
				if(obj.getButton().isSelected())	 
				    obj.set(true);				    
				else
				    obj.set(false);				    

				return;
			    }					
		    }

		// Look at Point Lights Button
		for (Iterator it = pointLightsVector.iterator(); it.hasNext();) 
		    {
			PointLightObject obj = (PointLightObject) it.next();
			if(source == obj.getButton())
			    {
				if(obj.getButton().isSelected())	 
				    obj.set(true);				    
				else
				    obj.set(false);				    

				return;
			    }					
		    }

		// Look at Spot Lights Button
		for (Iterator it = spotLightsVector.iterator(); it.hasNext();) 
		    {
			SpotLightObject obj = (SpotLightObject) it.next();
			if(source == obj.getButton())
			    {
				if(obj.getButton().isSelected())	 
				    obj.set(true);				    
				else
				    obj.set(false);				    

				return;
			    }					
		    }
		
	    }
    }


    /**
     * 3D Scene Creation.
     * @param top Top canvas.
     * @param right Right canvas.
     * @param left Left canvas. 
     * @param front Front canvas. 
     * @param big Big canvas. 
     * @return 3D Scene.
     */
    public BranchGroup createSceneGraph(Canvas3D top,
					Canvas3D right,
					Canvas3D left,
					Canvas3D front,
					Canvas3D big					
					) {
	
	// Creation of the parent object
	BranchGroup parent = new BranchGroup();

	// Creation of the scene transform
	sceneTransform = new TransformGroup();
	
	sceneTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	sceneTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	sceneTransform.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
	sceneTransform.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);

	BoundingSphere infiniteBounds = new BoundingSphere( new Point3d(), Double.MAX_VALUE );

	// Creation mouse rotation behaviour
	// MouseRotate rotate = new MouseRotate(sceneTransform);
	CustomMouseRotate rotate = new CustomMouseRotate(sceneTransform,
							 top,
							 right,
							 left,
							 front,
							 big
							 );
	rotate.setSchedulingBounds(infiniteBounds);
	rotate.setEnable(true);
	parent.addChild(rotate);

	// Creation mouse translation behaviour
	// MouseTranslate translate = new MouseTranslate(sceneTransform);
	CustomMouseTranslate translate = new CustomMouseTranslate(sceneTransform,
								  top,
								  right,
								  left,
								  front,
								  big
								  );	       
	translate.setSchedulingBounds(infiniteBounds);
	translate.setEnable(true);
	parent.addChild(translate);

	// Creation mouse zoom behaviour
	// MouseZoom zoom = new MouseZoom(sceneTransform);
	CustomMouseZoom zoom = new CustomMouseZoom(sceneTransform,
						   top,
						   right,
						   left,
						   front,
						   big						   
						   );
	zoom.setSchedulingBounds(infiniteBounds);
	zoom.setEnable(true);
	parent.addChild(zoom);

	// Scene objects Creation 
	objectsSwitch = new Switch();   
	objectsSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);
	appearanceObjects.setCapability(Appearance.ALLOW_TEXTURE_WRITE);
	appearanceObjects.setCapability(Appearance.ALLOW_TEXGEN_WRITE);
	appearanceObjects.setCapability(Appearance.ALLOW_TEXTURE_ATTRIBUTES_WRITE);	
	appearanceObjects.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
	appearanceObjects.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
	appearanceObjects.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_WRITE);
	appearanceObjects.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);		
	
	Material m = new Material();	
	m.setCapability(Material.ALLOW_COMPONENT_WRITE); 	
	appearanceObjects.setMaterial(m);
	
	TransparencyAttributes t = new TransparencyAttributes();
	t.setCapability(TransparencyAttributes.ALLOW_BLEND_FUNCTION_WRITE);	
	t.setCapability(TransparencyAttributes.ALLOW_MODE_WRITE);	
	t.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);	
	appearanceObjects.setTransparencyAttributes(t);
				
	createTexture();

	//ColorCube cube = new ColorCube(0.3);
	com.sun.j3d.utils.geometry.Box box = new com.sun.j3d.utils.geometry.Box(0.5f,0.5f,0.5f, Primitive.GENERATE_TEXTURE_COORDS 
										+ Primitive.GENERATE_NORMALS, appearanceObjects);
	Sphere sphere = new Sphere(0.6f,Primitive.GENERATE_TEXTURE_COORDS + Primitive.GENERATE_NORMALS, 30, appearanceObjects);
	Cylinder cylinder = new Cylinder(0.5f,1.0f,Primitive.GENERATE_TEXTURE_COORDS + Primitive.GENERATE_NORMALS,appearanceObjects);
	Cone cone = new Cone(0.5f,1.2f,Primitive.GENERATE_TEXTURE_COORDS + Primitive.GENERATE_NORMALS,appearanceObjects);		

	objectsSwitch.addChild(box);
	objectsSwitch.addChild(sphere);
	objectsSwitch.addChild(cylinder);
	objectsSwitch.addChild(cone);

	objectsSwitch.setWhichChild(0);
	sceneTransform.addChild(objectsSwitch);
    
	// Create Axis
	createAxis();
	sceneTransform.addChild(axisFrontend);

	// Add to the Scene root
	parent.addChild(sceneTransform);
	
	// Create the Background Color
	createBackground();				
	parent.addChild(background);
	

	return parent;
    }

    /**
	 *
	 * Background creation for the scene objects.
	 * 
	 */
	protected void createBackground()
	{	
		// Create application bounds
		BoundingSphere worldBounds = new BoundingSphere(new Point3d(0.0, 0.0,
				0.0), // Center
				1000.0); // Extent
		Color3f color = new Color3f((float) SPINNER_BACKGROUNDCOLOR_R_INIT /255,
				(float)SPINNER_BACKGROUNDCOLOR_G_INIT /255,
				(float)SPINNER_BACKGROUNDCOLOR_B_INIT /255) ;
		// Set the background color and its application bounds
		background = new Background();
		background.setColor(color);
		background.setCapability(Background.ALLOW_COLOR_WRITE);
		background.setApplicationBounds(worldBounds);
	}
	
    /**
	 *
	 * Set the Background color.
	 * @param c the Color to set.
	 * 
	 */
	protected void setBackgroundColor(Color3f c)
	{	
		background.setColor(c);
	}
	
    /**
     *
     * Texture creation for the scene objects
     * 
     */
    protected void createTexture()
    {	
	URL urlTexture = null;

	try
	    {							
		urlTexture = DefaultTexture;
		texture = new TextureLoader(urlTexture, this).getTexture();
	    }
	catch (Exception ex1) {
	    System.err.println("Error : Cannot load Texture File : " + DefaultTexture);
	}
       
	textureTransform = new Transform3D();		
	textureAttributes = new TextureAttributes();	    	    	    
	textureAttributes.setCapability(TextureAttributes.ALLOW_BLEND_COLOR_WRITE);
	textureAttributes.setCapability(TextureAttributes.ALLOW_MODE_WRITE);
	textureAttributes.setCapability(TextureAttributes.ALLOW_COMBINE_WRITE);
	textureAttributes.setCapability(TextureAttributes.ALLOW_TRANSFORM_WRITE);
	textureAttributes.setTextureTransform(textureTransform);	    
	textureAttributes.setTextureMode(TextureAttributes.REPLACE);		           
	appearanceObjects.setTextureAttributes(textureAttributes);
		
	texture.setCapability(Texture.ALLOW_ENABLE_WRITE);
	texture.setCapability(Texture.ALLOW_IMAGE_WRITE);     	

	appearanceObjects.setTexture(texture);	
	texture.setEnable(false);
    }


    /**
     * Show a defined object. Ie Sphere, Cone, Cylinder or Cube
     * @param i object number to draw
     */
    public void setObject(int i)
    {
	if(i>3) i= 0;
	objectsSwitch.setWhichChild(i);	    
    }


    /**
     *
     * 3D Axis Creation for the scene. Each axis is 1.0 in length.
     * 
     */
    protected void createAxis() 
    {   
	TransformGroup axisFrontendTG = new TransformGroup();	
	
	axisFrontend = new Switch();	
	axisFrontend.setWhichChild(Switch.CHILD_MASK);
	axisFrontend.setCapability(Switch.ALLOW_SWITCH_WRITE);
	LineArray la = new LineArray(6,LineArray.COORDINATES|LineArray.COLOR_3);
	Point3f origin = new Point3f(0.0f,0.0f,0.0f);
	Point3f x = new Point3f(1.0f,0.0f,0.0f);
	Point3f y = new Point3f(0.0f,1.0f,0.0f);
	Point3f z = new Point3f(0.0f,0.0f,1.0f);
	Point3f coords[] = {origin, x, origin, y, origin, z};       
	la.setCoordinates(0,coords);       
	
	Color3f red = new Color3f(1.0f, 0.0f, 0.0f);
	Color3f green = new Color3f(0.0f, 1.0f, 0.0f);
	Color3f blue = new Color3f(0.0f, 0.0f, 1.0f);

	Color3f [] colors = {red,red, green,green, blue,blue};
	la.setColors(0,colors);

	LineAttributes laAttr = new LineAttributes();
	laAttr.setLineAntialiasingEnable(true);
	laAttr.setLineWidth(2.0f);
	Appearance laApp = new Appearance();
	laApp.setLineAttributes(laAttr);
	Shape3D axisShape = new Shape3D(la, laApp);
	axisFrontendTG.addChild(axisShape);

	//Font3D font3D = new Font3D(new Font("Helvetica", Font.PLAIN, 1),new FontExtrusion());
	Font3D font3D = new Font3D(new Font("Dialog", Font.BOLD, 1), new FontExtrusion());
	Text3D text3DX = new Text3D(font3D, "X", new Point3f(5.3f, -0.3f, -0.12f));
	text3DX.setAlignment(Text3D.ALIGN_CENTER);
	Shape3D XShape = new Shape3D();
	XShape.setGeometry(text3DX);	  	    	   
	Text3D text3DY = new Text3D(font3D, "Y", new Point3f(0.0f, 5.1f, -0.12f));
	text3DY.setAlignment(Text3D.ALIGN_CENTER);
	Shape3D YShape = new Shape3D();
	YShape.setGeometry(text3DY);	 
	Text3D text3DZ = new Text3D(font3D, "Z", new Point3f(0.0f, -0.3f, 5.1f));
	text3DZ.setAlignment(Text3D.ALIGN_CENTER);
	Shape3D ZShape = new Shape3D();
	ZShape.setGeometry(text3DZ);

	Transform3D scale3D = new Transform3D();
	scale3D.setScale(new Vector3d(0.2f,0.2f,0.2f));
	TransformGroup tgScale = new TransformGroup();
	tgScale.setTransform(scale3D);
	tgScale.addChild(XShape);
	tgScale.addChild(YShape);
	tgScale.addChild(ZShape);
	axisFrontendTG.addChild(tgScale);
	axisFrontend.addChild(axisFrontendTG);	
		
    }


    /**
     * Hide or Show the 3D Axis in the Scene.
     */
    public void showAxis() {
	BitSet visibilityMask = axisFrontend.getChildMask();
	visibilityMask.flip(0);
	axisFrontend.setChildMask(visibilityMask);
    }

	/**
	 * Set some default lights.
	 */
	public void initLights()  
	{
	Iterator it;
		
		// Set the Ambient Lights Button
		it = ambientLightsVector.iterator();		
		if (it.hasNext()) 
		{
			AmbientLightObject obj = (AmbientLightObject)it.next();
			obj.getButton().setSelected(true);			
			obj.set(true);						
		}
		
		// Set the Directional Lights Button
		it = directionalLightsVector.iterator();		
		if (it.hasNext()) 
		{
			DirectionalLightObject obj = (DirectionalLightObject)it.next();
			obj.getButton().setSelected(true);			
			obj.set(true);						
		}
		
		// Set the Point Lights Button
		it = pointLightsVector.iterator();		
		if (it.hasNext()) 
		{
			PointLightObject obj = (PointLightObject)it.next();
			obj.getButton().setSelected(true);			
			obj.set(true);						
		}
		
		// Set the Spot Lights Button
		it = spotLightsVector.iterator();		
		if (it.hasNext()) 
		{
			SpotLightObject obj = (SpotLightObject)it.next();
			obj.getButton().setSelected(true);			
			obj.set(true);						
		}
	}

    /**
     * Set Antialiasing on the scene.
     * @param on true if requested, false otherwise.
     */
    public void setSceneAntialiasing(boolean on)
    {
	vueTop.getView().setSceneAntialiasingEnable(on);	
	vueFront.getView().setSceneAntialiasingEnable(on);	
	vueLeft.getView().setSceneAntialiasingEnable(on);	
	vueRight.getView().setSceneAntialiasingEnable(on);	
	bigVue.getView().setSceneAntialiasingEnable(on);	
	//repaint();	
    }


    /**
     * Set the Ambient Color for the Scene objects.
     * @param c the Ambient Color to set.
     */
    public void setObjectsAmbientColor(Color3f c)
    {	
	appearanceObjects.getMaterial().setAmbientColor(c);
	//repaint();
    }

    /**
     * Set the Diffuse Color for the Scene objects.
     * @param c the Diffuse Color to set.
     */
    public void setObjectsDiffuseColor(Color3f c)
    {	
	appearanceObjects.getMaterial().setDiffuseColor(c);
	//repaint();
    }

    /**
     * Set the Specular Color for the Scene objects.
     * @param c the Specular Color to set.
     */
    public void setObjectsSpecularColor(Color3f c)
    {	
	appearanceObjects.getMaterial().setSpecularColor(c);
	//repaint();
    }

    /**
     * Set the Emissive Color for the Scene objects.
     * @param c the Emissive Color to set.
     */
    public void setObjectsEmissiveColor(Color3f c)
    {	
	appearanceObjects.getMaterial().setEmissiveColor(c);
	//repaint();
    }   

    /**
     * Set or Uset the lighting for the Scene objects.
     * @param b true if lighting is set, false otherwise.
     */
    public void setObjectsLighting(boolean b)
    {
	appearanceObjects.getMaterial().setLightingEnable(b);		
	//repaint();	
    }

    /**
     * Set the Shininess for the Scene objects.
     * @param s the Shininess.
     */
    public void setObjectsShininess(float s)
    {
	appearanceObjects.getMaterial().setShininess(s);
	//repaint();
    }

    /**
     * Set the Transparency value for the Scene objects.
     * @param s the Transparency value.
     */
    public void setObjectsTransparency(float s)
    {
	appearanceObjects.getTransparencyAttributes().setTransparency(s);
	//repaint();
    }

    /**
     * Set the Transparency mode for the Scene objects.
     * @param m the Transparency Mode.
     */
    public void setObjectsTransparencyMode(int m)
    {
	appearanceObjects.getTransparencyAttributes().setTransparencyMode(m);
	//repaint();
    }

    /**
     * Set the Transparency Src Blend Function for the Scene objects.
     * @param f the Transparency Src Blend Function.
     */
    public void setObjectsTransparencySrcBlendFunction(int f)
    {
	appearanceObjects.getTransparencyAttributes().setSrcBlendFunction(f);
	//repaint();
    }

    /**
     * Set the Transparency Dst Blend Function for the Scene objects.
     * @param f the Transparency Dst Blend Function.
     */
    public void setObjectsTransparencyDstBlendFunction(int f)
    {
	appearanceObjects.getTransparencyAttributes().setDstBlendFunction(f);
	//repaint();
    }
    

    /**
     * Set the Texture on the Scene objects.
     * @param urlTexture the Texture URL.
     */
    public void setTexture(URL urlTexture)
    {
	
	try {	    	  
	    /*TextureLoader loader = new TextureLoader(urlTexture, this);
	      texture.setImage(0,  loader.getTexture().getImage(0)); 
	      texture.setEnable(true);*/
	    // Recreate the texture because of a bug in some version for updating image 
	    // with a different size in the Texture Class
	    
	    TextureLoader loader = new TextureLoader(urlTexture, this);
	    texture = loader.getTexture();
	    texture.setCapability(Texture.ALLOW_ENABLE_WRITE);
	    texture.setCapability(Texture.ALLOW_IMAGE_WRITE);   	
	
	    appearanceObjects.setTexture(texture);

	}
	catch (Exception ex1) {
	    System.err.println("Error : Cannot load texture File : " + urlTexture);
	}       
	
	//repaint();		
    }

    
    /**
     * Unset the Texture use on the Scene objects.
     */
    public void unsetTexture()
    {
	try {	
	    appearanceObjects.getTexture().setEnable(false);
	}
	
	catch (Exception ex1) {	    
	}
	
	//repaint();
    }


    /**
     * Set the Texture transform to use on the Scene objects.
     * @param t the Texture transform.
     */
    public void setTextureTransform(Transform3D t)
    {
	textureTransform = t;
	textureAttributes.setTextureTransform(textureTransform);    
	//repaint();
    }


    /**
     * Set the Texture Perspective Correction Mode to use on the Scene objects.
     * @param m the Texture Perspective Correction Mode.
     */
    public void setObjectsTexturePerspectiveCorrectionMode(int m)
    {
	textureAttributes.setPerspectiveCorrectionMode(m);	
    }


    /**
     * Set the Texture Mode to use on the Scene objects.
     * @param m the Texture Mode.
     */
    public void setObjectsTextureMode(int m)
    {
	textureAttributes.setTextureMode(m);	
    }


    /**
     * Set the Texture COMBINE Mode for the RGB Components of the output color to use on the Scene objects.
     * @param m the Texture COMBINE Mode.
     */
    public void setObjectsTextureCombineRGBMode(int m)
    {
	textureAttributes.setCombineRgbMode(m);	
    }

    
    /**
     * Set the Texture COMBINE Source for the RGB Components to use on the Scene objects.
     * @param s the Texture COMBINE Source.
     */
    public void setObjectsTextureCombineRGBSource(int []s)
    {
	textureAttributes.setCombineRgbSource(0,s[0]);	
	textureAttributes.setCombineRgbSource(1,s[1]);	
	textureAttributes.setCombineRgbSource(2,s[2]);	
    }


    /**
     * Set the Texture COMBINE Function for the RGB Components to use on the Scene objects.
     * @param f the Texture COMBINE Function.
     */
    public void setObjectsTextureCombineRGBFunction(int []f)
    {
	textureAttributes.setCombineRgbFunction(0,f[0]);	
	textureAttributes.setCombineRgbFunction(1,f[1]);	
	textureAttributes.setCombineRgbFunction(2,f[2]);	
    }


    /**
     * Set the Texture COMBINE Scale for the RGB Components of the output color to use on the Scene objects.
     * @param m the Texture COMBINE Scale.
     */
    public void setObjectsTextureCombineRGBScale(int m)
    {
	textureAttributes.setCombineRgbScale(m);	
    }


    /**
     * Set the Texture COMBINE Mode for the Alpha Components of the output color to use on the Scene objects.
     * @param m the Texture COMBINE Mode.
     */
    public void setObjectsTextureCombineAlphaMode(int m)
    {
	textureAttributes.setCombineAlphaMode(m);	
    }


    /**
     * Set the Texture COMBINE Source for the Alpha Components to use on the Scene objects.
     * @param s the Texture COMBINE Source.
     */
    public void setObjectsTextureCombineAlphaSource(int []s)
    {
	textureAttributes.setCombineAlphaSource(0,s[0]);	
	textureAttributes.setCombineAlphaSource(1,s[1]);	
	textureAttributes.setCombineAlphaSource(2,s[2]);	
    }


    /**
     * Set the Texture COMBINE Function for the Alpha Components to use on the Scene objects.
     * @param f the Texture COMBINE Function.
     */
    public void setObjectsTextureCombineAlphaFunction(int []f)
    {
	textureAttributes.setCombineAlphaFunction(0,f[0]);	
	textureAttributes.setCombineAlphaFunction(1,f[1]);	
	textureAttributes.setCombineAlphaFunction(2,f[2]);	
    }


    /**
     * Set the Texture COMBINE Scale for the Alpha Components of the output color to use on the Scene objects.
     * @param m the Texture COMBINE Scale.
     */
    public void setObjectsTextureCombineAlphaScale(int m)
    {
	textureAttributes.setCombineAlphaScale(m);	
    }


    /**
     * Set the Texture Blend Color to use on the Scene objects.
     * @param r the R value of the Texture Blend Color.
     * @param g the G value of the Texture Blend Color.
     * @param b the B value of the Texture Blend Color.
     * @param a the Alpha value of the Texture Blend Color.
     */
    public void setObjectsTextureBlendColor(float r, float g, float b, float a)
    {	
	textureAttributes.setTextureBlendColor(r,g,b,a);
    }


    /**
     * Main method for a Test purpose.
     * @param args arguments used for the call.
     */
    public static void main(String[] args) {
	JFrame frame = new JFrame();
	RenderingViewPanel renderingViewPanel = 
	    new RenderingViewPanel((new Info()).getClass().getResource("resources/DefaultTexture.png"));
	frame.add(renderingViewPanel);	
    }
}


/**
 * Class used to Create a View in the 3D Scene 
 */
class SceneView {

    private static final long serialVersionUID = 1L;

    // PhysicalBody and PhysicalEnvironment are common to all instances    
    protected static final PhysicalBody physBody = new PhysicalBody();
    protected static final PhysicalEnvironment physEnv =
	new PhysicalEnvironment();

    protected BranchGroup parent = null;
    protected TransformGroup vpTG = null;
    protected ViewPlatform viewPlatform = null;
    protected View view = null;
    protected Canvas3D canvas = null;   

    /**
     * Default constructor to create a point of view in the 3D Scene.
     */
    public SceneView() {

	// Graphical configuration creation
	GraphicsConfigTemplate3D gconfigTemplate = new GraphicsConfigTemplate3D();
	GraphicsConfiguration gconfig =
	    GraphicsEnvironment.getLocalGraphicsEnvironment().
	    getDefaultScreenDevice().getBestConfiguration(gconfigTemplate);

	// Creation of the 3D canvas, the viewPlatform and the related view object
	canvas = new Canvas3D(gconfig);   

	viewPlatform = new ViewPlatform();	
	view = new View();

	// Initialisation of the view object
	view.addCanvas3D(canvas);
	view.attachViewPlatform(viewPlatform);
	view.setPhysicalBody(physBody);
	view.setPhysicalEnvironment(physEnv);


	// Creation of the transformation for the camera
	vpTG = new TransformGroup();
	vpTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	vpTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	vpTG.addChild(viewPlatform);


	// Creation of the parent object 
	parent = new BranchGroup();	
	parent.addChild(vpTG);
    }


    /**
     * Give a reference on the camera transformation group (viewPlatform).
     * @return reference on the camera transformation group (viewPlatform).
     */
    public TransformGroup getViewPlatformTransformGroup() {
	return vpTG;
    }

    /**
     * Give a reference on the parent object.
     * @return a reference on the parent object..
     */
    public BranchGroup getParent() {
	return parent;
    }

    /**
     * Give a reference on the 3D canvas.
     * @return a reference on the 3D canvas.
     */
    public Canvas3D getCanvas3D() {
	return canvas;
    }

    /**
     * Give a reference on the view.
     * @return reference on the view.
     */
    public View getView() {
	return view;
    }
}    
 


/**
 * Class representing an Ambient light in the 3D Scene.
 * Parameters for controlling this Ambient light are set in a separate control light Panel.
 */
class AmbientLightObject
{

    private static final long serialVersionUID = 1L;

    private JRadioButton ambientLightButton;
    private JSpinner ambientLightsColorRSpinner; 
    private JSpinner ambientLightsColorGSpinner; 
    private JSpinner ambientLightsColorBSpinner;
    private BranchGroup light;
    private TransformGroup sceneTransform;

    /**
     * Constructor for having an Ambient light in the 3D Scene.
     * @param ambientLightButton the JRadioButton to set or unset this light.
     * @param ambientLightsColorRSpinner a spinner to control the R Value of the light.
     * @param ambientLightsColorGSpinner a spinner to control the G Value of the light.
     * @param ambientLightsColorBSpinner a spinner to control the B Value of the light.
     * @param sceneTransform the transformation for the lights. 
     *
     */
    public AmbientLightObject(JRadioButton ambientLightButton, 
			      JSpinner ambientLightsColorRSpinner, 
			      JSpinner ambientLightsColorGSpinner, 
			      JSpinner ambientLightsColorBSpinner,
			      TransformGroup sceneTransform
			      )
    {
	this.ambientLightButton = ambientLightButton;
	this.ambientLightsColorRSpinner = ambientLightsColorRSpinner;
	this.ambientLightsColorGSpinner = ambientLightsColorGSpinner;
	this.ambientLightsColorBSpinner = ambientLightsColorBSpinner;
	this.sceneTransform = sceneTransform;

	// Create an Ambient light
	light = new BranchGroup();	
	light.setCapability(BranchGroup.ALLOW_DETACH);
	BoundingSphere infiniteBounds = new BoundingSphere(new Point3d(), Double.MAX_VALUE);
	AmbientLight ambientLight = new AmbientLight(new Color3f( (float)(Integer)ambientLightsColorRSpinner.getModel().getValue() /255, 
								  (float)(Integer)ambientLightsColorGSpinner.getModel().getValue() /255, 
								  (float)(Integer)ambientLightsColorBSpinner.getModel().getValue() /255 )
						     );
	ambientLight.setInfluencingBounds(infiniteBounds);
	ambientLight.setCapability(Light.ALLOW_COLOR_WRITE);
	light.addChild(ambientLight);		
    }    

    /**
     *
     * Get the JRadioButton to set or unset this light.
     *
     */
    public JRadioButton getButton()
    {
	return ambientLightButton;
    }

    /**
     *
     * Get the spinner to control the R Value of the light.
     *
     */
    public JSpinner getColorRSpinner()
    {
	return ambientLightsColorRSpinner;
    }

    /**
     *
     * Get the spinner to control the G Value of the light.
     *
     */
    public JSpinner getColorGSpinner()
    {
	return ambientLightsColorGSpinner;
    }

    /**
     *
     * Get the spinner to control the B Value of the light.
     *
     */
    public JSpinner getColorBSpinner()
    {
	return ambientLightsColorBSpinner;
    }

    /**
     *
     * Set or unset the light.
     *
     */
    public void set(boolean on)
    {
	if (on)
	    sceneTransform.addChild(light);
	else	  
	    light.detach();
    }


    /**
     *
     * Update the light color according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */
    public void updateColor()
    {
	((AmbientLight)light.getChild(0)).setColor(new Color3f( (float)(Integer)ambientLightsColorRSpinner.getModel().getValue() /255, 
								(float)(Integer)ambientLightsColorGSpinner.getModel().getValue() /255, 
								(float)(Integer)ambientLightsColorBSpinner.getModel().getValue() /255 ));
	
	/*
	  System.out.println("AMBIANT LIGHT - Update Color (" + 
	  (Integer)ambientLightsColorRSpinner.getModel().getValue() + ", " + 
	  (Integer)ambientLightsColorGSpinner.getModel().getValue() + ", " +
	  (Integer)ambientLightsColorBSpinner.getModel().getValue() + ")");
	*/

    }
}


/**
 * Class representing a Directional light in the 3D Scene.
 * Parameters for controlling this Directional light are set in a separate control light Panel.
 */
class DirectionalLightObject
{
    private static final long serialVersionUID = 1L;

    private JRadioButton directionalLightButton;
    private JSpinner directionalLightsColorRSpinner; 
    private JSpinner directionalLightsColorGSpinner; 
    private JSpinner directionalLightsColorBSpinner;
    private JSpinner directionalLightsDirectionXSpinner;
    private JSpinner directionalLightsDirectionYSpinner;
    private JSpinner directionalLightsDirectionZSpinner;  
    private BranchGroup light;
    private TransformGroup sceneTransform;


    /**
     * Constructor for having a Directional light in the 3D Scene.
     * @param directionalLightButton the JRadioButton to set or unset this light.
     * @param directionalLightsColorRSpinner a spinner to control the R Value of the light.
     * @param directionalLightsColorGSpinner a spinner to control the G Value of the light.
     * @param directionalLightsColorBSpinner a spinner to control the B Value of the light.
     * @param directionalLightsDirectionXSpinner a spinner to control the light direction according the X axis.
     * @param directionalLightsDirectionYSpinner a spinner to control the light direction according the Y axis.
     * @param directionalLightsDirectionZSpinner a spinner to control the light direction according the Z axis.
     * @param sceneTransform the transformation for the lights. 
     *
     */
    public DirectionalLightObject(JRadioButton directionalLightButton, 
				  JSpinner directionalLightsColorRSpinner, 
				  JSpinner directionalLightsColorGSpinner, 
				  JSpinner directionalLightsColorBSpinner,
				  JSpinner directionalLightsDirectionXSpinner,
				  JSpinner directionalLightsDirectionYSpinner,
				  JSpinner directionalLightsDirectionZSpinner,				  
				  TransformGroup sceneTransform
				  )
    {
	this.directionalLightButton = directionalLightButton;
	this.directionalLightsColorRSpinner = directionalLightsColorRSpinner;
	this.directionalLightsColorGSpinner = directionalLightsColorGSpinner;
	this.directionalLightsColorBSpinner = directionalLightsColorBSpinner;
	this.directionalLightsDirectionXSpinner = directionalLightsDirectionXSpinner;
	this.directionalLightsDirectionYSpinner = directionalLightsDirectionYSpinner;
	this.directionalLightsDirectionZSpinner = directionalLightsDirectionZSpinner;
	this.sceneTransform = sceneTransform;

	// Create a Directional light
	light = new BranchGroup();	
	light.setCapability(BranchGroup.ALLOW_DETACH);
	BoundingSphere infiniteBounds = new BoundingSphere(new Point3d(), Double.MAX_VALUE);
	DirectionalLight directionalLight = new DirectionalLight(new Color3f( (float)(Integer)directionalLightsColorRSpinner.getModel().getValue() /255, 
									      (float)(Integer)directionalLightsColorGSpinner.getModel().getValue() /255, 
									      (float)(Integer)directionalLightsColorBSpinner.getModel().getValue() /255), 
								 new Vector3f((Float)directionalLightsDirectionXSpinner.getModel().getValue(), 
									      (Float)directionalLightsDirectionYSpinner.getModel().getValue(), 
									      (Float)directionalLightsDirectionZSpinner.getModel().getValue())
								 );
	directionalLight.setInfluencingBounds(infiniteBounds);
	directionalLight.setCapability(Light.ALLOW_COLOR_WRITE);
	directionalLight.setCapability(DirectionalLight.ALLOW_DIRECTION_WRITE);
	light.addChild(directionalLight);	
    }    

    /**
     *
     * Get the JRadioButton to set or unset this light.
     *
     */
    public JRadioButton getButton()
    {
	return directionalLightButton;
    }

    /**
     *
     * Get the spinner to control the R Value of the light.
     *
     */
    public JSpinner getColorRSpinner()
    {
	return directionalLightsColorRSpinner;
    }

    /**
     *
     * Get the spinner to control the G Value of the light.
     *
     */
    public JSpinner getColorGSpinner()
    {
	return directionalLightsColorGSpinner;
    }

    /**
     *
     * Get the spinner to control the B Value of the light.
     *
     */
    public JSpinner getColorBSpinner()
    {
	return directionalLightsColorBSpinner;
    }

    /**
     *
     * Get the spinner to control the X Direction of the light.
     *
     */
    public JSpinner getDirectionXSpinner()
    {
	return directionalLightsDirectionXSpinner;
    }
	
    /**
     *
     * Get the spinner to control the Y Direction of the light.
     *
     */
    public JSpinner getDirectionYSpinner()
    {
	return directionalLightsDirectionYSpinner;
    }

    /**
     *
     * Get the spinner to control the Z Direction of the light.
     *
     */
    public JSpinner getDirectionZSpinner()
    {
	return directionalLightsDirectionZSpinner;
    }
	
    /**
     *
     * Set or unset the light.
     *
     */
    public void set(boolean on)
    {
	if (on)
	    sceneTransform.addChild(light);
	else
	    light.detach();
    }


    /**
     *
     * Update the light color according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */
    public void updateColor()
    {
	((DirectionalLight)light.getChild(0)).setColor(new Color3f( 
								   (float)(Integer)directionalLightsColorRSpinner.getModel().getValue() /255, 
								   (float)(Integer)directionalLightsColorGSpinner.getModel().getValue() /255, 
								   (float)(Integer)directionalLightsColorBSpinner.getModel().getValue() /255 ));

	/*
	  System.out.println("DIRECTIONAL LIGHT - Update Color (" + 
	  (Integer)directionalLightsColorRSpinner.getModel().getValue() + ", " + 
	  (Integer)directionalLightsColorGSpinner.getModel().getValue() + ", " +
	  (Integer)directionalLightsColorBSpinner.getModel().getValue() + ")");
	*/
    }


    /**
     *
     * Update the light direction according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */   
    public void updateDirection()
    {
	((DirectionalLight)light.getChild(0)).setDirection(new Vector3f(
									(Float)directionalLightsDirectionXSpinner.getModel().getValue(), 
									(Float)directionalLightsDirectionYSpinner.getModel().getValue(), 
									(Float)directionalLightsDirectionZSpinner.getModel().getValue()));

	/*
	  System.out.println("DIRECTIONAL LIGHT - Update Direction (" + 
	  (Float)directionalLightsDirectionXSpinner.getModel().getValue() + ", " + 
	  (Float)directionalLightsDirectionYSpinner.getModel().getValue() + ", " +
	  (Float)directionalLightsDirectionZSpinner.getModel().getValue() + ")");
	*/

    }
}



/**
 * Class representing a Point light in the 3D Scene.
 * Parameters for controlling this Point light are set in a separate control light Panel.
 */
class PointLightObject
{
    private static final long serialVersionUID = 1L;

    private JRadioButton pointLightButton;
    private JSpinner pointLightsColorRSpinner; 
    private JSpinner pointLightsColorGSpinner; 
    private JSpinner pointLightsColorBSpinner;
    private JSpinner pointLightsPositionXSpinner;
    private JSpinner pointLightsPositionYSpinner;
    private JSpinner pointLightsPositionZSpinner;
    private JSpinner pointLightsAttenuationXSpinner;
    private JSpinner pointLightsAttenuationYSpinner;
    private JSpinner pointLightsAttenuationZSpinner;
    private BranchGroup light;
    private TransformGroup sceneTransform;
	

    /**
     * Constructor for having a Point light in the 3D Scene.
     * @param pointLightButton the JRadioButton to set or unset this light.
     * @param pointLightsColorRSpinner a spinner to control the R Value of the light.
     * @param pointLightsColorGSpinner a spinner to control the G Value of the light.
     * @param pointLightsColorBSpinner a spinner to control the B Value of the light.
     * @param pointLightsPositionXSpinner a spinner to control the light position according the X axis.
     * @param pointLightsPositionYSpinner a spinner to control the light position according the Y axis.
     * @param pointLightsPositionZSpinner a spinner to control the light position according the Z axis.
     * @param pointLightsAttenuationXSpinner a spinner to control the light attenuation according the X axis.
     * @param pointLightsAttenuationYSpinner a spinner to control the light attenuation according the Y axis.
     * @param pointLightsAttenuationZSpinner a spinner to control the light attenuation according the Z axis.
     * @param sceneTransform the transformation for the lights. 
     *
     */
    public PointLightObject(JRadioButton pointLightButton, 
			    JSpinner pointLightsColorRSpinner, 
			    JSpinner pointLightsColorGSpinner, 
			    JSpinner pointLightsColorBSpinner,
			    JSpinner pointLightsPositionXSpinner,
			    JSpinner pointLightsPositionYSpinner,
			    JSpinner pointLightsPositionZSpinner,
			    JSpinner pointLightsAttenuationXSpinner,
			    JSpinner pointLightsAttenuationYSpinner,
			    JSpinner pointLightsAttenuationZSpinner,
			    TransformGroup sceneTransform
			    )
    {
	this.pointLightButton = pointLightButton;
	this.pointLightsColorRSpinner = pointLightsColorRSpinner;
	this.pointLightsColorGSpinner = pointLightsColorGSpinner;
	this.pointLightsColorBSpinner = pointLightsColorBSpinner;
	this.pointLightsPositionXSpinner = pointLightsPositionXSpinner;
	this.pointLightsPositionYSpinner = pointLightsPositionYSpinner;
	this.pointLightsPositionZSpinner = pointLightsPositionZSpinner;
	this.pointLightsAttenuationXSpinner = pointLightsAttenuationXSpinner;
	this.pointLightsAttenuationYSpinner = pointLightsAttenuationYSpinner;
	this.pointLightsAttenuationZSpinner = pointLightsAttenuationZSpinner;
	this.sceneTransform = sceneTransform;

	// Create a Point light
	light = new BranchGroup();	
	light.setCapability(BranchGroup.ALLOW_DETACH);
	BoundingSphere infiniteBounds = new BoundingSphere(new Point3d(), Double.MAX_VALUE);
	PointLight pointLight = new PointLight(new Color3f( (float)(Integer)pointLightsColorRSpinner.getModel().getValue() /255, 
							    (float)(Integer)pointLightsColorGSpinner.getModel().getValue() /255, 
							    (float)(Integer)pointLightsColorBSpinner.getModel().getValue() /255), 
					       new Point3f((Float)pointLightsPositionXSpinner.getModel().getValue(), 
							   (Float)pointLightsPositionYSpinner.getModel().getValue(), 
							   (Float)pointLightsPositionZSpinner.getModel().getValue()),
					       new Point3f((Float)pointLightsAttenuationXSpinner.getModel().getValue(), 
							   (Float)pointLightsAttenuationYSpinner.getModel().getValue(), 
							   (Float)pointLightsAttenuationZSpinner.getModel().getValue())
					       );
	pointLight.setInfluencingBounds(infiniteBounds);
	pointLight.setCapability(Light.ALLOW_COLOR_WRITE);
	pointLight.setCapability(PointLight.ALLOW_POSITION_WRITE);
	pointLight.setCapability(PointLight.ALLOW_ATTENUATION_WRITE);
	light.addChild(pointLight);	
    }    

    /**
     *
     * Get the JRadioButton to set or unset this light.
     *
     */
    public JRadioButton getButton()
    {
	return pointLightButton;
    }

    /**
     *
     * Get the spinner to control the R Value of the light.
     *
     */
    public JSpinner getColorRSpinner()
    {
	return pointLightsColorRSpinner;
    }

    /**
     *
     * Get the spinner to control the G Value of the light.
     *
     */
    public JSpinner getColorGSpinner()
    {
	return pointLightsColorGSpinner;
    }

    /**
     *
     * Get the spinner to control the B Value of the light.
     *
     */
    public JSpinner getColorBSpinner()
    {
	return pointLightsColorBSpinner;
    }

    /**
     *
     * Get the spinner to control the X Position of the light.
     *
     */
    public JSpinner getPositionXSpinner()
    {
	return pointLightsPositionXSpinner;
    }
	
    /**
     *
     * Get the spinner to control the Y Position of the light.
     *
     */
    public JSpinner getPositionYSpinner()
    {
	return pointLightsPositionYSpinner;
    }

    /**
     *
     * Get the spinner to control the Z Position of the light.
     *
     */
    public JSpinner getPositionZSpinner()
    {
	return pointLightsPositionZSpinner;
    }

    /**
     *
     * Get the spinner to control the X Attenuation of the light.
     *
     */
    public JSpinner getAttenuationXSpinner()
    {
	return pointLightsAttenuationXSpinner;
    }
	
    /**
     *
     * Get the spinner to control the Y Attenuation of the light.
     *
     */
    public JSpinner getAttenuationYSpinner()
    {
	return pointLightsAttenuationYSpinner;
    }

    /**
     *
     * Get the spinner to control the Z Attenuation of the light.
     *
     */
    public JSpinner getAttenuationZSpinner()
    {
	return pointLightsAttenuationZSpinner;
    }
	
    /**
     *
     * Set or unset the light.
     *
     */
    public void set(boolean on)
    {
	if (on)
	    sceneTransform.addChild(light);
	else
	    light.detach();
    }

    /**
     *
     * Update the light color according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */
    public void updateColor()
    {
	((PointLight)light.getChild(0)).setColor(new Color3f( 
							     (float)(Integer)pointLightsColorRSpinner.getModel().getValue() /255, 
							     (float)(Integer)pointLightsColorGSpinner.getModel().getValue() /255, 
							     (float)(Integer)pointLightsColorBSpinner.getModel().getValue() /255 ));

	/*
	  System.out.println("POINT LIGHT - Update Color (" + 
	  (Integer)pointLightsColorRSpinner.getModel().getValue() + ", " + 
	  (Integer)pointLightsColorGSpinner.getModel().getValue() + ", " +
	  (Integer)pointLightsColorBSpinner.getModel().getValue() + ")");
	*/
    }
    
    /**
     *
     * Update the light position according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */   
    public void updatePosition()
    {
	((PointLight)light.getChild(0)).setPosition(new Point3f(
								(Float)pointLightsPositionXSpinner.getModel().getValue(), 
								(Float)pointLightsPositionYSpinner.getModel().getValue(), 
								(Float)pointLightsPositionZSpinner.getModel().getValue()));

	/*
	  System.out.println("POINT LIGHT - Update Position (" + 
	  (Float)pointLightsPositionXSpinner.getModel().getValue() + ", " + 
	  (Float)pointLightsPositionYSpinner.getModel().getValue() + ", " +
	  (Float)pointLightsPositionZSpinner.getModel().getValue() + ")");
	*/
    }

    /**
     *
     * Update the light attenuation according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */   
    public void updateAttenuation()
    {
	((PointLight)light.getChild(0)).setAttenuation(new Point3f(
								   (Float)pointLightsAttenuationXSpinner.getModel().getValue(), 
								   (Float)pointLightsAttenuationYSpinner.getModel().getValue(), 
								   (Float)pointLightsAttenuationZSpinner.getModel().getValue()));
	
	/*
	  System.out.println("POINT LIGHT - Update Attenuation (" + 
	  (Float)pointLightsAttenuationXSpinner.getModel().getValue() + ", " + 
	  (Float)pointLightsAttenuationYSpinner.getModel().getValue() + ", " +
	  (Float)pointLightsAttenuationZSpinner.getModel().getValue() + ")");
	*/
    }
}


/**
 * Class representing a Spot light in the 3D Scene.
 * Parameters for controlling this Spot light are set in a separate control light Panel.
 */
class SpotLightObject
{
    private static final long serialVersionUID = 1L;

    private JRadioButton spotLightButton;
    private JSpinner spotLightsColorRSpinner; 
    private JSpinner spotLightsColorGSpinner; 
    private JSpinner spotLightsColorBSpinner;
    private JSpinner spotLightsPositionXSpinner;
    private JSpinner spotLightsPositionYSpinner;
    private JSpinner spotLightsPositionZSpinner;
    private JSpinner spotLightsAttenuationXSpinner;
    private JSpinner spotLightsAttenuationYSpinner;
    private JSpinner spotLightsAttenuationZSpinner;
    private JSpinner spotLightsDirectionXSpinner;
    private JSpinner spotLightsDirectionYSpinner;
    private JSpinner spotLightsDirectionZSpinner;
    private JSpinner spotLightsSpreadAngleSpinner;
    private JSpinner spotLightsConcentrationSpinner;
 
    private BranchGroup light;
    private TransformGroup sceneTransform;

    /**
     * Constructor for having a Spot light in the 3D Scene.
     * @param spotLightButton the JRadioButton to set or unset this light.
     * @param spotLightsColorRSpinner a spinner to control the R Value of the light.
     * @param spotLightsColorGSpinner a spinner to control the G Value of the light.
     * @param spotLightsColorBSpinner a spinner to control the B Value of the light.
     * @param spotLightsPositionXSpinner a spinner to control the light position according the X axis.
     * @param spotLightsPositionYSpinner a spinner to control the light position according the Y axis.
     * @param spotLightsPositionZSpinner a spinner to control the light position according the Z axis.
     * @param spotLightsAttenuationXSpinner a spinner to control the light attenuation according the X axis.
     * @param spotLightsAttenuationYSpinner a spinner to control the light attenuation according the Y axis.
     * @param spotLightsAttenuationZSpinner a spinner to control the light attenuation according the Z axis.
     * @param spotLightsDirectionXSpinner a spinner to control the light direction according the X axis.
     * @param spotLightsDirectionYSpinner a spinner to control the light direction according the Y axis.
     * @param spotLightsDirectionZSpinner a spinner to control the light direction according the Z axis.
     * @param spotLightsSpreadAngleSpinner a spinner to control the Spread Angle of the light.
     * @param spotLightsConcentrationSpinner a spinner to control the Concentration of the light.
     * @param sceneTransform the transformation for the lights. 
     *
     */	
    public SpotLightObject(JRadioButton spotLightButton, 
			   JSpinner spotLightsColorRSpinner, 
			   JSpinner spotLightsColorGSpinner, 
			   JSpinner spotLightsColorBSpinner,
			   JSpinner spotLightsPositionXSpinner,
			   JSpinner spotLightsPositionYSpinner,
			   JSpinner spotLightsPositionZSpinner,
			   JSpinner spotLightsAttenuationXSpinner,
			   JSpinner spotLightsAttenuationYSpinner,
			   JSpinner spotLightsAttenuationZSpinner,
			   JSpinner spotLightsDirectionXSpinner,
			   JSpinner spotLightsDirectionYSpinner,
			   JSpinner spotLightsDirectionZSpinner,
			   JSpinner spotLightsSpreadAngleSpinner,
			   JSpinner spotLightsConcentrationSpinner,
			   TransformGroup sceneTransform
			   )
    {
	this.spotLightButton = spotLightButton;
	this.spotLightsColorRSpinner = spotLightsColorRSpinner;
	this.spotLightsColorGSpinner = spotLightsColorGSpinner;
	this.spotLightsColorBSpinner = spotLightsColorBSpinner;
	this.spotLightsPositionXSpinner = spotLightsPositionXSpinner;
	this.spotLightsPositionYSpinner = spotLightsPositionYSpinner;
	this.spotLightsPositionZSpinner = spotLightsPositionZSpinner;
	this.spotLightsAttenuationXSpinner = spotLightsAttenuationXSpinner;
	this.spotLightsAttenuationYSpinner = spotLightsAttenuationYSpinner;
	this.spotLightsAttenuationZSpinner = spotLightsAttenuationZSpinner;
	this.spotLightsDirectionXSpinner = spotLightsDirectionXSpinner;
	this.spotLightsDirectionYSpinner = spotLightsDirectionYSpinner;
	this.spotLightsDirectionZSpinner = spotLightsDirectionZSpinner;
	this.spotLightsSpreadAngleSpinner = spotLightsSpreadAngleSpinner;
	this.spotLightsConcentrationSpinner = spotLightsConcentrationSpinner;
	this.sceneTransform = sceneTransform;
	
	light = new BranchGroup();	
	light.setCapability(BranchGroup.ALLOW_DETACH);
	BoundingSphere infiniteBounds = new BoundingSphere(new Point3d(), Double.MAX_VALUE);
	SpotLight spotLight = new SpotLight(new Color3f( (float)(Integer)spotLightsColorRSpinner.getModel().getValue() /255, 
							 (float)(Integer)spotLightsColorGSpinner.getModel().getValue() /255, 
							 (float)(Integer)spotLightsColorBSpinner.getModel().getValue() /255), 
					    new Point3f((Float)spotLightsPositionXSpinner.getModel().getValue(), 
							(Float)spotLightsPositionYSpinner.getModel().getValue(), 
							(Float)spotLightsPositionZSpinner.getModel().getValue()),
					    new Point3f((Float)spotLightsAttenuationXSpinner.getModel().getValue(), 
							(Float)spotLightsAttenuationYSpinner.getModel().getValue(), 
							(Float)spotLightsAttenuationZSpinner.getModel().getValue()),
					    new Vector3f((Float)spotLightsDirectionXSpinner.getModel().getValue(), 
							 (Float)spotLightsDirectionYSpinner.getModel().getValue(), 
							 (Float)spotLightsDirectionZSpinner.getModel().getValue()),
					    new Double(Math.toRadians((Float)spotLightsSpreadAngleSpinner.getModel().getValue())).floatValue(),
					    (Float)spotLightsConcentrationSpinner.getModel().getValue()
					    );
	spotLight.setInfluencingBounds(infiniteBounds);
	spotLight.setCapability(Light.ALLOW_COLOR_WRITE);
	spotLight.setCapability(SpotLight.ALLOW_POSITION_WRITE);
	spotLight.setCapability(SpotLight.ALLOW_ATTENUATION_WRITE);
    spotLight.setCapability(SpotLight.ALLOW_DIRECTION_WRITE);
    spotLight.setCapability(SpotLight.ALLOW_SPREAD_ANGLE_WRITE);
    spotLight.setCapability(SpotLight.ALLOW_CONCENTRATION_WRITE);
	light.addChild(spotLight);	
    }    

    /**
     *
     * Get the JRadioButton to set or unset this light.
     *
     */
    public JRadioButton getButton()
    {
	return spotLightButton;
    }

    /**
     *
     * Get the spinner to control the R Value of the light.
     *
     */
    public JSpinner getColorRSpinner()
    {
	return spotLightsColorRSpinner;
    }

    /**
     *
     * Get the spinner to control the G Value of the light.
     *
     */
    public JSpinner getColorGSpinner()
    {
	return spotLightsColorGSpinner;
    }

    /**
     *
     * Get the spinner to control the B Value of the light.
     *
     */
    public JSpinner getColorBSpinner()
    {
	return spotLightsColorBSpinner;
    }

    /**
     *
     * Get the spinner to control the X Position of the light.
     *
     */
    public JSpinner getPositionXSpinner()
    {
	return spotLightsPositionXSpinner;
    }
	
    /**
     *
     * Get the spinner to control the Y Position of the light.
     *
     */
    public JSpinner getPositionYSpinner()
    {
	return spotLightsPositionYSpinner;
    }

    /**
     *
     * Get the spinner to control the Z Position of the light.
     *
     */
    public JSpinner getPositionZSpinner()
    {
	return spotLightsPositionZSpinner;
    }
    
    /**
     *
     * Get the spinner to control the X Direction of the light.
     *
     */
    public JSpinner getDirectionXSpinner()
    {
	return spotLightsDirectionXSpinner;
    }
	
    /**
     *
     * Get the spinner to control the Y Direction of the light.
     *
     */
    public JSpinner getDirectionYSpinner()
    {
	return spotLightsDirectionYSpinner;
    }

    /**
     *
     * Get the spinner to control the Z Direction of the light.
     *
     */
    public JSpinner getDirectionZSpinner()
    {
	return spotLightsDirectionZSpinner;
    }

    /**
     *
     * Get the spinner to control the X Attenuation of the light.
     *
     */
    public JSpinner getAttenuationXSpinner()
    {
	return spotLightsAttenuationXSpinner;
    }
	
    /**
     *
     * Get the spinner to control the Y Attenuation of the light.
     *
     */
    public JSpinner getAttenuationYSpinner()
    {
	return spotLightsAttenuationYSpinner;
    }

    /**
     *
     * Get the spinner to control the Z Attenuation of the light.
     *
     */
    public JSpinner getAttenuationZSpinner()
    {
	return spotLightsAttenuationZSpinner;
    }

    /**
     *
     * Get the spinner to control the Spread Angle of the light.
     *
     */
    public JSpinner getSpreadAngleSpinner()
    {
	return spotLightsSpreadAngleSpinner;
    }
    
    /**
     *
     * Get the spinner to control the Concentration of the light.
     *
     */
    public JSpinner getConcentrationSpinner()
    {
	return spotLightsConcentrationSpinner;
    }
	
    /**
     *
     * Set or unset the light.
     *
     */
    public void set(boolean on)
    {
	if (on)
	    sceneTransform.addChild(light);
	else
	    light.detach();
    }

    /**
     *
     * Update the light color according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */
    public void updateColor()
    {
	((SpotLight)light.getChild(0)).setColor(new Color3f( 
							    (float)(Integer)spotLightsColorRSpinner.getModel().getValue() /255, 
							    (float)(Integer)spotLightsColorGSpinner.getModel().getValue() /255, 
							    (float)(Integer)spotLightsColorBSpinner.getModel().getValue() /255 ));

	/*
	  System.out.println("SPOT LIGHT - Update Color (" + 
	  (Integer)spotLightsColorRSpinner.getModel().getValue() + ", " + 
	  (Integer)spotLightsColorGSpinner.getModel().getValue() + ", " +
	  (Integer)spotLightsColorBSpinner.getModel().getValue() + ")");
	*/
    }
    
    /**
     *
     * Update the light position according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */   
    public void updatePosition()
    {
	((SpotLight)light.getChild(0)).setPosition(new Point3f(
							       (Float)spotLightsPositionXSpinner.getModel().getValue(), 
							       (Float)spotLightsPositionYSpinner.getModel().getValue(), 
							       (Float)spotLightsPositionZSpinner.getModel().getValue()));
	
	/*
	  System.out.println("SPOT LIGHT - Update Position (" + 
	  (Float)spotLightsPositionXSpinner.getModel().getValue() + ", " + 
	  (Float)spotLightsPositionYSpinner.getModel().getValue() + ", " +
	  (Float)spotLightsPositionZSpinner.getModel().getValue() + ")");
	*/
    }

    /**
     *
     * Update the light attenuation according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */   
    public void updateAttenuation()
    {
	((SpotLight)light.getChild(0)).setAttenuation(new Point3f(
								  (Float)spotLightsAttenuationXSpinner.getModel().getValue(), 
								  (Float)spotLightsAttenuationYSpinner.getModel().getValue(), 
								  (Float)spotLightsAttenuationZSpinner.getModel().getValue()));

	/*
	  System.out.println("SPOT LIGHT - Update Attenuation (" + 
	  (Float)spotLightsAttenuationXSpinner.getModel().getValue() + ", " + 
	  (Float)spotLightsAttenuationYSpinner.getModel().getValue() + ", " +
	  (Float)spotLightsAttenuationZSpinner.getModel().getValue() + ")");
	*/
    }

    /**
     *
     * Update the light direction according to the associated Spinners values.
     * Called when an event is detected on the corresponding Spinner.
     *
     */   
    public void updateDirection()
    {
	((SpotLight)light.getChild(0)).setDirection(new Vector3f(
								 (Float)spotLightsDirectionXSpinner.getModel().getValue() , 
								 (Float)spotLightsDirectionYSpinner.getModel().getValue() , 
								 (Float)spotLightsDirectionZSpinner.getModel().getValue() ));

	/*
	  System.out.println("SPOT LIGHT - Update Direction (" + 
	  (Float)spotLightsDirectionXSpinner.getModel().getValue() + ", " + 
	  (Float)spotLightsDirectionYSpinner.getModel().getValue() + ", " +
	  (Float)spotLightsDirectionZSpinner.getModel().getValue() + ")");
	*/	
    }

    /**
     *
     * Update the light spread angle according to the associated Spinner value.
     * Called when an event is detected on the corresponding Spinner.
     *
     */   
    public void updateSpreadAngle()
    {
	((SpotLight)light.getChild(0)).setSpreadAngle(new Double(Math.toRadians((Float)spotLightsSpreadAngleSpinner.getModel().getValue())).floatValue());

	/*
	  System.out.println("SPOT LIGHT - Update Spread Angle (" 
	  + new Double(Math.toRadians((Float)spotLightsSpreadAngleSpinner.getModel().getValue())).floatValue() 
	  + ")");
	*/					      
    }

    /**
     *
     * Update the light concentration according to the associated Spinner value.
     * Called when an event is detected on the corresponding Spinner.
     *
     */   
    public void updateConcentration()
    {
	((SpotLight)light.getChild(0)).setConcentration((Float)spotLightsConcentrationSpinner.getModel().getValue() );
	/*
	  System.out.println("SPOT LIGHT - Update Concentration (" + (Float)spotLightsConcentrationSpinner.getModel().getValue() + ")");
	*/
    }
}






