/**
   ##############################################################################
   ##                                                                          ##
   ## Appearance3DChooser                                                      ##
   ##                                                                          ##
   ## Copyright (C) 2009  ederic Roudaut  <ederic.roudaut@ee.>         ##
   ##                                                                          ##
   ##                                                                          ##
   ## This program is ee software: you can redistribute it and/or modify     ##
   ## it under the terms of the GNU General Public License as published by     ##
   ## the ee Software Foundation, either version 3 of the License, or        ##
   ## (at your option) any later version.                                      ##
   ##                                                                          ##
   ## This program is distributed in the hope that it will be useful,          ##
   ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
   ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
   ## GNU General Public License for more details.                             ##
   ##                                                                          ##
   ## You should have received a copy of the GNU General Public License        ##
   ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
   ##                                                                          ##
   ##############################################################################
**/


package com.appearance3Dchooser;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.media.j3d.Transform3D;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;
import javax.vecmath.AxisAngle4d;
import javax.vecmath.Matrix3f;
import java.net.URL;


import javax.media.j3d.TextureAttributes;

import com.colorpicker.swing.*;

/**
 * Panel creation for the Textures.
 * 
 */
public class TexturePane extends JTabbedPane implements ActionListener, ChangeListener
{
    private static final long serialVersionUID = 1L;

    public static final String TEXTURE_FILE = "texture_file";
    public static final String TEXTURE_USE = "texture_use";
    public static final String TEXTURE_TRANSFORM = "texture_transform";
    public static final String TEXTURE_PERPSPECTIVE_CORRECTION_MODE = "texture_perspective_correction_mode";
    public static final String TEXTURE_MODE = "texture_mode";
    public static final String TEXTURE_BLEND = "texture_blend";
    public static final String TEXTURE_COMBINE_RGB_MODE = "texture_combine_rgb_mode";
    public static final String TEXTURE_COMBINE_RGB_SOURCE = "texture_combine_rgb_source";
    public static final String TEXTURE_COMBINE_RGB_FUNCTION = "texture_combine_rgb_function";
    public static final String TEXTURE_COMBINE_RGB_SCALE = "texture_combine_rgb_scale";    
    public static final String TEXTURE_COMBINE_ALPHA_MODE = "texture_combine_alpha_mode";
    public static final String TEXTURE_COMBINE_ALPHA_SOURCE = "texture_combine_alpha_source";    
    public static final String TEXTURE_COMBINE_ALPHA_FUNCTION = "texture_combine_alpha_function";    
    public static final String TEXTURE_COMBINE_ALPHA_SCALE = "texture_combine_alpha_scale";    
    
    public static final int TEXTURE_MODE_MODULATE = 0;
    public static final int TEXTURE_MODE_DECAL = 1;
    public static final int TEXTURE_MODE_BLEND = 2;
    public static final int TEXTURE_MODE_REPLACE = 3;
    public static final int TEXTURE_MODE_COMBINE = 4;

    public static final int COMBINE_MODE_REPLACE = 0;
    public static final int COMBINE_MODE_MODULATE = 1;
    public static final int COMBINE_MODE_ADD = 2;
    public static final int COMBINE_MODE_ADD_SIGNED = 3;
    public static final int COMBINE_MODE_SUBTRACT = 4;
    public static final int COMBINE_MODE_INTERPOLATE = 5;
    public static final int COMBINE_MODE_DOT3 = 6;

    public static final int COMBINE_FUNCTION_SRC_ALPHA = 0;
    public static final int COMBINE_FUNCTION_ONE_MINUS_SRC_ALPHA = 1;
    public static final int COMBINE_FUNCTION_SRC_COLOR = 2; // not available for alpha
    public static final int COMBINE_FUNCTION_ONE_MINUS_SRC_COLOR = 3; // not available for alpha

    
    public static final int COMBINE_SOURCE_OBJECT_COLOR = 0;
    public static final int COMBINE_SOURCE_TEXTURE_COLOR = 1;
    public static final int COMBINE_SOURCE_CONSTANT_COLOR = 2;
    public static final int COMBINE_SOURCE_PREVIOUS_TEXTURE_UNIT_STATE = 3;

    public static final int PERSPECTIVE_CORRECTION_MODE_FASTEST = 0;
    public static final int PERSPECTIVE_CORRECTION_MODE_NICEST = 1;

    protected JTextField textureNameJText;
    protected JButton fileChooserButton;
    protected JButton OKButton;
    protected JFileChooser fileChooser;
    protected JRadioButton setTextureButton;
    protected ImageIcon imageIcon;
    protected URL textureName;	
    protected URL DefaultTexture;
    protected boolean useTexture=false;
    protected Image image;
    protected int textureMode = TEXTURE_MODE_REPLACE;
    protected int perspectiveCorrectionMode = PERSPECTIVE_CORRECTION_MODE_NICEST;
    protected int combineRGBMode = COMBINE_MODE_MODULATE;
    protected int combineAlphaMode = COMBINE_MODE_MODULATE;
    protected int combineRGBFunctionC0 = COMBINE_FUNCTION_SRC_COLOR;
    protected int combineRGBFunctionC1 = COMBINE_FUNCTION_SRC_COLOR;
    protected int combineRGBFunctionC2 = COMBINE_FUNCTION_SRC_COLOR;
    protected int combineAlphaFunctionC0 = COMBINE_FUNCTION_SRC_ALPHA;
    protected int combineAlphaFunctionC1 = COMBINE_FUNCTION_SRC_ALPHA;
    protected int combineAlphaFunctionC2 = COMBINE_FUNCTION_SRC_ALPHA;
    protected int combineRGBSourceC0 = COMBINE_SOURCE_TEXTURE_COLOR;
    protected int combineRGBSourceC1 = COMBINE_SOURCE_PREVIOUS_TEXTURE_UNIT_STATE;
    protected int combineRGBSourceC2 = COMBINE_SOURCE_CONSTANT_COLOR;
    protected int combineAlphaSourceC0 = COMBINE_SOURCE_TEXTURE_COLOR;
    protected int combineAlphaSourceC1 = COMBINE_SOURCE_PREVIOUS_TEXTURE_UNIT_STATE;
    protected int combineAlphaSourceC2 = COMBINE_SOURCE_CONSTANT_COLOR;
    protected int combineRGBScale = 0;
    protected int combineAlphaScale = 0;    
    protected Transform3D textureTransform;
    protected boolean textureTransformUniformScale=false;    
    protected float[] blendColor = {0.5f,0.5f,0.5f,0.5f};

    protected JComboBox listTextureMode;
    protected JComboBox listCombineRGBMode;
    protected JComboBox listCombineAlphaMode;
    protected JComboBox listCombineRGBFunctionC0;
    protected JComboBox listCombineRGBFunctionC1;
    protected JComboBox listCombineRGBFunctionC2;
    protected JComboBox listCombineAlphaFunctionC0;
    protected JComboBox listCombineAlphaFunctionC1;
    protected JComboBox listCombineAlphaFunctionC2;
    protected JComboBox listCombineRGBSourceC0;
    protected JComboBox listCombineRGBSourceC1;
    protected JComboBox listCombineRGBSourceC2;
    protected JComboBox listCombineAlphaSourceC0;
    protected JComboBox listCombineAlphaSourceC1;
    protected JComboBox listCombineAlphaSourceC2;
    protected JComboBox listPerspectiveCorrectionMode;
    protected JComboBox listCombineRGBScale;
    protected JComboBox listCombineAlphaScale;
    protected JSpinner textureSpinnerTransformRotZ;
    protected JSpinner textureSpinnerTransformScaleX;
    protected JSpinner textureSpinnerTransformScaleY;
    protected JSpinner textureSpinnerTransformTranslatX;
    protected JSpinner textureSpinnerTransformTranslatY;
    protected JRadioButton textureButtonTransformReset;
    protected JRadioButton textureButtonTransformUniformScale;

    protected JTabbedPane textureTabbedPane;
    protected ColorPicker blendColorPicker;


    /**
     * Constructor for creating a Texture panel
     * @param DefaultTextureName Default Texture URL.
     */    
    public TexturePane(URL DefaultTextureName)
    {	 		
	setTabPlacement(JTabbedPane.LEFT);
	DefaultTexture = DefaultTextureName;	
	makeBasicTexturePane();
	makeBlendColorPane();	
	makeCombineModePane();
    }


    /**
     * Basic parameters Panel for the Texture : use, don't use, Texture tranformation, Texture Mode, 
     * Texture Perspective Correction Mode.
     */        
    private void makeBasicTexturePane ()
    {       	
	// Panel For texture color
	JPanel basicTexturePanel = new JPanel();    
		
	GridBagConstraints c = new GridBagConstraints();

	basicTexturePanel.setLayout(new GridBagLayout());
	
	c.gridx = 0; c.gridy = 0;
	c.weightx = 0; c.weighty = 0; 
	c.insets = new Insets(0,0,0,0);
	c.fill = GridBagConstraints.WEST;
	c.anchor = GridBagConstraints.WEST;
		
	setTextureButton = new JRadioButton(Info.ressources.getObject("Use_Texture").toString());

	setTextureButton.addActionListener(this);
	basicTexturePanel.add(setTextureButton,c);

	c.gridx = 0; c.gridy ++;
	JPanel fileChooserPanel = new JPanel();	       
	textureNameJText = new JTextField();
	textureNameJText.addActionListener(this);
	textureNameJText.setColumns(20);
	fileChooserPanel.add(textureNameJText);	       
	fileChooserButton = new JButton(Info.ressources.getObject("Search").toString());
	fileChooser = new JFileChooser();	
	fileChooserButton.addActionListener(this);
	fileChooserPanel.add(fileChooserButton);
	OKButton = new JButton(Info.ressources.getObject("OK").toString());
	OKButton.addActionListener(this);
	fileChooserPanel.add(OKButton);
	
	basicTexturePanel.add(fileChooserPanel,c);	
	
	//Create the texture Transform
	textureTransform = new Transform3D();
	
	// Create The Transform Panel
	JPanel textureTransformPanel = new JPanel();    		
	GridBagConstraints ct = new GridBagConstraints();
	textureTransformPanel.setLayout(new GridBagLayout());
	
	ct.gridy = 0;
	ct.weightx = 0; ct.weighty = 0; 
	ct.insets = new Insets(0,0,0,0);
	ct.fill = GridBagConstraints.EAST;
	ct.anchor = GridBagConstraints.EAST;

	textureButtonTransformReset = new JRadioButton(Info.ressources.getObject("Reset").toString());
	ct.gridx = 4;
	textureTransformPanel.add(textureButtonTransformReset, ct);
	textureButtonTransformReset.addActionListener(this);

	JLabel textureSpinnerTransformRotZLabel = new JLabel(Info.ressources.getObject("Rotation").toString());       
	ct.gridy ++; ct.gridx = 0;
	textureTransformPanel.add(textureSpinnerTransformRotZLabel, ct);

	SpinnerModel textureSpinnerModelTransformRotZ = new SpinnerNumberModel(Float.valueOf(0f), //initial value
									       Float.valueOf(-360f), //min
									       Float.valueOf(360f), //max
									       Float.valueOf(0.1f)); //step
	textureSpinnerTransformRotZ = new JSpinner(textureSpinnerModelTransformRotZ);
	((JSpinner.DefaultEditor)textureSpinnerTransformRotZ.getEditor()).getTextField().setColumns(3);
	textureSpinnerTransformRotZ.addChangeListener(this);
	ct.gridy ++; ct.gridx = 1;
	textureTransformPanel.add(textureSpinnerTransformRotZ,ct);

	JLabel textureSpinnerTransformTranslatLabel = new JLabel(Info.ressources.getObject("Translation").toString());
	ct.gridy ++; ct.gridx = 0;
	textureTransformPanel.add(textureSpinnerTransformTranslatLabel, ct);

	JLabel textureSpinnerTransformTranslatXLabel = new JLabel("X");
	ct.gridy ++;
	ct.insets = new Insets(0,10,0,5);
	textureTransformPanel.add(textureSpinnerTransformTranslatXLabel, ct);
	ct.insets = new Insets(0,0,0,0);
	SpinnerModel textureSpinnerModelTransformTranslatX = new SpinnerNumberModel(Float.valueOf(0f), //initial value
										    Float.valueOf(-100f), //min
										    Float.valueOf(100f), //max
										    Float.valueOf(0.1f)); //step
	textureSpinnerTransformTranslatX = new JSpinner(textureSpinnerModelTransformTranslatX);
	((JSpinner.DefaultEditor)textureSpinnerTransformTranslatX.getEditor()).getTextField().setColumns(3);
	textureSpinnerTransformTranslatX.addChangeListener(this);
	ct.gridx ++;
	textureTransformPanel.add(textureSpinnerTransformTranslatX,ct);

	JLabel textureSpinnerTransformTranslatYLabel = new JLabel("Y");
	ct.gridx ++;
	ct.insets = new Insets(0,10,0,5);
	textureTransformPanel.add(textureSpinnerTransformTranslatYLabel, ct);
	ct.insets = new Insets(0,0,0,0);
	SpinnerModel textureSpinnerModelTransformTranslatY = new SpinnerNumberModel(Float.valueOf(0f), //initial value
										    Float.valueOf(-100f), //min
										    Float.valueOf(100f), //max
										    Float.valueOf(0.1f)); //step
	textureSpinnerTransformTranslatY = new JSpinner(textureSpinnerModelTransformTranslatY);
	((JSpinner.DefaultEditor)textureSpinnerTransformTranslatY.getEditor()).getTextField().setColumns(3);
	textureSpinnerTransformTranslatY.addChangeListener(this);
	ct.gridx ++;
	textureTransformPanel.add(textureSpinnerTransformTranslatY,ct);
	
	JLabel textureSpinnerTransformScaleLabel = new JLabel(Info.ressources.getObject("Scale").toString());
	ct.gridy ++; ct.gridx = 0;
	textureTransformPanel.add(textureSpinnerTransformScaleLabel, ct);
	
	textureButtonTransformUniformScale = new JRadioButton();
	ct.gridy ++; 
	textureTransformPanel.add(textureButtonTransformUniformScale, ct);
	textureButtonTransformUniformScale.addActionListener(this);
	JLabel textureButtonTransformUniformScaleLabel = new JLabel(Info.ressources.getObject("Uniforme").toString());
	ct.gridx++;
	textureTransformPanel.add(textureButtonTransformUniformScaleLabel, ct);
	


	JLabel textureSpinnerTransformScaleXLabel = new JLabel("X");
	ct.gridy ++; ct.gridx = 0;
	ct.insets = new Insets(0,10,0,5);
	textureTransformPanel.add(textureSpinnerTransformScaleXLabel, ct);
	ct.insets = new Insets(0,0,0,0);
	SpinnerModel textureSpinnerModelTransformScaleX = new SpinnerNumberModel(Float.valueOf(1.0f), //initial value
										 Float.valueOf(-100f), //min
										 Float.valueOf(100f), //max
										 Float.valueOf(0.1f)); //step
	textureSpinnerTransformScaleX = new JSpinner(textureSpinnerModelTransformScaleX);
	((JSpinner.DefaultEditor)textureSpinnerTransformScaleX.getEditor()).getTextField().setColumns(3);
	textureSpinnerTransformScaleX.addChangeListener(this);
	ct.gridx ++;
	textureTransformPanel.add(textureSpinnerTransformScaleX,ct);

	JLabel textureSpinnerTransformScaleYLabel = new JLabel("Y");
	ct.gridx ++;
	ct.insets = new Insets(0,10,0,5);
	textureTransformPanel.add(textureSpinnerTransformScaleYLabel, ct);
	ct.insets = new Insets(0,0,0,0);
	SpinnerModel textureSpinnerModelTransformScaleY = new SpinnerNumberModel(Float.valueOf(1.0f), //initial value
										 Float.valueOf(-100f), //min
										 Float.valueOf(100f), //max
										 Float.valueOf(0.1f)); //step

	textureSpinnerTransformScaleY = new JSpinner(textureSpinnerModelTransformScaleY);
	((JSpinner.DefaultEditor)textureSpinnerTransformScaleY.getEditor()).getTextField().setColumns(3);
	textureSpinnerTransformScaleY.addChangeListener(this);
	ct.gridx ++;
	textureTransformPanel.add(textureSpinnerTransformScaleY,ct);



	//Add the Texture transform Panel
	c.gridx=0; c.gridy++;
	c.insets = new Insets(0,10,0,0);
	textureTransformPanel.setBorder(BorderFactory.createTitledBorder(Info.ressources.getObject("Transform").toString()));
	basicTexturePanel.add(textureTransformPanel,c);

	JLabel perspectiveCorrectionModeLabel = new JLabel(Info.ressources.getObject("Perspective_Correction_Mode").toString());
	c.gridx=0;c.gridy ++;
	c.insets = new Insets(10,10,0,0);
	basicTexturePanel.add(perspectiveCorrectionModeLabel, c);

	String [] perspectiveCorrectionModeChoices = new String[2];
	c.gridy++;
	c.insets = new Insets(0,10,0,0);
	perspectiveCorrectionModeChoices[PERSPECTIVE_CORRECTION_MODE_NICEST] = "NICEST";
	perspectiveCorrectionModeChoices[PERSPECTIVE_CORRECTION_MODE_FASTEST] = "FASTEST";	       
	listPerspectiveCorrectionMode = new JComboBox(perspectiveCorrectionModeChoices);
	listPerspectiveCorrectionMode.setSelectedIndex(perspectiveCorrectionMode);
	listPerspectiveCorrectionMode.addActionListener(this);		
	basicTexturePanel.add(listPerspectiveCorrectionMode, c);


	JLabel textureModeLabel = new JLabel(Info.ressources.getObject("Texture_Mode").toString());
	c.gridy ++;
	c.insets = new Insets(10,10,0,0);
	basicTexturePanel.add(textureModeLabel, c);
	
	c.gridy ++;c.gridx = 0;
	c.insets = new Insets(0,10,0,0);
	String [] textureModeChoices = new String[5];
	textureModeChoices[TEXTURE_MODE_MODULATE] = "MODULATE";
	textureModeChoices[TEXTURE_MODE_DECAL] = "DECAL";
	textureModeChoices[TEXTURE_MODE_REPLACE] = "REPLACE";
	textureModeChoices[TEXTURE_MODE_BLEND] = "BLEND";       
	textureModeChoices[TEXTURE_MODE_COMBINE] = "COMBINE";       
	listTextureMode = new JComboBox(textureModeChoices);
	listTextureMode.setSelectedIndex(textureMode);
	listTextureMode.addActionListener(this);		
	basicTexturePanel.add(listTextureMode, c);
	

	c.gridx = 1; c.gridy = 0;
	c.gridheight = 15;
	c.insets=new Insets(0,20,0,0);
	c.fill = GridBagConstraints.EAST;
	c.anchor = GridBagConstraints.EAST;       	
	JPanel imagePanel = createImagePanel();
	basicTexturePanel.add(imagePanel,c);       		

	addTab(Info.ressources.getObject("Texture").toString(),basicTexturePanel);			
    }


    /**
     * COMBINE parameters Panel for the Texture.
     */        
    private void makeCombineModePane()
    {       	
	// Panel For blend mode
	JPanel combineModePanel = new JPanel();       
	
	GridBagConstraints c = new GridBagConstraints();
	GridBagConstraints c2;

	combineModePanel.setLayout(new GridBagLayout());
		
	c.weightx = 0; c.weighty = 0; 
	c.fill = GridBagConstraints.WEST;
	c.anchor = GridBagConstraints.WEST;			

	// RGB Combine Mode	
	c.insets = new Insets(0,0,0,20);
	JLabel combineRGBModeLabel = new JLabel(Info.ressources.getObject("Combine_RGB_Mode").toString());
	c.gridx=0;c.gridy =0;      
	combineModePanel.add(combineRGBModeLabel, c);

	String [] combineRGBModeChoices = new String[7];	
	combineRGBModeChoices[COMBINE_MODE_REPLACE] = "REPLACE";
	combineRGBModeChoices[COMBINE_MODE_MODULATE] = "MODULATE";
	combineRGBModeChoices[COMBINE_MODE_ADD] = "ADD";
	combineRGBModeChoices[COMBINE_MODE_ADD_SIGNED] = "ADD_SIGNED";       
	combineRGBModeChoices[COMBINE_MODE_SUBTRACT] = "SUBTRACT";       
	combineRGBModeChoices[COMBINE_MODE_INTERPOLATE] = "INTERPOLATE";       
	combineRGBModeChoices[COMBINE_MODE_DOT3] = "DOT3";       

	listCombineRGBMode = new JComboBox(combineRGBModeChoices);
	listCombineRGBMode.setSelectedIndex(combineRGBMode);
	listCombineRGBMode.addActionListener(this);	
	c.gridy ++;	
	c.insets = new Insets(0,10,0,20);
	combineModePanel.add(listCombineRGBMode, c);
		
	c.insets = new Insets(10,0,0,20);
	JLabel combineRGBSourceLabel = new JLabel(Info.ressources.getObject("Combine_RGB_Source").toString());
	c.gridy ++;	
	combineModePanel.add(combineRGBSourceLabel, c);
	c.insets = new Insets(0,10,0,20);
	JPanel combineRGBSourcePanel = new JPanel();
	combineRGBSourcePanel.setLayout(new GridBagLayout());
	c2 = new GridBagConstraints();
	c2.gridx = 0; c2.gridy = 0;
	c2.insets = new Insets(0,0,0,5);

	JLabel combineRGBSourceLabelC0 = new JLabel("C0");
	combineRGBSourcePanel.add(combineRGBSourceLabelC0, c2);
	c2.gridx ++;
	String [] combineRGBSourceChoices = new String[4];	
	combineRGBSourceChoices[COMBINE_SOURCE_OBJECT_COLOR] = "OBJECT_COLOR";
	combineRGBSourceChoices[COMBINE_SOURCE_TEXTURE_COLOR] = "TEXTURE_COLOR";
	combineRGBSourceChoices[COMBINE_SOURCE_CONSTANT_COLOR] = "CONSTANT_COLOR";
	combineRGBSourceChoices[COMBINE_SOURCE_PREVIOUS_TEXTURE_UNIT_STATE] = "PREVIOUS_TEXTURE_UNIT_STATE";       
	listCombineRGBSourceC0 = new JComboBox(combineRGBSourceChoices);
	listCombineRGBSourceC0.setSelectedIndex(combineRGBSourceC0);
	listCombineRGBSourceC0.addActionListener(this);			
	combineRGBSourcePanel.add(listCombineRGBSourceC0, c2);
	
	c2.gridx = 0; c2.gridy ++;
	JLabel combineRGBSourceLabelC1 = new JLabel("C1");
	combineRGBSourcePanel.add(combineRGBSourceLabelC1, c2);
	c2.gridx ++;
	listCombineRGBSourceC1 = new JComboBox(combineRGBSourceChoices);
	listCombineRGBSourceC1.setSelectedIndex(combineRGBSourceC1);
	listCombineRGBSourceC1.addActionListener(this);	
	combineRGBSourcePanel.add(listCombineRGBSourceC1, c2);

	c2.gridx = 0; c2.gridy ++;
	JLabel combineRGBSourceLabelC2 = new JLabel("C2");
	combineRGBSourcePanel.add(combineRGBSourceLabelC2, c2);
	c2.gridx ++;
	listCombineRGBSourceC2 = new JComboBox(combineRGBSourceChoices);
	listCombineRGBSourceC2.setSelectedIndex(combineRGBSourceC2);
	listCombineRGBSourceC2.addActionListener(this);	
	combineRGBSourcePanel.add(listCombineRGBSourceC2, c2);

	c.gridy ++;	
	combineModePanel.add(combineRGBSourcePanel, c);

	c.insets = new Insets(10,0,0,20);
	JLabel combineRGBFunctionLabel = new JLabel(Info.ressources.getObject("Combine_RGB_Function").toString());
	c.gridy ++;	
	combineModePanel.add(combineRGBFunctionLabel, c);
	c.insets = new Insets(0,10,0,20);
	JPanel combineRGBFunctionPanel = new JPanel();
	combineRGBFunctionPanel.setLayout(new GridBagLayout());
	c2 = new GridBagConstraints();
	c2.gridx = 0; c2.gridy = 0;
	c2.insets = new Insets(0,0,0,5);
	
	JLabel combineRGBFunctionLabelC0 = new JLabel("C0");
	combineRGBFunctionPanel.add(combineRGBFunctionLabelC0, c2);
	c2.gridx ++;
	String [] combineRGBFunctionChoices = new String[4];	
	combineRGBFunctionChoices[COMBINE_FUNCTION_SRC_COLOR] = "SRC_COLOR";
	combineRGBFunctionChoices[COMBINE_FUNCTION_ONE_MINUS_SRC_COLOR] = "ONE_MINUS_SRC_COLOR";
	combineRGBFunctionChoices[COMBINE_FUNCTION_SRC_ALPHA] = "SRC_ALPHA";
	combineRGBFunctionChoices[COMBINE_FUNCTION_ONE_MINUS_SRC_ALPHA] = "ONE_MINUS_SRC_ALPHA";       
	listCombineRGBFunctionC0 = new JComboBox(combineRGBFunctionChoices);
	listCombineRGBFunctionC0.setSelectedIndex(combineRGBFunctionC0);
	listCombineRGBFunctionC0.addActionListener(this);	
	combineRGBFunctionPanel.add(listCombineRGBFunctionC0, c2);	

	c2.gridx = 0; c2.gridy ++;
	JLabel combineRGBFunctionLabelC1 = new JLabel("C1");
	combineRGBFunctionPanel.add(combineRGBFunctionLabelC1, c2);
	listCombineRGBFunctionC1 = new JComboBox(combineRGBFunctionChoices);
	listCombineRGBFunctionC1.setSelectedIndex(combineRGBFunctionC1);	
	listCombineRGBFunctionC1.addActionListener(this);		
	c2.gridx ++;
	combineRGBFunctionPanel.add(listCombineRGBFunctionC1, c2);	

	c2.gridx = 0; c2.gridy ++;
	JLabel combineRGBFunctionLabelC2 = new JLabel("C2");
	combineRGBFunctionPanel.add(combineRGBFunctionLabelC2, c2);
	listCombineRGBFunctionC2 = new JComboBox(combineRGBFunctionChoices);
	listCombineRGBFunctionC2.setSelectedIndex(combineRGBFunctionC2);
	listCombineRGBFunctionC2.addActionListener(this);	
	c2.gridx ++;
	combineRGBFunctionPanel.add(listCombineRGBFunctionC2, c2);	

	c.gridy ++;	
	combineModePanel.add(combineRGBFunctionPanel, c);

	c.insets = new Insets(10,0,0,20);
	JLabel combineRGBScaleLabel = new JLabel(Info.ressources.getObject("Combine_RGB_Scale").toString());
	c.gridy ++;	
	combineModePanel.add(combineRGBScaleLabel, c);

	String [] combineRGBScaleChoices = new String[3];	
	combineRGBScaleChoices[0] = "1";
	combineRGBScaleChoices[1] = "2";
	combineRGBScaleChoices[2] = "4";	
	listCombineRGBScale = new JComboBox(combineRGBScaleChoices);
	listCombineRGBScale.setSelectedIndex(combineRGBScale);
	listCombineRGBScale.addActionListener(this);		
	c.gridy ++;	
	c.insets = new Insets(0,10,0,20);
	combineModePanel.add(listCombineRGBScale, c);

	
	// Alpha Combine Mode
	c.insets = new Insets(0,20,0,0);
	JLabel combineAlphaModeLabel = new JLabel(Info.ressources.getObject("Combine_Alpha_Mode").toString());
	c.gridx=1;c.gridy =0;
	combineModePanel.add(combineAlphaModeLabel, c);

	String [] combineAlphaModeChoices = new String[7];	
	combineAlphaModeChoices[COMBINE_MODE_REPLACE] = "REPLACE";
	combineAlphaModeChoices[COMBINE_MODE_MODULATE] = "MODULATE";
	combineAlphaModeChoices[COMBINE_MODE_ADD] = "ADD";
	combineAlphaModeChoices[COMBINE_MODE_ADD_SIGNED] = "ADD_SIGNED";       
	combineAlphaModeChoices[COMBINE_MODE_SUBTRACT] = "SUBTRACT";       
	combineAlphaModeChoices[COMBINE_MODE_INTERPOLATE] = "INTERPOLATE";       
	combineAlphaModeChoices[COMBINE_MODE_DOT3] = "DOT3";       

	listCombineAlphaMode = new JComboBox(combineAlphaModeChoices);
	listCombineAlphaMode.setSelectedIndex(combineAlphaMode);
	listCombineAlphaMode.addActionListener(this);		
	c.gridy ++;	
	c.insets = new Insets(0,30,0,0);
	combineModePanel.add(listCombineAlphaMode, c);
	
	c.insets = new Insets(10,20,0,0);
	JLabel combineAlphaSourceLabel = new JLabel(Info.ressources.getObject("Combine_Alpha_Source").toString());
	c.gridy ++;	
	combineModePanel.add(combineAlphaSourceLabel, c);
	
	JPanel combineAlphaSourcePanel = new JPanel();
	combineAlphaSourcePanel.setLayout(new GridBagLayout());
	c2 = new GridBagConstraints();
	c2.gridx = 0; c2.gridy = 0;
	c2.insets = new Insets(0,0,0,5);

	String [] combineAlphaSourceChoices = new String[4];	
	c.insets = new Insets(0,30,0,0);
	combineAlphaSourceChoices[COMBINE_SOURCE_OBJECT_COLOR] = "OBJECT_COLOR";
	combineAlphaSourceChoices[COMBINE_SOURCE_TEXTURE_COLOR] = "TEXTURE_COLOR";
	combineAlphaSourceChoices[COMBINE_SOURCE_CONSTANT_COLOR] = "CONSTANT_COLOR";
	combineAlphaSourceChoices[COMBINE_SOURCE_PREVIOUS_TEXTURE_UNIT_STATE] = "PREVIOUS_TEXTURE_UNIT_STATE";       
	JLabel combineAlphaSourceLabelC0 = new JLabel("C0");
	combineAlphaSourcePanel.add(combineAlphaSourceLabelC0, c2);
	listCombineAlphaSourceC0 = new JComboBox(combineAlphaSourceChoices);
	listCombineAlphaSourceC0.setSelectedIndex(combineAlphaSourceC0);
	listCombineAlphaSourceC0.addActionListener(this);		
	c2.gridx ++;
	combineAlphaSourcePanel.add(listCombineAlphaSourceC0, c2);

	c2.gridx = 0; c2.gridy ++;
	JLabel combineAlphaSourceLabelC1 = new JLabel("C1");
	combineAlphaSourcePanel.add(combineAlphaSourceLabelC1, c2);
	listCombineAlphaSourceC1 = new JComboBox(combineAlphaSourceChoices);
	listCombineAlphaSourceC1.setSelectedIndex(combineAlphaSourceC1);
	listCombineAlphaSourceC1.addActionListener(this);	
	c2.gridx ++;
	combineAlphaSourcePanel.add(listCombineAlphaSourceC1, c2);

	c2.gridx = 0; c2.gridy ++;
	JLabel combineAlphaSourceLabelC2 = new JLabel("C2");
	combineAlphaSourcePanel.add(combineAlphaSourceLabelC2, c2);
	listCombineAlphaSourceC2 = new JComboBox(combineAlphaSourceChoices);
	listCombineAlphaSourceC2.setSelectedIndex(combineAlphaSourceC2);
	listCombineAlphaSourceC2.addActionListener(this);	
	c2.gridx ++;
	combineAlphaSourcePanel.add(listCombineAlphaSourceC2, c2);

	c.gridy ++;	
	combineModePanel.add(combineAlphaSourcePanel, c);

	c.insets = new Insets(10,20,0,0);
	JLabel combineAlphaFunctionLabel = new JLabel(Info.ressources.getObject("Combine_Alpha_Function").toString());
	c.gridy ++;	
	combineModePanel.add(combineAlphaFunctionLabel, c);
	
	JPanel combineAlphaFunctionPanel = new JPanel();
	combineAlphaFunctionPanel.setLayout(new GridBagLayout());
	c2 = new GridBagConstraints();
	c2.gridx = 0; c2.gridy = 0;
	c2.insets = new Insets(0,0,0,5);

	String [] combineAlphaFunctionChoices = new String[2];	
	c.insets = new Insets(0,30,0,0);	
	combineAlphaFunctionChoices[COMBINE_FUNCTION_SRC_ALPHA] = "SRC_ALPHA";
	combineAlphaFunctionChoices[COMBINE_FUNCTION_ONE_MINUS_SRC_ALPHA] = "ONE_MINUS_SRC_ALPHA";       
	JLabel combineAlphaFunctionLabelC0 = new JLabel("C0");
	combineAlphaFunctionPanel.add(combineAlphaFunctionLabelC0, c2);
	listCombineAlphaFunctionC0 = new JComboBox(combineAlphaFunctionChoices);
	listCombineAlphaFunctionC0.setSelectedIndex(combineAlphaFunctionC0);
	listCombineAlphaFunctionC0.addActionListener(this);	
	c2.gridx ++;
	combineAlphaFunctionPanel.add(listCombineAlphaFunctionC0, c2);
	
	c2.gridx = 0; c2.gridy ++;
	JLabel combineAlphaFunctionLabelC1 = new JLabel("C1");
	combineAlphaFunctionPanel.add(combineAlphaFunctionLabelC1, c2);
	listCombineAlphaFunctionC1 = new JComboBox(combineAlphaFunctionChoices);
	listCombineAlphaFunctionC1.setSelectedIndex(combineAlphaFunctionC1);
	listCombineAlphaFunctionC1.addActionListener(this);		
	c2.gridx ++;
	combineAlphaFunctionPanel.add(listCombineAlphaFunctionC1, c2);

	c2.gridx = 0; c2.gridy ++;
	JLabel combineAlphaFunctionLabelC2 = new JLabel("C2");
	combineAlphaFunctionPanel.add(combineAlphaFunctionLabelC2, c2);
	listCombineAlphaFunctionC2 = new JComboBox(combineAlphaFunctionChoices);
	listCombineAlphaFunctionC2.setSelectedIndex(combineAlphaFunctionC2);
	listCombineAlphaFunctionC2.addActionListener(this);	
	c2.gridx ++;
	combineAlphaFunctionPanel.add(listCombineAlphaFunctionC2, c2);

	c.gridy ++;	
	combineModePanel.add(combineAlphaFunctionPanel, c);

	c.insets = new Insets(10,20,0,0);
	JLabel combineAlphaScaleLabel = new JLabel(Info.ressources.getObject("Combine_Alpha_Scale").toString());
	c.gridy ++;	
	combineModePanel.add(combineAlphaScaleLabel, c);

	String [] combineAlphaScaleChoices = new String[3];
	c.gridy ++;	
	combineAlphaScaleChoices[0] = "1";
	combineAlphaScaleChoices[1] = "2";
	combineAlphaScaleChoices[2] = "4";	
	listCombineAlphaScale = new JComboBox(combineAlphaScaleChoices);
	listCombineAlphaScale.setSelectedIndex(combineAlphaScale);
	listCombineAlphaScale.addActionListener(this);		
	c.insets = new Insets(0,30,0,0);
	combineModePanel.add(listCombineAlphaScale, c);	
	
	addTab("Combine",combineModePanel);			
    }


    /**
     * Blend Color Panel for the Texture.
     */            
    private void makeBlendColorPane()
    {
	// Panel For blend color
	JPanel blendColorPanel = new JPanel();       
	blendColorPanel.setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.CENTER;
	blendColorPicker = new ColorPicker(true,true);
	blendColorPanel.add(blendColorPicker,c);	
	// Init Blend Color to {0.5,0.5,0.5,0.5}
	setTextureBlendColor(blendColor);

	blendColorPicker.addPropertyChangeListener(ColorPicker.HEX_COLOR_PROPERTY, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
		    float rgb[] = new float[4];
		    rgb[0]=(float)blendColorPicker.getRGB()[0] / 255;
		    rgb[1]=(float)blendColorPicker.getRGB()[1] / 255;
		    rgb[2]=(float)blendColorPicker.getRGB()[2] / 255;	
		    rgb[3]=blendColorPicker.getOpacity();
		    updateTextureBlendColor(rgb);		    
		    
		}});	

	blendColorPicker.addPropertyChangeListener(ColorPicker.OPACITY_PROPERTY, new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {				
		    float rgb[] = new float[4];
		    rgb[0]=(float)blendColorPicker.getRGB()[0] / 255;
		    rgb[1]=(float)blendColorPicker.getRGB()[1] / 255;
		    rgb[2]=(float)blendColorPicker.getRGB()[2] / 255;	
		    rgb[3]=blendColorPicker.getOpacity();
		    updateTextureBlendColor(rgb);
		}});	

	addTab("Blend",blendColorPanel);			
    }

    /**
     * Panel creation for the texture image. A default image is used and shown here.
     * @return the Panel created.
     */                
    public JPanel createImagePanel()
    {
	JPanel imagePanel = new JPanel();

	image = Toolkit.getDefaultToolkit().getImage(DefaultTexture);
	/* Wait for the loading */
	MediaTracker tracker=new MediaTracker(this);
	tracker.addImage(image,0);
	try {
	    tracker.waitForID(0);
	}
	catch(InterruptedException e) 
	    {}

	// Resize the image			
	BufferedImage imageResize = new BufferedImage(300, 350, BufferedImage.TYPE_INT_ARGB);				    
	Graphics2D g = (Graphics2D) imageResize.getGraphics();
	g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
	g.drawImage(image, 0, 0, 300, 350, null);				
	image=imageResize;
	
	imageIcon = new ImageIcon(image);
	JLabel imageLabel = new JLabel(imageIcon);
	imagePanel.add(imageLabel,BorderLayout.EAST);
	g.dispose();

	textureName = DefaultTexture;

	return imagePanel;
    }


    /**
     * Update the texture image shown and the Texture itself.
     * @param texture the new Texture File.
     */                
    public void updateImage(URL texture)
    {
	image = Toolkit.getDefaultToolkit().getImage(texture);
	
	/* Wait for the loading */
	MediaTracker tracker=new MediaTracker(this);
	tracker.addImage(image,0);
	try {
	    tracker.waitForID(0);
	}
	catch(InterruptedException e) 
	    {}
    
	// Resize the image			
	BufferedImage imageResize = new BufferedImage(300, 350, BufferedImage.TYPE_INT_ARGB);				    
	Graphics2D g = (Graphics2D) imageResize.getGraphics();
	g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
	g.drawImage(image, 0, 0, 300, 350, null);					
	image=imageResize;	
	
	imageIcon.setImage(image);
	g.dispose();
	repaint();
	
	URL lastTextureName = textureName;
	textureName = texture;
	firePropertyChange(TEXTURE_FILE,lastTextureName,textureName);		
    }


    /**
     * Called when a modification is done on the Texture transformation.
     * @param e the related event.
     */                
    public void stateChanged(ChangeEvent e) 
    {
	Object source = e.getSource();
	Transform3D lastTextureTransform = new Transform3D(textureTransform);	

	if(source == textureSpinnerTransformRotZ)
	    textureTransform.setRotation(new AxisAngle4d(new Vector3d(0,0,1),Math.toRadians((Float)textureSpinnerTransformRotZ.getValue())));
	else if(source == textureSpinnerTransformTranslatX || source == textureSpinnerTransformTranslatY)
	    /*textureTransform.setTranslation(new Vector3d((float)(Math.round((Float)textureSpinnerTransformTranslatX.getValue()*10))/10,
	      (float)(Math.round((Float)textureSpinnerTransformTranslatY.getValue()*10))/10,
	      0.0f));  */
	    textureTransform.setTranslation(new Vector3d((Float)textureSpinnerTransformTranslatX.getValue(),
							 (Float)textureSpinnerTransformTranslatY.getValue(),
							 0.0f)); 
	else if(source == textureSpinnerTransformScaleX || source == textureSpinnerTransformScaleY) 
	    {
		if(textureTransformUniformScale)
		    {
			if(source == textureSpinnerTransformScaleX)
			    textureSpinnerTransformScaleY.setValue((Float)textureSpinnerTransformScaleX.getValue());
			else if(source == textureSpinnerTransformScaleY)
			    textureSpinnerTransformScaleX.setValue((Float)textureSpinnerTransformScaleY.getValue());
		    }

		textureTransform.setScale(new Vector3d((Float)textureSpinnerTransformScaleX.getValue(),
						       (Float)textureSpinnerTransformScaleY.getValue(),
						       1.0f));
	    }		
	firePropertyChange(TEXTURE_TRANSFORM,lastTextureTransform,textureTransform);
    }


    /**
     * Called when an action is performed : the OK button is clicked, the Texture is set or changed, the 
     * Texture Mode or the Texture Perspective correction Mode is modified ...
     * @param e the related action.
     */                
    public void actionPerformed(ActionEvent e) 
    {
	Object source = e.getSource();
	if(source == fileChooserButton)
	    {				
		int option = fileChooser.showOpenDialog(this);
		if (option == JFileChooser.APPROVE_OPTION) 
		    {
			File fileOutput = fileChooser.getSelectedFile();
			textureNameJText.setText(fileOutput.getPath());				
			try
			    {		
				updateImage(fileOutput.toURL());
			    }
			catch (Exception f)
			    {
				System.err.println("Error : cannot use the Choosen image" + fileOutput.getPath());
			    }
		    }		
	    }

	else if(source == OKButton)
	    {				
		if (textureNameJText.getText().compareTo("") ==0)
		    updateImage(DefaultTexture);
		else
		    {
			try
			    {
				File fileOutput = new File(textureNameJText.getText());				
				updateImage(fileOutput.toURL());
			    }
			catch (Exception g)
			    {
				System.err.println("Error : cannot use the Choosen image : " +  textureNameJText.getText());
			    }

		    }
	    }
	
	else if(source == setTextureButton)
	    {
		if(setTextureButton.isSelected() && textureNameJText.getText().compareTo("") == 0)
		    updateImage(DefaultTexture);

		firePropertyChange(TEXTURE_USE,useTexture,!useTexture);	
		useTexture=!useTexture;
	    }	       	   

	else if(source == textureButtonTransformReset) 
	    {
		if(textureButtonTransformReset.isSelected())
		    {
			Transform3D lastTextureTransform = new Transform3D(textureTransform);					       
			textureButtonTransformReset.setSelected(false);
			textureTransform.setIdentity();
			textureSpinnerTransformRotZ.setValue(Float.valueOf(0.0f));
			textureSpinnerTransformScaleX.setValue(Float.valueOf(1.0f));
			textureSpinnerTransformScaleY.setValue(Float.valueOf(1.0f));
			textureSpinnerTransformTranslatX.setValue(Float.valueOf(0.0f));
			textureSpinnerTransformTranslatY.setValue(Float.valueOf(0.0f));			
			firePropertyChange(TEXTURE_TRANSFORM,lastTextureTransform,textureTransform);		
		    }
	    }	
	
	else if (source == textureButtonTransformUniformScale)
	    {		
		textureTransformUniformScale = textureButtonTransformUniformScale.isSelected();
	    }

	else if (source == listPerspectiveCorrectionMode)
	    {
		int lastPerspectiveCorrectionMode = perspectiveCorrectionMode;
		perspectiveCorrectionMode = (Integer) listPerspectiveCorrectionMode.getSelectedIndex();		
		firePropertyChange(TEXTURE_PERPSPECTIVE_CORRECTION_MODE,lastPerspectiveCorrectionMode, perspectiveCorrectionMode);
	    }	

	else if (source == listTextureMode)
	    {
		int lastTextureMode = textureMode;
		textureMode = (Integer) listTextureMode.getSelectedIndex();		
		firePropertyChange(TEXTURE_MODE,lastTextureMode, textureMode);		
	    }
	
	else if (source == listCombineRGBMode)
	    {
		int lastCombineRGBMode = combineRGBMode;
		combineRGBMode = (Integer) listCombineRGBMode.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_RGB_MODE,lastCombineRGBMode, combineRGBMode);				       
	    }

	else if (source == listCombineAlphaMode)
	    {
		int lastCombineAlphaMode = combineAlphaMode;
		combineAlphaMode = (Integer) listCombineAlphaMode.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_ALPHA_MODE,lastCombineAlphaMode, combineAlphaMode);						
	    }
	else if (source == listCombineRGBSourceC0)
	    {
		int lastCombineRGBSourceC0 = combineRGBSourceC0;
		combineRGBSourceC0 = (Integer) listCombineRGBSourceC0.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_RGB_SOURCE, lastCombineRGBSourceC0, combineRGBSourceC0);   
	    }

	else if (source == listCombineRGBSourceC1)
	    {
		int lastCombineRGBSourceC1 = combineRGBSourceC1;
		combineRGBSourceC1 = (Integer) listCombineRGBSourceC1.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_RGB_SOURCE, lastCombineRGBSourceC1, combineRGBSourceC1);   
	    }

	else if (source == listCombineRGBSourceC2)
	    {
		int lastCombineRGBSourceC2 = combineRGBSourceC2;
		combineRGBSourceC2 = (Integer) listCombineRGBSourceC2.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_RGB_SOURCE, lastCombineRGBSourceC2, combineRGBSourceC2);   
	    }

	else if (source == listCombineAlphaSourceC0)
	    {
		int lastCombineAlphaSourceC0 = combineAlphaSourceC0;
		combineAlphaSourceC0 = (Integer) listCombineAlphaSourceC0.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_ALPHA_SOURCE, lastCombineAlphaSourceC0, combineAlphaSourceC0);   
	    }

	else if (source == listCombineAlphaSourceC1)
	    {
		int lastCombineAlphaSourceC1 = combineAlphaSourceC1;
		combineAlphaSourceC1 = (Integer) listCombineAlphaSourceC1.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_ALPHA_SOURCE, lastCombineAlphaSourceC1, combineAlphaSourceC1);   
	    }

	else if (source == listCombineAlphaSourceC2)
	    {
		int lastCombineAlphaSourceC2 = combineAlphaSourceC2;
		combineAlphaSourceC2 = (Integer) listCombineAlphaSourceC2.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_ALPHA_SOURCE, lastCombineAlphaSourceC2, combineAlphaSourceC2);   
	    }

	else if (source == listCombineRGBFunctionC0)
	    {
		int lastCombineRGBFunctionC0 = combineRGBFunctionC0;
		combineRGBFunctionC0 = (Integer) listCombineRGBFunctionC0.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_RGB_FUNCTION, lastCombineRGBFunctionC0, combineRGBFunctionC0);   
	    }

	else if (source == listCombineRGBFunctionC1)
	    {
		int lastCombineRGBFunctionC1 = combineRGBFunctionC1;
		combineRGBFunctionC1 = (Integer) listCombineRGBFunctionC1.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_RGB_FUNCTION, lastCombineRGBFunctionC1, combineRGBFunctionC1);   
	    }

	else if (source == listCombineRGBFunctionC2)
	    {
		int lastCombineRGBFunctionC2 = combineRGBFunctionC2;
		combineRGBFunctionC2 = (Integer) listCombineRGBFunctionC2.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_RGB_FUNCTION, lastCombineRGBFunctionC2, combineRGBFunctionC2);   
	    }

	else if (source == listCombineAlphaFunctionC0)
	    {
		int lastCombineAlphaFunctionC0 = combineAlphaFunctionC0;
		combineAlphaFunctionC0 = (Integer) listCombineAlphaFunctionC0.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_ALPHA_FUNCTION, lastCombineAlphaFunctionC0, combineAlphaFunctionC0);   
	    }

	else if (source == listCombineAlphaFunctionC1)
	    {
		int lastCombineAlphaFunctionC1 = combineAlphaFunctionC1;
		combineAlphaFunctionC1 = (Integer) listCombineAlphaFunctionC1.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_ALPHA_FUNCTION, lastCombineAlphaFunctionC1, combineAlphaFunctionC1);   
	    }

	else if (source == listCombineAlphaFunctionC2)
	    {
		int lastCombineAlphaFunctionC2 = combineAlphaFunctionC2;
		combineAlphaFunctionC2 = (Integer) listCombineAlphaFunctionC2.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_ALPHA_FUNCTION, lastCombineAlphaFunctionC2, combineAlphaFunctionC2);   
	    }
	
	else if (source == listCombineRGBScale)
	    {
		int lastCombineRGBScale = combineRGBScale;
		combineRGBScale = (Integer) listCombineRGBScale.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_RGB_SCALE, lastCombineRGBScale, combineRGBScale);   
	    }
	
	else if (source == listCombineAlphaScale)
	    {
		int lastCombineAlphaScale = combineAlphaScale;
		combineAlphaScale = (Integer) listCombineAlphaScale.getSelectedIndex();	
		firePropertyChange(TEXTURE_COMBINE_ALPHA_SCALE, lastCombineAlphaScale, combineAlphaScale);   
	    }

    }
    

    /**
     * 
     * Give the current Texture file.
     * @return the Texture file.
     *
     */                
    public URL getTextureFile()
    {
	return textureName;
    }

    /**
     * 
     * Set the current Texture file.
     * @param f the Texture file to set.
     *
     */                
    public void setTextureFile(URL f)
    {
	updateImage(f);	
	textureNameJText.setText(f.getPath());
    }

    /**
     * 
     * Set the current default Texture file.
     * @param f the Texture default file to set.
     *
     */                
    public void setDefaultTextureFile(URL f)
    {
	updateImage(f);	
	DefaultTexture = f;	
    }

    /**
     * 
     * Check is the Texture is used.
     * @return true if the Texture is used, false otherwise.
     *
     */                
    public boolean isTextureSet()
    {
	return setTextureButton.isSelected();
    }

    /**
     * 
     * use or not the current Texture.
     * @param b true if the currect Texture has to be used, false otherwise.
     *
     */                
    public void setTextureUse(boolean b)
    {
	setTextureButton.setSelected(b);
	
	useTexture = b;
	if(b && textureNameJText.getText().compareTo("") ==0)
	    updateImage(DefaultTexture);

	firePropertyChange(TEXTURE_USE,useTexture,!useTexture);		
    }

    /**
     * 
     * Get the current Texture transformation.
     * @return the current Texture transformation.
     *
     */                
    public Transform3D getTextureTransform()
    {
	return new Transform3D(textureTransform);
    }

    /**
     * 
     * Set the Texture transformation.
     * @param t the Texture transformation to use.
     *
     */                
    public void setTextureTransform(Transform3D t)
    {
	Transform3D lastTextureTransform = new Transform3D(textureTransform);
	textureTransform = new Transform3D(t);		
	Vector3f trans = new  Vector3f();
	Vector3d scale = new Vector3d();
	Matrix3f rot = new Matrix3f();
	textureTransform.get(trans);	
	textureTransform.get(rot);	
	textureTransform.getScale(scale);
	
	float [] transV = new float[3];
	double [] scaleV = new double[3];
	trans.get(transV);
	scale.get(scaleV);

	// If Sinus is >= 0 compute acos
	if (rot.getElement(1,0)>=0)	    
	    {
		textureSpinnerTransformRotZ.setValue((float)Math.toDegrees(Math.acos(rot.getElement(0,0))));
	    }
	else 
	    // If Sinus is < 0; angle is 360 degres - acos
	    {
		textureSpinnerTransformRotZ.setValue((float) (360 - Math.toDegrees(Math.acos(rot.getElement(0,0)))));
	    }

	textureSpinnerTransformTranslatX.setValue(transV[0]);
	textureSpinnerTransformTranslatY.setValue(transV[1]);
	textureSpinnerTransformScaleX.setValue((float)scaleV[0]);
	textureSpinnerTransformScaleY.setValue((float)scaleV[1]);
	
	firePropertyChange(TEXTURE_TRANSFORM,lastTextureTransform,textureTransform);	
    }

    /**
     * 
     * Get the current Texture scale transformation on the X axis.
     * @return the current Texture scale transformation on the X axis.
     *
     */                
    public float getTextureTransformScaleX()
    {
	return (Float)textureSpinnerTransformScaleX.getValue();
    }

    /**
     * 
     * Set the Texture scale transformation on the X axis.
     * @param v the Texture scale transformation on the X axis to use.
     *
     */                
    public void setTextureTransformScaleX(float v)
    {
	textureSpinnerTransformScaleX.setValue(v);
    }

    /**
     * 
     * Get the current Texture scale transformation on the Y axis.
     * @return the current Texture scale transformation on the Y axis.
     *
     */                
    public float getTextureTransformScaleY()
    {
	return (Float)textureSpinnerTransformScaleY.getValue();
    }

    /**
     * 
     * Set the Texture scale transformation on the Y axis.
     * @param v the Texture scale transformation on the Y axis to use.
     *
     */                
    public void setTextureTransformScaleY(float v)
    {
	textureSpinnerTransformScaleY.setValue(v);
    }

    /**
     * 
     * Get the current Texture translation transformation on the X axis.
     * @return the current Texture translation transformation on the X axis.
     *
     */                
    public float getTextureTransformTranslatX()
    {
	return (Float)textureSpinnerTransformTranslatX.getValue();
    }

    /**
     * 
     * Set the Texture translation transformation on the X axis.
     * @param v the Texture translation transformation on the X axis to use.
     *
     */                
    public void setTextureTransformTranslatX(float v)
    {
	textureSpinnerTransformTranslatX.setValue(v);
    }

    /**
     * 
     * Get the current Texture translation transformation on the Y axis.
     * @return the current Texture translation transformation on the Y axis.
     *
     */                
    public float getTextureTransformTranslatY()
    {
	return (Float)textureSpinnerTransformTranslatY.getValue();
    }

    /**
     * 
     * Set the Texture translation transformation on the Y axis.
     * @param v the Texture translation transformation on the Y axis to use.
     *
     */                
    public void setTextureTransformTranslatY(float v)
    {
	textureSpinnerTransformTranslatY.setValue(v);
    }

    /**
     * 
     * Get the current Texture rotation transformation on the Z axis.
     * @return the current Texture rotation transformation on the Z axis.
     *
     */                
    public float getTextureTransformRotZ()
    {			
	return (float)Math.toRadians((Float)textureSpinnerTransformRotZ.getValue());
    }

    /**
     * 
     * Set the Texture rotation transformation on the Z axis.
     * @param v the Texture rotation transformation on the Z axis to use.
     *
     */                
    public void setTextureTransformRotZ(float v)
    {
	textureSpinnerTransformRotZ.setValue((float)Math.toDegrees(v));
    }

    /**
     * 
     * Get the current Texture Mode.
     * @return the current Texture Mode.
     *
     */                   
    public int getTextureMode()
    {
	return convertExternTextureMode(textureMode);    
    }

    /**
     * 
     * Set the Texture Mode.
     * @param m the Texture Mode to use.
     *
     */                
    public void setTextureMode(int m)
    {
	listTextureMode.setSelectedIndex(convertInternTextureMode(m));	
	textureMode = convertInternTextureMode(m);    
    }
        
    /**
     * 
     * Get the current Texture Perspective Correction Mode.
     * @return the current Texture Perspective Correction Mode.
     *
     */                
    public int getTexturePerspectiveCorrectionMode()
    {
	return convertExternPerspectiveCorrectionMode(perspectiveCorrectionMode);    
    }

    /**
     * 
     * Set the Texture Perspective Correction Mode.
     * @param m the Texture Perspective Correction Mode to use.
     *
     */                
    public void setTexturePerspectiveCorrectionMode(int m)
    {
	listPerspectiveCorrectionMode.setSelectedIndex(convertInternPerspectiveCorrectionMode(m));	
	perspectiveCorrectionMode = convertInternPerspectiveCorrectionMode(m);   
    }

    /**
     * 
     * Get the current Texture blend color.
     * @return the current Texture blend color.
     *
     */                
    public float [] getTextureBlendColor ()
    {
	return blendColor;
    }

    /**
     * 
     * Set the Texture blend color.
     * @param c the Texture blend color to use.
     *
     */                	
    public void setTextureBlendColor (float []c)
    {
	blendColorPicker.setRGB((int)(c[0] * 255), (int)(c[1] * 255), (int)(c[2] * 255));
	blendColorPicker.setOpacity(c[3]);	
	blendColor = c;	
    }

    /**
     * 
     * Get the current Texture Combine RGB Mode.
     * @return the current Texture Combine RGB Mode.
     *
     */                
    public int getTextureCombineRGBMode ()
    {
	return convertExternTextureCombineMode(combineRGBMode);
    }

    /**
     * 
     * Set the Texture Combine RGB Mode.
     * @param m the Texture Combine RGB Mode to use.
     *
     */                	
    public void setTextureCombineRGBMode (int m)
    {
	listCombineRGBMode.setSelectedIndex(convertInternTextureCombineMode(m));	
	combineRGBMode = convertInternTextureCombineMode(m);   
    }

    /**
     * 
     * Get the current Texture Combine RGB Source.
     * @return the current Texture Combine RGB Source (C0,C1,C2).
     *
     */                
    public int[] getTextureCombineRGBSource ()
    {
	int [] res = new int[3];
	res[0] = convertExternTextureCombineSource(combineRGBSourceC0);
	res[1] = convertExternTextureCombineSource(combineRGBSourceC1);
	res[2] = convertExternTextureCombineSource(combineRGBSourceC2);
	
	return res;
    }

    /**
     * 
     * Set the Texture Combine RGB Source.
     * @param c the Texture Combine RGB Source Mode (C0,C1,C2) to use.     
     *
     */                	
    public void setTextureCombineRGBSource (int []c)
    {
	listCombineRGBSourceC0.setSelectedIndex(convertInternTextureCombineSource(c[0]));	
	combineRGBSourceC0 = convertInternTextureCombineSource(c[0]);   
	listCombineRGBSourceC1.setSelectedIndex(convertInternTextureCombineSource(c[1]));	
	combineRGBSourceC1 = convertInternTextureCombineSource(c[1]);   
	listCombineRGBSourceC2.setSelectedIndex(convertInternTextureCombineSource(c[2]));	
	combineRGBSourceC2 = convertInternTextureCombineSource(c[2]);   
    }

    /**
     * 
     * Get the current Texture Combine RGB Function.
     * @return the current Texture Combine RGB Function (C0,C1,C2).
     *
     */                
    public int[] getTextureCombineRGBFunction ()
    {
	int [] res = new int[3];
	res[0] = convertExternTextureCombineFunction(combineRGBFunctionC0);
	res[1] = convertExternTextureCombineFunction(combineRGBFunctionC1);
	res[2] = convertExternTextureCombineFunction(combineRGBFunctionC2);
	
	return res;
    }

    /**
     * 
     * Set the Texture Combine RGB Function.
     * @param c the Texture Combine RGB Function Mode (C0,C1,C2) to use.     
     *
     */                	
    public void setTextureCombineRGBFunction (int []c)
    {
	listCombineRGBFunctionC0.setSelectedIndex(convertInternTextureCombineFunction(c[0]));	
	combineRGBFunctionC0 = convertInternTextureCombineFunction(c[0]);   
	listCombineRGBFunctionC1.setSelectedIndex(convertInternTextureCombineFunction(c[1]));	
	combineRGBFunctionC1 = convertInternTextureCombineFunction(c[1]);   
	listCombineRGBFunctionC2.setSelectedIndex(convertInternTextureCombineFunction(c[2]));	
	combineRGBFunctionC2 = convertInternTextureCombineFunction(c[2]);   
    }

    /**
     * 
     * Get the current Texture Combine RGB Scale Factor.
     * @return the current Texture Combine RGB Scale Factor.
     *
     */                
    public int getTextureCombineRGBScale ()
    {
	return convertExternTextureCombineScale(combineRGBScale);
    }

    /**
     * 
     * Set the Texture Combine RGB Scale Factor.
     * @param m the Texture Combine RGB Scale Factor to use.
     *
     */                	
    public void setTextureCombineRGBScale (int m)
    {
	listCombineRGBScale.setSelectedIndex(convertInternTextureCombineScale(m));	
	combineRGBScale = convertInternTextureCombineScale(m);   
    }

    /**
     * 
     * Get the current Texture Combine Alpha Mode.
     * @return the current Texture Combine Alpha Mode.
     *
     */                
    public int getTextureCombineAlphaMode ()
    {
	return convertExternTextureCombineMode(combineAlphaMode);
    }

    /**
     * 
     * Set the Texture Combine Alpha Mode.
     * @param m the Texture Combine Alpha Mode to use.
     *
     */                	
    public void setTextureCombineAlphaMode (int m)
    {
	listCombineAlphaMode.setSelectedIndex(convertInternTextureCombineMode(m));	
	combineAlphaMode = convertInternTextureCombineMode(m);   
    }

    /**
     * 
     * Get the current Texture Combine Alpha Source.
     * @return the current Texture Combine Alpha Source (C0,C1,C2).
     *
     */                
    public int[] getTextureCombineAlphaSource ()
    {
	int [] res = new int[3];
	res[0] = convertExternTextureCombineSource(combineAlphaSourceC0);
	res[1] = convertExternTextureCombineSource(combineAlphaSourceC1);
	res[2] = convertExternTextureCombineSource(combineAlphaSourceC2);
	
	return res;
    }

    /**
     * 
     * Set the Texture Combine Alpha Source.
     * @param c the Texture Combine Alpha Source Mode (C0,C1,C2) to use.
     *
     */                	
    public void setTextureCombineAlphaSource (int []c)
    {
	listCombineAlphaSourceC0.setSelectedIndex(convertInternTextureCombineSource(c[0]));	
	combineAlphaSourceC0 = convertInternTextureCombineSource(c[0]);   
	listCombineAlphaSourceC1.setSelectedIndex(convertInternTextureCombineSource(c[1]));	
	combineAlphaSourceC1 = convertInternTextureCombineSource(c[1]);   
	listCombineAlphaSourceC2.setSelectedIndex(convertInternTextureCombineSource(c[2]));	
	combineAlphaSourceC2 = convertInternTextureCombineSource(c[2]);   
    }

    /**
     * 
     * Get the current Texture Combine Alpha Function.
     * @return the current Texture Combine Alpha Function (C0,C1,C2).
     *
     */                
    public int[] getTextureCombineAlphaFunction ()
    {
	int [] res = new int[3];
	res[0] = convertExternTextureCombineFunction(combineAlphaFunctionC0);
	res[1] = convertExternTextureCombineFunction(combineAlphaFunctionC1);
	res[2] = convertExternTextureCombineFunction(combineAlphaFunctionC2);
	
	return res;
    }

    /**
     * 
     * Set the Texture Combine Alpha Function.
     * @param c the Texture Combine Alpha Function Mode (C0,C1,C2) to use.
     *
     */                	
    public void setTextureCombineAlphaFunction (int []c)
    {
	listCombineAlphaFunctionC0.setSelectedIndex(convertInternTextureCombineFunction(c[0]));	
	combineAlphaFunctionC0 = convertInternTextureCombineFunction(c[0]);   
	listCombineAlphaFunctionC1.setSelectedIndex(convertInternTextureCombineFunction(c[1]));	
	combineAlphaFunctionC1 = convertInternTextureCombineFunction(c[1]);   
	listCombineAlphaFunctionC2.setSelectedIndex(convertInternTextureCombineFunction(c[2]));	
	combineAlphaFunctionC2 = convertInternTextureCombineFunction(c[2]);   
    }

    /**
     * 
     * Get the current Texture Combine Alpha Scale Factor.
     * @return the current Texture Combine Alpha Scale Factor.
     *
     */                
    public int getTextureCombineAlphaScale ()
    {
	return convertExternTextureCombineScale(combineAlphaScale);
    }

    /**
     * 
     * Set the Texture Combine Alpha Scale Factor.
     * @param m the Texture Combine Alpha Scale Factor to use.
     *
     */                	
    public void setTextureCombineAlphaScale (int m)
    {
	listCombineAlphaScale.setSelectedIndex(convertInternTextureCombineScale(m));	
	combineAlphaScale = convertInternTextureCombineScale(m);   
    }
    
    /**
     * 
     * Update the Texture blend color. Called when a blend color parameter is modified.
     * @param b the (R,G,B,A) value of the new Texture blend color to use.
     *
     */                	
    private void updateTextureBlendColor (float []b)
    {
	float []lastBlend = blendColor;
	blendColor = b;	
	firePropertyChange(TEXTURE_BLEND,lastBlend,blendColor);
    }

    /**
     * 
     * Convert a Texture Perspective Correction Mode om TextureAttributes representation to Internal representation.
     * @param m a Texture Perspective Correction Mode in TextureAttributes representation.
     * @return a Texture Perspective Correction Mode in Internal representation.
     *
     */                	
    public static int convertInternPerspectiveCorrectionMode (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.NICEST:
		return PERSPECTIVE_CORRECTION_MODE_NICEST;
	    case TextureAttributes.FASTEST:
		return PERSPECTIVE_CORRECTION_MODE_FASTEST;	
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Perspective Correction Mode om Internal representation to TextureAttributes representation.
     * @param m a Texture Perspective Correction Mode in Internal representation.
     * @return a Texture Perspective Correction Mode in TextureAttributes representation.
     *
     */                	
    public static int convertExternPerspectiveCorrectionMode (int m)
    {
	switch (m)
	    {
	    case PERSPECTIVE_CORRECTION_MODE_NICEST:
		return TextureAttributes.NICEST;
	    case PERSPECTIVE_CORRECTION_MODE_FASTEST:
		return TextureAttributes.FASTEST;	
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Mode om TextureAttributes representation to Internal representation.
     * @param m a Texture Mode in TextureAttributes representation.
     * @return a Texture Mode in Internal representation.
     *
     */                	
    public static int convertInternTextureMode (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.MODULATE:
		return TEXTURE_MODE_MODULATE;
	    case TextureAttributes.DECAL:
		return TEXTURE_MODE_DECAL;	
	    case TextureAttributes.BLEND:
		return TEXTURE_MODE_BLEND;	
	    case TextureAttributes.REPLACE:
		return TEXTURE_MODE_REPLACE;	
	    case TextureAttributes.COMBINE:
		return TEXTURE_MODE_COMBINE;	
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Mode om Internal representation to TextureAttributes representation.
     * @param m a Texture Mode in Internal representation.
     * @return a Texture Mode in TextureAttributes representation.
     *
     */                	
    public static int convertExternTextureMode (int m)
    {
	switch (m)
	    {
	    case TEXTURE_MODE_MODULATE:
		return TextureAttributes.MODULATE;
	    case TEXTURE_MODE_DECAL:
		return TextureAttributes.DECAL;	
	    case TEXTURE_MODE_BLEND:
		return TextureAttributes.BLEND;	
	    case TEXTURE_MODE_REPLACE:
		return TextureAttributes.REPLACE;	
	    case TEXTURE_MODE_COMBINE:
		return TextureAttributes.COMBINE;	
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Combine RGB/Alpha Mode om TextureAttributes representation to Internal representation.
     * @param m a Texture Combine RGB/Alpha Mode in TextureAttributes representation.
     * @return a Texture Combine RGB/Alpha Mode in Internal representation.
     *
     */                	
    public static int convertInternTextureCombineMode (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.COMBINE_REPLACE:
		return COMBINE_MODE_REPLACE;
	    case TextureAttributes.COMBINE_MODULATE:
		return COMBINE_MODE_MODULATE;	
	    case TextureAttributes.COMBINE_ADD:
		return COMBINE_MODE_ADD;	
	    case TextureAttributes.COMBINE_ADD_SIGNED:
		return COMBINE_MODE_ADD_SIGNED;	
	    case TextureAttributes.COMBINE_SUBTRACT:
		return COMBINE_MODE_SUBTRACT;	
	    case TextureAttributes.COMBINE_INTERPOLATE:
		return COMBINE_MODE_INTERPOLATE;	
	    case TextureAttributes.COMBINE_DOT3:
		return COMBINE_MODE_DOT3;	
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Combine RGB/Alpha Mode om Internal representation to TextureAttributes representation.
     * @param m a Texture Combine RGB/Alpha Mode in Internal representation.
     * @return a Texture Combine RGB/Alpha Mode in TextureAttributes representation.
     *
     */                	
    public static int convertExternTextureCombineMode (int m)
    {
	switch (m)
	    {
	    case COMBINE_MODE_REPLACE:
		return TextureAttributes.COMBINE_REPLACE;
	    case COMBINE_MODE_MODULATE:
		return TextureAttributes.COMBINE_MODULATE;	
	    case COMBINE_MODE_ADD:
		return TextureAttributes.COMBINE_ADD;	
	    case COMBINE_MODE_ADD_SIGNED:
		return TextureAttributes.COMBINE_ADD_SIGNED;	
	    case  COMBINE_MODE_SUBTRACT:
		return TextureAttributes.COMBINE_SUBTRACT;	
	    case COMBINE_MODE_INTERPOLATE:
		return TextureAttributes.COMBINE_INTERPOLATE;	
	    case COMBINE_MODE_DOT3:
		return TextureAttributes.COMBINE_DOT3;	
	    default :
		return -1;
	    }
    }


    /**
     * 
     * Convert a Texture Combine RGB/Alpha Color Source om TextureAttributes representation to Internal representation.
     * @param m a Texture Combine RGB/Alpha Color Source in TextureAttributes representation.
     * @return a Texture Combine RGB/Alpha Color Source in Internal representation.
     *
     */                	
    public static int convertInternTextureCombineSource (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.COMBINE_OBJECT_COLOR:
		return COMBINE_SOURCE_OBJECT_COLOR;
	    case TextureAttributes.COMBINE_TEXTURE_COLOR:
		return COMBINE_SOURCE_TEXTURE_COLOR;	
	    case TextureAttributes.COMBINE_CONSTANT_COLOR:
		return COMBINE_SOURCE_CONSTANT_COLOR;	
	    case TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE:
		return COMBINE_SOURCE_PREVIOUS_TEXTURE_UNIT_STATE;		    
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Combine RGB/Alpha Color Source om Internal representation to TextureAttributes representation.
     * @param m a Texture Combine RGB/Alpha Color Source in Internal representation.
     * @return a Texture Combine RGB/Alpha Color Source in TextureAttributes representation.
     *
     */                	
    public static int convertExternTextureCombineSource (int m)
    {
	switch (m)
	    {
	    case COMBINE_SOURCE_OBJECT_COLOR:
		return TextureAttributes.COMBINE_OBJECT_COLOR;
	    case COMBINE_SOURCE_TEXTURE_COLOR:
		return TextureAttributes.COMBINE_TEXTURE_COLOR;	
	    case COMBINE_SOURCE_CONSTANT_COLOR:
		return TextureAttributes.COMBINE_CONSTANT_COLOR;	
	    case COMBINE_SOURCE_PREVIOUS_TEXTURE_UNIT_STATE:
		return TextureAttributes.COMBINE_PREVIOUS_TEXTURE_UNIT_STATE;		    
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Combine RGB/Alpha Color Function om TextureAttributes representation to Internal representation.
     * @param m a Texture Combine RGB/Alpha Color Function in TextureAttributes representation.
     * @return a Texture Combine RGB/Alpha Color Function in Internal representation.
     *
     */                	
    public static int convertInternTextureCombineFunction (int m)
    {
	switch (m)
	    {
	    case TextureAttributes.COMBINE_SRC_COLOR:
		return COMBINE_FUNCTION_SRC_COLOR;
	    case TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR:
		return COMBINE_FUNCTION_ONE_MINUS_SRC_COLOR;	
	    case TextureAttributes.COMBINE_SRC_ALPHA:
		return COMBINE_FUNCTION_SRC_ALPHA;	
	    case TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA:
		return COMBINE_FUNCTION_ONE_MINUS_SRC_ALPHA;		    
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Combine RGB/Alpha Color Function om Internal representation to TextureAttributes representation.
     * @param m a Texture Combine RGB/Alpha Color Function in Internal representation.
     * @return a Texture Combine RGB/Alpha Color Function in TextureAttributes representation.
     *
     */                	
    public static int convertExternTextureCombineFunction (int m)
    {
	switch (m)
	    {
	    case COMBINE_FUNCTION_SRC_COLOR:
		return TextureAttributes.COMBINE_SRC_COLOR;
	    case COMBINE_FUNCTION_ONE_MINUS_SRC_COLOR:
		return TextureAttributes.COMBINE_ONE_MINUS_SRC_COLOR;	
	    case COMBINE_FUNCTION_SRC_ALPHA:
		return TextureAttributes.COMBINE_SRC_ALPHA;	
	    case COMBINE_FUNCTION_ONE_MINUS_SRC_ALPHA:
		return TextureAttributes.COMBINE_ONE_MINUS_SRC_ALPHA;		    
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Combine Scale Factor om TextureAttributes representation to Internal representation.
     * @param m a Texture Combine Scale Factor in TextureAttributes representation.
     * @return a Texture Combine Scale Factor in Internal representation.
     *
     */                	
    public static int convertInternTextureCombineScale (int m)
    {
	switch (m)
	    {
	    case 1:
		return 0;
	    case 2:
		return 1;	
	    case 4:
		return 2;	
	    default :
		return -1;
	    }
    }

    /**
     * 
     * Convert a Texture Combine Scale Factor from Internal representation to TextureAttributes representation.
     * @param m a Texture Combine Scale Factor in Internal representation.
     * @return a Texture Combine Scale Factor in TextureAttributes representation.
     *
     */                	
    public static int convertExternTextureCombineScale (int m)
    {
	switch (m)
	    {
	    case 0:
		return 1;
	    case 1:
		return 2;	
	    case 2:
		return 4;	
	    default :
		return -1;
	    }
    }


}