/**
    ##############################################################################
    ##                                                                          ##
    ## EnvInfo                                                                  ##
    ##                                                                          ##
    ## Copyright (C) 2009  Frederic Roudaut  <frederic.roudaut@free.fr>         ##
    ##                                                                          ##
    ## This class is an adaption of the one done by Nicolas Richasse for his    ##
    ## Java Iperf frontend.                                                     ##
    ##                                                                          ##
    ##                                                                          ##
    ## This program is free software: you can redistribute it and/or modify     ##
    ## it under the terms of the GNU General Public License as published by     ##
    ## the Free Software Foundation, either version 3 of the License, or        ##
    ## (at your option) any later version.                                      ##
    ##                                                                          ##
    ## This program is distributed in the hope that it will be useful,          ##
    ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
    ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
    ## GNU General Public License for more details.                             ##
    ##                                                                          ##
    ## You should have received a copy of the GNU General Public License        ##
    ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
    ##                                                                          ##
    ##                                                                          ##
    ##############################################################################
 **/

    package com.envInfo;

/**
 * Class used to represent a single Memory Measurement.
 * <br/><br/>
 * This class is an adaption of the one done by Nicolas Richasse for his   
 * Java Iperf frontend.
 *
 **/
public class MemoryMeasurement
{
    private double	start;
    private double	end;
    private double	value;
    private String	units;

    /**
     * Constructor.
     * @param start the start time of the measurement.
     * @param end the end time of the measurement.
     * @param value the memory value.
     * @param units the memory Units.
     *
     **/
    public MemoryMeasurement(double start, double end, double value, String units)
    {
	this.start = start;
	this.end = end;
	this.value = value;
	this.units = units;
    }

    /**
     * Return the start time.
     * @return the start time.	 
     *
     **/
    public double getStartTime()
    {
	return start;
    }

    /**
     * Return the end time.
     * @return the end time.	 
     *
     **/
    public double getEndTime()
    {
	return end;
    }

    /**
     * Return the measurement value.
     * @return the measurement value.	 
     *
     **/
    public double getValue()
    {
	return value;
    }

    /**
     * Return the measurement unit.
     * @return the measurement unit.	 
     *
     **/
    public String getUnits()
    {
	return units;
    }

    /**
     * Print the measurement content.
     *
     **/
    public void print()
    {
	System.out.println("Start Time: " + start);
	System.out.println("End Time: " + end);
	System.out.println("Value: " + value);
	System.out.println("Units: " + units);
    }
}
