/**
   ##############################################################################
   ##                                                                          ##
   ## EnvInfo                                                                  ##
   ##                                                                          ##
   ## Copyright (C) 2009  Frederic Roudaut  <frederic.roudaut@free.fr>         ##
   ##                                                                          ##
   ##                                                                          ##
   ## This program is free software: you can redistribute it and/or modify     ##
   ## it under the terms of the GNU General Public License as published by     ##
   ## the Free Software Foundation, either version 3 of the License, or        ##
   ## (at your option) any later version.                                      ##
   ##                                                                          ##
   ## This program is distributed in the hope that it will be useful,          ##
   ## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
   ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
   ## GNU General Public License for more details.                             ##
   ##                                                                          ##
   ## You should have received a copy of the GNU General Public License        ##
   ## along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
   ##                                                                          ##
   ##                                                                          ##
   ##############################################################################
**/



package com.envInfo;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.*;
import java.awt.Insets;
import java.awt.Font;
import java.awt.Dimension;
import java.awt.Color;
import javax.swing.*;
import javax.swing.event.*;

import com.awt.GridBagLayoutConstraint;

import java.util.*;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.lang.management.ThreadMXBean;  
import java.lang.management.ThreadInfo;  



/**
 * A Class for showing in real time some System (OS, CPU, Java Memory) Info.
 *
 */
public class SystemInfo extends JFrame
{
	  
    private static final long serialVersionUID = 1L;   
    
    protected JTextField totalMemoryJText;
    protected JTextField freeMemoryJText;
    protected JTextField maxMemoryJText;
    protected JTextField usedMemoryJText;
    protected JComboBox outputUnit;
    protected float memoryTimer = 1.0f;  // Timer to update memory info (sec)
    protected float threadsTimer = 60.0f;  // Timer to update threads info (sec)
    protected MemoryChartPanel chartPanel = null;
    protected long chartDateStart=0;    
    protected JPanel threadsTextPanel;

    /**
     * Draw a Frame with System (OS, CPU, Java Memory) Info.
     *
     */       
    public SystemInfo() 
    {
	super(Info.ressources.getObject("System_Info").toString());
	setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);	
	setVisible(true);

	/**
	 * Class containing a thread in order to update the current Memory Info .
	 *
	 */       
	class updateMemoryInfo extends Thread 
	{
	    /**
	     * Start a thread for updating Memory Info every 1s.
	     *
	     */       
	    public void run() {		
		while(true)
		    {
			try {
			    Thread.sleep(Math.round(memoryTimer * 1000));
			}
			catch (Exception e)
			    {
			    }
			updateMemoryInfo();
		    }
	    }
	}

	final updateMemoryInfo mem = new updateMemoryInfo();
	Runtime r = Runtime.getRuntime();
	OperatingSystemMXBean bean = ManagementFactory.getOperatingSystemMXBean();	

   
	JPanel systemInfoBoxPanel = new JPanel(new BorderLayout());
	getContentPane().add(systemInfoBoxPanel, BorderLayout.CENTER);
	JLabel entryText;
			
	JPanel textPanel, textPanel2 ;
	textPanel = new JPanel();	
	GridBagLayout gbMain = new GridBagLayout();
	textPanel.setLayout(gbMain);
	systemInfoBoxPanel.add(textPanel, BorderLayout.WEST);
			
	textPanel2 = new JPanel();
	GridBagLayout gb = new GridBagLayout();
	textPanel2.setLayout(gb);
	textPanel2.setBorder(BorderFactory.createTitledBorder(Info.ressources.getObject("System").toString()));

	if (bean != null)
	    {		
		//  Returns the System architecture.
		entryText = new JLabel(Info.ressources.getObject("System_architecture").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel2.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,0,
									 new Insets(15,8,0,15)));

      
		entryText = new JLabel("= " + bean.getArch());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel2.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,0,
									 new Insets(15,8,0,15)));
	
		/*
		  entryText = new JLabel(Utils.ressources.getObject("CPU").toString());
		  entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		  textPanel2.add(entryText);
		  gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,1,
		  new Insets(0,8,0,15)));
	
		
		  entryText = new JLabel("= " + System.getProperty("sun.cpu.isalist"));
		  entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		  textPanel2.add(entryText);
		  gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,1,
		  new Insets(0,8,0,15)));		

		*/
		//  Returns the Operating system name
		entryText = new JLabel(Info.ressources.getObject("Operating_System_Name").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel2.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,2,
									 new Insets(0,8,0,15)));
	
	
		entryText = new JLabel("= " + bean.getName());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel2.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,2,
									 new Insets(0,8,0,15)));
	
		//  Returns the Operating system version
		entryText = new JLabel(Info.ressources.getObject("Operating_System_Version").toString());
		entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
		textPanel2.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,3,
									 new Insets(0,8,0,15)));
	
	
		entryText = new JLabel("= " + bean.getVersion());
		entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));
		textPanel2.add(entryText);
		gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,3,
									 new Insets(0,8,0,15)));		
	    }       

	//  Returns the number of processors available to the Java virtual machine.
	entryText = new JLabel(Info.ressources.getObject("Available_Processors").toString());
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,4,
								 new Insets(20,8,0,15)));
	
	entryText = new JLabel("= " + r.availableProcessors());
	entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));		
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,4,
								 new Insets(20,8,0,15)));
	  	
	
	/*
	  if (bean != null)
	  {
	  try {
	  //  Returns the System load average for the last minute (Java 6 only)
	  double i = bean.getSystemLoadAverage();
	  if (i > 0)
	  {
	  entryText = new JLabel(Utils.ressources.getObject("System_Load").toString());
	  entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	  textPanel2.add(entryText);
	  gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,5,
	  new Insets(0,8,0,15)));
	
			    
	  entryText = new JLabel("= " + i);
	  entryText.setFont(new Font("SansSerif", Font.PLAIN, 10));		
	  textPanel2.add(entryText);
	  gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,5,
	  new Insets(0,8,0,15)));
	  }
					    
	  }
	  catch (Exception e)
	  {
	  }
		
	  }
	*/

	textPanel.add(textPanel2);
	gbMain.setConstraints(textPanel2, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
								      new Insets(8,8,0,8)));

	textPanel2 = new JPanel();
	gb = new GridBagLayout();
	textPanel2.setLayout(gb);
	textPanel2.setBorder(BorderFactory.createTitledBorder(Info.ressources.getObject("Memory").toString()));

	// compute the different memory in MB
	chartDateStart = (new Date()).getTime() / 1000;		
	float totalMemory = r.totalMemory() / (1024 * 1024);
	float freeMemory = r.freeMemory() / (1024 * 1024);
	float maxMemory = r.maxMemory() / (1024 * 1024);
	float usedMemory = totalMemory - freeMemory;;
	
	// Returns the maximum amount of memory that the Java virtual machine will attempt to use.
	entryText = new JLabel(Info.ressources.getObject("Max_Memory").toString());
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,6,
								 new Insets(15,8,0,15)));
	
	maxMemoryJText = new JTextField(Float.valueOf((float)Math.round(maxMemory * 100) / 100).toString());
	maxMemoryJText.setFont(new Font("SansSerif", Font.PLAIN, 10));
	maxMemoryJText.setColumns(10);
	maxMemoryJText.setEditable(false);
	maxMemoryJText.setHorizontalAlignment(JTextField.RIGHT);
	textPanel2.add(maxMemoryJText);
	gb.setConstraints(maxMemoryJText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,6,
								      new Insets(15,8,0,15)));
	
	// Returns the total amount of memory in the Java virtual machine.	  	
	entryText = new JLabel(Info.ressources.getObject("Total_Memory").toString());
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,7,
								 new Insets(0,8,0,15)));
	
	totalMemoryJText = new JTextField(Float.valueOf((float)Math.round(totalMemory * 100) / 100).toString());
	totalMemoryJText.setFont(new Font("SansSerif", Font.PLAIN, 10));
	totalMemoryJText.setColumns(10);
	totalMemoryJText.setEditable(false);
	totalMemoryJText.setHorizontalAlignment(JTextField.RIGHT);
	textPanel2.add(totalMemoryJText);
	gb.setConstraints(totalMemoryJText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,7,
									new Insets(0,8,0,15)));
	
	// Returns the amount of free memory in the Java Virtual Machine.
	entryText = new JLabel(Info.ressources.getObject("Free_Memory").toString());
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,8,
								 new Insets(0,8,0,15)));
	
	freeMemoryJText = new JTextField(Float.valueOf((float)Math.round(freeMemory * 100) / 100).toString());
	freeMemoryJText.setFont(new Font("SansSerif", Font.PLAIN, 10));
	freeMemoryJText.setColumns(10);
	freeMemoryJText.setEditable(false);
	freeMemoryJText.setHorizontalAlignment(JTextField.RIGHT);
	textPanel2.add(freeMemoryJText);
	gb.setConstraints(freeMemoryJText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,8,
								       new Insets(0,8,0,15)));      
    			
	// Returns the amount of used memory in the Java Virtual Machine.	
	entryText = new JLabel(Info.ressources.getObject("Used_Memory").toString());
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,9,
								 new Insets(0,8,0,15)));
	
	
	usedMemoryJText = new JTextField(Float.valueOf((float)Math.round(usedMemory * 100) / 100).toString());
	usedMemoryJText.setFont(new Font("SansSerif", Font.PLAIN, 10));
	usedMemoryJText.setColumns(10);
	usedMemoryJText.setEditable(false);
	usedMemoryJText.setHorizontalAlignment(JTextField.RIGHT);
	textPanel2.add(usedMemoryJText);
	gb.setConstraints(usedMemoryJText, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,9,
								       new Insets(0,8,0,15)));
	
	// Set the output Unit
	entryText = new JLabel(Info.ressources.getObject("Output_Unit").toString());
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,10,
								 new Insets(0,8,0,15)));
	
	String [] outputUnitValues = {Info.ressources.getObject("Bytes").toString(), 
				      Info.ressources.getObject("KB").toString(), 
				      Info.ressources.getObject("MB").toString()};
	outputUnit = new JComboBox(outputUnitValues);
	outputUnit.setFont(new Font("SansSerif", Font.PLAIN, 10));
	outputUnit.setSelectedIndex(2);
	outputUnit.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    chartPanel.reconfigure(Info.ressources.getObject("Memory_Status").toString(),
					   (String)outputUnit.getSelectedItem(),
					   Info.ressources.getObject("Time").toString(), 
					   Info.ressources.getObject("Memory").toString() + " (" 
					   + (String)outputUnit.getSelectedItem() + ")", 120.0); 
		    chartPanel.maybeAddNewSeries("0", Info.ressources.getObject("Total_Memory2").toString() +": ", 
						 SeriesColorGenerator.nextColor());
		    chartPanel.maybeAddNewSeries("1", Info.ressources.getObject("Used_Memory2").toString() + ": ", 
						 SeriesColorGenerator.nextColor());
		    chartPanel.maybeAddNewSeries("2", Info.ressources.getObject("Free_Memory2").toString() + ": ", 
						 SeriesColorGenerator.nextColor());
		    chartPanel.start();	
		    chartDateStart = (new Date()).getTime() / 1000;
		    updateMemoryInfo();
		    pack();
		}
	    });

	textPanel2.add(outputUnit);
	gb.setConstraints(outputUnit, new GridBagLayoutConstraint(GridBagConstraints.WEST,2,10,
								  new Insets(0,8,0,15)));
	textPanel.add(textPanel2);
	gbMain.setConstraints(textPanel2, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,0,
								      new Insets(8,8,0,8)));
	textPanel2 = new JPanel();
	gb = new GridBagLayout();
	textPanel2.setLayout(gb);

	// Set the memory update Timer
	entryText = new JLabel(Info.ressources.getObject("Memory_Timer").toString());
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
								 new Insets(0,8,0,8)));
	
	SpinnerModel memoryTimerSpinnerModel = new SpinnerNumberModel(Float.valueOf(memoryTimer), //initial value
								      Float.valueOf(0.5f), //min
								      Float.valueOf(60.0f), //max
								      Float.valueOf(0.5f)); //step      
	final JSpinner memoryTimerSpinner = new JSpinner(memoryTimerSpinnerModel);
	((JSpinner.DefaultEditor)memoryTimerSpinner.getEditor()).getTextField().setColumns(3);
	memoryTimerSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent e) 
		{
		    memoryTimer = (Float)memoryTimerSpinner.getValue();
		}
	    }
	    );

	memoryTimerSpinner.setFont(new Font("SansSerif", Font.PLAIN, 10));
	textPanel2.add(memoryTimerSpinner);
	gb.setConstraints(memoryTimerSpinner, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,0,
									  new Insets(0,8,0,8)));

	// Set the threads update Timer
	entryText = new JLabel(Info.ressources.getObject("Threads_Timer").toString());
	entryText.setFont(new Font("SansSerif", Font.BOLD, 11));
	textPanel2.add(entryText);
	gb.setConstraints(entryText, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,1,
								 new Insets(0,8,0,8)));
	
	SpinnerModel threadsTimerSpinnerModel = new SpinnerNumberModel(Float.valueOf(threadsTimer), //initial value
								       Float.valueOf(0.5f), //min
								       Float.valueOf(3600.0f), //max
								       Float.valueOf(0.5f)); //step      
	final JSpinner threadsTimerSpinner = new JSpinner(threadsTimerSpinnerModel);
	((JSpinner.DefaultEditor)threadsTimerSpinner.getEditor()).getTextField().setColumns(3);
	threadsTimerSpinner.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent e) 
		{
		    threadsTimer = (Float)threadsTimerSpinner.getValue();
		}
	    }
	    );

	threadsTimerSpinner.setFont(new Font("SansSerif", Font.PLAIN, 10));
	textPanel2.add(threadsTimerSpinner);
	gb.setConstraints(threadsTimerSpinner, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,1,
									   new Insets(0,8,0,8)));
	
	textPanel.add(textPanel2);
	gbMain.setConstraints(textPanel2, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,1,
								      new Insets(8,8,0,8)));

	// Add the Memory Chart Panel
	chartPanel = new MemoryChartPanel(Info.ressources.getObject("Memory_Status").toString(), 
					  (String)outputUnit.getSelectedItem(), 
					  Info.ressources.getObject("Time").toString(), 
					  Info.ressources.getObject("Memory").toString() + " (" 
					  + (String)outputUnit.getSelectedItem() + ")",  
					  memoryTimer, 120.0,  
					  Color.black, Color.white, Color.gray);      
	chartPanel.maybeAddNewSeries("0", Info.ressources.getObject("Total_Memory2").toString() + ": ", 
				     SeriesColorGenerator.nextColor());
	chartPanel.maybeAddNewSeries("1", Info.ressources.getObject("Used_Memory2").toString() + ": ", 
				     SeriesColorGenerator.nextColor());
	chartPanel.maybeAddNewSeries("2", Info.ressources.getObject("Free_Memory2").toString() + ": ", 
				     SeriesColorGenerator.nextColor());
	chartPanel.start();		
	MemoryMeasurement M;

	M = new MemoryMeasurement(0, 0, 
				  Double.valueOf(r.totalMemory()/(1024*1024)), 
				  (String)outputUnit.getSelectedItem());
	chartPanel.addSeriesMemoryMeasurement("0", M);
	M = new MemoryMeasurement(0, 0, 
				  Double.valueOf((r.totalMemory() - r.freeMemory())/(1024*1024)), 
				  (String)outputUnit.getSelectedItem());
	chartPanel.addSeriesMemoryMeasurement("1", M);
	M = new MemoryMeasurement(0, 0, 
				  Double.valueOf(r.freeMemory()/(1024*1024)), 
				  (String)outputUnit.getSelectedItem());
	chartPanel.addSeriesMemoryMeasurement("2", M);
	
	textPanel.add(chartPanel);
	GridBagConstraints gbc = new GridBagLayoutConstraint(GridBagConstraints.WEST,0,2,
							     new Insets(15,8,0,8));
	gbc.gridwidth = 2;
	gbMain.setConstraints(chartPanel, gbc);

	textPanel2 = new JPanel();
	gb = new GridBagLayout();
	textPanel2.setLayout(gb);

	JButton threadsbutton = new JButton("Threads");
	textPanel2.add(threadsbutton);
	gb.setConstraints(threadsbutton, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
								     new Insets(0,0,0,2)));
	threadsbutton.addActionListener(new ActionListener() {	
		public void actionPerformed(ActionEvent e) {
		    showJavaThreadsInfo();
		}});	 
			
	
	// Garbage Collector call
	JButton gcbutton = new JButton("GC");
	textPanel2.add(gcbutton);
	gb.setConstraints(gcbutton, new GridBagLayoutConstraint(GridBagConstraints.WEST,1,0,
								new Insets(0,2,0,0)));
	gcbutton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Runtime r = Runtime.getRuntime();
		    r.gc();
		    updateMemoryInfo();
		}
	    });


	textPanel.add(textPanel2);
	gbMain.setConstraints(textPanel2, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,3,
								      new Insets(15,8,8,8)));

	JButton okbutton = new JButton(Info.ressources.getObject("OK").toString());
	textPanel.add(okbutton);
	gbMain.setConstraints(okbutton, new GridBagLayoutConstraint(GridBagConstraints.EAST,1,3,
								    new Insets(15,8,8,8)));
	okbutton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    mem.stop();
		    setVisible(false);
		    dispose();
		}
	    });
	 
	mem.start();

	pack();
	setResizable(false);
	setVisible(true);
    }



    /**
     * Update the current Memory Info.
     *
     */       
    private void updateMemoryInfo()
    {	
	int unit = outputUnit.getSelectedIndex();
	Runtime r = Runtime.getRuntime();		    
	float totalMemory = r.totalMemory();
	float freeMemory = r.freeMemory();
	float maxMemory = r.maxMemory();
	float usedMemory = 0;

	switch (unit)
	    {
	    case 0 : 
		// Already in bytes
		break;
		
	    case 1 : 
		// In KBytes
		totalMemory = totalMemory / 1024;	       
		freeMemory = freeMemory / 1024;
		maxMemory = maxMemory / 1024;
		break;

	    case 2 :
		// In MBytes
		totalMemory = totalMemory / (1024 * 1024);	       
		freeMemory = freeMemory / (1024 * 1024);
		maxMemory = maxMemory / (1024 * 1024);
		break;

	    default :
		// Leaves in bytes
		break;		
	    }

	totalMemoryJText.setText(Float.valueOf((float)Math.round(totalMemory * 100) / 100).toString());
	freeMemoryJText.setText(Float.valueOf((float)Math.round(freeMemory * 100) / 100).toString());
	maxMemoryJText.setText(Float.valueOf((float)Math.round(maxMemory * 100) / 100).toString());
	usedMemory = totalMemory - freeMemory;
	usedMemoryJText.setText(Float.valueOf((float)Math.round(usedMemory * 100) / 100).toString());

	long ts = (new Date()).getTime() / 1000;	
	MemoryMeasurement M;	 
	M = new MemoryMeasurement(ts - chartDateStart, ts - chartDateStart, totalMemory , (String)outputUnit.getSelectedItem());
	chartPanel.addSeriesMemoryMeasurement("0", M);
	M = new MemoryMeasurement(ts - chartDateStart, ts - chartDateStart, usedMemory, (String)outputUnit.getSelectedItem());
	chartPanel.addSeriesMemoryMeasurement("1", M);
	M = new MemoryMeasurement(ts - chartDateStart, ts - chartDateStart, freeMemory, (String)outputUnit.getSelectedItem());
	chartPanel.addSeriesMemoryMeasurement("2", M);   
    }

    /**
     * Draw a Frame with the Threads Info.
     *
     */      
    public void showJavaThreadsInfo()
    {
	showJavaThreadsInfo(new Dimension(650, 800));
    }

    /**
     * Draw a Frame with the Threads Info.
     * @param d the text Panel Dimension
     *
     */      
    public void showJavaThreadsInfo(Dimension d)
    {
	/**
	 * Class containing a thread in order to update the current Threads Info .
	 *
	 */  
	final Dimension PanelSize = d;
     
	class updateJavaThreadsInfo extends Thread 
	{
	    /**
	     * Start a thread for updating Threads Info every 1s.
	     *
	     */       
	    public void run() {		
		while(true)
		    {
			try {
			    Thread.sleep(Math.round(threadsTimer * 1000));
			}
			catch (Exception e)
			    {
			    }
			updateJavaThreadsInfo(PanelSize);
		    }
	    }
	}
	
	final updateJavaThreadsInfo threads = new updateJavaThreadsInfo();	
	final JFrame threadsFrame = new JFrame(Info.ressources.getObject("Threads_Info").toString());	
	threadsFrame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);	
	
	threadsTextPanel = new JPanel();
	JPanel threadsPanel = new JPanel();
	GridBagLayout gb = new GridBagLayout();
	threadsPanel.setLayout(gb);				
	updateJavaThreadsInfo(PanelSize);
	threadsPanel.add(threadsTextPanel);
	gb.setConstraints(threadsTextPanel, new GridBagLayoutConstraint(GridBagConstraints.WEST,0,0,
									new Insets(15,15,15,15)));
			
	JButton okbutton = new JButton(Info.ressources.getObject("OK").toString());
	threadsPanel.add(okbutton);
	gb.setConstraints(okbutton, new GridBagLayoutConstraint(GridBagConstraints.EAST,0,1,
								new Insets(15,8,8,8)));
	okbutton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    threads.stop();
		    threadsFrame.setVisible(false);
		    threadsFrame.dispose();
		}
	    });
	
	threadsFrame.getContentPane().add(threadsPanel);		
	threadsFrame.setResizable(false);
	threadsFrame.pack();
	threadsFrame.setVisible(true);	
	threads.start();
    }


    /**
     * Update the current Java threads Info.
     *
     */       
    protected void updateJavaThreadsInfo(Dimension d)    
    {	
	JTextArea text = new JTextArea();
	text.setFont(new Font("SansSerif", Font.PLAIN, 12));       
	ThreadMXBean bean = ManagementFactory.getThreadMXBean();  
	if (!bean.isThreadCpuTimeSupported()) {  
	    System.err.println("Sorry, the Java virtual machine implementation does not support CPU time measurement for any thread.");
	    return;
	}  
	if (!bean.isCurrentThreadCpuTimeSupported()) {  
	    System.err.println("Sorry, JVM does not supports CPU time measurement for the current thread");  
	    return;
	}  
	if (!bean.isThreadCpuTimeEnabled()) {  
	    System.err.println("Sorry, Thread CPU measurement is not enabled");  
	    return;
	}  
	
	String [] threadsTab = new String [bean.getThreadCount()];
	String textContent = "";
	int cpt = 0;	
	
	for (long id : bean.getAllThreadIds()) {  

	    String idth ="";
	    if(id < 100)
		idth += " ";
	    if(id < 10)
		idth += " "; 
	    idth += id;			
		    
	    ThreadInfo info = bean.getThreadInfo(id);         
		
	    threadsTab[cpt] = "   ID " + idth + ":\n     java thread name: " + info.getThreadName() 
		+ "\n     java thread state: " + info.getThreadState() 
		+ "\n     Thread CPU Time : "+ bean.getThreadCpuTime(id)
		+ "\n     Thread peak time: "+bean.getPeakThreadCount()
		+"\n";
	    
	    cpt ++;
	}
	
	JScrollPane textScrolled = new JScrollPane(text);		
	textScrolled.setPreferredSize(d);   	
	Arrays.sort(threadsTab);
	for(int i=0; i < cpt; i++)
	    {
		textContent += threadsTab[i] + "\n";
	    }
	
	text.append(textContent);
	text.setCaretPosition(0);

	//remove all components in panel.
	threadsTextPanel.removeAll();
	threadsTextPanel.setPreferredSize(d);
	// refresh the panel.		
	threadsTextPanel.add(textScrolled);	
	threadsTextPanel.updateUI(); 
    }    
       
}